@@ -4,7 +4,7 @@ use strict;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '3.008'; # $Id: //depot/Tk8/DragDrop/DragDrop/Common.pm#8$
+$VERSION = '4.002'; # $Id: //depot/Tkutf8/DragDrop/DragDrop/Common.pm#2 $
 
 sub Type
 {
@@ -15,7 +15,7 @@ sub Type
  unless (exists $hash->{$name})
   {
    push(@$array,$name);
-   $class = (caller(0))[0] unless (@_ > 2); 
+   $class = (caller(0))[0] unless (@_ > 2);
    $hash->{$name} = $class;
    # confess "Strange class $class for $base/$name" unless ($class =~ /^Tk/);
    # print "$base $name is ",$class,"\n";
@@ -39,12 +39,11 @@ sub import
      else
       {
        my ($kind) = $class =~ /([A-Z][a-z]+)$/;
-       # print "$type $class is kind of '$kind'\n";
        my $file = Tk->findINC("DragDrop/${type}${kind}.pm");
        if (defined $file)
         {
-         # print "Loading $file\n"; 
-         require $file; 
+         # print "Loading $file\n";
+         require $file;
         }
        else
         {
@@ -58,14 +57,3 @@ sub import
 1;
 __END__
 
-=head1 NAME
-
-Tk::DragDrop::Common - private class used by Drag&Drop
-
-=for category User Interaction
-
-=head1 DESCRIPTION
-
-This class provides methods to automate the the loading and declaring 
-of Drop and Site 'types'.
-
