@@ -1,7 +1,7 @@
 package Tk::HList;
 
 use vars qw($VERSION);
-$VERSION = '3.034'; # $Id: //depot/Tk8/HList/HList.pm#34 $
+$VERSION = '4.008'; # $Id: //depot/Tkutf8/HList/HList.pm#8 $
 
 use Tk qw(Ev $XS_VERSION);
 
@@ -50,6 +50,7 @@ sub ClassInit
  $mw->bind($class,'<Shift-ButtonPress-1>',[ 'ShiftButton1' ] );
  $mw->bind($class,'<Control-ButtonRelease-1>','Control_ButtonRelease_1');
  $mw->bind($class,'<ButtonRelease-1>','ButtonRelease_1');
+ $mw->bind($class,'<Double-ButtonRelease-1>','NoOp');
  $mw->bind($class,'<B1-Motion>',[ 'Button1Motion' ] );
  $mw->bind($class,'<B1-Leave>',[ 'AutoScan' ] );
 
@@ -77,6 +78,9 @@ sub ClassInit
  $mw->bind($class,'<Home>',   ['KeyboardHome']);
  $mw->bind($class,'<End>',    ['KeyboardEnd']);
 
+ $mw->YMouseWheelBind($class);
+ $mw->XMouseWheelBind($class);
+
  return $class;
 }
 
@@ -263,7 +267,7 @@ sub ButtonRelease1
   }
  return unless (defined($ent) and length($ent));
 
- if(exists $w->{tixindicator})
+ if (exists $w->{tixindicator})
   {
    return unless delete($w->{tixindicator}) eq $ent;
    my @info = $w->info('item',$Ev->x, $Ev->y);
@@ -271,6 +275,10 @@ sub ButtonRelease1
     {
      $w->Callback(-indicatorcmd => $ent, '<Activate>');
     }
+   else
+    {
+     $w->Callback(-indicatorcmd => $ent, '<Disarm>');
+    }
    return;
   }
 
@@ -368,7 +376,7 @@ sub Double1
 	unless(defined $w->info('anchor'));
 
  $w->selectionSet($ent);
- 
+
  $w->Callback(-command => $ent);
 }
 
@@ -579,7 +587,7 @@ sub KeyboardActivate
 {
  my $w = shift;
 
- my $anchor = $w->info('anchor'); 
+ my $anchor = $w->info('anchor');
 
  return unless (defined($anchor) and length($anchor));
 
@@ -614,7 +622,7 @@ sub KeyboardBrowse
 }
 
 sub AutoScan
-{                                                  
+{
  my ($w,$x,$y) = @_;
 
  return if ($w->cget('-selectmode') eq 'dragdrop');
@@ -661,3 +669,4 @@ sub children
 
 1;
 
+
