@@ -1,9 +1,9 @@
 package Tk::TList;
 
 use vars qw($VERSION);
-$VERSION = '3.012'; # $Id: //depot/Tk8/TList/TList.pm#12$
+$VERSION = '4.004'; # $Id: //depot/Tkutf8/TList/TList.pm#4 $
 
-use Tk qw(Ev);
+use Tk qw(Ev $XS_VERSION);
 
 use base  qw(Tk::Widget);
 
@@ -11,11 +11,11 @@ use strict;
 
 Construct Tk::Widget 'TList';
 
-bootstrap Tk::TList $Tk::VERSION;
+bootstrap Tk::TList;
 
 sub Tk_cmd { \&Tk::tlist }
 
-Tk::Methods qw(insert anchor delete dragsite dropsite entrycget
+Tk::Methods qw(insert index anchor delete dragsite dropsite entrycget
                entryconfigure info nearest see selection xview yview);
 
 use Tk::Submethods ( 'delete' => [qw(all entry offsprings siblings)],
