@@ -5,7 +5,7 @@
 package Tk::Balloon;
 
 use vars qw($VERSION);
-$VERSION = '3.035'; # $Id: //depot/Tk8/Tixish/Balloon.pm#35 $
+$VERSION = '4.002'; # $Id: //depot/Tkutf8/Tixish/Balloon.pm#2 $
 
 use Tk qw(Ev Exists);
 use Carp;
@@ -108,7 +108,7 @@ sub detach {
       $w->Deactivate if ($client->IS($w->{'client'}));
      }
     delete $w->{'clients'}{$client};
-}                                    
+}
 
 sub GetOption
 {
@@ -116,11 +116,11 @@ sub GetOption
  $client = $w->{'client'} unless defined $client;
  if (defined $client)
   {
-   my $info = $w->{'clients'}{$client};         
+   my $info = $w->{'clients'}{$client};
    return $info->{$opt} if exists $info->{$opt};
   }
  return $w->cget($opt);
-} 
+}
 
 sub Motion {
     my ($ewin, $x, $y, $s) = @_;
@@ -154,20 +154,21 @@ sub Motion {
 	    }
 	    # Deactivate it if the motioncommand says to:
             my $command = $w->GetOption(-motioncommand => $client);
-	    $deactivate = $command->Call if defined $command;
+	    $deactivate = $command->Call($client, $x, $y) if defined $command;
             if ($deactivate)
              {
               $w->Deactivate;
              }
             else
              {
+              # warn "deact: $client $w->{'client'}";
               $w->Deactivate unless $client->IS($w->{'client'});
-              my $msg = $client->BalloonInfo($w,$x,$y,'-statusmsg','-balloonmsg');    
+              my $msg = $client->BalloonInfo($w,$x,$y,'-statusmsg','-balloonmsg');
               if (defined($msg))
-               {   
+               {
                 my $delay = delete $w->{'delay'};
                 $delay->cancel if defined $delay;
-                my $initwait = $w->GetOption(-initwait => $client);                   
+                my $initwait = $w->GetOption(-initwait => $client);
                 $w->{'delay'} = $client->after($initwait, sub {$w->SwitchToClient($client);});
                 $w->{'client'} = $client;
                }
@@ -202,7 +203,7 @@ sub SwitchToClient {
     my $command = $w->GetOption(-postcommand => $client);
     if (defined $command) {
 	# Execute the user's command and return if it returns false:
-	my $pos = $command->Call;
+	my $pos = $command->Call($client);
 	return if not $pos;
 	if ($pos =~ /^(\d+),(\d+)$/) {
 	    # Save the returned position so the Popup method can use it:
@@ -222,17 +223,19 @@ sub Subclient
  if (defined($w->{'subclient'}) && (!defined($data) || $w->{'subclient'} ne $data))
   {
    $w->Deactivate;
-  } 
+  }
  $w->{'subclient'} = $data;
 }
 
-sub Verify {                
+sub Verify {
     my $w      = shift;
     my $client = shift;
     my ($X,$Y) = (@_) ? @_ : ($w->pointerxy);
     my $over = $w->Containing($X,$Y);
     return if not defined $over or ($over->toplevel eq $w);
-    my $deactivate = ($over ne $client) or not $client->IS($w->{'client'}) 
+    my $deactivate = # DELETE? or move it to the isa-Menu section?:
+	             # ($over ne $client) or
+	             not $client->IS($w->{'client'})
                      or (!$client->isa('Tk::Menu') && $w->grabCurrent);
     if ($deactivate)
      {
@@ -244,7 +247,7 @@ sub Verify {
      }
 }
 
-sub Deactivate {                 
+sub Deactivate {
     my ($w) = @_;
     my $delay = delete $w->{'delay'};
     $delay->cancel if defined $delay;
@@ -253,7 +256,7 @@ sub Deactivate {
 	my $command = $w->GetOption(-cancelcommand => $client);
 	if (defined $command) {
 	    # Execute the user's command and return if it returns false:
-	    return if not $command->Call;
+	    return if not $command->Call($client);
 	}
 	$w->withdraw;
 	$w->ClearStatus;
@@ -313,7 +316,7 @@ sub Popup {
     $w->deiconify();
     $w->raise;
     #$w->update;  # This can cause confusion by processing more Motion events before this one has finished.
-}                                           
+}
 
 sub SetStatus {
     my ($w) = @_;
