@@ -1,7 +1,7 @@
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994 Sun Microsystems, Inc.
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 
 # modify it under the same terms as Perl itself, subject
@@ -9,7 +9,7 @@
 # derivation from Tk8.0 sources.
 #
 package Tk;
-require 5.00404;
+require 5.007;
 use     Tk::Event ();
 use     AutoLoader qw(AUTOLOAD);
 use     DynaLoader;
@@ -17,10 +17,32 @@ use base qw(Exporter DynaLoader);
 
 *fileevent = \&Tk::Event::IO::fileevent;
 
-BEGIN { $Tk::platform = ($^O eq 'MSWin32') ? $^O : 'unix' };
+use Encode;
+$Tk::encode_fallback = Encode::FB_QUIET();
+
+our %font_encoding = ('jis0208' => 'jis0208-raw',
+                      'jis0212' => 'jis0212-raw',
+                      'ksc5601' => 'ksc5601-raw',
+                      'gb2312'  => 'gb2312-raw',
+                      'unicode' => 'ucs-2le',
+                     );
+
+BEGIN {
+ if($^O eq 'cygwin')
+  {
+   require Tk::Config;
+   $Tk::platform = $Tk::Config::win_arch;
+   $Tk::platform = 'unix' if $Tk::platform eq 'x';
+  }
+ else
+  {
+   $Tk::platform = ($^O eq 'MSWin32') ? $^O : 'unix';
+  }
+};
 
 $Tk::tearoff = 1 if ($Tk::platform eq 'unix');
 
+
 @EXPORT    = qw(Exists Ev exit MainLoop DoOneEvent tkinit);
 @EXPORT_OK = qw(NoOp after *widget *event lsearch catch $XS_VERSION
                 DONT_WAIT WINDOW_EVENTS  FILE_EVENTS TIMER_EVENTS
@@ -40,14 +62,14 @@ use Carp;
 
 # $tk_version and $tk_patchLevel are reset by pTk when a mainwindow
 # is created, $VERSION is checked by bootstrap
-$Tk::version     = '8.0';
-$Tk::patchLevel  = '8.0';
-$Tk::VERSION     = '800.021';
+$Tk::version     = '8.4';
+$Tk::patchLevel  = '8.4';
+$Tk::VERSION     = '804.024';
 $Tk::XS_VERSION  = $Tk::VERSION;
 $Tk::strictMotif = 0;
 
 {($Tk::library) = __FILE__ =~ /^(.*)\.pm$/;}
-$Tk::library = Tk->findINC('.') unless (-d $Tk::library);
+$Tk::library = Tk->findINC('.') unless (defined($Tk::library) && -d $Tk::library);
 
 $Tk::widget  = undef;
 $Tk::event   = undef;
@@ -218,6 +240,23 @@ sub Methods
   }
 }
 
+my %dialog = ( tk_chooseColor => 'ColorDialog',
+               tk_messageBox  => 'MessageBox',
+               tk_getOpenFile => 'FDialog',
+               tk_getSaveFile => 'FDialog',
+               tk_chooseDirectory => 'FDialog'
+             );
+
+foreach my $dialog (keys %dialog)
+ {
+  no strict 'refs';
+  unless (defined &$dialog)
+   {
+    my $kind = $dialog;
+    my $code = \&{"Tk::$dialog{$dialog}"};
+    *$dialog = sub { &$code($kind,@_) };
+   }
+ }
 
 sub MessageBox {
     my ($kind,%args) = @_;
@@ -356,6 +395,47 @@ sub TranslateFileName
  return $_;
 }
 
+sub findINC
+{
+ my $file = join('/',@_);
+ my $dir;
+ $file  =~ s,::,/,g;
+ foreach $dir (@INC)
+  {
+   my $path;
+   return $path if (-e ($path = "$dir/$file"));
+  }
+ return undef;
+}
+
+sub idletasks
+{
+ shift->update('idletasks');
+}
+
+sub backtrace
+{
+ my ($self,$msg) = @_;
+ my $i = 1;
+ my ($pack,$file,$line,$sub) = caller($i++);
+ while (1)
+  {
+   my $loc = "at $file line $line";
+   ($pack,$file,$line,$sub) = caller($i++);
+   last unless defined($sub);
+   $msg .= " $sub $loc\n";
+  }
+ return $msg;
+}
+
+sub die_with_trace
+{
+ my ($self,$msg) = @_;
+ die $self->backtrace($msg);
+}
+
+
+
 1;
 
 __END__
@@ -605,11 +685,13 @@ sub Selection
  croak "Use Selection\u$cmd()";
 }
 
-sub Clipboard
-{my $w = shift;
- my $cmd    = shift;
- croak "Use clipboard\u$cmd()";
-}
+# If we have sub Clipboard in Tk then use base qw(Tk::Clipboard ....)
+# calls it when it does its eval "require $base"
+#sub Clipboard
+#{my $w = shift;
+# my $cmd    = shift;
+# croak "Use clipboard\u$cmd()";
+#}
 
 sub Receive
 {
@@ -623,11 +705,6 @@ sub break
  die "_TK_BREAK_\n";
 }
 
-sub idletasks
-{
- shift->update('idletasks');
-}
-
 sub updateWidgets
 {
  my ($w) = @_;
@@ -653,19 +730,6 @@ sub interps
  return $w->winfo('interps','-displayof');
 }
 
-sub findINC
-{
- my $file = join('/',@_);
- my $dir;
- $file  =~ s,::,/,g;
- foreach $dir (@INC)
-  {
-   my $path;
-   return $path if (-e ($path = "$dir/$file"));
-  }
- return undef;
-}
-
 sub lsearch
 {my $ar = shift;
  my $x  = shift;
@@ -678,5 +742,40 @@ sub lsearch
 }
 
 
+sub getEncoding
+{
+ my ($class,$name) = @_;
+ eval { require Encode };
+ if ($@)
+  {
+   require Tk::DummyEncode;
+   return Tk::DummyEncode->getEncoding($name);
+  }
+ $name = $Tk::font_encoding{$name} if exists $Tk::font_encoding{$name};
+ my $enc = Encode::find_encoding($name);
+
+ unless ($enc)
+  {
+   $enc = Encode::find_encoding($name) if ($name =~ s/[-_]\d+$//)
+  }
+# if ($enc)
+#  {
+#   print STDERR "Lookup '$name' => ".$enc->name."\n";
+#  }
+# else
+#  {
+#   print STDERR "Failed '$name'\n";
+#  }
+ unless ($enc)
+  {
+   if ($name eq 'X11ControlChars')
+    {
+     require Tk::DummyEncode;
+     $Encode::encoding{$name} = $enc = Tk::DummyEncode->getEncoding($name);
+    }
+  }
+ return $enc;
+}
+
 
 
