@@ -2,7 +2,7 @@ package Tk::ColorSelect;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.034'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#34 $
+$VERSION = '4.004'; # $Id: //depot/Tkutf8/Tk/ColorEditor.pm#4 $
 
 use Tk qw(Ev);
 
@@ -442,7 +442,7 @@ sub Show
 package Tk::ColorEditor;
 
 use vars qw($VERSION $SET_PALETTE);
-$VERSION = '3.034'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#34 $
+$VERSION = '4.004'; # $Id: //depot/Tkutf8/Tk/ColorEditor.pm#4 $
 
 use Tk qw(lsearch Ev);
 use Tk::Toplevel;
@@ -658,7 +658,6 @@ sub Populate
             sub {
                 my ($objref) = @_;
                 $objref->Callback(-command => ($objref->{'highlight'}, $objref->cget('-color')));
-		$cw->{'done'} = 1;
             }, $cw,
         ],
     );
@@ -712,9 +711,9 @@ sub Populate
 
 sub Show {
 
-    my($objref, @args) = @_;
+    my($objref) = @_;
 
-    Tk::ColorDialog::Show(@_);
+    $objref->deiconify;
 
 } # end show
 
