@@ -1,7 +1,7 @@
 package Tk::FileSelect;
 
 use vars qw($VERSION @EXPORT_OK);
-$VERSION = '3.045'; # $Id: //depot/Tk8/Tk/FileSelect.pm#45 $
+$VERSION = '4.010'; # $Id: //depot/Tkutf8/Tk/FileSelect.pm#10 $
 @EXPORT_OK = qw(glob_to_re);
 
 use Tk qw(Ev);
@@ -237,7 +237,7 @@ sub Populate {
         -filelistlabel    => [ 'PASSIVE', undef, undef, 'Files' ],
         -filter           => [ 'METHOD',  undef, undef, undef ],
         -defaultextension => [ 'METHOD',  undef, undef, undef ],
-        -regexp           => [ 'PASSIVE', undef, undef, undef ],
+        -regexp           => [ 'METHOD', undef, undef, undef ],
         -dirlistlabel     => [ 'PASSIVE', undef, undef, 'Directories'],
         -dirlabel         => [ 'PASSIVE', undef, undef, 'Directory'],
         '-accept'         => [ 'CALLBACK',undef,undef, undef ],
@@ -284,7 +284,24 @@ sub filter
   {
    $val = '*' unless defined $val;
    $$var = $val;
-   $cw->{'match'} = glob_to_re($val);
+   $cw->{'match'} = glob_to_re($val)  unless defined $cw->{'match'};
+   unless ($cw->{'reread'}++)
+    {
+     $cw->Busy;
+     $cw->afterIdle(['reread',$cw,$cw->cget('-directory')])
+    }
+  }
+ return $$var;
+}
+
+sub regexp
+{
+ my ($cw,$val) = @_;
+ my $var = \$cw->{Configure}{'-regexp'};
+ if (@_ > 1)
+  {
+   $$var = $val;
+   $cw->{'match'} = (defined $val) ? sub { shift =~ m|^${val}$| } : sub { 1 };
    unless ($cw->{'reread'}++)
     {
      $cw->Busy;
@@ -299,6 +316,7 @@ sub defaultextension
  my ($cw,$val) = @_;
  if (@_ > 1)
   {
+   $val = '' unless defined $val;
    $val = ".$val" if ($val !~ /^\./);
    $cw->filter("*$val");
   }
@@ -320,20 +338,37 @@ sub directory
     {
      if (substr($dir,1,1) eq '/')
       {
-       $dir = $ENV{'HOME'} . substr($dir,1);
+       $dir = (defined $ENV{'HOME'} ? $ENV{'HOME'} : '') . substr($dir,1);
       }
      else
       {my ($uid,$rest) = ($dir =~ m#^~([^/]+)(/.*$)#);
        $dir = (getpwnam($uid))[7] . $rest;
       }
     }
+   my $revert_dir = sub
+    {
+     my $message = shift;
+     $$var = $cw->{OldDirectory};
+     $cw->messageBox(-message => $message, -icon => 'error');
+     if (!defined $$var)
+      {
+       # OldDirectory was never set, so force reread...
+       $$var = $cw->{OldDirectory} = Cwd::getcwd(); # XXX maybe use check like code below...
+       unless ($cw->{'reread'}++)
+        {
+         $cw->Busy;
+         $cw->afterIdle(['reread',$cw])
+        }
+      }
+     $$var;
+    };
    $dir =~ s#([^/\\])[\\/]+$#$1#;
    if (-d $dir)
     {
      unless (Tk::tainting())
       {
        my $pwd = Cwd::getcwd();
-       if (chdir($dir))
+       if (chdir( (defined($dir) ? $dir : '') ) )
         {
          my $new = Cwd::getcwd();
          if ($new)
@@ -342,17 +377,20 @@ sub directory
           }
          else
           {
-           carp "Cannot getcwd in '$dir'";
+	   return $revert_dir->("Cannot getcwd in '$dir'");
           }
-         chdir($pwd) || carp "Cannot chdir($pwd) : $!";
-         $cw->{Configure}{'-directory'} = $dir;
+         if (!chdir($pwd))
+          {
+	   return $revert_dir->("Cannot change directory to $pwd:\n$!");
+          }
+	 $$var = $dir;
         }
        else
         {
-         $cw->BackTrace("Cannot chdir($dir) :$!");
+	 return $revert_dir->("Cannot change directory to $dir:\n$!");
         }
+       $$var = $cw->{OldDirectory} = $dir;
       }
-     $$var = $dir;
      unless ($cw->{'reread'}++)
       {
        $cw->Busy;
@@ -375,6 +413,7 @@ sub reread
     }
    my $dl = $w->Subwidget('dir_list');
    $dl->delete(0, 'end');
+   $dl->selectionClear(0,'end');
    my $fl = $w->Subwidget('file_list');
    $fl->delete(0, 'end');
    local *DIR;
@@ -414,10 +453,10 @@ sub reread
        $w->configure(-initialfile => undef) unless $w->cget('-create');
       }
     }
-   $w->{DirectoryString} = $dir . '/' . $w->cget('-filter');
+   $w->{DirectoryString} = $dir . ($dir ne '/' ? '/' : '') . $w->cget('-filter');
   }
  $w->{'reread'} = 0;
- $w->Unbusy;
+ $w->Unbusy if $w->{'Busy'};
 }
 
 sub validateDir
