@@ -1,12 +1,13 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Optionmenu;
 require Tk::Menubutton;
 require Tk::Menu;
+use Carp;
 
 use vars qw($VERSION);
-$VERSION = '3.020'; # $Id: //depot/Tk8/Tk/Optionmenu.pm#20 $
+$VERSION = '4.009'; # $Id: //depot/Tkutf8/Tk/Optionmenu.pm#9 $
 
 use base  qw(Tk::Derived Tk::Menubutton);
 
@@ -19,18 +20,7 @@ sub Populate
  my ($w,$args) = @_;
  $w->SUPER::Populate($args);
  $args->{-indicatoron} = 1;
- my $var = delete $args->{-textvariable};
- unless (defined $var && exists $args->{-variable})
-  {
-   $var = $args->{-variable};
-  }
- unless (defined $var)
-  {
-   my $gen = undef;
-   $var = \$gen;
-  }
  my $menu = $w->menu(-tearoff => 0);
- $w->configure(-textvariable => $var);
 
  # Should we allow -menubackground etc. as in -label* of Frame ?
 
@@ -45,38 +35,74 @@ sub Populate
 
                 );
 
- $w->configure(-variable => delete $args->{-variable});
+ # configure -variable and -command now so that when -options
+ # is set by main-line configure they are there to be set/called.
+
+ my $tvar = delete $args->{-textvariable};
+ my $vvar = delete $args->{-variable};
+ if (!defined($vvar))
+  {
+   if (defined $tvar)
+    {
+     $vvar = $tvar;
+    }
+   else
+    {
+     my $new;
+     $vvar = \$new;
+    }
+  }
+ $tvar = $vvar if (!defined($tvar));
+ $w->configure(-textvariable => $tvar, -variable => $vvar);
+ $w->configure(-command  => $vvar) if ($vvar = delete $args->{-command});
 }
 
 sub setOption
 {
  my ($w, $label, $val) = @_;
- $val = $label if @_ == 2;
- my $var = $w->cget(-textvariable);
- $$var = $label;
- $var = $w->cget(-variable);
- $$var = $val if $var;
+ my $tvar = $w->cget(-textvariable);
+ my $vvar = $w->cget(-variable);
+ if (@_ == 2)
+  {
+   $val = $label;
+  }
+ $$tvar = $label if $tvar;
+ $$vvar = $val   if $vvar;
  $w->Callback(-command => $val);
 }
 
 sub addOptions
 {
- my $w = shift;
- my $menu = $w->menu;
- my $var = $w->cget(-textvariable);
+ my $w     = shift;
+ my $menu  = $w->menu;
+ my $tvar  = $w->cget(-textvariable);
+ my $vvar  = $w->cget(-variable);
+ my $oldt  = $$tvar;
  my $width = $w->cget('-width');
+ my %hash;
+ my $first;
  while (@_)
   {
    my $val = shift;
    my $label = $val;
    if (ref $val)
     {
+     if ($vvar == $tvar)
+      {
+       my $new = $label;
+       $w->configure(-textvariable => ($tvar = \$new));
+      }
      ($label, $val) = @$val;
     }
    my $len = length($label);
    $width = $len if (!defined($width) || $len > $width);
    $menu->command(-label => $label, -command => [ $w , 'setOption', $label, $val ]);
-   $w->setOption($label, $val) unless (defined $$var);
+   $hash{$label} = $val;
+   $first = $label unless defined $first;
+  }
+ if (!defined($oldt) || !exists($hash{$oldt}))
+  {
+   $w->setOption($first, $hash{$first}) if defined $first;
   }
  $w->configure('-width' => $width);
 }
