@@ -1,14 +1,14 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
-require Tk;
 package Tk::ROText;
-require Tk::Text;
 
 use vars qw($VERSION);
-$VERSION = '3.023'; # $Id: //depot/Tk8/Tk/ROText.pm#23 $
+$VERSION = '4.005'; # $Id: //depot/Tkutf8/Tk/ROText.pm#5 $
 
+use Tk::Text;
 use base  qw(Tk::Text);
+
 Construct Tk::Widget 'ROText';
 
 sub clipEvents
