@@ -1,4 +1,4 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons.
+# Copyright (c) 1995-2000 Nick Ing-Simmons.
 # Copyright (c) 1999 Greg London.
 # All rights reserved.
 # This program is free software; you can redistribute it and/or
@@ -6,12 +6,13 @@
 package Tk::TextUndo;
 
 use vars qw($VERSION $DoDebug);
-$VERSION = '3.047'; # $Id: //depot/Tk8/Tk/TextUndo.pm#47 $
+$VERSION = '4.009'; # $Id: //depot/Tkutf8/Tk/TextUndo.pm#9 $
 $DoDebug = 0;
 
 use Tk qw (Ev);
 use AutoLoader;
 
+use Tk::Text ();
 use base qw(Tk::Text);
 
 Construct Tk::Widget 'TextUndo';
@@ -172,7 +173,7 @@ sub CheckForRedoShuffle
  while ($w->SizeRedo)
   {
    my $ref = $w->ShiftRedo;
-   $w->PushUndo($ref);		
+   $w->PushUndo($ref);
   }
 
  # Finally undo whatever we did to compensate for doing it
@@ -850,6 +851,12 @@ sub CreateFileSelect
    require File::Basename;
    my $sfx;
    ($name,$dir,$sfx) = File::Basename::fileparse($name,'\..*');
+   #
+   # it should never happen where we have a file suffix and
+   # no file name... but fileparse() screws this up with dotfiles.
+   #
+   if (length($sfx) && !length($name)) { ($name, $sfx) = ($sfx, $name) }
+
    if (defined($sfx) && length($sfx))
     {
      unshift(@types,['Similar Files',[$sfx]]);
