@@ -1,4 +1,4 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Wm;
@@ -14,17 +14,17 @@ use strict qw(vars);
 
 
 use vars qw($VERSION);
-$VERSION = '3.018'; # $Id: //depot/Tk8/Tk/Wm.pm#18$
+$VERSION = '4.008'; # $Id: //depot/Tkutf8/Tk/Wm.pm#8 $
 
 use Tk::Submethods ( 'wm' => [qw(grid tracing)] );
 
 Direct Tk::Submethods ('wm' => [qw(aspect client colormapwindows command
                        deiconify focusmodel frame geometry group
-                       iconbitmap iconify iconmask iconname
+                       iconbitmap iconify iconimage iconmask iconname
                        iconwindow maxsize minsize overrideredirect positionfrom
-                       protocol resizable saveunder sizefrom state title transient
+                       protocol resizable sizefrom state title transient
                        withdraw wrapper)]);
-
+		
 sub SetBindtags
 {
  my ($obj) = @_;
@@ -49,6 +49,20 @@ sub MoveResizeWindow
  $w->deiconify;
 }
 
+sub WmDeleteWindow
+{
+ my ($w) = @_;
+ my $cb  = $w->protocol('WM_DELETE_WINDOW');
+ if (defined $cb)
+  {
+   $cb->Call;
+  }
+ else
+  {
+   $w->destroy;
+  }
+}
+
 
 1;
 
@@ -109,7 +123,11 @@ sub Popup
   }
  my ($X,$Y) = AnchorAdjust($w->cget('-overanchor'),$rx,$ry,$rw,$rh);
  ($X,$Y)    = AnchorAdjust($w->cget('-popanchor'),$X,$Y,-$mw,-$mh);
+ # adjust to not cross screen borders
+ if ($mw > $w->screenwidth)  { $X = 0 }
+ if ($mh > $w->screenheight) { $Y = 0 }
  $w->Post($X,$Y);
+ $w->waitVisibility;
 }
 
 sub FullScreen
@@ -148,3 +166,4 @@ sub iconposition
   }
  $w->wm('iconposition',@_);
 }
+
