@@ -1,10 +1,10 @@
 package Tk::install;
 require Exporter;
 
-use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '3.004'; # $Id: //depot/Tk8/Tk/install.pm#4$
+use vars qw($VERSION @EXPORT);
+$VERSION = '4.002'; # $Id: //depot/Tkutf8/Tk/install.pm#2 $
 
-@ISA = qw(Exporter);
+use base  qw(Exporter);
 @EXPORT = qw(installbin);
 
 use Config;
@@ -13,17 +13,18 @@ sub installbin
 {
  my $prog  = shift(@ARGV);
  my $start = $Config{'startperl'};
- my $perl  = $Config{'perl'};
+ my $perl  = $Config{'perl'} || 'perl';
  $start =~ s/$perl$/$prog/;
  while (($src,$dst) = splice(@ARGV,0,2))
   {
    open(SRC,"<$src") || die "Cannot open $src:$!";
    my $line = <SRC>;
    $line =~ s/^#!\s*\S+/$start/;
+   warn $line;
    chmod(0755,$dst) if (-f $dst);
    open(DST,">$dst") || die "Cannot open $dst:$!";
    print "installbin $src => $dst\n";
-   do 
+   do
     {
      print DST $line;
     } while (defined($line = <SRC>));
