@@ -2,16 +2,16 @@ package Tk::X;
 
 use strict;
 use Carp;
-use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);
-$VERSION = '3.010'; # $Id: //depot/Tk8/Xlib/X/X.pm#10$
-require Tk;
+use vars qw($VERSION @EXPORT $AUTOLOAD);
+$VERSION = '4.002'; # $Id: //depot/Tkutf8/Xlib/X/X.pm#2 $
+use Tk qw($XS_VERSION);
 
 require Exporter;
 require DynaLoader;
 require AutoLoader;
 
 
-@ISA = qw(Exporter DynaLoader);
+use base  qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
@@ -385,7 +385,7 @@ sub AUTOLOAD {
     goto &$AUTOLOAD;
 }
 
-bootstrap Tk::X $Tk::VERSION;
+bootstrap Tk::X;
 
 # Preloaded methods go here.
 
