@@ -1,9 +1,8 @@
 # floor.pl
-use strict;
 
 use subs qw/floor_bg1 floor_bg2 floor_bg3 floor_display floor_fg1 floor_fg2
 	    floor_fg3 floor_room_changed/;
-use vars qw/$MW $TOP/;
+use vars qw/$TOP/;
 
 sub floor {
 
@@ -11,13 +10,12 @@ sub floor {
     # floorplan for DEC's Western Research Laboratory.
 
     my($demo) = @_;
-    my $demo_widget = $MW->WidgetDemo(
+    $TOP = $MW->WidgetDemo(
         -name     => $demo,
         -text     => ['This window contains a canvas widget showing the floorplan of Digital Equipment Corporation\'s Western Research Laboratory.  It has three levels.  At any given time one of the levels is active, meaning that you can see its room structure.  To activate a level, click the left mouse button anywhere on it.  As the mouse moves over the active level, the room under the mouse lights up and its room number appears in the "Room:" entry.  You can also type a room number in the entry and the room will light up.', qw/-wraplength 8i/],
         -title    => 'Floorplan Canvas Demonstration',
         -iconname => 'floor',
     );
-    $TOP = $demo_widget->Top;	# get geometry master
 
     my $c = $TOP->Scrolled(qw/Canvas -width 900 -height 500 -relief sunken
 			   -borderwidth 2 -scrollbars se/);
@@ -63,7 +61,7 @@ sub floor {
     my $floor_number;
     for $floor_number (1..3) {
 	$c->bind("floor${floor_number}", '<1>' =>
-            [\&floor_display, $floor_number, \%floor_labels, \%floor_items, 
+            [\&floor_display, $floor_number, \%floor_labels, \%floor_items,
 	    \%cinfo, \$active_floor, $c_entry],
         );
     }
@@ -126,7 +124,7 @@ sub floor_display {
     # Add the walls and labels for the active floor, along with transparent
     # polygons that define the rooms on the floor.  Make sure that the room
     # polygons are on top.
-	
+
     my $cmd = "floor_fg${active}";
     {
 	no strict qw(refs);
@@ -150,7 +148,7 @@ sub floor_display {
 
 sub floor_room_changed {
 
-    # Whenever the current_room variable changes, this procedure highlights 
+    # Whenever the current_room variable changes, this procedure highlights
     # the current room and unhighlights any previous room.
 
     my($w, $floor_items, $cinfo) = @_;
@@ -171,7 +169,7 @@ sub floor_room_changed {
 } # end floor_room_changed
 
 # The following procedures are invoked to instantiate various portions of
-# the building floorplan.  The bodies of these procedures were generated 
+# the building floorplan.  The bodies of these procedures were generated
 # automatically from database files describing the building.
 
 
@@ -1340,7 +1338,7 @@ sub TIESCALAR {
 sub FETCH {
 
     # Method to handle reads of the tied variable:  simply return it's value.
-   
+
     my($current_room) = @_;
     return $$current_room;
 
