@@ -4,7 +4,7 @@ use vars qw/$TOP/;
 
 sub form {
 
-    # Create a top-level window that displays a bunch of entries with 
+    # Create a top-level window that displays a bunch of entries with
     # tabs set up to move between them.
 
     my($demo) = @_;
@@ -14,14 +14,14 @@ sub form {
         -title    => 'Form Demonstration',
         -iconname => 'form',
     );
-
+    my $f = $TOP->Frame->pack(-fill => 'both');
+    my $row = 0;
     foreach ('Name:', 'Address:', '', '', 'Phone:') {
-	my $f = $TOP->Frame(qw/-borderwidth 2/);
 	my $e = $f->Entry(qw/-relief sunken -width 40/);
-	my $l = $f->Label(-text => $_);
-	$f->pack(qw/-side top -fill x/);
-	$e->pack(qw/-side right/);
-	$l->pack(qw/-side left/);
+	my $l = $f->Label(-text => $_, -anchor => 'e', -justify => 'right');
+        Tk::grid( $l, -row => $row, -column => 0, -sticky => 'e');
+        Tk::grid( $e, -row => $row++, -column => 1,-sticky => 'ew');
+        $f->gridRowconfigure(1,-weight => 1);
 	$e->focus if $_ eq 'Name:';
     }
     $TOP->bind('<Return>' => [$TOP => 'destroy']);
