@@ -3,6 +3,7 @@
   This program is free software; you can redistribute it and/or
   modify it under the same terms as Perl itself.
 */
+/* #define PERL_NO_GET_CONTEXT */
 
 #include <EXTERN.h>
 #include <perl.h>
@@ -14,23 +15,37 @@
 #include "pTk/tkInt.h"
 #include "tkGlue.h"
 
-#ifndef newSVpvn
-static SV *
-newSVpvn(char *s,STRLEN len)
-{
- SV *sv = newSVpv("",0);
- sv_setpvn(sv,s,len);
- return sv;
-}
-#endif
-
-
 static int
 Expire(int code)
 {
  return code;
 }
 
+int
+has_highbit(CONST char *s,int l)
+{
+ CONST char *e = s+l;
+ while (s < e)
+  {
+   if (*s++ & 0x80)
+    return 1;
+  }
+ return 0;
+}
+
+SV *
+sv_maybe_utf8(SV *sv)
+{
+#ifdef SvUTF8_on
+ if (SvPOK(sv))
+  {
+   if (has_highbit(SvPVX(sv),SvCUR(sv)))
+    SvUTF8_on(sv);
+  }
+#endif
+ return sv;
+}
+
 #define EXPIRE(args) \
   ( Tcl_SprintfResult args, Expire(TCL_ERROR) )
 
@@ -57,6 +72,12 @@ Expire(int code)
  *
  */
 
+int
+Tcl_IsShared(Tcl_Obj *objPtr)
+{
+ return SvREFCNT(objPtr) > 1;
+}
+
 void
 Tcl_IncrRefCount(Tcl_Obj *objPtr)
 {
@@ -84,7 +105,16 @@ Scalarize(SV *sv, AV *av)
    SV **svp;
    if (n == 1 && (svp = av_fetch(av, 0, 0)))
     {
-     sv_setsv(sv,*svp);
+     STRLEN len = 0;
+     char *s  = SvPV(*svp,len);
+#ifdef SvUTF8
+     int utf8 = SvUTF8(*svp);
+     sv_setpvn(sv,s,len);
+     if (utf8)
+      SvUTF8_on(sv);
+#else
+     sv_setpvn(sv,s,len);
+#endif
     }
    else
     {
@@ -103,12 +133,13 @@ Scalarize(SV *sv, AV *av)
            temp = 1;
            Scalarize(el,(AV *) SvRV(*svp));
           }
-         Tcl_DStringAppendElement(&ds,LangString(el));
+         Tcl_DStringAppendElement(&ds,Tcl_GetString(el));
          if (temp)
           SvREFCNT_dec(el);
         }
       }
      sv_setpvn(sv,Tcl_DStringValue(&ds), Tcl_DStringLength(&ds));
+     sv_maybe_utf8(sv);
      Tcl_DStringFree(&ds);
     }
   }
@@ -142,7 +173,13 @@ ForceScalar(SV *sv)
    else if (!SvOK(sv))
     {
      /* Map undef to null string */
-     sv_setpvn(sv,"",0);
+     if (SvREADONLY(sv))
+      {
+       SV *nsv = newSVpv("",0);
+       return sv_2mortal(nsv);
+      }
+     else
+      sv_setpvn(sv,"",0);
     }
    return sv;
   }
@@ -190,11 +227,13 @@ Tcl_SetLongObj (Tcl_Obj *objPtr, long value)
 }
 
 void
-Tcl_SetStringObj (Tcl_Obj *objPtr, char *bytes, int length)
+Tcl_SetStringObj (Tcl_Obj *objPtr, CONST char *bytes, int length)
 {
  if (length < 0)
   length = strlen(bytes);
- sv_setpvn(ForceScalarLvalue(objPtr), bytes, length);
+ objPtr = ForceScalarLvalue(objPtr);
+ sv_setpvn(objPtr, bytes, length);
+ sv_maybe_utf8(objPtr);
 }
 
 int
@@ -287,7 +326,7 @@ Tcl_NewBooleanObj (int value)
 Tcl_Obj *
 Tcl_NewObj(void)
 {
- return newSVpvn("",0);
+ return newSVsv(&PL_sv_undef);
 }
 
 Tcl_Obj *
@@ -303,13 +342,16 @@ Tcl_NewDoubleObj(double value)
 }
 
 Tcl_Obj *
-Tcl_NewStringObj (char *bytes, int length)
+Tcl_NewStringObj (CONST char *bytes, int length)
 {
  if (bytes)
   {
+   SV *sv;
    if (length < 0)
     length = strlen(bytes);
-   return newSVpvn(bytes,length);
+   sv = newSV(length);
+   sv_setpvn(sv,(char *)bytes,length);
+   return sv_maybe_utf8(sv);
   }
  else
   return &PL_sv_undef;
@@ -331,7 +373,7 @@ Tcl_NewListObj (int objc, Tcl_Obj *CONST objv[])
         */
        if (SvREFCNT(sv) <= 0 || SvTEMP(sv))
         {
-         LangDebug("%s %d:\n",__FUNCTION__, objc);
+         LangDebug(__FUNCTION__ " %d:\n",objc);
          sv_dump(sv);
         }
        av_store(av,objc,sv);
@@ -341,29 +383,132 @@ Tcl_NewListObj (int objc, Tcl_Obj *CONST objv[])
  return MakeReference((SV *) av);
 }
 
+static char *
+LangString(SV *sv)
+{
+ STRLEN na;
+ if (!sv)
+  return "";
+ if (SvGMAGICAL(sv)) mg_get(sv);
+ if (SvPOK(sv))
+  return SvPV(sv, na);
+ else
+  {
+   if (SvROK(sv))
+    {
+     SV *rv = SvRV(sv);
+     if (SvTYPE(rv) == SVt_PVCV || SvTYPE(rv) == SVt_PVAV)
+      return SvPV(sv, na);
+     else
+      {
+       if (SvOBJECT(rv))
+        {
+         if (SvTYPE(rv) == SVt_PVHV)
+          {
+           SV **p = hv_fetch((HV *) rv,"_TkValue_",9,0);
+           if (p)
+            {
+             return SvPV(*p,na);
+            }
+           else
+            {
+             Lang_CmdInfo *info = WindowCommand(sv, NULL, 0);
+             if (info)
+              {
+               if (info->tkwin)
+                {
+                 char *val = Tk_PathName(info->tkwin);
+                 hv_store((HV *) rv,"_TkValue_",9,Tcl_NewStringObj(val,strlen(val)),0);
+                 return val;
+                }
+               if (info->image)
+                {
+                 return SvPV(info->image,na);
+                }
+              }
+            }
+          }
+         else if (SvPOK(rv))
+          {
+#ifdef SvUTF8
+           if (!SvUTF8(sv))
+            sv_utf8_upgrade(rv);
+#endif
+           return SvPV(rv,na);
+          }
+         else
+          LangDumpVec("Odd object type", 1, &rv);
+        }
+      }
+    }
+   if (SvOK(sv))
+    {
+#ifdef SvUTF8
+     if (!SvUTF8(sv))
+      sv_utf8_upgrade(sv);
+#endif
+     return SvPV(sv, na);
+    }
+   else
+    return "";
+  }
+}
+
 char *
 Tcl_GetStringFromObj (Tcl_Obj *objPtr, int *lengthPtr)
 {
- char *s;
- if ((SvROK(objPtr) && SvTYPE(SvRV(objPtr)) == SVt_PVAV) ||
-      (SvTYPE(objPtr) == SVt_PVAV))
-  objPtr = ForceScalar(objPtr);
- if (SvPOK(objPtr))
+ if (objPtr)
   {
-   STRLEN len;
-   s = SvPV(objPtr, len);
-   if (lengthPtr)
-    *lengthPtr = len;
+   char *s;
+   if ((SvROK(objPtr) && SvTYPE(SvRV(objPtr)) == SVt_PVAV) ||
+        (SvTYPE(objPtr) == SVt_PVAV))
+    objPtr = ForceScalar(objPtr);
+   if (SvPOK(objPtr))
+    {
+     STRLEN len;
+#ifdef SvUTF8
+     if (!SvUTF8(objPtr))
+      sv_utf8_upgrade(objPtr);
+#endif
+     s = SvPV(objPtr, len);
+#ifdef SvUTF8
+     if (!is_utf8_string(s,len))
+      {
+       LangDebug(__FUNCTION__ " not utf8\n");
+       sv_dump(objPtr);
+       croak("Not UTF-8");
+      }
+#endif
+     if (lengthPtr)
+      *lengthPtr = len;
+    }
+   else
+    {
+     s = LangString(objPtr);
+#ifdef SvUTF8
+     if (!is_utf8_string(s,strlen(s)))
+      {
+       LangDebug(__FUNCTION__ " not utf8\n");
+       sv_dump(objPtr);
+       abort();
+      }
+#endif
+     if (lengthPtr)
+      *lengthPtr = strlen(s);
+    }
+   return s;
   }
- else
-  {
-   s = LangString(objPtr);
-   if (lengthPtr)
-    *lengthPtr = strlen(s);
-  }
- return s;
+ return NULL;
 }
 
+
+char *
+Tcl_GetString(Tcl_Obj *objPtr)
+{
+ return Tcl_GetStringFromObj(objPtr, NULL);
+}
+
+
 AV *
 ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
 {
@@ -400,7 +545,7 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
             else { $tmp = $2 ; $tmp =~ s/\\([\s\\])/$1/g; push @arr, $tmp }
           }
        */
-       unsigned char *s = (unsigned char *) LangString(sv);
+       unsigned char *s = (unsigned char *) Tcl_GetString(sv);
        int i = 0;
        while (*s)
         {
@@ -443,7 +588,7 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
              s++;
             }
           }
-         av_store(av,i++,newSVpvn(base,(s-base)));
+         av_store(av,i++,Tcl_NewStringObj(base,(s-base)));
          if (*s == '}')
           s++;
         }
@@ -457,7 +602,7 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
      else
       {
        SV *ref = MakeReference((SV *) av);
-       sv_setsv(sv,ref);
+       SvSetMagicSV(sv,ref);
        SvREFCNT_dec(ref);
       }
      return (AV *) SvRV(sv);
@@ -465,6 +610,20 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
   }
 }
 
+void
+Tcl_SetListObj(Tcl_Obj * objPtr,int objc, Tcl_Obj *CONST objv[])
+{
+ AV *av = ForceList(NULL,objPtr);
+ av_clear(av);
+ while (objc-- > 0)
+  {
+   /* Used by tkListbox.c passing in array from Tcl_ListObjGetEelements()
+    * so we need to increment REFCNT
+    */
+   av_store(av,objc,SvREFCNT_inc(objv[objc]));
+  }
+}
+
 int
 Tcl_ListObjAppendElement (Tcl_Interp *interp, Tcl_Obj *listPtr,
 			    Tcl_Obj *objPtr)
@@ -510,7 +669,7 @@ MaybeForceList(Tcl_Interp *interp, Tcl_Obj *sv)
    if (av && av_len(av) > 0)
     {
      /* AV is mortal - so we want newRV not MakeReference as we need extra REFCNT */
-     sv_setsv(sv,newRV((SV *) av));
+     SvSetMagicSV(sv,newRV((SV *) av));
     }
    return av;
   }
@@ -602,13 +761,39 @@ Tcl_ListObjReplace (Tcl_Interp *interp, Tcl_Obj *listPtr, int first, int count,
    /* Store new values */
    for (i=0; i < objc; i++)
     {
-     av_store(av,first+i,objv[i]);
+     /* In tkListbox.c used with incoming objv
+      * so we need to make copies
+      */
+     av_store(av,first+i,newSVsv(objv[i]));
     }
    return TCL_OK;
   }
  return TCL_ERROR;
 }
 
+int
+Tcl_ListObjAppendList(Tcl_Interp * interp, Tcl_Obj * listPtr,Tcl_Obj * elemListPtr)
+{
+ Tcl_Obj **objv;
+ int objc = 0;
+ int code;
+ AV *av = ForceList(interp,listPtr);
+ if ((code = Tcl_ListObjGetElements(interp,elemListPtr,&objc,&objv)) == TCL_OK)
+  {
+   dTHX;
+   int j = av_len(av)+1;
+   int i;
+   for (i=0; i < objc; i++)
+    {
+     av_store(av,j++,objv[i]);
+    }
+  }
+ return code;
+}
+
+
+
+
 Tcl_Obj *
 Tcl_ConcatObj (int objc, Tcl_Obj *CONST objv[])
 {
@@ -633,10 +818,10 @@ Tcl_ConcatObj (int objc, Tcl_Obj *CONST objv[])
 char *
 Tcl_DStringAppendElement(dsPtr, string)
     Tcl_DString *dsPtr;		/* Structure describing dynamic string. */
-    char *string;		/* String to append.  Must be
+    CONST char *string;		/* String to append.  Must be
 				 * null-terminated. */
 {
-    char *s = string;
+    CONST char *s = string;
     int ch;
     while ((ch = *s))
      {
@@ -666,11 +851,11 @@ Tcl_AppendStringsToObj (Tcl_Obj *obj,...)
  va_start(ap,obj);
  while ((s = va_arg(ap,char *)))
   {
-   sv_catpv(sv,s);
+   Tcl_AppendToObj(sv,s,-1);
   }
  va_end(ap);
  if (sv != obj && SvROK(obj))
-  sv_setsv(obj,sv);
+  SvSetMagicSV(obj,sv);
 }
 
 /*
@@ -704,15 +889,15 @@ int
 Tcl_GetIndexFromObj(interp, objPtr, tablePtr, msg, flags, indexPtr)
     Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
     Tcl_Obj *objPtr;		/* Object containing the string to lookup. */
-    char **tablePtr;		/* Array of strings to compare against the
+    CONST char **tablePtr;		/* Array of strings to compare against the
 				 * value of objPtr; last entry must be NULL
 				 * and there must not be duplicate entries. */
-    char *msg;			/* Identifying word to use in error messages. */
+    CONST char *msg;			/* Identifying word to use in error messages. */
     int flags;			/* 0 or TCL_EXACT */
     int *indexPtr;		/* Place to store resulting integer index. */
 {
     int index, length, i, numAbbrev;
-    char *key, *p1, *p2, **entryPtr;
+    CONST char *key, *p1, *p2, **entryPtr;
     Tcl_Obj *resultPtr;
 
     /*
@@ -773,20 +958,41 @@ Tcl_GetIndexFromObj(interp, objPtr, tablePtr, msg, flags, indexPtr)
 void
 Tcl_AppendToObj(objPtr, bytes, length)
     register Tcl_Obj *objPtr;	/* Points to the object to append to. */
-    char *bytes;		/* Points to the bytes to append to the
+    CONST char *bytes;		/* Points to the bytes to append to the
 				 * object. */
     register int length;	/* The number of bytes to append from
 				 * "bytes". If < 0, then append all bytes
 				 * up to NULL byte. */
 {
  SV *sv = ForceScalar(objPtr);
+ int hi;
  if (length < 0)
   length = strlen(bytes);
+#ifdef SvUTF8
+ if ((hi = has_highbit(bytes,length)))
+  {
+   sv_utf8_upgrade(sv);
+  }
  sv_catpvn(sv, bytes, length);
+ if (hi)
+  SvUTF8_on(sv);
+#else
+ sv_catpvn(sv, bytes, length);
+#endif
  if (sv != objPtr && SvROK(objPtr))
-  sv_setsv(objPtr,sv);
+  SvSetMagicSV(objPtr,sv);
 }
 
+void
+Tcl_AppendObjToObj(Tcl_Obj * objPtr,Tcl_Obj * appendObjPtr)
+{
+ int len = 0;
+ char *s = Tcl_GetStringFromObj(appendObjPtr,&len);
+ Tcl_AppendToObj(objPtr,s,len);
+}
+
+
+
 void
 Tcl_WrongNumArgs(interp, objc, objv, message)
     Tcl_Interp *interp;			/* Current interpreter. */
@@ -795,7 +1001,7 @@ Tcl_WrongNumArgs(interp, objc, objv, message)
     Tcl_Obj *CONST objv[];		/* Initial argument objects, which
 					 * should be included in the error
 					 * message. */
-    char *message;			/* Error message to print after the
+    CONST char *message;		/* Error message to print after the
 					 * leading objects in objv. The
 					 * message may be NULL. */
 {
@@ -838,20 +1044,29 @@ Tcl_DbDStringInit(Tcl_DString *svp,char *file,int line)
 void
 Tcl_DStringFree(Tcl_DString *svp)
 {
- if (*svp)
+ SV *sv;
+ if ((sv = *svp))
   {
-   SvREFCNT_dec(*svp);
-   *svp = NULL;
+   SvREFCNT_dec(sv);
+   *svp = Nullsv;
   }
 }
 
-char *
-Tcl_DStringAppend(Tcl_DString *svp, char *s, int len)
+void
+Tcl_DStringResult(Tcl_Interp *interp, Tcl_DString *svp)
 {
  SV *sv = DStringSV(svp);
- if (len < 0)
-  len = strlen(s);
- sv_catpvn(sv,s,len);
+ /* Tcl8.1+ strings are UTF-8 */
+ Tcl_SetObjResult(interp,sv_maybe_utf8(sv));
+ /* Now "free" the DString - the SvREFCNT_dec has been done by SetObjResult */
+ *svp = Nullsv;
+}
+
+char *
+Tcl_DStringAppend(Tcl_DString *svp, CONST char *s, int len)
+{
+ SV *sv = DStringSV(svp);
+ Tcl_AppendToObj(sv,(char *)s,len);
  return SvPVX(sv);
 }
 
@@ -861,19 +1076,11 @@ Tcl_DStringLength(Tcl_DString *svp)
  return (int) ((*svp) ? SvCUR(DStringSV(svp)) : 0);
 }
 
-void
-Tcl_DStringResult(Tcl_Interp *interp, Tcl_DString *svp)
-{
- SV *sv = DStringSV(svp);
- Tcl_ArgResult(interp,sv);
- Tcl_DStringFree(svp);
-}
-
 void
 Tcl_DStringSetLength(Tcl_DString *svp,int len)
 {
  SV *sv = DStringSV(svp);
- char *s = SvGROW(sv,(Size_t)(len+1));
+ char *s = SvGROW(sv,len+1);
  s[len] = '\0';
  SvCUR(sv) = len;
 }
@@ -881,6 +1088,7 @@ Tcl_DStringSetLength(Tcl_DString *svp,int len)
 char *
 Tcl_DStringValue(Tcl_DString *svp)
 {
+ dTHX;
  SV *sv = DStringSV(svp);
  STRLEN len;
  return SvPV(sv,len);
@@ -889,7 +1097,411 @@ Tcl_DStringValue(Tcl_DString *svp)
 void
 Tcl_DStringGetResult(Tcl_Interp *interp, Tcl_DString *svp)
 {
- SV *sv = DStringSV(svp);
- sv_setsv(sv,LangScalarResult(interp));
+ int len;
+ char *s = Tcl_GetStringFromObj(Tcl_GetObjResult(interp),&len);
+ Tcl_DStringAppend(svp,s,len);
 }
 
+/* Now fake Tcl_Obj * internals routines */
+
+static void
+DummyFreeProc(Tcl_Obj *obj)
+{
+}
+
+static void
+IntUpdateStringProc(Tcl_Obj *obj)
+{
+ dTHX;
+ STRLEN len;
+ (void) SvPV(obj,len);
+}
+
+static void
+IntDupProc(Tcl_Obj *src,Tcl_Obj *dst)
+{
+ dTHX;
+ SvSetMagicSV(dst,src);
+ TclObjSetType(dst,TclObjGetType(src));
+}
+
+static int
+IntSetFromAnyProc(Tcl_Interp *interp, Tcl_Obj *obj)
+{
+ Tcl_ObjType *typePtr;
+ Tcl_GetString(obj);
+ typePtr = TclObjGetType(obj);
+ if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
+	(*typePtr->freeIntRepProc)(obj);
+  }
+ TclObjSetType(obj,&tclIntType);
+ return TCL_OK;
+}
+
+extern Tcl_ObjType   tclDoubleType;
+
+static int
+DoubleSetFromAnyProc(Tcl_Interp *interp, Tcl_Obj *obj)
+{
+ Tcl_ObjType *typePtr;
+ Tcl_GetString(obj);
+ typePtr = TclObjGetType(obj);
+ if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
+	(*typePtr->freeIntRepProc)(obj);
+  }
+ TclObjSetType(obj,&tclDoubleType);
+ return TCL_OK;
+}
+
+Tcl_ObjType tclIntType = {
+  "int",
+  DummyFreeProc,
+  IntDupProc,
+  IntUpdateStringProc,
+  IntSetFromAnyProc
+};
+
+Tcl_ObjType tclDoubleType = {
+  "double",
+  DummyFreeProc,
+  IntDupProc,
+  IntUpdateStringProc,
+  DoubleSetFromAnyProc
+};
+
+Tcl_ObjType perlDummyType = {
+  "scalar",
+  DummyFreeProc,
+  IntDupProc,
+  IntUpdateStringProc,
+  IntSetFromAnyProc
+};
+
+typedef struct
+{
+ Tcl_ObjType *type;
+ Tcl_InternalRep internalRep;
+} TclObjMagic_t;
+
+static int
+TclObj_get(pTHX_ SV *sv, MAGIC *mg)
+{
+ TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
+ if (info->type == &tclIntType)
+  {
+   SvIV_set(sv,info->internalRep.longValue);
+   SvIOK_on(sv);
+   LangDebug(__FUNCTION__ " %p %s %ld'\n",sv,info->type->name,SvIV(sv));
+   return 0;
+  }
+ else if (info->type == &tclDoubleType)
+  {
+   SvNV_set(sv,info->internalRep.doubleValue);
+   SvNOK_on(sv);
+   LangDebug(__FUNCTION__ " %p %s %g'\n",sv,info->type->name,SvNV(sv));
+   return 0;
+  }
+ else if (SvROK(sv) || info->type == &perlDummyType)
+  {
+   if (!SvPOK(sv) && SvPOKp(sv))
+    SvPOK_on(sv);
+
+   if (!SvNOK(sv) && SvNOKp(sv))
+    SvNOK_on(sv);
+
+   if (!SvIOK(sv) && SvIOKp(sv))
+    SvIOK_on(sv);
+  }
+ else
+  {
+   Tcl_GetString(sv);
+   SvPOK_on(sv);
+#if 0
+   LangDebug(__FUNCTION__ " %p %s '%s'\n",sv,info->type->name,SvPV_nolen(sv));
+#endif
+  }
+ return 0;
+}
+
+static int
+TclObj_free(pTHX_ SV *sv, MAGIC *mg)
+{
+ TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
+ if (info->type)
+  {
+#ifdef DEBUG_TCLOBJ
+   LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+#endif
+   if (info->type->freeIntRepProc != NULL)
+    {
+     /* We _use_ MAGIC chain to locate interal rep so
+      * re-link mg for duration of callback
+      */
+     MAGIC *save = SvMAGIC(sv);
+     SvMAGIC(sv) = mg;
+     mg->mg_moremagic = NULL;
+     (*info->type->freeIntRepProc)(sv);
+     SvMAGIC(sv) = save;
+    }
+  }
+ else
+  {
+   /* We can have pretened we are double or int without setting a type */
+#if 0
+   LangDebug(__FUNCTION__ " %p NULL\n",sv);
+   sv_dump(sv);
+#endif
+  }
+ return 0;
+}
+
+static int
+TclObj_set(pTHX_ SV *sv, MAGIC *mg)
+{
+#ifdef DEBUG_TCLOBJ
+ TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
+ LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+#endif
+ sv_unmagic(sv,'~');  /* sv_unmagic calls free proc */
+ return 0;
+}
+
+static U32
+TclObj_len(pTHX_ SV *sv, MAGIC *mg)
+{
+#ifdef DEBUG_TCLOBJ
+ TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
+ LangDebug(__FUNCTION__ " %s\n",info->type->name);
+#endif
+ return 0;
+}
+
+static int
+TclObj_clear(pTHX_ SV *sv, MAGIC *mg)
+{
+#ifdef DEBUG_TCLOBJ
+ TclObjMagic_t *info = (TclObjMagic_t *)SvPVX(mg->mg_obj);
+ LangDebug(__FUNCTION__ " %p %s\n",sv,info->type->name);
+#endif
+ sv_unmagic(sv,'~');  /* sv_unmagic calls free proc */
+ return 0;
+}
+
+
+MGVTBL TclObj_vtab = {
+ TclObj_get,
+ TclObj_set,
+ NULL, /* TclObj_len, */
+ TclObj_clear,
+ TclObj_free
+};
+
+static TclObjMagic_t *
+Tcl_ObjMagic(Tcl_Obj *obj,int add)
+{
+ dTHX;
+ MAGIC *mg = (SvTYPE(obj) >= SVt_PVMG) ? mg_find(obj,'~') : NULL;
+ SV *data = NULL;
+ TclObjMagic_t *iv;
+ if (mg)
+  {
+   if (mg->mg_virtual == &TclObj_vtab)
+    {
+     data = mg->mg_obj;
+    }
+   else
+    {
+     if (add)
+      {
+       warn("Wrong kind of '~' magic on %_",obj);
+       sv_dump(obj);
+       abort();
+      }
+    }
+  }
+ else if (add)
+  {
+   Tcl_ObjType *type =  TclObjGetType(obj);
+   int rdonly = SvREADONLY(obj);
+   data = newSV(sizeof(TclObjMagic_t));
+   Zero(SvPVX(data),sizeof(TclObjMagic_t),char);
+   if (rdonly)
+    SvREADONLY_off(obj);
+   sv_upgrade(obj,SVt_PVMG);
+   sv_magic(obj,data,'~',NULL,0);
+   SvREFCNT_dec(data);
+   SvRMAGICAL_off(obj);
+   mg = mg_find(obj,'~');
+   if (mg->mg_obj != data)
+    abort();
+   mg->mg_virtual = &TclObj_vtab;
+   mg_magical(obj);
+   if (rdonly)
+    SvREADONLY_on(obj);
+   iv = (TclObjMagic_t *) SvPVX(data);
+   iv->type = type;
+   if (iv->type == &tclIntType)
+    {
+     iv->internalRep.longValue = SvIV(obj);
+    }
+   else if (iv->type == &tclDoubleType)
+    {
+     iv->internalRep.doubleValue = SvNV(obj);
+    }
+   return iv;
+  }
+ if (data)
+  {
+   TclObjMagic_t *iv = (TclObjMagic_t *) SvPVX(data);
+   return iv;
+  }
+ return NULL;
+}
+
+Tcl_Obj *
+Tcl_DuplicateObj(Tcl_Obj *src)
+{
+ dTHX;
+ /* We get AVs either from SvRV test below, or
+  * "suspect" ResultAv scheme
+  */
+ int object = sv_isobject(src);
+ if (SvTYPE(src) == SVt_PVAV)
+  {
+   abort();
+  }
+ if (!object && SvROK(src) && SvTYPE(SvRV(src)) == SVt_PVAV)
+  {
+   AV *av  = (AV *) SvRV(src);
+   IV max  = av_len(av);
+   AV *dst = newAV();
+   int i;
+   for (i=0; i <= max; i++)
+    {
+     /* Do a deep copy and hope there are no loops */
+     SV **svp = av_fetch(av,i,0);
+     SV *d    = (svp && *svp) ? Tcl_DuplicateObj(*svp) : &PL_sv_undef;
+     av_store(dst,i,d);
+    }
+   return MakeReference((SV *) dst);
+  }
+ else
+  {
+   SV *dup = newSVsv(src);
+   TclObjMagic_t *m = Tcl_ObjMagic(src,0);
+   if (m && m->type)
+    {
+     if (m->type->dupIntRepProc)
+      {
+       (*m->type->dupIntRepProc)(src,dup);
+      }
+     else
+      {
+       TclObjMagic_t *n = Tcl_ObjMagic(dup,1);
+       n->type = m->type;
+       n->internalRep = m->internalRep;
+      }
+    }
+   return dup;
+  }
+}
+
+Tcl_ObjType *
+Tcl_GetObjType(CONST char *name)
+{
+ if (strEQ(name,"int"))
+  return &tclIntType;
+ if (strEQ(name,"double"))
+  return &tclDoubleType;
+ LangDebug(__FUNCTION__ " wanted %s\n",name);
+ return &perlDummyType;
+}
+
+static void
+NoFreeProc(Tcl_Obj *obj)
+{
+ TclObjMagic_t *m = Tcl_ObjMagic(obj,1);
+ LangDebug(__FUNCTION__ " %p %s\n",obj,m->type->name);
+}
+
+Tcl_ObjType *
+TclObjGetType(Tcl_Obj *obj)
+{
+ TclObjMagic_t *m = Tcl_ObjMagic(obj,0);
+ if (m)
+  {
+#ifdef DEBUG_TCLOBJ
+   if (!m->type->freeIntRepProc)
+    m->type->freeIntRepProc = &NoFreeProc;
+#endif
+   return m->type;
+  }
+ if (SvNOK(obj))
+  {
+   return &tclDoubleType;
+  }
+ else if (SvIOK(obj))
+  {
+   return &tclIntType;
+  }
+ return &perlDummyType;
+}
+
+int
+TclObjLength(Tcl_Obj *obj)
+{
+ dTHX;
+ STRLEN len;
+ char *s = SvPV(obj,len);
+ return len;
+}
+
+void
+TclObjSetType(Tcl_Obj *obj,Tcl_ObjType *type)
+{
+ TclObjMagic_t *m = Tcl_ObjMagic(obj,1);
+#ifdef DEBUG_TCLOBJ
+ if (m->type)
+  {
+   LangDebug(__FUNCTION__ " %p was %s\n",obj,m->type->name);
+  }
+ LangDebug(__FUNCTION__ " %p now %s\n",obj,type->name);
+#endif
+ m->type = type;
+}
+
+Tcl_InternalRep *
+TclObjInternal(Tcl_Obj *obj)
+{
+ TclObjMagic_t *m = Tcl_ObjMagic(obj,1);
+ return &(m->internalRep);
+}
+
+void
+Tcl_RegisterObjType(Tcl_ObjType *type)
+{
+}
+
+
+Arg
+LangCopyArg(sv)
+SV *sv;
+{
+ if (sv)
+  {
+   dTHX;
+   MAGIC *mg = (SvTYPE(sv) >= SVt_PVMG) ? mg_find(sv,'~') : NULL;
+   if (mg && mg->mg_virtual == &TclObj_vtab)
+    {
+     return Tcl_DuplicateObj(sv);
+    }
+   sv = newSVsv(sv);
+  }
+ return sv;
+}
+
+
+
+
+
+
