@@ -1,20 +1,20 @@
-/* 
+/*
  * tkCanvImg.c --
  *
  *	This file implements image items for canvas widgets.
  *
  * Copyright (c) 1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkCanvImg.c 1.18 96/05/03 10:49:09
+ * RCS: @(#) $Id: tkCanvImg.c,v 1.5 2002/08/05 04:30:38 dgp Exp $
  */
 
 #include "tkInt.h"
 #include "tkPort.h"
-#include "tkCanvas.h"
+#include "tkCanvases.h"
 
 /*
  * The structure below defines the record for each image item.
@@ -30,25 +30,49 @@ typedef struct ImageItem  {
 				 * (x,y). */
     char *imageString;		/* String describing -image option (malloc-ed).
 				 * NULL means no image right now. */
+    char *activeImageString;	/* String describing -activeimage option.
+				 * NULL means no image right now. */
+    char *disabledImageString;	/* String describing -disabledimage option.
+				 * NULL means no image right now. */
     Tk_Image image;		/* Image to display in window, or NULL if
 				 * no image at present. */
+    Tk_Image activeImage;	/* Image to display in window, or NULL if
+				 * no image at present. */
+    Tk_Image disabledImage;	/* Image to display in window, or NULL if
+				 * no image at present. */
 } ImageItem;
 
 /*
  * Information used for parsing configuration specs:
  */
 
-static Tk_CustomOption tagsOption = {Tk_CanvasTagsParseProc,
+static Tk_CustomOption stateOption = {
+    TkStateParseProc,
+    TkStatePrintProc, (ClientData) 2
+};
+static Tk_CustomOption tagsOption = {
+    Tk_CanvasTagsParseProc,
     Tk_CanvasTagsPrintProc, (ClientData) NULL
 };
 
 static Tk_ConfigSpec configSpecs[] = {
+    {TK_CONFIG_STRING, "-activeimage", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(ImageItem, activeImageString),
+	TK_CONFIG_NULL_OK},
     {TK_CONFIG_ANCHOR, "-anchor", (char *) NULL, (char *) NULL,
 	"center", Tk_Offset(ImageItem, anchor), TK_CONFIG_DONT_SET_DEFAULT},
+    {TK_CONFIG_OBJECT, "-disabledimage", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(ImageItem, disabledImageString),
+	TK_CONFIG_NULL_OK},
     {TK_CONFIG_OBJECT, "-image", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(ImageItem, imageString), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(Tk_Item, state), TK_CONFIG_NULL_OK,
+	&stateOption},
     {TK_CONFIG_CUSTOM, "-tags", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
+    {TK_CONFIG_CALLBACK, "-updatecommand", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(Tk_Item, updateCmd), TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, 0}
 };
@@ -62,19 +86,21 @@ static void		ImageChangedProc _ANSI_ARGS_((ClientData clientData,
 			    int imgHeight));
 static int		ImageCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv));
+			    Tcl_Obj *CONST objv[]));
 static int		ImageToArea _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *rectPtr));
 static double		ImageToPoint _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *coordPtr));
+static int		ImageToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
 static void		ComputeImageBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    ImageItem *imgPtr));
 static int		ConfigureImage _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv, int flags));
+			    Tcl_Obj *CONST objv[], int flags));
 static int		CreateImage _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int argc, Tcl_Obj *CONST objv[]));
 static void		DeleteImage _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display));
 static void		DisplayImage _ANSI_ARGS_((Tk_Canvas canvas,
@@ -100,10 +126,10 @@ Tk_ItemType tkImageType = {
     ImageCoords,			/* coordProc */
     DeleteImage,			/* deleteProc */
     DisplayImage,			/* displayProc */
-    0,					/* alwaysRedraw */
+    TK_CONFIG_OBJS,			/* flags */
     ImageToPoint,			/* pointProc */
     ImageToArea,			/* areaProc */
-    (Tk_ItemPostscriptProc *) NULL,	/* postscriptProc */
+    ImageToPostscript,	                /* postscriptProc */
     ScaleImage,				/* scaleProc */
     TranslateImage,			/* translateProc */
     (Tk_ItemIndexProc *) NULL,		/* indexProc */
@@ -111,9 +137,9 @@ Tk_ItemType tkImageType = {
     (Tk_ItemSelectionProc *) NULL,	/* selectionProc */
     (Tk_ItemInsertProc *) NULL,		/* insertProc */
     (Tk_ItemDCharsProc *) NULL,		/* dTextProc */
-    (Tk_ItemType *) NULL		/* nextPtr */
+    (Tk_ItemType *) NULL,		/* nextPtr */
 };
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -125,7 +151,7 @@ Tk_ItemType tkImageType = {
  * Results:
  *	A standard Tcl return value.  If an error occurred in
  *	creating the item, then an error message is left in
- *	interp->result;  in this case itemPtr is left uninitialized,
+ *	the interp's result;  in this case itemPtr is left uninitialized,
  *	so it can be safely freed by the caller.
  *
  * Side effects:
@@ -135,17 +161,30 @@ Tk_ItemType tkImageType = {
  */
 
 static int
-CreateImage(interp, canvas, itemPtr, argc, argv)
+CreateImage(interp, canvas, itemPtr, argc, objv)
     Tcl_Interp *interp;			/* Interpreter for error reporting. */
     Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
     int argc;				/* Number of arguments in argv. */
-    char **argv;			/* Arguments describing rectangle. */
+    Tcl_Obj *CONST objv[];		/* Arguments describing rectangle. */
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
+    int i;
 
-    if (argc < 2) {
+    if (argc==1) {
+	i = 1;
+    } else {
+	char *arg = Tcl_GetStringFromObj(objv[1], NULL);
+	if (((argc>1) && (arg[0] == '-')
+		&& (arg[1] >= 'a') && (arg[1] <= 'z'))) {
+	    i = 1;
+	} else {
+	    i = 2;
+	}
+    }
+
+    if (argc < i) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
 		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
 		itemPtr->typePtr->name, " x y ?options?\"",
@@ -160,25 +199,28 @@ CreateImage(interp, canvas, itemPtr, argc, argv)
     imgPtr->canvas = canvas;
     imgPtr->anchor = TK_ANCHOR_CENTER;
     imgPtr->imageString = NULL;
+    imgPtr->activeImageString = NULL;
+    imgPtr->disabledImageString = NULL;
     imgPtr->image = NULL;
+    imgPtr->activeImage = NULL;
+    imgPtr->disabledImage = NULL;
 
     /*
      * Process the arguments to fill in the item record.
      */
 
-    if ((Tk_CanvasGetCoord(interp, canvas, argv[0], &imgPtr->x) != TCL_OK)
-	    || (Tk_CanvasGetCoord(interp, canvas, argv[1], &imgPtr->y)
-		!= TCL_OK)) {
-	return TCL_ERROR;
+    if ((ImageCoords(interp, canvas, itemPtr, i, argv) != TCL_OK)) {
+	goto error;
+    }
+    if (ConfigureImage(interp, canvas, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
+	return TCL_OK;
     }
 
-    if (ConfigureImage(interp, canvas, itemPtr, argc-2, argv+2, 0) != TCL_OK) {
-	DeleteImage(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
-	return TCL_ERROR;
-    }
-    return TCL_OK;
+    error:
+    DeleteImage(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
+    return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -189,7 +231,7 @@ CreateImage(interp, canvas, itemPtr, argc, argv)
  *	details on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets interp->result.
+ *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
  *	The coordinates for the given item may be changed.
@@ -198,36 +240,53 @@ CreateImage(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-ImageCoords(interp, canvas, itemPtr, argc, argv)
+ImageCoords(interp, canvas, itemPtr, argc, objv)
     Tcl_Interp *interp;			/* Used for error reporting. */
     Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item whose coordinates are to be
 					 * read or modified. */
     int argc;				/* Number of coordinates supplied in
 					 * argv. */
-    char **argv;			/* Array of coordinates: x1, y1,
+    Tcl_Obj *CONST objv[];		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
-    char x[TCL_DOUBLE_SPACE], y[TCL_DOUBLE_SPACE];
 
     if (argc == 0) {
-	Tcl_DoubleResults(interp, 2, 1, imgPtr->x, imgPtr->y); 
-    } else if (argc == 2) {
-	if ((Tk_CanvasGetCoord(interp, canvas, argv[0], &imgPtr->x) != TCL_OK)
-		|| (Tk_CanvasGetCoord(interp, canvas, argv[1],
-		    &imgPtr->y) != TCL_OK)) {
+	Tcl_Obj *obj = Tcl_NewObj();
+	Tcl_Obj *subobj = Tcl_NewDoubleObj(imgPtr->x);
+	Tcl_ListObjAppendElement(interp, obj, subobj);
+	subobj = Tcl_NewDoubleObj(imgPtr->y);
+	Tcl_ListObjAppendElement(interp, obj, subobj);
+	Tcl_SetObjResult(interp, obj);
+    } else if (argc < 3) {
+	if (argc==1) {
+	    if (Tcl_ListObjGetElements(interp, objv[0], &argc, (Tcl_Obj ***)&objv) != TCL_OK) {
+		return TCL_ERROR;
+	    } else if (argc != 2) {
+		char buf[64];
+
+		sprintf(buf, "wrong # coordinates: expected 2, got %d", argc);
+		Tcl_SetResult(interp, buf, TCL_VOLATILE);
+		return TCL_ERROR;
+	    }
+	}
+	if ((Tk_CanvasGetCoordFromObj(interp, canvas, objv[0], &imgPtr->x) != TCL_OK)
+		|| (Tk_CanvasGetCoordFromObj(interp, canvas, objv[1],
+  		    &imgPtr->y) != TCL_OK)) {
 	    return TCL_ERROR;
 	}
 	ComputeImageBbox(canvas, imgPtr);
     } else {
-	sprintf(interp->result,
-		"wrong # coordinates: expected 0 or 2, got %d", argc);
+	char buf[64];
+
+	sprintf(buf, "wrong # coordinates: expected 0 or 2, got %d", argc);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_ERROR;
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -238,7 +297,7 @@ ImageCoords(interp, canvas, itemPtr, argc, argv)
  *
  * Results:
  *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in interp->result.
+ *	an error message is left in the interp's result.
  *
  * Side effects:
  *	Configuration information may be set for itemPtr.
@@ -247,12 +306,12 @@ ImageCoords(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
+ConfigureImage(interp, canvas, itemPtr, argc, objv, flags)
     Tcl_Interp *interp;		/* Used for error reporting. */
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Image item to reconfigure. */
     int argc;			/* Number of elements in argv.  */
-    char **argv;		/* Arguments describing things to configure. */
+    Tcl_Obj *CONST objv[];	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
@@ -260,8 +319,8 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
     Tk_Image image;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc,
-	    argv, (char *) imgPtr, flags) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, 
+	    argv, (char *) imgPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -272,6 +331,11 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
      * if it hasn't changed.
      */
 
+    if (imgPtr->activeImageString != NULL) {
+	itemPtr->redraw_flags |= TK_ITEM_STATE_DEPENDANT;
+    } else {
+	itemPtr->redraw_flags &= ~TK_ITEM_STATE_DEPENDANT;
+    }
     if (imgPtr->imageString != NULL) {
 	image = Tk_GetImage(interp, tkwin, imgPtr->imageString,
 		ImageChangedProc, (ClientData) imgPtr);
@@ -285,10 +349,36 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
 	Tk_FreeImage(imgPtr->image);
     }
     imgPtr->image = image;
+    if (imgPtr->activeImageString != NULL) {
+	image = Tk_GetImage(interp, tkwin, imgPtr->activeImageString,
+		ImageChangedProc, (ClientData) imgPtr);
+	if (image == NULL) {
+	    return TCL_ERROR;
+	}
+    } else {
+	image = NULL;
+    }
+    if (imgPtr->activeImage != NULL) {
+	Tk_FreeImage(imgPtr->activeImage);
+    }
+    imgPtr->activeImage = image;
+    if (imgPtr->disabledImageString != NULL) {
+	image = Tk_GetImage(interp, tkwin, imgPtr->disabledImageString,
+		ImageChangedProc, (ClientData) imgPtr);
+	if (image == NULL) {
+	    return TCL_ERROR;
+	}
+    } else {
+	image = NULL;
+    }
+    if (imgPtr->disabledImage != NULL) {
+	Tk_FreeImage(imgPtr->disabledImage);
+    }
+    imgPtr->disabledImage = image;
     ComputeImageBbox(canvas, imgPtr);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -318,11 +408,23 @@ DeleteImage(canvas, itemPtr, display)
     if (imgPtr->imageString != NULL) {
 	ckfree(imgPtr->imageString);
     }
+    if (imgPtr->activeImageString != NULL) {
+	ckfree(imgPtr->activeImageString);
+    }
+    if (imgPtr->disabledImageString != NULL) {
+	ckfree(imgPtr->disabledImageString);
+    }
     if (imgPtr->image != NULL) {
 	Tk_FreeImage(imgPtr->image);
     }
+    if (imgPtr->activeImage != NULL) {
+	Tk_FreeImage(imgPtr->activeImage);
+    }
+    if (imgPtr->disabledImage != NULL) {
+	Tk_FreeImage(imgPtr->disabledImage);
+    }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -352,11 +454,24 @@ ComputeImageBbox(canvas, imgPtr)
 {
     int width, height;
     int x, y;
+    Tk_Image image;
+    Tk_State state = Tk_GetItemState(canvas, &imgPtr->header);
+
+    image = imgPtr->image;
+    if (((TkCanvas *)canvas)->currentItemPtr == (Tk_Item *)imgPtr) {
+	if (imgPtr->activeImage != NULL) {
+	    image = imgPtr->activeImage;
+	}
+    } else if (state == TK_STATE_DISABLED) {
+	if (imgPtr->disabledImage != NULL) {
+	    image = imgPtr->disabledImage;
+	}
+    }
 
     x = (int) (imgPtr->x + ((imgPtr->x >= 0) ? 0.5 : - 0.5));
     y = (int) (imgPtr->y + ((imgPtr->y >= 0) ? 0.5 : - 0.5));
 
-    if (imgPtr->image == None) {
+    if ((state == TK_STATE_HIDDEN) || (image == None)) {
 	imgPtr->header.x1 = imgPtr->header.x2 = x;
 	imgPtr->header.y1 = imgPtr->header.y2 = y;
 	return;
@@ -366,7 +481,7 @@ ComputeImageBbox(canvas, imgPtr)
      * Compute location and size of image, using anchor information.
      */
 
-    Tk_SizeOfImage(imgPtr->image, &width, &height);
+    Tk_SizeOfImage(image, &width, &height);
     switch (imgPtr->anchor) {
 	case TK_ANCHOR_N:
 	    x -= width/2;
@@ -409,7 +524,7 @@ ComputeImageBbox(canvas, imgPtr)
     imgPtr->header.x2 = x + width;
     imgPtr->header.y2 = y + height;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -440,8 +555,21 @@ DisplayImage(canvas, itemPtr, display, drawable, x, y, width, height)
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
     short drawableX, drawableY;
+    Tk_Image image;
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
-    if (imgPtr->image == NULL) {
+    image = imgPtr->image;
+    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+	if (imgPtr->activeImage != NULL) {
+	    image = imgPtr->activeImage;
+	}
+    } else if (state == TK_STATE_DISABLED) {
+	if (imgPtr->disabledImage != NULL) {
+	    image = imgPtr->disabledImage;
+	}
+    }
+
+    if (image == NULL) {
 	return;
     }
 
@@ -451,10 +579,10 @@ DisplayImage(canvas, itemPtr, display, drawable, x, y, width, height)
 
     Tk_CanvasDrawableCoords(canvas, (double) x, (double) y,
 	    &drawableX, &drawableY);
-    Tk_RedrawImage(imgPtr->image, x - imgPtr->header.x1, y - imgPtr->header.y1,
+    Tk_RedrawImage(image, x - imgPtr->header.x1, y - imgPtr->header.y1,
 	    width, height, drawable, drawableX, drawableY);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -511,7 +639,7 @@ ImageToPoint(canvas, itemPtr, coordPtr)
 
     return hypot(xDiff, yDiff);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -556,7 +684,93 @@ ImageToArea(canvas, itemPtr, rectPtr)
     }
     return 0;
 }
-
+
+/*
+ *--------------------------------------------------------------
+ *
+ * ImageToPostscript --
+ *
+ *	This procedure is called to generate Postscript for
+ *	image items.
+ *
+ * Results:
+ *	The return value is a standard Tcl result.  If an error
+ *	occurs in generating Postscript then an error message is
+ *	left in interp->result, replacing whatever used to be there.
+ *	If no error occurs, then Postscript for the item is appended
+ *	to the result.
+ *
+ * Side effects:
+ *	None.
+ *
+ *--------------------------------------------------------------
+ */
+
+static int
+ImageToPostscript(interp, canvas, itemPtr, prepass)
+    Tcl_Interp *interp;			/* Leave Postscript or error message
+					 * here. */
+    Tk_Canvas canvas;			/* Information about overall canvas. */
+    Tk_Item *itemPtr;			/* Item for which Postscript is
+					 * wanted. */
+    int prepass;			/* 1 means this is a prepass to
+					 * collect font information;  0 means
+					 * final Postscript is being created.*/
+{
+    ImageItem *imgPtr = (ImageItem *)itemPtr;
+    Tk_Window canvasWin = Tk_CanvasTkwin(canvas);
+
+    char buffer[256];
+    double x, y;
+    int width, height;
+    Tk_Image image;
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
+
+    image = imgPtr->image;
+    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+	if (imgPtr->activeImage != NULL) {
+	    image = imgPtr->activeImage;
+	}
+    } else if (state == TK_STATE_DISABLED) {
+	if (imgPtr->disabledImage != NULL) {
+	    image = imgPtr->disabledImage;
+	}
+    }
+    Tk_SizeOfImage(image, &width, &height);
+
+    /*
+     * Compute the coordinates of the lower-left corner of the image,
+     * taking into account the anchor position for the image.
+     */
+
+    x = imgPtr->x;
+    y = Tk_CanvasPsY(canvas, imgPtr->y);
+
+    switch (imgPtr->anchor) {
+	case TK_ANCHOR_NW:			y -= height;		break;
+	case TK_ANCHOR_N:	x -= width/2.0; y -= height;		break;
+	case TK_ANCHOR_NE:	x -= width;	y -= height;		break;
+	case TK_ANCHOR_E:	x -= width;	y -= height/2.0;	break;
+	case TK_ANCHOR_SE:	x -= width;				break;
+	case TK_ANCHOR_S:	x -= width/2.0;				break;
+	case TK_ANCHOR_SW:						break;
+	case TK_ANCHOR_W:			y -= height/2.0;	break;
+	case TK_ANCHOR_CENTER:	x -= width/2.0; y -= height/2.0;	break;
+    }
+
+    if (image == NULL) {
+        return TCL_OK;
+    }
+
+    if (!prepass) {
+	sprintf(buffer, "%.15g %.15g", x, y);
+	Tcl_AppendResult(interp, buffer, " translate\n", (char *) NULL);
+    }
+
+    return Tk_PostscriptImage(image, interp, canvasWin,
+	    ((TkCanvas *) canvas)->psInfo, 0, 0, width, height, prepass);
+}
+
 /*
  *--------------------------------------------------------------
  *
@@ -590,7 +804,7 @@ ScaleImage(canvas, itemPtr, originX, originY, scaleX, scaleY)
     imgPtr->y = originY + scaleY*(imgPtr->y - originY);
     ComputeImageBbox(canvas, imgPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -622,7 +836,7 @@ TranslateImage(canvas, itemPtr, deltaX, deltaY)
     imgPtr->y += deltaY;
     ComputeImageBbox(canvas, imgPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -666,9 +880,10 @@ ImageChangedProc(clientData, x, y, width, height, imgWidth, imgHeight)
 	height = imgHeight;
 	Tk_CanvasEventuallyRedraw(imgPtr->canvas, imgPtr->header.x1,
 		imgPtr->header.y1, imgPtr->header.x2, imgPtr->header.y2);
-    } 
+    }
     ComputeImageBbox(imgPtr->canvas, imgPtr);
     Tk_CanvasEventuallyRedraw(imgPtr->canvas, imgPtr->header.x1 + x,
 	    imgPtr->header.y1 + y, (int) (imgPtr->header.x1 + x + width),
 	    (int) (imgPtr->header.y1 + y + height));
 }
+
