@@ -4,12 +4,13 @@
  *	This file implements line items for canvas widgets.
  *
  * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvLine.c,v 1.3 1999/02/04 20:51:23 stanton Exp $
+ * RCS: @(#) $Id: tkCanvLine.c,v 1.11 2002/08/05 04:30:38 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -81,13 +82,13 @@ static int		ArrowheadPostscript _ANSI_ARGS_((Tcl_Interp *interp,
 static void		ComputeLineBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    LineItem *linePtr));
 static int		ConfigureLine _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv, int flags));
+			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
+			    Tcl_Obj *CONST objv[], int flags));
 static int		ConfigureArrows _ANSI_ARGS_((Tk_Canvas canvas,
 			    LineItem *linePtr));
 static int		CreateLine _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int objc, Tcl_Obj *CONST objv[]));
 static void		DeleteLine _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display));
 static void		DisplayLine _ANSI_ARGS_((Tk_Canvas canvas,
@@ -98,7 +99,7 @@ static int		GetLineIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *obj, int *indexPtr));
 static int		LineCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int objc, Tcl_Obj *CONST objv[]));
 static void		LineDeleteCoords _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, int first, int last));
 static void		LineInsert _ANSI_ARGS_((Tk_Canvas canvas,
@@ -146,16 +147,16 @@ static Tk_CustomOption smoothOption = {
     TkSmoothPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
+    TkStateParseProc,
+    TkStatePrintProc, (ClientData) 2
 };
 static Tk_CustomOption tagsOption = {
     Tk_CanvasTagsParseProc,
     Tk_CanvasTagsPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption dashOption = {
-    Tk_CanvasDashParseProc,
-    Tk_CanvasDashPrintProc, (ClientData) NULL
+    TkCanvasDashParseProc,
+    TkCanvasDashPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption tileOption = {
     Tk_TileParseProc,
@@ -181,9 +182,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-activestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(LineItem, outline.activeStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-activetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(LineItem, outline.activeTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-activewidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(LineItem, outline.activeWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -211,9 +209,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-disabledstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(LineItem, outline.disabledStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledtile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(LineItem, outline.disabledTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-disabledwidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(LineItem, outline.disabledWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -235,9 +230,6 @@ static Tk_ConfigSpec configSpecs[] = {
 	TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-tags", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
-    {TK_CONFIG_CUSTOM, "-tile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(LineItem, outline.tile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-width", (char *) NULL, (char *) NULL,
 	"1.0", Tk_Offset(LineItem, outline.width),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -261,7 +253,7 @@ Tk_ItemType tkLineType = {
     LineCoords,				/* coordProc */
     DeleteLine,				/* deleteProc */
     DisplayLine,			/* displayProc */
-    TK_ITEM_VISITOR_SUPPORT|TK_CONFIG_OBJS,/* flags */
+    TK_CONFIG_OBJS,			/* flags */
     LineToPoint,			/* pointProc */
     LineToArea,				/* areaProc */
     LineToPostscript,			/* postscriptProc */
@@ -286,7 +278,7 @@ Tk_ItemType tkLineType = {
  */
 
 #define MAX_STATIC_POINTS 200
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -298,7 +290,7 @@ Tk_ItemType tkLineType = {
  * Results:
  *	A standard Tcl return value.  If an error occurred in
  *	creating the item, then an error message is left in
- *	interp->result;  in this case itemPtr is left uninitialized,
+ *	the interp's result;  in this case itemPtr is left uninitialized,
  *	so it can be safely freed by the caller.
  *
  * Side effects:
@@ -308,13 +300,13 @@ Tk_ItemType tkLineType = {
  */
 
 static int
-CreateLine(interp, canvas, itemPtr, argc, argv)
+CreateLine(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Interpreter for error reporting. */
     Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
-    int argc;				/* Number of arguments in argv. */
-    char **argv;			/* Arguments describing line. */
+    int objc;				/* Number of arguments in objv. */
+    Tcl_Obj *CONST objv[];		/* Arguments describing line. */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
     int i;
@@ -347,17 +339,16 @@ CreateLine(interp, canvas, itemPtr, argc, argv)
      * start with a digit or a minus sign followed by a digit.
      */
 
-    for (i = 0; i < argc; i++) {
-	char *arg = Tcl_GetStringFromObj(args[i], NULL);
-	if ((arg[0] == '-') && (arg[1] >= 'a')
-		&& (arg[1] <= 'z')) {
+    for (i = 0; i < objc; i++) {
+	char *arg = Tcl_GetString(objv[i]);
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    break;
 	}
     }
-    if (i && (LineCoords(interp, canvas, itemPtr, i, argv) != TCL_OK)) {
+    if (i && (LineCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
 	goto error;
     }
-    if (ConfigureLine(interp, canvas, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
+    if (ConfigureLine(interp, canvas, itemPtr, objc-i, objv+i, 0) == TCL_OK) {
 	return TCL_OK;
     }
 
@@ -365,7 +356,7 @@ CreateLine(interp, canvas, itemPtr, argc, argv)
     DeleteLine(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -376,7 +367,7 @@ CreateLine(interp, canvas, itemPtr, argc, argv)
  *	on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets interp->result.
+ *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
  *	The coordinates for the given item may be changed.
@@ -385,21 +376,21 @@ CreateLine(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-LineCoords(interp, canvas, itemPtr, argc, argv)
+LineCoords(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Used for error reporting. */
     Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item whose coordinates are to be
 					 * read or modified. */
-    int argc;				/* Number of coordinates supplied in
-					 * argv. */
-    char **argv;			/* Array of coordinates: x1, y1,
+    int objc;				/* Number of coordinates supplied in
+					 * objv. */
+    Tcl_Obj *CONST objv[];		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
     int i, numPoints;
     double *coordPtr;
 
-    if (argc == 0) {
+    if (objc == 0) {
 	int numCoords;
 	Tcl_Obj *subobj, *obj = Tcl_NewObj();
 
@@ -422,21 +413,27 @@ LineCoords(interp, canvas, itemPtr, argc, argv)
 	Tcl_SetObjResult(interp, obj);
 	return TCL_OK;
     }
-    if (argc == 1) {
-	if (Tcl_ListObjGetElements(interp, args[0], &argc, &args) != TCL_OK) {
+    if (objc == 1) {
+	if (Tcl_ListObjGetElements(interp, objv[0], &objc,
+		(Tcl_Obj ***) &objv) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     }
-    if (argc & 1) {
+    if (objc & 1) {
 	Tcl_AppendResult(interp,
 		"odd number of coordinates specified for line",
 		(char *) NULL);
 	return TCL_ERROR;
+    } else if (objc < 4) {
+	Tcl_AppendResult(interp,
+		"too few coordinates specified for line",
+		(char *) NULL);
+	return TCL_ERROR;
     } else {
-	numPoints = argc/2;
+	numPoints = objc/2;
 	if (linePtr->numPoints != numPoints) {
 	    coordPtr = (double *) ckalloc((unsigned)
-		    (sizeof(double) * argc));
+		    (sizeof(double) * objc));
 	    if (linePtr->coordPtr != NULL) {
 		ckfree((char *) linePtr->coordPtr);
 	    }
@@ -444,8 +441,8 @@ LineCoords(interp, canvas, itemPtr, argc, argv)
 	    linePtr->numPoints = numPoints;
 	}
 	coordPtr = linePtr->coordPtr;
-	for (i = 0; i <argc; i++) {
-	    if (Tk_CanvasGetCoordFromObj(interp, canvas, args[i],
+	for (i = 0; i <objc; i++) {
+	    if (Tk_CanvasGetCoordFromObj(interp, canvas, objv[i],
 		    coordPtr++) != TCL_OK) {
   		return TCL_ERROR;
   	    }
@@ -471,7 +468,7 @@ LineCoords(interp, canvas, itemPtr, argc, argv)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -482,7 +479,7 @@ LineCoords(interp, canvas, itemPtr, argc, argv)
  *
  * Results:
  *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in interp->result.
+ *	an error message is left in the interp's result.
  *
  * Side effects:
  *	Configuration information, such as colors and stipple
@@ -492,12 +489,12 @@ LineCoords(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-ConfigureLine(interp, canvas, itemPtr, argc, argv, flags)
+ConfigureLine(interp, canvas, itemPtr, objc, objv, flags)
     Tcl_Interp *interp;		/* Used for error reporting. */
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Line item to reconfigure. */
-    int argc;			/* Number of elements in argv.  */
-    char **argv;		/* Arguments describing things to configure. */
+    int objc;			/* Number of elements in objv.  */
+    Tcl_Obj *CONST objv[];	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
@@ -508,8 +505,8 @@ ConfigureLine(interp, canvas, itemPtr, argc, argv, flags)
     Tk_State state;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, argv,
-	    (char *) linePtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, tkwin, configSpecs, objc, 
+            objv, (char *) linePtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -521,8 +518,7 @@ ConfigureLine(interp, canvas, itemPtr, argc, argv, flags)
     state = Tk_GetItemState(canvas, itemPtr);
 
     if (linePtr->outline.activeWidth > linePtr->outline.width ||
-	    linePtr->outline.activeTile != None ||
-	    linePtr->outline.activeDash.number > 0 ||
+	    linePtr->outline.activeDash.number != 0 ||
 	    linePtr->outline.activeColor != NULL ||
 	    linePtr->outline.activeStipple != None) {
 	itemPtr->redraw_flags |= TK_ITEM_STATE_DEPENDANT;
@@ -567,6 +563,7 @@ ConfigureLine(interp, canvas, itemPtr, argc, argv, flags)
 	ComputeLineBbox(canvas, linePtr);
 	return TCL_OK;
     }
+
     /*
      * Setup arrowheads, if needed.  If arrowheads are turned off,
      * restore the line's endpoints (they were shortened when the
@@ -602,7 +599,7 @@ ConfigureLine(interp, canvas, itemPtr, argc, argv, flags)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -643,7 +640,7 @@ DeleteLine(canvas, itemPtr, display)
 	ckfree((char *) linePtr->lastArrowPtr);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -711,7 +708,7 @@ ComputeLineBbox(canvas, linePtr)
 	    i++, coordPtr += 2) {
 	TkIncludePoint((Tk_Item *) linePtr, coordPtr);
     }
-
+    width = linePtr->outline.width;
     if (width < 1.0) {
 	width = 1.0;
     }
@@ -822,7 +819,7 @@ ComputeLineBbox(canvas, linePtr)
     linePtr->header.y1 -= 1;
     linePtr->header.y2 += 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -867,25 +864,19 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
 
     linewidth = linePtr->outline.width;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (linePtr->outline.activeWidth>linewidth) {
-	    linewidth = linePtr->outline.activeWidth;
-	}
-	if (linePtr->outline.activeTile!=NULL) {
-	    tile = linePtr->outline.activeTile;
-	}
-	if (linePtr->outline.activeStipple!=None) {
+	if (linePtr->outline.activeStipple != None) {
 	    stipple = linePtr->outline.activeStipple;
 	}
+	if (linePtr->outline.activeWidth != linewidth) {
+	    linewidth = linePtr->outline.activeWidth;
+	}
     } else if (state==TK_STATE_DISABLED) {
-	if (linePtr->outline.disabledWidth>linewidth) {
-	    linewidth = linePtr->outline.disabledWidth;
-	}
-	if (linePtr->outline.disabledTile!=NULL) {
-	    tile = linePtr->outline.disabledTile;
-	}
-	if (linePtr->outline.disabledStipple!=None) {
+	if (linePtr->outline.disabledStipple != None) {
 	    stipple = linePtr->outline.disabledStipple;
 	}
+	if (linePtr->outline.disabledWidth != linewidth) {
+	    linewidth = linePtr->outline.disabledWidth;
+	}
     }
     /*
      * Build up an array of points in screen coordinates.  Use a
@@ -935,8 +926,12 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
 	    CoordModeOrigin);
     } else {
 	int intwidth = (int) (linewidth + 0.5);
-	XFillArc(display, drawable, linePtr->outline.gc, pointPtr->x - intwidth/2,
-		pointPtr->y - intwidth/2, intwidth+1, intwidth+1, 0, 64*360);
+        if (intwidth<1) {
+	    intwidth=1;
+	}
+	XFillArc(display, drawable, linePtr->outline.gc,
+		pointPtr->x - intwidth/2, pointPtr->y - intwidth/2,
+		(unsigned int)intwidth+1, (unsigned int)intwidth+1, 0, 64*360);
     }
     if (pointPtr != staticPoints) {
 	ckfree((char *) pointPtr);
@@ -958,7 +953,7 @@ DisplayLine(canvas, itemPtr, display, drawable, x, y, width, height)
 	XSetTSOrigin(display, linePtr->arrowGC, 0, 0);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -984,14 +979,18 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
     Tcl_Obj *obj;		/* New coordinates to be inserted. */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
-    int length, argc, i;
+    int length, objc, i;
     double *new, *coordPtr;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
     Tcl_Obj **objv;
 
-    if (!obj || (Tcl_ListObjGetElements((Tcl_Interp *) NULL, obj, &argc, &objv) != TCL_OK)
-	    || !argc || argc&1) {
+    if(state == TK_STATE_NULL) {
+	state = ((TkCanvas *)canvas)->canvas_state;
+    }
+
+    if (!obj || (Tcl_ListObjGetElements((Tcl_Interp *) NULL, obj, &objc, &objv) != TCL_OK)
+	    || !objc || objc&1) {
 	return;
     }
     length = 2*linePtr->numPoints;
@@ -1009,11 +1008,11 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 	linePtr->coordPtr[length-2] = linePtr->lastArrowPtr[0];
 	linePtr->coordPtr[length-1] = linePtr->lastArrowPtr[1];
     }
-    new = (double *) ckalloc((unsigned)(sizeof(double) * (length + argc)));
+    new = (double *) ckalloc((unsigned)(sizeof(double) * (length + objc)));
     for(i=0; i<beforeThis; i++) {
 	new[i] = linePtr->coordPtr[i];
     }
-    for(i=0; i<argc; i++) {
+    for(i=0; i<objc; i++) {
 	if (Tcl_GetDoubleFromObj((Tcl_Interp *) NULL,objv[i],
 		new+(i+beforeThis))!=TCL_OK) {
 	    Tcl_ResetResult(((TkCanvas *)canvas)->interp);
@@ -1023,11 +1022,11 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
     }
 
     for(i=beforeThis; i<length; i++) {
-	new[i+argc] = linePtr->coordPtr[i];
+	new[i+objc] = linePtr->coordPtr[i];
     }
     if(linePtr->coordPtr) ckfree((char *)linePtr->coordPtr);
     linePtr->coordPtr = new;
-    linePtr->numPoints = (length + argc)/2;
+    linePtr->numPoints = (length + objc)/2;
 
     if ((length>3) && (state != TK_STATE_HIDDEN)) {
 	/*
@@ -1040,14 +1039,14 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 	 */
 	itemPtr->redraw_flags |= TK_ITEM_DONT_REDRAW;
 
-	if (beforeThis>0) {beforeThis -= 2; argc+=2; }
-	if ((beforeThis+argc)<length) argc+=2;
+	if (beforeThis>0) {beforeThis -= 2; objc+=2; }
+	if ((beforeThis+objc)<length) objc+=2;
 	if (linePtr->smooth) {
 	    if(beforeThis>0) {
-		beforeThis-=2; argc+=2;
+		beforeThis-=2; objc+=2;
 	    }
-	    if((beforeThis+argc+2)<length) {
-		argc+=2;
+	    if((beforeThis+objc+2)<length) {
+		objc+=2;
 	    }
 	}
 	itemPtr->x1 = itemPtr->x2 = (int) linePtr->coordPtr[beforeThis];
@@ -1059,7 +1058,7 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 		TkIncludePoint(itemPtr, coordPtr);
 	    }
 	}
-	if ((linePtr->lastArrowPtr != NULL) && ((beforeThis+argc)>=length)) {
+	if ((linePtr->lastArrowPtr != NULL) && ((beforeThis+objc)>=length)) {
 		/* include old last arrow */
 	    for (i = 0, coordPtr = linePtr->lastArrowPtr; i < PTS_IN_ARROW;
 		    i++, coordPtr += 2) {
@@ -1067,7 +1066,7 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 	    }
 	}
 	coordPtr = linePtr->coordPtr+beforeThis+2;
-	for(i=2; i<argc; i+=2) {
+	for(i=2; i<objc; i+=2) {
 	    TkIncludePoint(itemPtr, coordPtr);
 		coordPtr+=2;
 	}
@@ -1094,7 +1093,7 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 		TkIncludePoint(itemPtr, coordPtr);
 	    }
 	}
-	if ((linePtr->lastArrowPtr != NULL) && ((beforeThis+argc)<(length-2))) {
+	if ((linePtr->lastArrowPtr != NULL) && ((beforeThis+objc)<(length-2))) {
 	    /* include new right arrow */
 	    for (i = 0, coordPtr = linePtr->lastArrowPtr; i < PTS_IN_ARROW;
 		    i++, coordPtr += 2) {
@@ -1123,7 +1122,7 @@ LineInsert(canvas, itemPtr, beforeThis, obj)
 
     ComputeLineBbox(canvas, linePtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1210,14 +1209,14 @@ LineDeleteCoords(canvas, itemPtr, first, last)
 	    }
 	}
 	coordPtr = linePtr->coordPtr+first1+2;
-	for(i=first1+2; i<=last1; i+=2) {
+	for (i=first1+2; i<=last1; i+=2) {
 	    TkIncludePoint(itemPtr, coordPtr);
-		coordPtr+=2;
+	    coordPtr+=2;
 	}
     }
 
     count = last + 2 - first;
-    for(i=last+2; i<length; i++) {
+    for (i=last+2; i<length; i++) {
 	linePtr->coordPtr[i-count] = linePtr->coordPtr[i];
     }
     linePtr->numPoints -= count/2;
@@ -1270,7 +1269,7 @@ LineDeleteCoords(canvas, itemPtr, first, last)
     }
     ComputeLineBbox(canvas, linePtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1499,7 +1498,7 @@ LineToPoint(canvas, itemPtr, pointPtr)
     }
     return bestDist;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1585,7 +1584,7 @@ LineToArea(canvas, itemPtr, rectPtr)
      * Check the segments of the line.
      */
 
-    if (width < 1.0) {
+     if (width < 1.0) {
 	width = 1.0;
     }
 
@@ -1623,7 +1622,7 @@ LineToArea(canvas, itemPtr, rectPtr)
     }
     return result;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1686,7 +1685,7 @@ ScaleLine(canvas, itemPtr, originX, originY, scaleX, scaleY)
     }
     ComputeLineBbox(canvas, linePtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1784,7 +1783,7 @@ GetLineIndex(interp, canvas, itemPtr, obj, indexPtr)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1835,7 +1834,7 @@ TranslateLine(canvas, itemPtr, deltaX, deltaY)
     }
     ComputeLineBbox(canvas, linePtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1879,7 +1878,7 @@ ParseArrowShape(clientData, interp, tkwin, value, recordPtr, offset)
     if (Tcl_ListObjGetElements(interp, value, &argc, &objv) != TCL_OK) {
 	syntaxError:
 	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp, "bad arrow shape \"", LangString(value),
+	Tcl_AppendResult(interp, "bad arrow shape \"", Tcl_GetString(value),
 		"\": must be list with three numbers", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -1898,7 +1897,7 @@ ParseArrowShape(clientData, interp, tkwin, value, recordPtr, offset)
     linePtr->arrowShapeC = (float) c;
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1934,8 +1933,8 @@ PrintArrowShape(clientData, tkwin, recordPtr, offset, freeProcPtr)
     Tcl_ListObjAppendElement(NULL,result,Tcl_NewDoubleObj(linePtr->arrowShapeC));
     return result;
 }
-
-
+
+
 /*
  *--------------------------------------------------------------
  *
@@ -1965,7 +1964,7 @@ ArrowParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
 {
     int c;
     size_t length;
-    char *value = LangString(ovalue);
+    char *value = Tcl_GetString(ovalue);
 
     register Arrows *arrowPtr = (Arrows *) (widgRec + offset);
 
@@ -2000,7 +1999,7 @@ ArrowParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
     *arrowPtr = ARROWS_NONE;
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -2047,7 +2046,7 @@ ArrowPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 	return LangStringArg("none");
   }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -2206,7 +2205,7 @@ ConfigureArrows(canvas, linePtr)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -2218,7 +2217,7 @@ ConfigureArrows(canvas, linePtr)
  * Results:
  *	The return value is a standard Tcl result.  If an error
  *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used
+ *	left in the interp's result, replacing whatever used
  *	to be there.  If no error occurs, then Postscript for the
  *	item is appended to the result.
  *
@@ -2240,7 +2239,7 @@ LineToPostscript(interp, canvas, itemPtr, prepass)
 					 * final Postscript is being created. */
 {
     LineItem *linePtr = (LineItem *) itemPtr;
-    char buffer[200];
+    char buffer[64 + TCL_INTEGER_SPACE];
     char *style;
 
     double width;
@@ -2389,7 +2388,7 @@ LineToPostscript(interp, canvas, itemPtr, prepass)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -2401,7 +2400,7 @@ LineToPostscript(interp, canvas, itemPtr, prepass)
  * Results:
  *	The return value is a standard Tcl result.  If an error
  *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used
+ *	left in the interp's result, replacing whatever used
  *	to be there.  If no error occurs, then Postscript for the
  *	arrowhead is appended to the result.
  *
@@ -2447,3 +2446,5 @@ ArrowheadPostscript(interp, canvas, linePtr, arrowPtr)
     }
     return TCL_OK;
 }
+
+
