@@ -5,11 +5,12 @@
  *
  * Copyright (c) 1991-1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-2000 Ajuba Solutions.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvPoly.c,v 1.2 1998/09/14 18:23:05 stanton Exp $
+ * RCS: @(#) $Id: tkCanvPoly.c,v 1.9 2002/08/05 04:30:38 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -33,9 +34,6 @@ typedef struct PolygonItem  {
 				 * X-coords are even-valued indices, y-coords
 				 * are corresponding odd-valued indices. */
     int joinStyle;		/* Join style for outline */
-    Tk_Tile fillTile;		/* Tile for filling rectangle/oval. */
-    Tk_Tile activeFillTile;	/* Tile for filling rectangle/oval if state is active. */
-    Tk_Tile disabledFillTile;	/* Tile for filling rectangle/oval if state is disabled. */
     Tk_TSOffset tsoffset;
     XColor *fillColor;		/* Foreground color for polygon. */
     XColor *activeFillColor;	/* Foreground color for polygon if state is active. */
@@ -49,6 +47,7 @@ typedef struct PolygonItem  {
     int splineSteps;		/* Number of steps in each spline segment. */
     int autoClosed;		/* Zero means the given polygon was closed,
 				   one means that we auto closed it. */
+    Tk_Tile fillTile;
 } PolygonItem;
 
 /*
@@ -60,16 +59,16 @@ static Tk_CustomOption smoothOption = {
     TkSmoothPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
+    TkStateParseProc,
+    TkStatePrintProc, (ClientData) 2
 };
 static Tk_CustomOption tagsOption = {
     Tk_CanvasTagsParseProc,
     Tk_CanvasTagsPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption dashOption = {
-    Tk_CanvasDashParseProc,
-    Tk_CanvasDashPrintProc, (ClientData) NULL
+    TkCanvasDashParseProc,
+    TkCanvasDashPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption tileOption = {
     Tk_TileParseProc,
@@ -98,15 +97,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-activeoutlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.activeStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-activeoutlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.activeTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_BITMAP, "-activestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, activeFillStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-activetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, activeFillTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-activewidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(PolygonItem, outline.activeWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -128,15 +121,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-disabledoutlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.disabledStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledoutlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.disabledTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_BITMAP, "-disabledstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, disabledFillStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledtile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, disabledFillTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-disabledwidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(PolygonItem, outline.disabledWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -156,9 +143,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-outlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.stipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-outlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.tile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-smooth", (char *) NULL, (char *) NULL,
 	"0", Tk_Offset(PolygonItem, smooth),
 	TK_CONFIG_DONT_SET_DEFAULT, &smoothOption},
@@ -190,11 +174,11 @@ static Tk_ConfigSpec configSpecs[] = {
 static void		ComputePolygonBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    PolygonItem *polyPtr));
 static int		ConfigurePolygon _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv, int flags));
+			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
+			    Tcl_Obj *CONST objv[], int flags));
 static int		CreatePolygon _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int objc, Tcl_Obj *CONST objv[]));
 static void		DeletePolygon _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr,  Display *display));
 static void		DisplayPolygon _ANSI_ARGS_((Tk_Canvas canvas,
@@ -205,7 +189,7 @@ static int		GetPolygonIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *obj, int *indexPtr));
 static int		PolygonCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int objc, Tcl_Obj *CONST objv[]));
 static void		PolygonDeleteCoords _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, int first, int last));
 static void		PolygonInsert _ANSI_ARGS_((Tk_Canvas canvas,
@@ -236,7 +220,7 @@ Tk_ItemType tkPolygonType = {
     PolygonCoords,			/* coordProc */
     DeletePolygon,			/* deleteProc */
     DisplayPolygon,			/* displayProc */
-    TK_ITEM_VISITOR_SUPPORT|TK_CONFIG_OBJS,/* flags */
+    TK_CONFIG_OBJS,			/* flags */
     PolygonToPoint,			/* pointProc */
     PolygonToArea,			/* areaProc */
     PolygonToPostscript,		/* postscriptProc */
@@ -248,10 +232,6 @@ Tk_ItemType tkPolygonType = {
     PolygonInsert,			/* insertProc */
     PolygonDeleteCoords,		/* dTextProc */
     (Tk_ItemType *) NULL,		/* nextPtr */
-    (Tk_ItemBboxProc *) ComputePolygonBbox,/* bboxProc */
-    Tk_Offset(Tk_VisitorType, visitPolygon), /* acceptProc */
-    (Tk_ItemGetCoordProc *) NULL,	/* getCoordProc */
-    (Tk_ItemSetCoordProc *) NULL	/* setCoordProc */
 };
 
 /*
@@ -261,7 +241,7 @@ Tk_ItemType tkPolygonType = {
  */
 
 #define MAX_STATIC_POINTS 200
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -273,7 +253,7 @@ Tk_ItemType tkPolygonType = {
  * Results:
  *	A standard Tcl return value.  If an error occurred in
  *	creating the item, then an error message is left in
- *	interp->result;  in this case itemPtr is
+ *	the interp's result;  in this case itemPtr is
  *	left uninitialized, so it can be safely freed by the
  *	caller.
  *
@@ -284,13 +264,13 @@ Tk_ItemType tkPolygonType = {
  */
 
 static int
-CreatePolygon(interp, canvas, itemPtr, argc, argv)
+CreatePolygon(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Interpreter for error reporting. */
     Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
-    int argc;				/* Number of arguments in argv. */
-    char **argv;			/* Arguments describing polygon. */
+    int objc;				/* Number of arguments in objv. */
+    Tcl_Obj *CONST objv[];		/* Arguments describing polygon. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
     int i;
@@ -305,9 +285,6 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
     polyPtr->pointsAllocated = 0;
     polyPtr->coordPtr = NULL;
     polyPtr->joinStyle = JoinRound;
-    polyPtr->fillTile = NULL;
-    polyPtr->activeFillTile = NULL;
-    polyPtr->disabledFillTile = NULL;
     polyPtr->tsoffset.flags = 0;
     polyPtr->tsoffset.xoffset = 0;
     polyPtr->tsoffset.yoffset = 0;
@@ -328,18 +305,17 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
      * start with a digit or a minus sign followed by a digit.
      */
 
-    for (i = 0; i < argc; i++) {
-	char *arg = Tcl_GetStringFromObj(objv[i], NULL);
-	if ((arg[0] == '-') && (arg[1] >= 'a')
-		&& (arg[1] <= 'z')) {
+    for (i = 0; i < objc; i++) {
+	char *arg = Tcl_GetString(objv[i]);
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    break;
 	}
     }
-    if (i && PolygonCoords(interp, canvas, itemPtr, i, argv) != TCL_OK) {
+    if (i && PolygonCoords(interp, canvas, itemPtr, i, objv) != TCL_OK) {
 	goto error;
     }
 
-    if (ConfigurePolygon(interp, canvas, itemPtr, argc-i, argv+i, 0)
+    if (ConfigurePolygon(interp, canvas, itemPtr, objc-i, objv+i, 0)
 	    == TCL_OK) {
 	return TCL_OK;
     }
@@ -348,7 +324,7 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
     DeletePolygon(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -359,7 +335,7 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
  *	on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets interp->result.
+ *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
  *	The coordinates for the given item may be changed.
@@ -368,20 +344,20 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-PolygonCoords(interp, canvas, itemPtr, argc, argv)
+PolygonCoords(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Used for error reporting. */
     Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item whose coordinates are to be
 					 * read or modified. */
-    int argc;				/* Number of coordinates supplied in
-					 * argv. */
-    char **argv;			/* Array of coordinates: x1, y1,
+    int objc;				/* Number of coordinates supplied in
+					 * objv. */
+    Tcl_Obj *CONST objv[];		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
     int i, numPoints;
 
-    if (argc == 0) {
+    if (objc == 0) {
 	/*
 	 * Print the coords used to create the polygon.  If we auto
 	 * closed the polygon then we don't report the last point.
@@ -394,18 +370,19 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 	Tcl_SetObjResult(interp, obj);
 	return TCL_OK;
     }
-    if (argc == 1) {
-	if (Tcl_ListObjGetElements(interp, objv[0], &argc, &objv) != TCL_OK) {
+    if (objc == 1) {
+	if (Tcl_ListObjGetElements(interp, objv[0], &objc,
+		(Tcl_Obj ***) &objv) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     }
-    if (argc & 1) {
+    if (objc & 1) {
 	Tcl_AppendResult(interp,
 		"odd number of coordinates specified for polygon",
 		(char *) NULL);
 	return TCL_ERROR;
     } else {
-	numPoints = argc/2;
+	numPoints = objc/2;
 	if (polyPtr->pointsAllocated <= numPoints) {
 	    if (polyPtr->coordPtr != NULL) {
 		ckfree((char *) polyPtr->coordPtr);
@@ -417,10 +394,10 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 	     */
 
 	    polyPtr->coordPtr = (double *) ckalloc((unsigned)
-		    (sizeof(double) * (argc+2)));
+		    (sizeof(double) * (objc+2)));
 	    polyPtr->pointsAllocated = numPoints+1;
 	}
-	for (i = argc-1; i >= 0; i--) {
+	for (i = objc-1; i >= 0; i--) {
 	    if (Tk_CanvasGetCoordFromObj(interp, canvas, objv[i],
 		    &polyPtr->coordPtr[i]) != TCL_OK) {
 		return TCL_ERROR;
@@ -433,18 +410,18 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 	 * Close the polygon if it isn't already closed.
 	 */
 
-	if (argc>2 && ((polyPtr->coordPtr[argc-2] != polyPtr->coordPtr[0])
-		|| (polyPtr->coordPtr[argc-1] != polyPtr->coordPtr[1]))) {
+	if (objc>2 && ((polyPtr->coordPtr[objc-2] != polyPtr->coordPtr[0])
+		|| (polyPtr->coordPtr[objc-1] != polyPtr->coordPtr[1]))) {
 	    polyPtr->autoClosed = 1;
 	    polyPtr->numPoints++;
-	    polyPtr->coordPtr[argc] = polyPtr->coordPtr[0];
-	    polyPtr->coordPtr[argc+1] = polyPtr->coordPtr[1];
+	    polyPtr->coordPtr[objc] = polyPtr->coordPtr[0];
+	    polyPtr->coordPtr[objc+1] = polyPtr->coordPtr[1];
 	}
 	ComputePolygonBbox(canvas, polyPtr);
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -455,7 +432,7 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
  *
  * Results:
  *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in interp->result.
+ *	an error message is left in the interp's result.
  *
  * Side effects:
  *	Configuration information, such as colors and stipple
@@ -465,12 +442,12 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
+ConfigurePolygon(interp, canvas, itemPtr, objc, objv, flags)
     Tcl_Interp *interp;		/* Interpreter for error reporting. */
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Polygon item to reconfigure. */
-    int argc;			/* Number of elements in argv.  */
-    char **argv;		/* Arguments describing things to configure. */
+    int objc;			/* Number of elements in objv.  */
+    Tcl_Obj *CONST objv[];	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
@@ -478,15 +455,13 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     GC newGC;
     unsigned long mask;
     Tk_Window tkwin;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state;
-    Pixmap pixmap;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, argv,
-	    (char *) polyPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK !=  Tk_ConfigureWidget(interp, tkwin, configSpecs, objc,
+             objv, (char *) polyPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -498,11 +473,9 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     state = Tk_GetItemState(canvas, itemPtr);
 
     if (polyPtr->outline.activeWidth > polyPtr->outline.width ||
-	    polyPtr->outline.activeDash.number > 0 ||
-	    polyPtr->outline.activeTile != NULL ||
+	    polyPtr->outline.activeDash.number != 0 ||
 	    polyPtr->outline.activeColor != NULL ||
 	    polyPtr->outline.activeStipple != None ||
-	    polyPtr->activeFillTile != NULL ||
 	    polyPtr->activeFillColor != NULL ||
 	    polyPtr->activeFillStipple != None) {
 	itemPtr->redraw_flags |= TK_ITEM_STATE_DEPENDANT;
@@ -529,13 +502,9 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     }
     polyPtr->outline.gc = newGC;
 
-    tile = polyPtr->fillTile;
     color = polyPtr->fillColor;
     stipple = polyPtr->fillStipple;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (polyPtr->activeFillTile!=NULL) {
-	    tile = polyPtr->activeFillTile;
-	}
 	if (polyPtr->activeFillColor!=NULL) {
 	    color = polyPtr->activeFillColor;
 	}
@@ -543,9 +512,6 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
 	    stipple = polyPtr->activeFillStipple;
 	}
     } else if (state==TK_STATE_DISABLED) {
-	if (polyPtr->disabledFillTile!=NULL) {
-	    tile = polyPtr->disabledFillTile;
-	}
 	if (polyPtr->disabledFillColor!=NULL) {
 	    color = polyPtr->disabledFillColor;
 	}
@@ -554,16 +520,7 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
 	}
     }
 
-    Tk_SetTileCanvasItem(polyPtr->fillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(polyPtr->activeFillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(polyPtr->disabledFillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(tile, canvas, itemPtr);
-
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	newGC = Tk_GetGC(tkwin, GCTile|GCFillStyle, &gcValues);
-    } else if (color == NULL) {
+    if (color == NULL) {
 	newGC = None;
     } else {
 	gcValues.foreground = color->pixel;
@@ -593,7 +550,7 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     ComputePolygonBbox(canvas, polyPtr);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -624,15 +581,6 @@ DeletePolygon(canvas, itemPtr, display)
     if (polyPtr->coordPtr != NULL) {
 	ckfree((char *) polyPtr->coordPtr);
     }
-    if (polyPtr->fillTile != NULL) {
-	Tk_FreeTile(polyPtr->fillTile);
-    }
-    if (polyPtr->activeFillTile != NULL) {
-	Tk_FreeTile(polyPtr->activeFillTile);
-    }
-    if (polyPtr->disabledFillTile != NULL) {
-	Tk_FreeTile(polyPtr->disabledFillTile);
-    }
     if (polyPtr->fillColor != NULL) {
 	Tk_FreeColor(polyPtr->fillColor);
     }
@@ -655,7 +603,7 @@ DeletePolygon(canvas, itemPtr, display)
 	Tk_FreeGC(display, polyPtr->fillGC);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -833,7 +781,7 @@ ComputePolygonBbox(canvas, polyPtr)
     polyPtr->header.y1 -= 1;
     polyPtr->header.y2 += 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -907,7 +855,7 @@ TkFillPolygon(canvas, coordPtr, numPoints, display, drawable, gc, outlineGC)
 	ckfree((char *) pointPtr);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -955,9 +903,6 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	if (polyPtr->activeFillStipple != None) {
 	    stipple = polyPtr->activeFillStipple;
 	}
-	if (polyPtr->activeFillTile != NULL) {
-	    tile = polyPtr->activeFillTile;
-	}
     } else if (state==TK_STATE_DISABLED) {
 	if (polyPtr->outline.disabledWidth>0.0) {
 	    linewidth = polyPtr->outline.disabledWidth;
@@ -965,9 +910,6 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	if (polyPtr->disabledFillStipple != None) {
 	    stipple = polyPtr->disabledFillStipple;
 	}
-	if (polyPtr->disabledFillTile != NULL) {
-	    tile = polyPtr->disabledFillTile;
-	}
     }
     /*
      * If we're stippling then modify the stipple offset in the GC.  Be
@@ -975,16 +917,12 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
      * read-only.
      */
 
-    if ((tile != NULL) || (stipple != None)) {
+    if ((stipple != None) && (polyPtr->fillGC != None)) {
 	Tk_TSOffset *tsoffset = &polyPtr->tsoffset;
 	int w=0; int h=0;
 	int flags = tsoffset->flags;
 	if (!(flags & TK_OFFSET_INDEX) && (flags & (TK_OFFSET_CENTER|TK_OFFSET_MIDDLE))) {
-	    if (tile != NULL) {
-		Tk_SizeOfTile(tile, &w, &h);
-	    } else {
-		Tk_SizeOfBitmap(display, stipple, &w, &h);
-	    }
+	    Tk_SizeOfBitmap(display, stipple, &w, &h);
 	    if (flags & TK_OFFSET_CENTER) {
 		w /= 2;
 	    } else {
@@ -1004,7 +942,7 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
     }
     Tk_ChangeOutlineGC(canvas, itemPtr, &(polyPtr->outline));
 
-    if (polyPtr->numPoints < 3) {
+    if(polyPtr->numPoints < 3) {
 	short x,y;
 	int intLineWidth = (int) (linewidth + 0.5);
 	if (intLineWidth < 1) {
@@ -1012,8 +950,10 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	}
 	Tk_CanvasDrawableCoords(canvas, polyPtr->coordPtr[0],
 		    polyPtr->coordPtr[1], &x,&y);
-	XFillArc(display, drawable, polyPtr->outline.gc, x - intLineWidth/2,
-		    y - intLineWidth/2, intLineWidth+1, intLineWidth+1, 0, 64*360);
+	XFillArc(display, drawable, polyPtr->outline.gc,
+		x - intLineWidth/2, y - intLineWidth/2,
+		(unsigned int)intLineWidth+1, (unsigned int)intLineWidth+1,
+		0, 64*360);
     } else if (!polyPtr->smooth || polyPtr->numPoints < 4) {
 	TkFillPolygon(canvas, polyPtr->coordPtr, polyPtr->numPoints,
 		    display, drawable, polyPtr->fillGC, polyPtr->outline.gc);
@@ -1052,11 +992,11 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	}
     }
     Tk_ResetOutlineGC(canvas, itemPtr, &(polyPtr->outline));
-    if ((tile != NULL) && (stipple != None)) {
+    if ((stipple != None) && (polyPtr->fillGC != None)) {
 	XSetTSOrigin(display, polyPtr->fillGC, 0, 0);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1082,23 +1022,27 @@ PolygonInsert(canvas, itemPtr, beforeThis, obj)
     Tcl_Obj *obj;		/* New coordinates to be inserted. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
-    int length, argc, i;
+    int length, objc, i;
     Tcl_Obj **objv;
     double *new;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
-    if (!obj || (Tcl_ListObjGetElements((Tcl_Interp *) NULL, obj, &argc, &objv) != TCL_OK)
-	    || !argc || argc&1) {
+    if (state == TK_STATE_NULL) {
+	state = ((TkCanvas *)canvas)->canvas_state;
+    }
+
+    if (!obj || (Tcl_ListObjGetElements((Tcl_Interp *) NULL, obj, &objc, &objv) != TCL_OK)
+	    || !objc || objc&1) {
 	return;
     }
     length = 2*(polyPtr->numPoints - polyPtr->autoClosed);
     while(beforeThis>length) beforeThis-=length;
     while(beforeThis<0) beforeThis+=length;
-    new = (double *) ckalloc((unsigned)(sizeof(double) * (length + 2 + argc)));
+    new = (double *) ckalloc((unsigned)(sizeof(double) * (length + 2 + objc)));
     for (i=0; i<beforeThis; i++) {
 	new[i] = polyPtr->coordPtr[i];
     }
-    for (i=0; i<argc; i++) {
+    for (i=0; i<objc; i++) {
 	if (Tcl_GetDoubleFromObj((Tcl_Interp *) NULL,objv[i],
 		new+(i+beforeThis))!=TCL_OK) {
 	    ckfree((char *) new);
@@ -1107,10 +1051,10 @@ PolygonInsert(canvas, itemPtr, beforeThis, obj)
     }
 
     for(i=beforeThis; i<length; i++) {
-	new[i+argc] = polyPtr->coordPtr[i];
+	new[i+objc] = polyPtr->coordPtr[i];
     }
     if(polyPtr->coordPtr) ckfree((char *) polyPtr->coordPtr);
-    length+=argc;
+    length+=objc;
     polyPtr->coordPtr = new;
     polyPtr->numPoints = (length/2) + polyPtr->autoClosed;
 
@@ -1134,7 +1078,7 @@ PolygonInsert(canvas, itemPtr, beforeThis, obj)
 
     new[length] = new[0];
     new[length+1] = new[1];
-    if (((length-argc)>3) && (state != TK_STATE_HIDDEN)) {
+    if (((length-objc)>3) && (state != TK_STATE_HIDDEN)) {
 	/*
 	 * This is some optimizing code that will result that only the part
 	 * of the polygon that changed (and the objects that are overlapping
@@ -1157,11 +1101,11 @@ PolygonInsert(canvas, itemPtr, beforeThis, obj)
 
 	itemPtr->x1 = itemPtr->x2 = (int) polyPtr->coordPtr[beforeThis];
 	itemPtr->y1 = itemPtr->y2 = (int) polyPtr->coordPtr[beforeThis+1];
-	beforeThis-=2; argc+=4;
+	beforeThis-=2; objc+=4;
 	if(polyPtr->smooth) {
-	    beforeThis-=2; argc+=4;
+	    beforeThis-=2; objc+=4;
 	} /* be carefull; beforeThis could now be negative */
-	for(i=beforeThis; i<beforeThis+argc; i+=2) {
+	for(i=beforeThis; i<beforeThis+objc; i+=2) {
 		j=i;
 		if(j<0) j+=length;
 		if(j>=length) j-=length;
@@ -1186,7 +1130,7 @@ PolygonInsert(canvas, itemPtr, beforeThis, obj)
 
     ComputePolygonBbox(canvas, polyPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1249,7 +1193,7 @@ PolygonDeleteCoords(canvas, itemPtr, first, last)
     polyPtr->numPoints -= count/2;
     ComputePolygonBbox(canvas, polyPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1287,19 +1231,19 @@ PolygonToPoint(canvas, itemPtr, pointPtr)
     int changedMiterToBevel;	/* Non-zero means that a mitered corner
 				 * had to be treated as beveled after all
 				 * because the angle was < 11 degrees. */
-    int width;
+    double width;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
     bestDist = 1.0e36;
 
-    width = (int) polyPtr->outline.width;
+    width = polyPtr->outline.width;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
 	if (polyPtr->outline.activeWidth>width) {
-	    width = (int) polyPtr->outline.activeWidth;
+	    width = polyPtr->outline.activeWidth;
 	}
     } else if (state==TK_STATE_DISABLED) {
 	if (polyPtr->outline.disabledWidth>0.0) {
-	    width = (int) polyPtr->outline.disabledWidth;
+	    width = polyPtr->outline.disabledWidth;
 	}
     }
     radius = width/2.0;
@@ -1439,7 +1383,7 @@ PolygonToPoint(canvas, itemPtr, pointPtr)
     }
     return bestDist;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1534,21 +1478,16 @@ PolygonToArea(canvas, itemPtr, rectPtr)
 	polyPoints = polyPtr->coordPtr;
     }
 
-    if (polyPtr->fillGC != None) {
-	inside = TkPolygonToArea(polyPoints, numPoints, rectPtr);
-	if (inside==0) goto donearea;
-    } else {
-	if ((polyPoints[0] >= rectPtr[0])
-		&& (polyPoints[0] <= rectPtr[2])
-		&& (polyPoints[1] >= rectPtr[1])
-		&& (polyPoints[1] <= rectPtr[3])) {
-	    inside = 1;
-	}
-    }
+    /*
+     * Simple test to see if we are in the polygon.  Polygons are
+     * different from othe canvas items in that they register points
+     * being inside even if it isn't filled.
+     */
+    inside = TkPolygonToArea(polyPoints, numPoints, rectPtr);
+    if (inside==0) goto donearea;
 
     if (polyPtr->outline.gc == None) goto donearea ;
 
-
     /*
      * Iterate through all of the edges of the line, computing a polygon
      * for each edge and testing the area against that polygon.  In
@@ -1640,7 +1579,7 @@ PolygonToArea(canvas, itemPtr, rectPtr)
     }
     return inside;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1680,7 +1619,7 @@ ScalePolygon(canvas, itemPtr, originX, originY, scaleX, scaleY)
     }
     ComputePolygonBbox(canvas, polyPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1786,7 +1725,7 @@ GetPolygonIndex(interp, canvas, itemPtr, obj, indexPtr)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1824,7 +1763,7 @@ TranslatePolygon(canvas, itemPtr, deltaX, deltaY)
     }
     ComputePolygonBbox(canvas, polyPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1836,7 +1775,7 @@ TranslatePolygon(canvas, itemPtr, deltaX, deltaY)
  * Results:
  *	The return value is a standard Tcl result.  If an error
  *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used
+ *	left in the interp's result, replacing whatever used
  *	to be there.  If no error occurs, then Postscript for the
  *	item is appended to the result.
  *
@@ -1857,8 +1796,8 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 					 * collect font information;  0 means
 					 * final Postscript is being created. */
 {
-    char string[100], *style;
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
+    char *style;
     XColor *color;
     XColor *fillColor;
     Pixmap stipple;
@@ -1908,8 +1847,8 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 	    fillStipple = polyPtr->disabledFillStipple;
 	}
     }
-
     if (polyPtr->numPoints==2) {
+	char string[128];
 	sprintf(string, "%.15g %.15g translate %.15g %.15g",
 		polyPtr->coordPtr[0], Tk_CanvasPsY(canvas, polyPtr->coordPtr[1]),
 		width/2.0, width/2.0);
@@ -1929,6 +1868,7 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 	}
 	return TCL_OK;
     }
+
     /*
      * Fill the area of the polygon.
      */
@@ -1963,6 +1903,7 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
      */
 
     if (color != NULL) {
+
 	if (!polyPtr->smooth || !polyPtr->smooth->postscriptProc) {
 	    Tk_CanvasPsPath(interp, canvas, polyPtr->coordPtr,
 		polyPtr->numPoints);
@@ -1987,3 +1928,4 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
     }
     return TCL_OK;
 }
+
