@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvWind.c,v 1.2 1998/09/14 18:23:06 stanton Exp $
+ * RCS: @(#) $Id: tkCanvWind.c,v 1.7 2002/08/05 04:30:38 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -41,8 +41,8 @@ typedef struct WindowItem  {
  */
 
 static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
+    TkStateParseProc,
+    TkStatePrintProc, (ClientData) 2
 };
 static Tk_CustomOption tagsOption = {
     Tk_CanvasTagsParseProc,
@@ -76,11 +76,11 @@ static Tk_ConfigSpec configSpecs[] = {
 static void		ComputeWindowBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    WindowItem *winItemPtr));
 static int		ConfigureWinItem _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv, int flags));
+			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
+			    Tcl_Obj *CONST objv[], int flags));
 static int		CreateWinItem _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int objc, Tcl_Obj *CONST objv[]));
 static void		DeleteWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display));
 static void		DisplayWinItem _ANSI_ARGS_((Tk_Canvas canvas,
@@ -92,8 +92,8 @@ static void		ScaleWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 static void		TranslateWinItem _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double deltaX, double deltaY));
 static int		WinItemCoords _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv));
+			    Tk_Canvas canvas, Tk_Item *itemPtr, int objc,
+			    Tcl_Obj *CONST objv[]));
 static void		WinItemLostSlaveProc _ANSI_ARGS_((
 			    ClientData clientData, Tk_Window tkwin));
 static void		WinItemRequestProc _ANSI_ARGS_((ClientData clientData,
@@ -128,7 +128,7 @@ Tk_ItemType tkWindowType = {
     WinItemCoords,			/* coordProc */
     DeleteWinItem,			/* deleteProc */
     DisplayWinItem,			/* displayProc */
-    TK_ITEM_ALWAYS_REDRAW|TK_ITEM_VISITOR_SUPPORT|TK_CONFIG_OBJS,/* flags */
+    1|TK_CONFIG_OBJS,			/* flags */
     WinItemToPoint,			/* pointProc */
     WinItemToArea,			/* areaProc */
     WinItemToPostscript,	        /* postscriptProc */
@@ -140,10 +140,6 @@ Tk_ItemType tkWindowType = {
     (Tk_ItemInsertProc *) NULL,		/* insertProc */
     (Tk_ItemDCharsProc *) NULL,		/* dTextProc */
     (Tk_ItemType *) NULL,		/* nextPtr */
-    (Tk_ItemBboxProc *) ComputeWindowBbox,/* bboxProc */
-    Tk_Offset(Tk_VisitorType, visitWindow), /* acceptProc */
-    (Tk_ItemGetCoordProc *) NULL,	/* getCoordProc */
-    (Tk_ItemSetCoordProc *) NULL	/* setCoordProc */
 };
 
 
@@ -157,7 +153,7 @@ static Tk_GeomMgr canvasGeomType = {
     WinItemRequestProc,			/* requestProc */
     WinItemLostSlaveProc,		/* lostSlaveProc */
 };
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -169,7 +165,7 @@ static Tk_GeomMgr canvasGeomType = {
  * Results:
  *	A standard Tcl return value.  If an error occurred in
  *	creating the item, then an error message is left in
- *	interp->result;  in this case itemPtr is
+ *	the interp's result;  in this case itemPtr is
  *	left uninitialized, so it can be safely freed by the
  *	caller.
  *
@@ -180,30 +176,27 @@ static Tk_GeomMgr canvasGeomType = {
  */
 
 static int
-CreateWinItem(interp, canvas, itemPtr, argc, argv)
+CreateWinItem(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Interpreter for error reporting. */
     Tk_Canvas canvas;			/* Canvas to hold new item. */
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
-    int argc;				/* Number of arguments in argv. */
-    char **argv;			/* Arguments describing window. */
+    int objc;				/* Number of arguments in objv. */
+    Tcl_Obj *CONST objv[];		/* Arguments describing window. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
-    int i;
+    int i = 2;
 
-    if (argc==1) {
+    if (objc == 1) {
 	i = 1;
-    } else {
-	char *arg = Tcl_GetStringFromObj(objv[1], NULL);
-	if (((argc>1) && (arg[0] == '-')
-		&& (arg[1] >= 'a') && (arg[1] <= 'z'))) {
+    } else if (objc > 1) {
+	char *arg = Tcl_GetString(objv[1]);
+	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    i = 1;
-	} else {
-	    i = 2;
 	}
     }
 
-    if (argc < i) {
+    if (objc < i) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
 		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
 		itemPtr->typePtr->name, " x y ?options?\"",
@@ -225,10 +218,10 @@ CreateWinItem(interp, canvas, itemPtr, argc, argv)
      * Process the arguments to fill in the item record.
      */
 
-    if ((WinItemCoords(interp, canvas, itemPtr, i, argv) != TCL_OK)) {
+    if ((WinItemCoords(interp, canvas, itemPtr, i, objv) != TCL_OK)) {
 	goto error;
     }
-    if (ConfigureWinItem(interp, canvas, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
+    if (ConfigureWinItem(interp, canvas, itemPtr, objc-i, objv+i, 0) == TCL_OK) {
 	return TCL_OK;
     }
 
@@ -236,7 +229,7 @@ CreateWinItem(interp, canvas, itemPtr, argc, argv)
     DeleteWinItem(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -247,7 +240,7 @@ CreateWinItem(interp, canvas, itemPtr, argc, argv)
  *	details on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets interp->result.
+ *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
  *	The coordinates for the given item may be changed.
@@ -256,34 +249,35 @@ CreateWinItem(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-WinItemCoords(interp, canvas, itemPtr, argc, argv)
+WinItemCoords(interp, canvas, itemPtr, objc, objv)
     Tcl_Interp *interp;			/* Used for error reporting. */
     Tk_Canvas canvas;			/* Canvas containing item. */
     Tk_Item *itemPtr;			/* Item whose coordinates are to be
 					 * read or modified. */
-    int argc;				/* Number of coordinates supplied in
-					 * argv. */
-    char **argv;			/* Array of coordinates: x1, y1,
+    int objc;				/* Number of coordinates supplied in
+					 * objv. */
+    Tcl_Obj *CONST objv[];		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
-    char x[TCL_DOUBLE_SPACE];
 
-    if (argc == 0) {
+    if (objc == 0) {
 	Tcl_Obj *obj = Tcl_NewObj();
 	Tcl_Obj *subobj = Tcl_NewDoubleObj(winItemPtr->x);
 	Tcl_ListObjAppendElement(interp, obj, subobj);
 	subobj = Tcl_NewDoubleObj(winItemPtr->y);
 	Tcl_ListObjAppendElement(interp, obj, subobj);
 	Tcl_SetObjResult(interp, obj);
-    } else if (argc < 3) {
-	if (argc==1) {
-	    if (Tcl_ListObjGetElements(interp, objv[0], &argc, &objv) != TCL_OK) {
+    } else if (objc < 3) {
+	if (objc==1) {
+	    if (Tcl_ListObjGetElements(interp, objv[0], &objc,
+		    (Tcl_Obj ***) &objv) != TCL_OK) {
 		return TCL_ERROR;
-	    } else if (argc != 2) {
-		sprintf(x,"%d",argc);
-		Tcl_AppendResult(interp, "wrong # coordinates: expected 2, got ",
-		x, (char *) NULL);
+	    } else if (objc != 2) {
+		char buf[64 + TCL_INTEGER_SPACE];
+
+		sprintf(buf, "wrong # coordinates: expected 2, got %d", objc);
+		Tcl_SetResult(interp, buf, TCL_VOLATILE);
 		return TCL_ERROR;
 	    }
 	}
@@ -294,14 +288,15 @@ WinItemCoords(interp, canvas, itemPtr, argc, argv)
 	}
 	ComputeWindowBbox(canvas, winItemPtr);
     } else {
-	sprintf(x,"%d",argc);
-	Tcl_AppendResult(interp, "wrong # coordinates: expected 0 or 2, got ",
-	x, (char *) NULL);
+	char buf[64 + TCL_INTEGER_SPACE];
+
+	sprintf(buf, "wrong # coordinates: expected 0 or 2, got %d", objc);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	return TCL_ERROR;
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -312,7 +307,7 @@ WinItemCoords(interp, canvas, itemPtr, argc, argv)
  *
  * Results:
  *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in interp->result.
+ *	an error message is left in the interp's result.
  *
  * Side effects:
  *	Configuration information may be set for itemPtr.
@@ -321,12 +316,12 @@ WinItemCoords(interp, canvas, itemPtr, argc, argv)
  */
 
 static int
-ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
+ConfigureWinItem(interp, canvas, itemPtr, objc, objv, flags)
     Tcl_Interp *interp;		/* Used for error reporting. */
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Window item to reconfigure. */
-    int argc;			/* Number of elements in argv.  */
-    char **argv;		/* Arguments describing things to configure. */
+    int objc;			/* Number of elements in objv.  */
+    Tcl_Obj *CONST objv[];	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     WindowItem *winItemPtr = (WindowItem *) itemPtr;
@@ -335,8 +330,8 @@ ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
 
     oldWindow = winItemPtr->tkwin;
     canvasTkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, canvasTkwin, configSpecs, argc, argv,
-	    (char *) winItemPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (TCL_OK != Tk_ConfigureWidget(interp, canvasTkwin, configSpecs, objc,
+	    (CONST84 Tcl_Obj *CONST *) objv, (char *) winItemPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
 
@@ -359,7 +354,7 @@ ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
 	    /*
 	     * Make sure that the canvas is either the parent of the
 	     * window associated with the item or a descendant of that
-	     * parent.  Also, don't allow a top-level window to be
+	     * parent.  Also, don't allow a top-of-hierarchy window to be
 	     * managed inside a canvas.
 	     */
 
@@ -369,7 +364,7 @@ ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
 		if (ancestor == parent) {
 		    break;
 		}
-		if (((Tk_FakeWin *) (ancestor))->flags & TK_TOP_LEVEL) {
+		if (((Tk_FakeWin *) (ancestor))->flags & TK_TOP_HIERARCHY) {
 		    badWindow:
 		    Tcl_AppendResult(interp, "can't use ",
 			    Tk_PathName(winItemPtr->tkwin),
@@ -378,7 +373,7 @@ ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
 		    return TCL_ERROR;
 		}
 	    }
-	    if (((Tk_FakeWin *) (winItemPtr->tkwin))->flags & TK_TOP_LEVEL) {
+	    if (((Tk_FakeWin *) (winItemPtr->tkwin))->flags & TK_TOP_HIERARCHY) {
 		goto badWindow;
 	    }
 	    if (winItemPtr->tkwin == canvasTkwin) {
@@ -395,7 +390,7 @@ ConfigureWinItem(interp, canvas, itemPtr, argc, argv, flags)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -434,7 +429,7 @@ DeleteWinItem(canvas, itemPtr, display)
 	Tk_UnmapWindow(winItemPtr->tkwin);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -547,7 +542,7 @@ ComputeWindowBbox(canvas, winItemPtr)
     winItemPtr->header.x2 = x + width;
     winItemPtr->header.y2 = y + height;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -636,7 +631,7 @@ DisplayWinItem(canvas, itemPtr, display, drawable, regionX, regionY,
 		width, height);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -693,7 +688,7 @@ WinItemToPoint(canvas, itemPtr, pointPtr)
 
     return hypot(xDiff, yDiff);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -738,7 +733,7 @@ WinItemToArea(canvas, itemPtr, rectPtr)
     }
     return 0;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -766,7 +761,7 @@ xerrorhandler(clientData, e)
 }
 #endif
 
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -844,14 +839,11 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
     double x, y;			/* origin of window. */
     int width, height;			/* width/height of window. */
 {
-#if defined(__WIN32__) || defined(MAC_TCL)
-    return TCL_OK;
-#else
     char buffer[256];
     TkWindow *winPtr;
     XImage *ximage;
     int result;
-    Tcl_DString buffer1, buffer2;
+    Tcl_Obj *savePtr, *buffer1;
 #ifdef X_GetImage
     Tk_ErrorHandler	handle;
 #endif
@@ -867,9 +859,9 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
      * when a pixmap is used.
      */
 
-    Tcl_DStringInit(&buffer1);
-    Tcl_DStringInit(&buffer2);
-    Tcl_DStringGetResult(interp, &buffer2);
+    savePtr = Tcl_GetObjResult(interp);
+    Tcl_IncrRefCount(savePtr);
+    Tcl_ResetResult(interp);
 #ifndef _LANG
     sprintf (buffer, "%s postscript -prolog 0\n", Tk_PathName(tkwin));
     result = Tcl_Eval(interp, buffer);
@@ -878,9 +870,10 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
     result = LangMethodCall(interp,obj,"postscript",1,2,"%s %d","-prolog",0);
     Tcl_DecrRefCount(obj);
 #endif /* _LANG */
-    Tcl_DStringGetResult(interp, &buffer1);
-    Tcl_DStringResult(interp, &buffer2);
-    Tcl_DStringFree(&buffer2);
+    buffer1 = Tcl_GetObjResult(interp);
+    Tcl_IncrRefCount(buffer1);
+    Tcl_SetObjResult(interp,savePtr);
+    Tcl_DecrRefCount(savePtr);
 
     if (result == TCL_OK) {
 	Tcl_AppendResult(interp,
@@ -892,9 +885,8 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
 	Tcl_AppendResult(interp, buffer, (char *) NULL);
 	Tcl_AppendResult(interp, " 0 rlineto closepath\n",
 		"1.000 1.000 1.000 setrgbcolor AdjustColor\nfill\ngrestore\n",
-		Tcl_DStringValue(&buffer1), "\nrestore\nend\n\n\n",
+		Tcl_GetString(buffer1), "\nrestore\nend\n\n\n",
 		(char *) NULL);
-	Tcl_DStringFree(&buffer1);
 #if 0
 	for (winPtr = ((TkWindow *) tkwin)->childList; winPtr != NULL;
 		winPtr = winPtr->nextPtr) {
@@ -905,7 +897,7 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
 #endif
 	return result;
     }
-    Tcl_DStringFree(&buffer1);
+    Tcl_DecrRefCount(buffer1);
 
     /*
      * If the window is off the screen it will generate an BadMatch/XError
@@ -922,7 +914,7 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
      */
 
     ximage = XGetImage(Tk_Display(tkwin), Tk_WindowId(tkwin), 0, 0,
-	    width, height, AllPlanes, ZPixmap);
+	    (unsigned int)width, (unsigned int)height, AllPlanes, ZPixmap);
 
 #ifdef X_GetImage
     Tk_DeleteErrorHandler(handle);
@@ -937,9 +929,8 @@ CanvasPsWindow(interp, tkwin, canvas, x, y, width, height)
 
     XDestroyImage(ximage);
     return result;
-#endif
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -980,7 +971,7 @@ ScaleWinItem(canvas, itemPtr, originX, originY, scaleX, scaleY)
     }
     ComputeWindowBbox(canvas, winItemPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1012,7 +1003,7 @@ TranslateWinItem(canvas, itemPtr, deltaX, deltaY)
     winItemPtr->y += deltaY;
     ComputeWindowBbox(canvas, winItemPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1044,7 +1035,7 @@ WinItemStructureProc(clientData, eventPtr)
 	winItemPtr->tkwin = NULL;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1075,7 +1066,7 @@ WinItemRequestProc(clientData, tkwin)
     DisplayWinItem(winItemPtr->canvas, (Tk_Item *) winItemPtr,
 	    (Display *) NULL, (Drawable) None, 0, 0, 0, 0);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1111,3 +1102,4 @@ WinItemLostSlaveProc(clientData, tkwin)
     Tk_UnmapWindow(winItemPtr->tkwin);
     winItemPtr->tkwin = NULL;
 }
+
