@@ -9,8 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkFileFilter.c,v 1.2 1998/09/14 18:23:10 stanton Exp $
- *
+ * RCS: @(#) $Id: tkFileFilter.c,v 1.5 2002/01/27 11:10:50 das Exp $
  */
 
 #include "tkInt.h"
@@ -26,8 +25,8 @@ static void		FreeGlobPatterns _ANSI_ARGS_((
 static void		FreeMacFileTypes _ANSI_ARGS_((
 			    FileFilterClause * clausePtr));
 static FileFilter *	GetFilter _ANSI_ARGS_((FileFilterList * flistPtr,
-			    char * name));
-
+			    CONST char * name));
+
 /*
  *----------------------------------------------------------------------
  *
@@ -58,7 +57,7 @@ TkInitFileFilters(flistPtr)
     flistPtr->filtersTail = NULL;
     flistPtr->numFilters = 0;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -88,8 +87,8 @@ TkGetFileFilters(interp, flistPtr, arg, isWindows)
 {
     int listArgc;
     Arg * listArgv = NULL;
-    int typeCount;
     Arg * typeInfo = NULL;
+    int typeCount;
     int code = TCL_OK;
     int i;
 
@@ -125,14 +124,14 @@ TkGetFileFilters(interp, flistPtr, arg, isWindows)
 	}
 	
 	if (typeCount != 2 && typeCount != 3) {
-	    Tcl_AppendResult(interp, "bad file type \"", LangString(listArgv[i]), "\", ",
+	    Tcl_AppendResult(interp, "bad file type \"", Tcl_GetString(listArgv[i]), "\", ",
 		"should be \"typeName {extension ?extensions ...?} ",
 		"?{macType ?macTypes ...?}?\"",	NULL);
 	    code = TCL_ERROR;
 	    goto done;
 	}
 
-	filterPtr = GetFilter(flistPtr, LangString(typeInfo[0]));
+	filterPtr = GetFilter(flistPtr, Tcl_GetString(typeInfo[0]));
 
 	if (typeCount == 2) {
 	    code = AddClause(interp, filterPtr, typeInfo[1], NULL,
@@ -151,7 +150,7 @@ TkGetFileFilters(interp, flistPtr, arg, isWindows)
   done:
     return code;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -183,7 +182,7 @@ TkFreeFileFilters(flistPtr)
     }
     flistPtr->filters = NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -238,7 +237,7 @@ static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
     }
 
     /*
-     * Add the clause into the list of clauses 
+     * Add the clause into the list of clauses
      */
 
     clausePtr = (FileFilterClause*)ckalloc(sizeof(FileFilterClause));
@@ -260,14 +259,14 @@ static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
 	    GlobPattern * globPtr = (GlobPattern*)ckalloc(sizeof(GlobPattern));
 	    int len;
 	    char *globi = Tcl_GetStringFromObj(globList[i],NULL);
-	    
+	
 	    len = (strlen(globi) + 1) * sizeof(char);
 
 	    if (globi[0] && globi[0] != '*') {
 		/*
 		 * Prepend a "*" to patterns that do not have a leading "*"
 		 */
-		globPtr->pattern = (char*)ckalloc(len+1);
+		globPtr->pattern = (char*)ckalloc((unsigned int) len+1);
 		globPtr->pattern[0] = '*';
 		strcpy(globPtr->pattern+1, globi);
 	    }
@@ -286,11 +285,11 @@ static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
 		    strcpy(globPtr->pattern, "*.");
 		}
 		else {
-		    globPtr->pattern = (char*)ckalloc(len);
+		    globPtr->pattern = (char*)ckalloc((unsigned int) len);
 		    strcpy(globPtr->pattern, globi);
 		}
 	    } else {
-		globPtr->pattern = (char*)ckalloc(len);
+		globPtr->pattern = (char*)ckalloc((unsigned int) len);
 		strcpy(globPtr->pattern, globi);
 	    }
 
@@ -329,8 +328,8 @@ static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
   done:
 
     return code;
-}	
-
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -349,7 +348,7 @@ static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
 static FileFilter * GetFilter(flistPtr, name)
     FileFilterList * flistPtr;	/* The FileFilterList that contains the
 				 * newly created filter */
-    char * name;		/* Name of the filter. It is usually displayed
+    CONST char * name;		/* Name of the filter. It is usually displayed
 				 * in the "File Types" listbox in the file
 				 * dialogs. */
 {
@@ -378,7 +377,7 @@ static FileFilter * GetFilter(flistPtr, name)
     ++flistPtr->numFilters;
     return filterPtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -411,7 +410,7 @@ FreeClauses(filterPtr)
     filterPtr->clauses = NULL;
     filterPtr->clausesTail = NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -443,7 +442,7 @@ FreeGlobPatterns(clausePtr)
     }
     clausePtr->patterns = NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -474,3 +473,4 @@ FreeMacFileTypes(clausePtr)
     }
     clausePtr->macTypes = NULL;
 }
+
