@@ -1,20 +1,24 @@
-/* 
+/*
  * tkGrab.c --
  *
  *	This file provides procedures that implement grabs for Tk.
  *
  * Copyright (c) 1992-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkGrab.c,v 1.2 1998/09/14 18:23:11 stanton Exp $
+ * RCS: @(#) $Id: tkGrab.c,v 1.9 2002/08/31 06:12:20 das Exp $
  */
 
 #include "tkPort.h"
 #include "tkInt.h"
 
+#if !(defined(__WIN32__) || defined(MAC_TCL) || defined(MAC_OSX_TK))
+#include "tkUnixInt.h"
+#endif
+
 /*
  * The grab state machine has four states: ungrabbed, button pressed,
  * grabbed, and button pressed while grabbed.  In addition, there are
@@ -37,7 +41,7 @@
  *
  * The transitions between different states is given in the following
  * table:
- * 
+ *
  * Event\State	U	B	G	GB
  * -----------	--	--	--	--
  * FirstPress	B	B	GB	GB
@@ -83,7 +87,7 @@
  */
 
 /*
- * The following structure is used to pass information to 
+ * The following structure is used to pass information to
  * GrabRestrictProc from EatGrabEvents.
  */
 
@@ -165,11 +169,11 @@ static void		MovePointer2 _ANSI_ARGS_((TkWindow *sourcePtr,
 static void		QueueGrabWindowChange _ANSI_ARGS_((TkDisplay *dispPtr,
 			    TkWindow *grabWinPtr));
 static void		ReleaseButtonGrab _ANSI_ARGS_((TkDisplay *dispPtr));
-
+
 /*
  *----------------------------------------------------------------------
  *
- * Tk_GrabCmd --
+ * Tk_GrabObjCmd --
  *
  *	This procedure is invoked to process the "grab" Tcl command.
  *	See the user documentation for details on what it does.
@@ -185,139 +189,195 @@ static void		ReleaseButtonGrab _ANSI_ARGS_((TkDisplay *dispPtr));
 
 	/* ARGSUSED */
 int
-Tk_GrabCmd(clientData, interp, argc, argv)
+Tk_GrabObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with
 				 * interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    int globalGrab, c;
+    int globalGrab;
     Tk_Window tkwin;
     TkDisplay *dispPtr;
-    size_t length;
+    char *arg;
+    int index;
+    int len;
+    static CONST char *optionStrings[] = { "current", "release",
+					 "set", "status", (char *) NULL };
 
-    if (argc < 2) {
-	badArgs:
+    static CONST char *flagStrings[] = { "-global", (char *) NULL };
+
+    enum options { GRABCMD_CURRENT, GRABCMD_RELEASE,
+		       GRABCMD_SET, GRABCMD_STATUS };
+
+    if (objc < 2) {
+	/*
+	 * Can't use Tcl_WrongNumArgs here because we want the message to
+	 * read:
+	 * wrong # args: should be "cmd ?-global window" or "cmd option
+	 *    ?arg arg ...?"
+	 * We can fake it with Tcl_WrongNumArgs if we assume the command name
+	 * is "grab", but if it has been aliased, the message will be
+	 * incorrect.
+	 */
+	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " ?-global? window\" or \"", argv[0],
-		" option ?arg arg ...?\"", (char *) NULL);
+		Tcl_GetString(objv[0]), " ?-global? window\" or \"",
+		Tcl_GetString(objv[0]), " option ?arg arg ...?\"",
+		(char *) NULL);
 	return TCL_ERROR;
     }
-    c = argv[1][0];
-    length = strlen(argv[1]);
-    if (c == '.') {
-	if (argc != 2) {
-	    goto badArgs;
+
+    /*
+     * First check for a window name or "-global" as the first argument.
+     */
+
+    arg = Tcl_GetStringFromObj(objv[1], &len);
+    if (arg[0] == '.') {
+	/* [grab window] */
+	if (objc != 2) {
+	    Tcl_WrongNumArgs(interp, 1, objv, "?-global? window");
+	    return TCL_ERROR;
 	}
-	tkwin = Tk_NameToWindow(interp, argv[1], (Tk_Window) clientData);
+	tkwin = Tk_NameToWindow(interp, arg, (Tk_Window) clientData);
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
 	}
 	return Tk_Grab(interp, tkwin, 0);
-    } else if ((c == '-') && (LangCmpOpt("-global", argv[1], length) == 0)
-	    && (length >= 2)) {
-	if (argc != 3) {
-	    goto badArgs;
+    } else if (arg[0] == '-' && len > 1) {
+	if (Tcl_GetIndexFromObj(interp, objv[1], flagStrings, "option", 0,
+		&index) != TCL_OK) {
+	    return TCL_ERROR;
 	}
-	tkwin = Tk_NameToWindow(interp, argv[2], (Tk_Window) clientData);
+
+	/* [grab -global window] */
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 1, objv, "?-global? window");
+	    return TCL_ERROR;
+	}
+	tkwin = Tk_NameToWindow(interp, Tcl_GetString(objv[2]),
+		(Tk_Window) clientData);
 	if (tkwin == NULL) {
 	    return TCL_ERROR;
 	}
 	return Tk_Grab(interp, tkwin, 1);
-    } else if ((c == 'c') && (strncmp(argv[1], "current", length) == 0)) {
-	if (argc > 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " current ?window?\"", (char *) NULL);
-	    return TCL_ERROR;
+    }
+
+    /*
+     * First argument is not a window name and not "-global", find out
+     * which option it is.
+     */
+
+    if (Tcl_GetIndexFromObj(interp, objv[1], optionStrings, "option", 0,
+	    &index) != TCL_OK) {
+	return TCL_ERROR;
+    }
+
+    switch ((enum options) index) {
+	case GRABCMD_CURRENT: {
+	    /* [grab current ?window?] */
+	    if (objc > 3) {
+		Tcl_WrongNumArgs(interp, 1, objv, "current ?window?");
+		return TCL_ERROR;
+	    }
+	    if (objc == 3) {
+		tkwin = Tk_NameToWindow(interp,
+			Tcl_GetString(objv[2]), (Tk_Window) clientData);
+		if (tkwin == NULL) {
+		    return TCL_ERROR;
+		}
+		dispPtr = ((TkWindow *) tkwin)->dispPtr;
+		if (dispPtr->eventualGrabWinPtr != NULL) {
+		    Tcl_SetResult(interp,
+			    dispPtr->eventualGrabWinPtr->pathName, TCL_STATIC);
+		}
+	    } else {
+		for (dispPtr = TkGetDisplayList(); dispPtr != NULL;
+		     dispPtr = dispPtr->nextPtr) {
+		    if (dispPtr->eventualGrabWinPtr != NULL) {
+			Tcl_AppendElement(interp,
+				dispPtr->eventualGrabWinPtr->pathName);
+		    }
+		}
+	    }
+	    return TCL_OK;
 	}
-	if (argc == 3) {
-	    tkwin = Tk_NameToWindow(interp, argv[2], (Tk_Window) clientData);
+
+	case GRABCMD_RELEASE: {
+	    /* [grab release window] */
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 1, objv, "release window");
+		return TCL_ERROR;
+	    }
+	    tkwin = Tk_NameToWindow(interp,
+		    Tcl_GetString(objv[2]), (Tk_Window) clientData);
+	    if (tkwin == NULL) {
+		Tcl_ResetResult(interp);
+	    } else {
+		Tk_Ungrab(tkwin);
+	    }
+	    break;
+	}
+
+	case GRABCMD_SET: {
+	    /* [grab set ?-global? window] */
+	    if ((objc != 3) && (objc != 4)) {
+		Tcl_WrongNumArgs(interp, 1, objv, "set ?-global? window");
+		return TCL_ERROR;
+	    }
+	    if (objc == 3) {
+		globalGrab = 0;
+		tkwin = Tk_NameToWindow(interp,
+			Tcl_GetString(objv[2]), (Tk_Window) clientData);
+	    } else {
+		globalGrab = 1;
+		/*
+		 * We could just test the argument by hand instead of using
+		 * Tcl_GetIndexFromObj; the benefit of using the function is
+		 * that it sets up the error message for us, so we are
+		 * certain to be consistant with the rest of Tcl.
+		 */
+		if (Tcl_GetIndexFromObj(interp, objv[2], flagStrings, "option",
+			0, &index) != TCL_OK) {
+		    return TCL_ERROR;
+		}
+		tkwin = Tk_NameToWindow(interp,
+			Tcl_GetString(objv[3]), (Tk_Window) clientData);
+	    }
 	    if (tkwin == NULL) {
 		return TCL_ERROR;
 	    }
-	    dispPtr = ((TkWindow *) tkwin)->dispPtr;
-	    if (dispPtr->eventualGrabWinPtr != NULL) {
-		interp->result = dispPtr->eventualGrabWinPtr->pathName;
-	    }
-	} else {
-	    for (dispPtr = tkDisplayList; dispPtr != NULL;
-		    dispPtr = dispPtr->nextPtr) {
-		if (dispPtr->eventualGrabWinPtr != NULL) {
-		    Tcl_AppendElement(interp,
-			    dispPtr->eventualGrabWinPtr->pathName);
-		}
-	    }
+	    return Tk_Grab(interp, tkwin, globalGrab);
 	}
-	return TCL_OK;
-    } else if ((c == 'r') && (strncmp(argv[1], "release", length) == 0)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " release window\"", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	tkwin = Tk_NameToWindow(interp, argv[2], (Tk_Window) clientData);
-	if (tkwin == NULL) {
-	    Tcl_ResetResult(interp);
-	} else {
-	    Tk_Ungrab(tkwin);
-	}
-    } else if ((c == 's') && (strncmp(argv[1], "set", length) == 0)
-	    && (length >= 2)) {
-	if ((argc != 3) && (argc != 4)) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " set ?-global? window\"", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	if (argc == 3) {
-	    globalGrab = 0;
-	    tkwin = Tk_NameToWindow(interp, argv[2], (Tk_Window) clientData);
-	} else {
-	    globalGrab = 1;
-	    length = strlen(argv[2]);
-	    if ((LangCmpOpt("-global", argv[2], length) != 0) || (length < 2)) {
-		Tcl_AppendResult(interp, "bad argument \"", argv[2],
-			"\": must be \"", argv[0], " set ?-global? window\"",
-			(char *) NULL);
+
+	case GRABCMD_STATUS: {
+	    /* [grab status window] */
+	    TkWindow *winPtr;
+
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 1, objv, "status window");
 		return TCL_ERROR;
 	    }
-	    tkwin = Tk_NameToWindow(interp, argv[3], (Tk_Window) clientData);
+	    winPtr = (TkWindow *) Tk_NameToWindow(interp,
+		    Tcl_GetString(objv[2]), (Tk_Window) clientData);
+	    if (winPtr == NULL) {
+		return TCL_ERROR;
+	    }
+	    dispPtr = winPtr->dispPtr;
+	    if (dispPtr->eventualGrabWinPtr != winPtr) {
+		Tcl_SetResult(interp, "none", TCL_STATIC);
+	    } else if (dispPtr->grabFlags & GRAB_GLOBAL) {
+		Tcl_SetResult(interp, "global", TCL_STATIC);
+	    } else {
+		Tcl_SetResult(interp, "local", TCL_STATIC);
+	    }
+	    break;
 	}
-	if (tkwin == NULL) {
-	    return TCL_ERROR;
-	}
-	return Tk_Grab(interp, tkwin, globalGrab);
-    } else if ((c == 's') && (strncmp(argv[1], "status", length) == 0)
-	    && (length >= 2)) {
-	TkWindow *winPtr;
-
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " status window\"", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	winPtr = (TkWindow *) Tk_NameToWindow(interp, argv[2],
-		(Tk_Window) clientData);
-	if (winPtr == NULL) {
-	    return TCL_ERROR;
-	}
-	dispPtr = winPtr->dispPtr;
-	if (dispPtr->eventualGrabWinPtr != winPtr) {
-	    interp->result = "none";
-	} else if (dispPtr->grabFlags & GRAB_GLOBAL) {
-	    interp->result = "global";
-	} else {
-	    interp->result = "local";
-	}
-    } else {
-	Tcl_AppendResult(interp, "unknown or ambiguous option \"", argv[1],
-		"\": must be current, release, set, or status",
-		(char *) NULL);
-	return TCL_ERROR;
     }
+
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -329,7 +389,7 @@ Tk_GrabCmd(clientData, interp, argc, argv)
  * Results:
  *	A standard Tcl result is returned.  TCL_OK is the normal return
  *	value;  if the grab could not be set then TCL_ERROR is returned
- *	and interp->result will hold an error message.
+ *	and the interp's result will hold an error message.
  *
  * Side effects:
  *	Once this call completes successfully, no window outside the
@@ -366,7 +426,8 @@ Tk_Grab(interp, tkwin, grabGlobal)
 	}
 	if (dispPtr->eventualGrabWinPtr->mainPtr != winPtr->mainPtr) {
 	    alreadyGrabbed:
-	    interp->result = "grab failed: another application has grab";
+	    Tcl_SetResult(interp, "grab failed: another application has grab",
+		    TCL_STATIC);
 	    return TCL_ERROR;
 	}
 	Tk_Ungrab((Tk_Window) dispPtr->eventualGrabWinPtr);
@@ -431,16 +492,19 @@ Tk_Grab(interp, tkwin, grabGlobal)
 	if (grabResult != 0) {
 	    grabError:
 	    if (grabResult == GrabNotViewable) {
-		interp->result = "grab failed: window not viewable";
+		Tcl_SetResult(interp, "grab failed: window not viewable",
+			TCL_STATIC);
 	    } else if (grabResult == AlreadyGrabbed) {
 		goto alreadyGrabbed;
 	    } else if (grabResult == GrabFrozen) {
-		interp->result = "grab failed: keyboard or pointer frozen";
+		Tcl_SetResult(interp,
+			"grab failed: keyboard or pointer frozen", TCL_STATIC);
 	    } else if (grabResult == GrabInvalidTime) {
-		interp->result = "grab failed: invalid time";
+		Tcl_SetResult(interp, "grab failed: invalid time",
+			TCL_STATIC);
 	    } else {
-		char msg[100];
-	
+		char msg[64 + TCL_INTEGER_SPACE];
+
 		sprintf(msg, "grab failed for unknown reason (code %d)",
 			grabResult);
 		Tcl_AppendResult(interp, msg, (char *) NULL);
@@ -494,7 +558,7 @@ Tk_Grab(interp, tkwin, grabGlobal)
     QueueGrabWindowChange(dispPtr, winPtr);
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -565,7 +629,7 @@ Tk_Ungrab(tkwin)
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -609,7 +673,7 @@ ReleaseButtonGrab(dispPtr)
 	EatGrabEvents(dispPtr, serial);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -677,7 +741,7 @@ TkPointerEvent(eventPtr, winPtr)
 
 	if (eventPtr->xcrossing.send_event != GENERATED_EVENT_MAGIC) {
 	    if ((eventPtr->type == LeaveNotify) &&
-		    (winPtr->flags & TK_TOP_LEVEL)) {
+		    (winPtr->flags & TK_TOP_HIERARCHY)) {
 		dispPtr->serverWinPtr = NULL;
 	    } else {
 		dispPtr->serverWinPtr = winPtr;
@@ -836,7 +900,7 @@ TkPointerEvent(eventPtr, winPtr)
 
     return 1;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -876,7 +940,7 @@ TkChangeEventWindow(eventPtr, winPtr)
 	eventPtr->xmotion.subwindow = None;
 	for (childPtr = winPtr->childList; childPtr != NULL;
 		childPtr = childPtr->nextPtr) {
-	    if (childPtr->flags & TK_TOP_LEVEL) {
+	    if (childPtr->flags & TK_TOP_HIERARCHY) {
 		continue;
 	    }
 	    x = eventPtr->xmotion.x - childPtr->changes.x;
@@ -901,7 +965,7 @@ TkChangeEventWindow(eventPtr, winPtr)
 	eventPtr->xbutton.same_screen = sameScreen;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -997,7 +1061,7 @@ TkInOutEvents(eventPtr, sourcePtr, destPtr, leaveType, enterType, position)
     }
 
     if (downLevels == 0) {
-    
+
 	/*
 	 * SourcePtr is an inferior of destPtr.
 	 */
@@ -1058,7 +1122,7 @@ TkInOutEvents(eventPtr, sourcePtr, destPtr, leaveType, enterType, position)
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1124,7 +1188,7 @@ MovePointer2(sourcePtr, destPtr, mode, leaveEvents, enterEvents)
     TkInOutEvents(&event, sourcePtr, destPtr, (leaveEvents) ? LeaveNotify : 0,
 	    (enterEvents) ? EnterNotify : 0, TCL_QUEUE_MARK);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1161,7 +1225,7 @@ TkGrabDeadWindow(winPtr)
 	ReleaseButtonGrab(dispPtr);
     }
     if (dispPtr->serverWinPtr == winPtr) {
-	if (winPtr->flags & TK_TOP_LEVEL) {
+	if (winPtr->flags & TK_TOP_HIERARCHY) {
 	    dispPtr->serverWinPtr = NULL;
 	} else {
 	    dispPtr->serverWinPtr = winPtr->parentPtr;
@@ -1171,7 +1235,7 @@ TkGrabDeadWindow(winPtr)
 	dispPtr->grabWinPtr = NULL;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1211,7 +1275,7 @@ EatGrabEvents(dispPtr, serial)
     }
     Tk_RestrictEvents(oldProc, oldArg, &dummy);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1262,7 +1326,7 @@ GrabRestrictProc(arg, eventPtr)
 	return TK_DISCARD_EVENT;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1305,7 +1369,7 @@ QueueGrabWindowChange(dispPtr, grabWinPtr)
     Tcl_QueueProcEvent(GrabWinEventProc, &grabEvPtr->header, TCL_QUEUE_MARK);
     dispPtr->eventualGrabWinPtr = grabWinPtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1339,7 +1403,7 @@ GrabWinEventProc(evPtr, flags)
 	    grabEvPtr->dispPtr->display, grabEvPtr->grabWindow);
     return 1;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1385,7 +1449,7 @@ FindCommonAncestor(winPtr1, winPtr2, countPtr1, countPtr2)
     if (winPtr1 != NULL) {
 	for (winPtr = winPtr1; winPtr != NULL; winPtr = winPtr->parentPtr) {
 	    winPtr->flags |= TK_GRAB_FLAG;
-	    if (winPtr->flags & TK_TOP_LEVEL) {
+	    if (winPtr->flags & TK_TOP_HIERARCHY) {
 		break;
 	    }
 	}
@@ -1405,7 +1469,7 @@ FindCommonAncestor(winPtr1, winPtr2, countPtr1, countPtr2)
 		ancestorPtr = winPtr;
 		break;
 	    }
-	    if (winPtr->flags & TK_TOP_LEVEL)  {
+	    if (winPtr->flags & TK_TOP_HIERARCHY)  {
 		count2++;
 		break;
 	    }
@@ -1427,7 +1491,7 @@ FindCommonAncestor(winPtr1, winPtr2, countPtr1, countPtr2)
 	    if (winPtr == ancestorPtr) {
 		count1 = i;
 	    }
-	    if (winPtr->flags & TK_TOP_LEVEL) {
+	    if (winPtr->flags & TK_TOP_HIERARCHY) {
 		if (count1 == -1) {
 		    count1 = i+1;
 		}
@@ -1440,7 +1504,7 @@ FindCommonAncestor(winPtr1, winPtr2, countPtr1, countPtr2)
     *countPtr2 = count2;
     return ancestorPtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1477,7 +1541,7 @@ TkPositionInTree(winPtr, treePtr)
 		if (winPtr2 == winPtr) {
 		    return TK_GRAB_ANCESTOR;
 		}
-		if (winPtr2->flags & TK_TOP_LEVEL) {
+		if (winPtr2->flags & TK_TOP_HIERARCHY) {
 		    break;
 		}
 	    }
@@ -1486,7 +1550,7 @@ TkPositionInTree(winPtr, treePtr)
     }
     return TK_GRAB_IN_TREE;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1531,3 +1595,4 @@ TkGrabState(winPtr)
 
     return TkPositionInTree(winPtr, grabWinPtr);
 }
+
