@@ -1,4 +1,4 @@
-/* 
+/*
  * tkImage.c --
  *
  *	This module implements the image protocol, which allows lots
@@ -6,12 +6,12 @@
  *	widgets.
  *
  * Copyright (c) 1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkImage.c,v 1.2 1998/09/14 18:23:12 stanton Exp $
+ * RCS: @(#) $Id: tkImage.c,v 1.17 2002/08/05 04:30:39 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -69,25 +69,30 @@ typedef struct ImageMaster {
 				 * entry). */
     Image *instancePtr;		/* Pointer to first in list of instances
 				 * derived from this name. */
+    int deleted;		/* Flag set when image is being deleted. */
+    TkWindow *winPtr;		/* Main window of interpreter (used to
+				 * detect when the world is falling apart.) */
 } ImageMaster;
 
-/*
- * The following variable points to the first in a list of all known
- * image types.
- */
-
-static Tk_ImageType *imageTypeList = NULL;
+typedef struct ThreadSpecificData {
+    Tk_ImageType *imageTypeList;/* First in a list of all known image
+				 * types. */
+    Tk_ImageType *oldImageTypeList;/* First in a list of all known old-style image
+				 * types. */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Prototypes for local procedures:
  */
 
-static void		DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
-
+static void	DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+static void	EventuallyDeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+
 /*
  *----------------------------------------------------------------------
  *
- * Tk_CreateImageType --
+ * Tk_CreateOldImageType, Tk_CreateImageType --
  *
  *	This procedure is invoked by an image manager to tell Tk about
  *	a new kind of image and the procedures that manage the new type.
@@ -103,6 +108,20 @@ static void		DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
  *----------------------------------------------------------------------
  */
 
+void
+Tk_CreateOldImageType(typePtr)
+    Tk_ImageType *typePtr;	/* Structure describing the type.  All of
+				 * the fields except "nextPtr" must be filled
+				 * in by caller.  Must not have been passed
+				 * to Tk_CreateImageType previously. */
+{
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+    typePtr->nextPtr = tsdPtr->oldImageTypeList;
+    tsdPtr->oldImageTypeList = typePtr;
+}
+
 void
 Tk_CreateImageType(typePtr)
     Tk_ImageType *typePtr;	/* Structure describing the type.  All of
@@ -110,10 +129,13 @@ Tk_CreateImageType(typePtr)
 				 * in by caller.  Must not have been passed
 				 * to Tk_CreateImageType previously. */
 {
-    typePtr->nextPtr = imageTypeList;
-    imageTypeList = typePtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+    typePtr->nextPtr = tsdPtr->imageTypeList;
+    tsdPtr->imageTypeList = typePtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -132,214 +154,296 @@ Tk_CreateImageType(typePtr)
  */
 
 int
-Tk_ImageObjCmd(clientData, interp, argc, objv)
+Tk_ImageObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Main window associated with interpreter. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
+    int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument strings. */
 {
+    static CONST char *imageOptions[] = {
+	"create", "delete", "height", "inuse", "names", "type", "types",
+	    "width", (char *) NULL
+    };
+    enum options {
+	IMAGE_CREATE, IMAGE_DELETE, IMAGE_HEIGHT, IMAGE_INUSE, IMAGE_NAMES,
+	IMAGE_TYPE, IMAGE_TYPES, IMAGE_WIDTH
+    };
     TkWindow *winPtr = (TkWindow *) clientData;
-    int c, i, new, firstOption;
-    size_t length;
+    int i, new, firstOption,  index;
     Tk_ImageType *typePtr;
     ImageMaster *masterPtr;
     Image *imagePtr;
     Tcl_HashEntry *hPtr;
     Tcl_HashSearch search;
-    char idString[30], *name;
-    static int id = 0;
-    static char **strv = NULL;
+    char idString[16 + TCL_INTEGER_SPACE], *name;
+    TkDisplay *dispPtr = winPtr->dispPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    if (argc < 2) {
-	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option ?args?");
 	return TCL_ERROR;
     }
-    if (strv) {
-	ckfree((char *) strv);
-    } 
-    strv = (char **) ckalloc((argc+1) * sizeof(char *));
-    strv[argc] = NULL;
-    for (i = 0; i < argc; i++) {
-	strv[i]=Tcl_GetStringFromObj(objv[i], (int *) NULL);
+
+    if (Tcl_GetIndexFromObj(interp, objv[1], imageOptions, "option", 0,
+	    &index) != TCL_OK) {
+	return TCL_ERROR;
     }
-    c = strv[1][0];
-    length = strlen(strv[1]);
-    if ((c == 'c') && (strncmp(strv[1], "create", length) == 0)) {
-	if (argc < 3) {
-	    Tcl_WrongNumArgs(interp, 2, objv, "type ?name? ?options?");
-	    return TCL_ERROR;
-	}
-	c = strv[2][0];
-
-	/*
-	 * Look up the image type.
-	 */
-
-	for (typePtr = imageTypeList; typePtr != NULL;
-		typePtr = typePtr->nextPtr) {
-	    if ((c == typePtr->name[0])
-		    && (strcmp(strv[2], typePtr->name) == 0)) {
-		break;
+    switch ((enum options) index) {
+	case IMAGE_CREATE: {
+	    char *arg;
+	    Tcl_Obj **args;
+	    int oldimage = 0;
+	    if (objc < 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "type ?name? ?options?");
+		return TCL_ERROR;
 	    }
-	}
-	if (typePtr == NULL) {
-	    Tcl_AppendResult(interp, "image type \"", strv[2],
-		    "\" doesn't exist", (char *) NULL);
-	    return TCL_ERROR;
-	}
 
-	/*
-	 * Figure out a name to use for the new image.
-	 */
-
-	if ((argc == 3) || (strv[3][0] == '-')) {
-	    id++;
-	    sprintf(idString, "image%d", id);
-	    name = idString;
-	    firstOption = 3;
-	} else {
-	    name = strv[3];
-	    firstOption = 4;
-	}
-
-	/*
-	 * Create the data structure for the new image.
-	 */
-
-	hPtr = Tcl_CreateHashEntry(&winPtr->mainPtr->imageTable, name, &new);
-	if (new) {
-	    masterPtr = (ImageMaster *) ckalloc(sizeof(ImageMaster));
-	    masterPtr->typePtr = NULL;
-	    masterPtr->masterData = NULL;
-	    masterPtr->width = masterPtr->height = 1;
-	    masterPtr->tablePtr = &winPtr->mainPtr->imageTable;
-	    masterPtr->hPtr = hPtr;
-	    masterPtr->instancePtr = NULL;
-	    Tcl_SetHashValue(hPtr, masterPtr);
-	} else {
 	    /*
-	     * An image already exists by this name.  Disconnect the
-	     * instances from the master.
+	     * Look up the image type.
 	     */
 
-	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	    if (masterPtr->typePtr != NULL) {
-		for (imagePtr = masterPtr->instancePtr; imagePtr != NULL;
-			imagePtr = imagePtr->nextPtr) {
-		   (*masterPtr->typePtr->freeProc)(
-			   imagePtr->instanceData, imagePtr->display);
-		   (*imagePtr->changeProc)(imagePtr->widgetClientData, 0, 0,
-			masterPtr->width, masterPtr->height, masterPtr->width,
-			masterPtr->height);
+	    arg = Tcl_GetString(objv[2]);
+	    for (typePtr = tsdPtr->imageTypeList; typePtr != NULL;
+		 typePtr = typePtr->nextPtr) {
+		if ((*arg == typePtr->name[0])
+			&& (strcmp(arg, typePtr->name) == 0)) {
+		    break;
 		}
-		(*masterPtr->typePtr->deleteProc)(masterPtr->masterData);
-		masterPtr->typePtr = NULL;
 	    }
-	}
+	    if (typePtr == NULL) {
+		oldimage = 1;
+		for (typePtr = tsdPtr->oldImageTypeList; typePtr != NULL;
+		     typePtr = typePtr->nextPtr) {
+		    if ((*arg == typePtr->name[0])
+			    && (strcmp(arg, typePtr->name) == 0)) {
+			break;
+		    }
+		}
+	    }
+	    if (typePtr == NULL) {
+		Tcl_AppendResult(interp, "image type \"", arg,
+			"\" doesn't exist", (char *) NULL);
+		return TCL_ERROR;
+	    }
 
-	/*
-	 * Call the image type manager so that it can perform its own
-	 * initialization, then re-"get" for any existing instances of
-	 * the image.
-	 */
+	    /*
+	     * Figure out a name to use for the new image.
+	     */
 
-	if ((*typePtr->createProc)(interp, name, argc-firstOption,
-		(Tcl_Obj **)(objv+firstOption), typePtr, (Tk_ImageMaster) masterPtr,
-		&masterPtr->masterData) != TCL_OK) {
-	    DeleteImage(masterPtr);
-	    return TCL_ERROR;
+	    if ((objc == 3) || (*(arg = Tcl_GetString(objv[3])) == '-')) {
+		dispPtr->imageId++;
+		sprintf(idString, "image%d", dispPtr->imageId);
+		name = idString;
+		firstOption = 3;
+	    } else {
+		name = arg;
+		firstOption = 4;
+	    }
+
+	    /*
+	     * Create the data structure for the new image.
+	     */
+
+	    hPtr = Tcl_CreateHashEntry(&winPtr->mainPtr->imageTable,
+		    name, &new);
+	    if (new) {
+		masterPtr = (ImageMaster *) ckalloc(sizeof(ImageMaster));
+		masterPtr->typePtr = NULL;
+		masterPtr->masterData = NULL;
+		masterPtr->width = masterPtr->height = 1;
+		masterPtr->tablePtr = &winPtr->mainPtr->imageTable;
+		masterPtr->hPtr = hPtr;
+		masterPtr->instancePtr = NULL;
+		masterPtr->deleted = 0;
+		masterPtr->winPtr = winPtr->mainPtr->winPtr;
+		Tcl_Preserve((ClientData) masterPtr->winPtr);
+		Tcl_SetHashValue(hPtr, masterPtr);
+	    } else {
+		/*
+		 * An image already exists by this name.  Disconnect the
+		 * instances from the master.
+		 */
+
+		masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
+		if (masterPtr->typePtr != NULL) {
+		    for (imagePtr = masterPtr->instancePtr; imagePtr != NULL;
+			 imagePtr = imagePtr->nextPtr) {
+			(*masterPtr->typePtr->freeProc)(
+			    imagePtr->instanceData, imagePtr->display);
+			(*imagePtr->changeProc)(imagePtr->widgetClientData,
+				0, 0, masterPtr->width, masterPtr->height,
+				masterPtr->width, masterPtr->height);
+		    }
+		    (*masterPtr->typePtr->deleteProc)(masterPtr->masterData);
+		    masterPtr->typePtr = NULL;
+		}
+	    }
+
+	    /*
+	     * Call the image type manager so that it can perform its own
+	     * initialization, then re-"get" for any existing instances of
+	     * the image.
+	     */
+
+	    objv += firstOption;
+	    objc -= firstOption;
+	    args = (Tcl_Obj **) objv;
+	    if (oldimage) {
+		int i;
+		args = (Tcl_Obj **) ckalloc((objc+1) * sizeof(char *));
+		for (i = 0; i < objc; i++) {
+		    args[i] = (Tcl_Obj *) Tcl_GetString(objv[i]);
+		}
+		args[objc] = NULL;
+	    }
+	    Tcl_Preserve((ClientData) masterPtr);
+	    if ((*typePtr->createProc)(interp, name, objc,
+		    args, typePtr, (Tk_ImageMaster) masterPtr,
+		    &masterPtr->masterData) != TCL_OK) {
+		EventuallyDeleteImage(masterPtr);
+		Tcl_Release((ClientData) masterPtr);
+		if (oldimage) {
+		    ckfree((char *) args);
+		}
+		return TCL_ERROR;
+	    }
+	    Tcl_Release((ClientData) masterPtr);
+	    if (oldimage) {
+		ckfree((char *) args);
+	    }
+	    masterPtr->typePtr = typePtr;
+	    for (imagePtr = masterPtr->instancePtr; imagePtr != NULL;
+		 imagePtr = imagePtr->nextPtr) {
+		imagePtr->instanceData = (*typePtr->getProc)(
+		    imagePtr->tkwin, masterPtr->masterData);
+	    }
+	    Tcl_SetResult(interp,
+		    Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr),
+		    TCL_STATIC);
+	    break;
 	}
-	masterPtr->typePtr = typePtr;
-	for (imagePtr = masterPtr->instancePtr; imagePtr != NULL;
-		imagePtr = imagePtr->nextPtr) {
-	   imagePtr->instanceData = (*typePtr->getProc)(
-		   imagePtr->tkwin, masterPtr->masterData);
+	case IMAGE_DELETE: {
+	    for (i = 2; i < objc; i++) {
+		char *arg = Tcl_GetString(objv[i]);
+		hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
+		if (hPtr == NULL) {
+		    Tcl_AppendResult(interp, "image \"", arg,
+			    "\" doesn't exist", (char *) NULL);
+		    return TCL_ERROR;
+		}
+		DeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
+	    }
+	    break;
 	}
-        Tcl_ArgResult(interp, LangObjectArg( interp, Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr)));
-    } else if ((c == 'd') && (strncmp(strv[1], "delete", length) == 0)) {
-	for (i = 2; i < argc; i++) {
-	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, strv[i]);
+	case IMAGE_HEIGHT: {
+	    char *arg;
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name");
+		return TCL_ERROR;
+	    }
+	    arg = Tcl_GetString(objv[2]);
+	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
 	    if (hPtr == NULL) {
-	    Tcl_AppendResult(interp, "image \"", strv[i],
-		    "\" doesn't exist", (char *) NULL);
+		Tcl_AppendResult(interp, "image \"", arg,
+			"\" doesn't exist", (char *) NULL);
 		return TCL_ERROR;
 	    }
 	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	    DeleteImage(masterPtr);
+	    Tcl_SetIntObj(Tcl_GetObjResult(interp), masterPtr->height);
+	    break;
 	}
-    } else if ((c == 'h') && (strncmp(strv[1], "height", length) == 0)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", strv[0],
-		    " height name\"", (char *) NULL);
-	    return TCL_ERROR;
+
+	case IMAGE_INUSE: {
+	    int count = 0;
+	    char *arg;
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name");
+		return TCL_ERROR;
+	    }
+	    arg = Tcl_GetString(objv[2]);
+	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
+	    if (hPtr == NULL) {
+		Tcl_AppendResult(interp, "image \"", arg,
+			"\" doesn't exist", (char *) NULL);
+		return TCL_ERROR;
+	    }
+	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
+	    if (masterPtr->typePtr != NULL && masterPtr->instancePtr != NULL) {
+		count = 1;
+	    }
+	    Tcl_SetBooleanObj(Tcl_GetObjResult(interp), count);
+	    break;
 	}
-	hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, strv[2]);
-	if (hPtr == NULL) {
-	    Tcl_AppendResult(interp, "image \"", strv[2],
-		    "\" doesn't exist", (char *) NULL);
-	    return TCL_ERROR;
+
+	case IMAGE_NAMES: {
+	    if (objc != 2) {
+		Tcl_WrongNumArgs(interp, 2, objv, NULL);
+		return TCL_ERROR;
+	    }
+	    hPtr = Tcl_FirstHashEntry(&winPtr->mainPtr->imageTable, &search);
+	    for ( ; hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
+		Tcl_AppendElement(interp, Tcl_GetHashKey(
+		    &winPtr->mainPtr->imageTable, hPtr));
+	    }
+	    break;
 	}
-	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	Tcl_SetIntObj(Tcl_GetObjResult(interp), masterPtr->height);
-    } else if ((c == 'n') && (strncmp(strv[1], "names", length) == 0)) {
-	if (argc != 2) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", strv[0],
-		    " names\"", (char *) NULL);
-	    return TCL_ERROR;
+
+	case IMAGE_TYPE: {
+	    char *arg;
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name");
+		return TCL_ERROR;
+	    }
+	    arg = Tcl_GetString(objv[2]);
+	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
+	    if (hPtr == NULL) {
+		Tcl_AppendResult(interp, "image \"", arg,
+			"\" doesn't exist", (char *) NULL);
+		return TCL_ERROR;
+	    }
+	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
+	    if (masterPtr->typePtr != NULL) {
+		Tcl_SetResult(interp, masterPtr->typePtr->name, TCL_STATIC);
+	    }
+	    break;
 	}
-	for (hPtr = Tcl_FirstHashEntry(&winPtr->mainPtr->imageTable, &search);
-		hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
-	    Tcl_AppendArg(interp, LangObjectArg(interp,Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr)));
+	case IMAGE_TYPES: {
+	    if (objc != 2) {
+		Tcl_WrongNumArgs(interp, 2, objv, NULL);
+		return TCL_ERROR;
+	    }
+	    for (typePtr = tsdPtr->imageTypeList; typePtr != NULL;
+		 typePtr = typePtr->nextPtr) {
+		Tcl_AppendElement(interp, typePtr->name);
+	    }
+	    for (typePtr = tsdPtr->oldImageTypeList; typePtr != NULL;
+		 typePtr = typePtr->nextPtr) {
+		Tcl_AppendElement(interp, typePtr->name);
+	    }
+	    break;
 	}
-    } else if ((c == 't') && (strcmp(strv[1], "type") == 0)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", strv[0],
-		    " type name\"", (char *) NULL);
-	    return TCL_ERROR;
+	case IMAGE_WIDTH: {
+	    char *arg;
+	    if (objc != 3) {
+		Tcl_WrongNumArgs(interp, 2, objv, "name");
+		return TCL_ERROR;
+	    }
+	    arg = Tcl_GetString(objv[2]);
+	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, arg);
+	    if (hPtr == NULL) {
+		Tcl_AppendResult(interp, "image \"", arg,
+			"\" doesn't exist", (char *) NULL);
+		return TCL_ERROR;
+	    }
+	    masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
+	    Tcl_SetIntObj(Tcl_GetObjResult(interp), masterPtr->width);
+	    break;
 	}
-	hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, strv[2]);
-	if (hPtr == NULL) {
-	    Tcl_AppendResult(interp, "image \"", strv[2],
-		    "\" doesn't exist", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	if (masterPtr->typePtr != NULL) {
-	    Tcl_AppendResult(interp, masterPtr->typePtr->name, (char *) NULL);
-	}
-    } else if ((c == 't') && (strcmp(strv[1], "types") == 0)) {
-	if (argc != 2) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", strv[0],
-		    " types\"", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	for (typePtr = imageTypeList; typePtr != NULL;
-		typePtr = typePtr->nextPtr) {
-	    Tcl_AppendElement(interp, typePtr->name);
-	}
-    } else if ((c == 'w') && (strncmp(strv[1], "width", length) == 0)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", strv[0],
-		    " width name\"", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, strv[2]);
-	if (hPtr == NULL) {
-	    Tcl_AppendResult(interp, "image \"", strv[2],
-		    "\" doesn't exist", (char *) NULL);
-	    return TCL_ERROR;
-	}
-	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	Tcl_SetIntObj(Tcl_GetObjResult(interp), masterPtr->width);
-    } else {
-	Tcl_AppendResult(interp, "bad option \"", strv[1],
-		"\": must be create, delete, height, names, type, types,",
-		" or width", (char *) NULL);
-	return TCL_ERROR;
     }
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -379,12 +483,12 @@ Tk_ImageChanged(imageMaster, x, y, width, height, imageWidth,
     masterPtr->width = imageWidth;
     masterPtr->height = imageHeight;
     for (imagePtr = masterPtr->instancePtr; imagePtr != NULL;
-	    imagePtr = imagePtr->nextPtr) {
+	 imagePtr = imagePtr->nextPtr) {
 	(*imagePtr->changeProc)(imagePtr->widgetClientData, x, y,
-	    width, height, imageWidth, imageHeight);
+		width, height, imageWidth, imageHeight);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -402,7 +506,7 @@ Tk_ImageChanged(imageMaster, x, y, width, height, imageWidth,
  *----------------------------------------------------------------------
  */
 
-char *
+CONST char *
 Tk_NameOfImage(imageMaster)
     Tk_ImageMaster imageMaster;		/* Token for image. */
 {
@@ -410,7 +514,7 @@ Tk_NameOfImage(imageMaster)
 
     return Tcl_GetHashKey(masterPtr->tablePtr, masterPtr->hPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -422,7 +526,7 @@ Tk_NameOfImage(imageMaster)
  * Results:
  *	The return value is a token for the image.  If there is no image
  *	by the given name, then NULL is returned and an error message is
- *	left in interp->result.
+ *	left in the interp's result.
  *
  * Side effects:
  *	Tk records the fact that the widget is using the image, and
@@ -440,7 +544,7 @@ Tk_GetImage(interp, tkwin, name, changeProc, clientData)
 				 * can't be found. */
     Tk_Window tkwin;		/* Token for window in which image will
 				 * be used. */
-    char *name;			/* Name of desired image. */
+    CONST char *name;		/* Name of desired image. */
     Tk_ImageChangedProc *changeProc;
 				/* Procedure to invoke when redisplay is
 				 * needed because image's pixels or size
@@ -476,7 +580,7 @@ Tk_GetImage(interp, tkwin, name, changeProc, clientData)
 	    (char *) NULL);
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -523,7 +627,7 @@ Tk_FreeImage(image)
     }
     ckfree((char *) imagePtr);
 
-    /* 
+    /*
      * If there are no more instances left for the master, and if the
      * master image has been deleted, then delete the master too.
      */
@@ -535,7 +639,102 @@ Tk_FreeImage(image)
 	ckfree((char *) masterPtr);
     }
 }
-
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_PostscriptImage --
+ *
+ *	This procedure is called by widgets that contain images in order
+ *	to redisplay an image on the screen or an off-screen pixmap.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The image's manager is notified, and it redraws the desired
+ *	portion of the image before returning.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
+    Tk_Image image;		/* Token for image to redisplay. */
+    Tcl_Interp *interp;
+    Tk_Window tkwin;
+    Tk_PostscriptInfo psinfo;	/* postscript info */
+    int x, y;			/* Upper-left pixel of region in image that
+				 * needs to be redisplayed. */
+    int width, height;		/* Dimensions of region to redraw. */
+    int prepass;
+{
+    Image *imagePtr = (Image *) image;
+    int result;
+    XImage *ximage;
+    Pixmap pmap;
+    GC newGC;
+    XGCValues gcValues;
+
+    if (imagePtr->masterPtr->typePtr == NULL) {
+	/*
+	 * No master for image, so nothing to display on postscript.
+	 */
+	return TCL_OK;
+    }
+
+    /*
+     * Check if an image specific postscript-generation function
+     * exists; otherwise go on with generic code.
+     */
+
+    if (imagePtr->masterPtr->typePtr->postscriptProc != NULL) {
+	return (*imagePtr->masterPtr->typePtr->postscriptProc)(
+	    imagePtr->masterPtr->masterData, interp, tkwin, psinfo,
+	    x, y, width, height, prepass);
+    }
+
+    if (prepass) {
+	return TCL_OK;
+    }
+
+    /*
+     * Create a Pixmap, tell the image to redraw itself there, and then
+     * generate an XImage from the Pixmap.  We can then read pixel
+     * values out of the XImage.
+     */
+
+    pmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
+                        width, height, Tk_Depth(tkwin));
+
+    gcValues.foreground = WhitePixelOfScreen(Tk_Screen(tkwin));
+    newGC = Tk_GetGC(tkwin, GCForeground, &gcValues);
+    if (newGC != None) {
+	XFillRectangle(Tk_Display(tkwin), pmap, newGC,
+		0, 0, (unsigned int)width, (unsigned int)height);
+	Tk_FreeGC(Tk_Display(tkwin), newGC);
+    }
+
+    Tk_RedrawImage(image, x, y, width, height, pmap, 0, 0);
+
+    ximage = XGetImage(Tk_Display(tkwin), pmap, 0, 0,
+	    (unsigned int)width, (unsigned int)height, AllPlanes, ZPixmap);
+
+    Tk_FreePixmap(Tk_Display(tkwin), pmap);
+
+    if (ximage == NULL) {
+	/* The XGetImage() function is apparently not
+	 * implemented on this system. Just ignore it.
+	 */
+	return TCL_OK;
+    }
+    result = TkPostscriptImage(interp, tkwin, psinfo, ximage, x, y,
+	    width, height);
+
+    XDestroyImage(ximage);
+    return result;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -603,7 +802,7 @@ Tk_RedrawImage(image, imageX, imageY, width, height, drawable,
 	    imagePtr->instanceData, imagePtr->display, drawable,
 	    imageX, imageY, width, height, drawableX, drawableY);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -632,7 +831,7 @@ Tk_SizeOfImage(image, widthPtr, heightPtr)
     *widthPtr = imagePtr->masterPtr->width;
     *heightPtr = imagePtr->masterPtr->height;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -656,7 +855,7 @@ void
 Tk_DeleteImage(interp, name)
     Tcl_Interp *interp;		/* Interpreter in which the image was
 				 * created. */
-    char *name;			/* Name of image. */
+    CONST char *name;		/* Name of image. */
 {
     Tcl_HashEntry *hPtr;
     TkWindow *winPtr;
@@ -669,9 +868,9 @@ Tk_DeleteImage(interp, name)
     if (hPtr == NULL) {
 	return;
     }
-    DeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
+    DeleteImage((ImageMaster *)Tcl_GetHashValue(hPtr));
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -711,13 +910,43 @@ DeleteImage(masterPtr)
 	(*typePtr->deleteProc)(masterPtr->masterData);
     }
     if (masterPtr->instancePtr == NULL) {
-        if (masterPtr->hPtr != NULL) {
+        if ((masterPtr->winPtr->flags & TK_ALREADY_DEAD) == 0) {
 	    Tcl_DeleteHashEntry(masterPtr->hPtr);
-        }
+	}
+	Tcl_Release((ClientData) masterPtr->winPtr);
 	ckfree((char *) masterPtr);
     }
 }
-
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * EventuallyDeleteImage --
+ *
+ *	Arrange for an image to be deleted when it is safe to do so.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Image will get freed, though not until it is no longer
+ *	Tcl_Preserve()d by anything.  May be called multiple times on
+ *	the same image without ill effects.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+EventuallyDeleteImage(masterPtr)
+    ImageMaster *masterPtr;	/* Pointer to main data structure for image. */
+{
+    if (!masterPtr->deleted) {
+	masterPtr->deleted = 1;
+	Tcl_EventuallyFree((ClientData) masterPtr,
+		(Tcl_FreeProc *)DeleteImage);
+    }
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -744,17 +973,14 @@ TkDeleteAllImages(mainPtr)
 {
     Tcl_HashSearch search;
     Tcl_HashEntry *hPtr;
-    ImageMaster *masterPtr;
 
     for (hPtr = Tcl_FirstHashEntry(&mainPtr->imageTable, &search);
 	    hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
-	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
-	DeleteImage(masterPtr);
-        masterPtr->hPtr = NULL; 
+	EventuallyDeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
     }
     Tcl_DeleteHashTable(&mainPtr->imageTable);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -780,7 +1006,7 @@ ClientData
 Tk_GetImageMasterData(interp, name, typePtrPtr)
     Tcl_Interp *interp;		/* Interpreter in which the image was
 				 * created. */
-    char *name;			/* Name of image. */
+    CONST char *name;		/* Name of image. */
     Tk_ImageType **typePtrPtr;	/* Points to location to fill in with
 				 * pointer to type information for image. */
 {
@@ -798,3 +1024,250 @@ Tk_GetImageMasterData(interp, name, typePtrPtr)
     *typePtrPtr = masterPtr->typePtr;
     return masterPtr->masterData;
 }
+
+/* ARGSUSED */
+static void
+TileChangedProc(clientData, x, y, width, height, imageWidth, imageHeight)
+    ClientData clientData;
+    int x, y, width, height;	/* Not used */
+    int imageWidth, imageHeight;
+{
+}
+
+/*========================================================================
+ * Tile support - reimplemented avoiding BLTisms
+ *========================================================================*/
+
+struct Tk_Tile_
+{
+ Tk_Image image;
+ int width;
+ int height;
+ Tk_Window tkwin;
+ Pixmap   pixmap;
+};
+
+static void		TileImageChanged _ANSI_ARGS_((ClientData clientData,
+			    int x, int y, int width, int height,
+			    int imgWidth, int imgHeight));
+
+void
+TileImageChanged(clientData, x, y, width, height, imgWidth, imgHeight)
+ClientData clientData;
+int x;
+int y;
+int width;
+int height;
+{
+#ifdef NOT_YET
+    Tk_Tile tile = (Tk_Tile) clientData;
+    if (tile->width < imgWidth || tile->height < imgHeight) {
+	Tk_Window tkwin = tile->tkwin;
+	if (tile->pixmap) {
+	    Tk_FreePixmap(Tk_Display(tkwin),tile->pixmap);
+	    tile->width  = 0;
+	    tile->height = 0;
+	}
+	tile->pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
+				imgWidth, imgHeight, Tk_Depth(tkwin));
+	if (tile->pixmap) {
+	    tile->width  = imgWidth;
+	    tile->height = imgHeight;
+	}
+    }
+    if (tile->pixmap) {
+	Tk_RedrawImage(tile->image, 0, 0, tile->width, tile->height,
+		       tile->pixmap, 0, 0);
+    }
+#endif
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_GetTile
+ *
+ *	Convert the named image into a tile.
+ *
+ * Results:
+ *	If the image is valid, a new tile is returned.  If the name
+ *	does not represent a proper image, an error message is left in
+ *	interp->result.
+ *
+ * Side Effects:
+ *	Memory and X resources are allocated.  Bookkeeping is
+ *	maintained on the tile (i.e. width, height, and name).
+ *
+ *----------------------------------------------------------------------
+ */
+Tk_Tile
+Tk_GetTile(interp, tkwin, imageName)
+    Tcl_Interp *interp;		/* Interpreter to report results back to */
+    Tk_Window tkwin;		/* Window on the same display as tile */
+    CONST char *imageName;	/* Name of image */
+{
+    Tk_Tile tile = (Tk_Tile) ckalloc(sizeof(struct Tk_Tile_));
+    memset(tile,0,sizeof(*tile));
+    tile->tkwin = tkwin;
+    tile->image = Tk_GetImage(interp, tkwin, (char *) imageName,
+			TileImageChanged, (ClientData) tile);
+    if (!tile->image) {
+	Tk_FreeTile(tile);
+	return NULL;
+    }
+    return tile;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_FreeTile
+ *
+ *	Release the resources associated with the tile.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Memory and X resources are freed.  Bookkeeping information
+ *	about the tile (i.e. width, height, and name) is discarded.
+ *
+ */
+void
+Tk_FreeTile(tile)
+    Tk_Tile tile;		/* Tile to be deleted */
+{
+ if (tile->image)
+  Tk_FreeImage(tile->image);
+ if (tile->pixmap)
+  Tk_FreePixmap(Tk_Display(tile->tkwin),tile->pixmap);
+ ckfree((char *) tile);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_NameOfTile
+ *
+ *	Returns the name of the image from which the tile was
+ *	generated.
+ *
+ * Results:
+ *	The name of the image is returned.  The name is not unique.
+ *	Many tiles may use the same image.
+ *
+ *----------------------------------------------------------------------
+ */
+CONST char *
+Tk_NameOfTile(tile)
+    Tk_Tile tile;		/* Tile to query */
+{
+ if (tile)
+  {
+   Image *imagePtr = (Image *) tile->image;
+   return Tk_NameOfImage((Tk_ImageMaster)(imagePtr->masterPtr));
+  }
+ return NULL;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_PixmapOfTile
+ *
+ *	Returns the pixmap of the tile.
+ *
+ * Results:
+ *	The X pixmap used as the tile is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+Pixmap
+Tk_PixmapOfTile(tile)
+    Tk_Tile tile;		/* Tile to query */
+{
+    return tile->pixmap;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_SizeOfTile
+ *
+ *	Returns the width and height of the tile.
+ *
+ * Results:
+ *	The width and height of the tile are returned.
+ *
+ *----------------------------------------------------------------------
+ */
+void
+Tk_SizeOfTile(tile, widthPtr, heightPtr)
+    Tk_Tile tile;		/* Tile to query */
+    int *widthPtr, *heightPtr;	/* Returned dimensions of the tile (out) */
+{
+    Tk_SizeOfImage(tile->image,widthPtr,heightPtr);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_SetTileChangedProc
+ *
+ *	Sets the routine to called when an image changes.  If
+ *	*changeProc* is NULL, no callback will be performed.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	The designated routine will be called the next time the
+ *	image associated with the tile changes.
+ *
+ *----------------------------------------------------------------------
+ */
+void
+Tk_SetTileChangedProc(tile, changeProc, clientData, itemPtr)
+    Tk_Tile tile;		/* Tile to query */
+    Tk_TileChangedProc *changeProc;
+    ClientData clientData;
+    Tk_Item *itemPtr;
+{
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_SetTSOrigin --
+ *
+ *	Set the pattern origin of the tile to a common point (i.e. the
+ *	origin (0,0) of the top level window) so that tiles from two
+ *	different widgets will match up.  This done by setting the
+ *	GCTileStipOrigin field is set to the translated origin of the
+ *	toplevel window in the hierarchy.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	The GCTileStipOrigin is reset in the GC.  This will cause the
+ *	tile origin to change when the GC is used for drawing.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+void
+Tk_SetTSOrigin(tkwin, gc, x, y)
+    Tk_Window tkwin;
+    GC gc;
+    int x, y;
+{
+    while (!Tk_TopWinHierarchy(tkwin)) {
+	x -= Tk_X(tkwin) + Tk_Changes(tkwin)->border_width;
+	y -= Tk_Y(tkwin) + Tk_Changes(tkwin)->border_width;
+	tkwin = Tk_Parent(tkwin);
+    }
+    XSetTSOrigin(Tk_Display(tkwin), gc, x, y);
+}
+
+
