@@ -1,4 +1,4 @@
-/* 
+/*
  * tkMenubutton.c --
  *
  *	This module implements button-like widgets that are used
@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMenubutton.c,v 1.2 1998/09/14 18:23:14 stanton Exp $
+ * RCS: @(#) $Id: tkMenubutton.c,v 1.12 2002/08/05 04:30:40 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -20,151 +20,151 @@
 #include "tkVMacro.h"
 
 /*
- * Uids internal to menubuttons.
+ * The following table defines the legal values for the -direction
+ * option.  It is used together with the "enum direction" declaration
+ * in tkMenubutton.h.
  */
 
-static Tk_Uid aboveUid = NULL;
-static Tk_Uid belowUid = NULL;
-static Tk_Uid leftUid = NULL;
-static Tk_Uid rightUid = NULL;
-static Tk_Uid flushUid = NULL;
+static char *directionStrings[] = {
+    "above", "below", "flush", "left", "right", (char *) NULL
+};
 
 /*
- * Custom option for handling "-state"
+ * The following table defines the legal values for the -state option.
+ * It is used together with the "enum state" declaration in tkMenubutton.h.
  */
 
-static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc,
-    (ClientData) 1	/* allow "normal", "active" and "disabled" */
+static char *stateStrings[] = {
+    "active", "disabled", "normal", (char *) NULL
 };
 
-static Tk_CustomOption tileOption = {
-    Tk_TileParseProc,
-    Tk_TilePrintProc,
-    (ClientData) NULL
-};
+/*
+ * The following table defines the legal values for the -compound option.
+ * It is used with the "enum compound" declaration in tkMenuButton.h
+ */
 
-static Tk_CustomOption offsetOption = {
-    Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
-    (ClientData) NULL
+static char *compoundStrings[] = {
+    "bottom", "center", "left", "none", "right", "top", (char *) NULL
 };
 
 /*
  * Information used for parsing configuration specs:
  */
 
-static Tk_ConfigSpec configSpecs[] = {
-    {TK_CONFIG_BORDER, "-activebackground", "activeBackground", "Foreground",
-	DEF_MENUBUTTON_ACTIVE_BG_COLOR, Tk_Offset(TkMenuButton, activeBorder),
-	TK_CONFIG_COLOR_ONLY},
-    {TK_CONFIG_BORDER, "-activebackground", "activeBackground", "Foreground",
-	DEF_MENUBUTTON_ACTIVE_BG_MONO, Tk_Offset(TkMenuButton, activeBorder),
-	TK_CONFIG_MONO_ONLY},
-    {TK_CONFIG_COLOR, "-activeforeground", "activeForeground", "Background",
-	DEF_MENUBUTTON_ACTIVE_FG_COLOR, Tk_Offset(TkMenuButton, activeFg),
-	TK_CONFIG_COLOR_ONLY},
-    {TK_CONFIG_COLOR, "-activeforeground", "activeForeground", "Background",
-	DEF_MENUBUTTON_ACTIVE_FG_MONO, Tk_Offset(TkMenuButton, activeFg),
-	TK_CONFIG_MONO_ONLY},
-    {TK_CONFIG_CUSTOM, "-activetile", "activeTile", "Tile", (char *) NULL,
-	Tk_Offset(TkMenuButton, activeTile),TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},
-    {TK_CONFIG_ANCHOR, "-anchor", "anchor", "Anchor",
-	DEF_MENUBUTTON_ANCHOR, Tk_Offset(TkMenuButton, anchor), 0},
-    {TK_CONFIG_BORDER, "-background", "background", "Background",
-	DEF_MENUBUTTON_BG_COLOR, Tk_Offset(TkMenuButton, normalBorder),
-	TK_CONFIG_COLOR_ONLY},
-    {TK_CONFIG_BORDER, "-background", "background", "Background",
-	DEF_MENUBUTTON_BG_MONO, Tk_Offset(TkMenuButton, normalBorder),
-	TK_CONFIG_MONO_ONLY},
-    {TK_CONFIG_SYNONYM, "-bd", "borderWidth", (char *) NULL,
-	(char *) NULL, 0, 0},
-    {TK_CONFIG_SYNONYM, "-bg", "background", (char *) NULL,
-	(char *) NULL, 0, 0},
-    {TK_CONFIG_BITMAP, "-bitmap", "bitmap", "Bitmap",
-	DEF_MENUBUTTON_BITMAP, Tk_Offset(TkMenuButton, bitmap),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_MENUBUTTON_BORDER_WIDTH, Tk_Offset(TkMenuButton, borderWidth), 0},
-    {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
-	DEF_MENUBUTTON_CURSOR, Tk_Offset(TkMenuButton, cursor),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_UID, "-direction", "direction", "Direction",
-    	DEF_MENUBUTTON_DIRECTION, Tk_Offset(TkMenuButton, direction), 
-	0},
-    {TK_CONFIG_COLOR, "-disabledforeground", "disabledForeground",
+static Tk_OptionSpec optionSpecs[] = {
+    {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
+        DEF_MENUBUTTON_ACTIVE_BG_COLOR, -1,
+        Tk_Offset(TkMenuButton, activeBorder), 0,
+        (ClientData) DEF_MENUBUTTON_ACTIVE_BG_MONO, 0},
+    {TK_OPTION_COLOR, "-activeforeground", "activeForeground", "Background",
+	DEF_MENUBUTTON_ACTIVE_FG_COLOR, -1,
+         Tk_Offset(TkMenuButton, activeFg),
+         0, (ClientData) DEF_MENUBUTTON_ACTIVE_FG_MONO, 0},
+    {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
+	DEF_MENUBUTTON_ANCHOR, -1,
+        Tk_Offset(TkMenuButton, anchor), 0, 0, 0},
+    {TK_OPTION_BORDER, "-background", "background", "Background",
+	DEF_MENUBUTTON_BG_COLOR, -1, Tk_Offset(TkMenuButton, normalBorder),
+        0, (ClientData) DEF_MENUBUTTON_BG_MONO, 0},
+    {TK_OPTION_SYNONYM, "-bd", (char *) NULL, (char *) NULL,
+	(char *) NULL, 0, -1, 0, (ClientData) "-borderwidth", 0},
+    {TK_OPTION_SYNONYM, "-bg", (char *) NULL, (char *) NULL,
+	(char *) NULL, 0, -1, 0, (ClientData) "-background", 0},
+    {TK_OPTION_BITMAP, "-bitmap", "bitmap", "Bitmap",
+	DEF_MENUBUTTON_BITMAP, -1, Tk_Offset(TkMenuButton, bitmap),
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
+	DEF_MENUBUTTON_BORDER_WIDTH, -1,
+        Tk_Offset(TkMenuButton, borderWidth), 0, 0, 0},
+    {TK_OPTION_CURSOR, "-cursor", "cursor", "Cursor",
+	DEF_MENUBUTTON_CURSOR, -1, Tk_Offset(TkMenuButton, cursor),
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_STRING_TABLE, "-direction", "direction", "Direction",
+    	DEF_MENUBUTTON_DIRECTION, -1, Tk_Offset(TkMenuButton, direction),
+	0, (ClientData) directionStrings, 0},
+    {TK_OPTION_COLOR, "-disabledforeground", "disabledForeground",
 	"DisabledForeground", DEF_MENUBUTTON_DISABLED_FG_COLOR,
-	Tk_Offset(TkMenuButton, disabledFg),
-	TK_CONFIG_COLOR_ONLY|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_COLOR, "-disabledforeground", "disabledForeground",
-	"DisabledForeground", DEF_MENUBUTTON_DISABLED_FG_MONO,
-	Tk_Offset(TkMenuButton, disabledFg),
-	TK_CONFIG_MONO_ONLY|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledtile", "disabledTile", "Tile", (char *) NULL,
-	Tk_Offset(TkMenuButton, disabledTile),TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},
-    {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
-	(char *) NULL, 0, 0},
-    {TK_CONFIG_FONT, "-font", "font", "Font",
-	DEF_MENUBUTTON_FONT, Tk_Offset(TkMenuButton, tkfont), 0},
-    {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
-	DEF_MENUBUTTON_FG, Tk_Offset(TkMenuButton, normalFg), 0},
-    {TK_CONFIG_LANGARG, "-height", "height", "Height",
-	DEF_MENUBUTTON_HEIGHT, Tk_Offset(TkMenuButton, heightString), 0},
-    {TK_CONFIG_COLOR, "-highlightbackground", "highlightBackground",
-	"HighlightBackground", DEF_MENUBUTTON_HIGHLIGHT_BG,
-	Tk_Offset(TkMenuButton, highlightBgColorPtr), 0},
-    {TK_CONFIG_COLOR, "-highlightcolor", "highlightColor", "HighlightColor",
-	DEF_MENUBUTTON_HIGHLIGHT, Tk_Offset(TkMenuButton, highlightColorPtr),
-	0},
-    {TK_CONFIG_PIXELS, "-highlightthickness", "highlightThickness",
+	-1, Tk_Offset(TkMenuButton, disabledFg), TK_OPTION_NULL_OK,
+	(ClientData) DEF_MENUBUTTON_DISABLED_FG_MONO, 0},
+    {TK_OPTION_SYNONYM, "-fg", "foreground", (char *) NULL,
+	(char *) NULL, 0, -1, 0, (ClientData) "-foreground", 0},
+    {TK_OPTION_FONT, "-font", "font", "Font",
+	DEF_MENUBUTTON_FONT, -1, Tk_Offset(TkMenuButton, tkfont), 0, 0, 0},
+    {TK_OPTION_COLOR, "-foreground", "foreground", "Foreground",
+	DEF_MENUBUTTON_FG, -1, Tk_Offset(TkMenuButton, normalFg), 0, 0, 0},
+    {TK_OPTION_OBJ, "-height", "height", "Height",
+	DEF_MENUBUTTON_HEIGHT, -1, Tk_Offset(TkMenuButton, heightString),
+        0, 0, 0},
+    {TK_OPTION_COLOR, "-highlightbackground", "highlightBackground",
+	"HighlightBackground", DEF_MENUBUTTON_HIGHLIGHT_BG_COLOR,
+	-1, Tk_Offset(TkMenuButton, highlightBgColorPtr), 0, 0, 0},
+    {TK_OPTION_COLOR, "-highlightcolor", "highlightColor", "HighlightColor",
+	DEF_MENUBUTTON_HIGHLIGHT, -1,
+        Tk_Offset(TkMenuButton, highlightColorPtr),	0, 0, 0},
+    {TK_OPTION_PIXELS, "-highlightthickness", "highlightThickness",
 	"HighlightThickness", DEF_MENUBUTTON_HIGHLIGHT_WIDTH,
-	Tk_Offset(TkMenuButton, highlightWidth), 0},
-    {TK_CONFIG_OBJECT, "-image", "image", "Image",
-	DEF_MENUBUTTON_IMAGE, Tk_Offset(TkMenuButton, imageString),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_BOOLEAN, "-indicatoron", "indicatorOn", "IndicatorOn",
-	DEF_MENUBUTTON_INDICATOR, Tk_Offset(TkMenuButton, indicatorOn), 0},
-    {TK_CONFIG_JUSTIFY, "-justify", "justify", "Justify",
-	DEF_MENUBUTTON_JUSTIFY, Tk_Offset(TkMenuButton, justify), 0},
-    {TK_CONFIG_LANGARG, "-menu", "menu", "Menu",
-	DEF_MENUBUTTON_MENU, Tk_Offset(TkMenuButton, menuName),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_PIXELS, "-padx", "padX", "Pad",
-	DEF_MENUBUTTON_PADX, Tk_Offset(TkMenuButton, padX), 0},
-    {TK_CONFIG_PIXELS, "-pady", "padY", "Pad",
-	DEF_MENUBUTTON_PADY, Tk_Offset(TkMenuButton, padY), 0},
-    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
-	Tk_Offset(TkMenuButton, tile),TK_CONFIG_DONT_SET_DEFAULT,
-	&offsetOption},
-    {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
-	DEF_MENUBUTTON_RELIEF, Tk_Offset(TkMenuButton, relief), 0},
-    {TK_CONFIG_CUSTOM, "-state", "state", "State",
-	DEF_MENUBUTTON_STATE, Tk_Offset(TkMenuButton, state), 0,
-	&stateOption},
-    {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
-	DEF_MENUBUTTON_TAKE_FOCUS, Tk_Offset(TkMenuButton, takeFocus),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_STRING, "-text", "text", "Text",
-	DEF_MENUBUTTON_TEXT, Tk_Offset(TkMenuButton, text), 0},
-    {TK_CONFIG_SCALARVAR, "-textvariable", "textVariable", "Variable",
-	DEF_MENUBUTTON_TEXT_VARIABLE, Tk_Offset(TkMenuButton, textVarName),
-	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(TkMenuButton, tile),TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},
-    {TK_CONFIG_INT, "-underline", "underline", "Underline",
-	DEF_MENUBUTTON_UNDERLINE, Tk_Offset(TkMenuButton, underline), 0},
-    {TK_CONFIG_LANGARG, "-width", "width", "Width",
-	DEF_MENUBUTTON_WIDTH, Tk_Offset(TkMenuButton, widthString), 0},
-    {TK_CONFIG_PIXELS, "-wraplength", "wrapLength", "WrapLength",
-	DEF_MENUBUTTON_WRAP_LENGTH, Tk_Offset(TkMenuButton, wrapLength), 0},
-    {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
+	-1, Tk_Offset(TkMenuButton, highlightWidth), 0, 0, 0},
+    {TK_OPTION_STRING, "-image", "image", "Image",
+	DEF_MENUBUTTON_IMAGE, -1, Tk_Offset(TkMenuButton, imageString),
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_BOOLEAN, "-indicatoron", "indicatorOn", "IndicatorOn",
+	DEF_MENUBUTTON_INDICATOR, -1, Tk_Offset(TkMenuButton, indicatorOn),
+        0, 0, 0},
+    {TK_OPTION_JUSTIFY, "-justify", "justify", "Justify",
+	DEF_BUTTON_JUSTIFY, -1, Tk_Offset(TkMenuButton, justify), 0, 0, 0},
+    {TK_OPTION_WINDOW, "-menu", "menu", "Menu",
+	DEF_MENUBUTTON_MENU, -1, Tk_Offset(TkMenuButton, menuName),
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_PIXELS, "-padx", "padX", "Pad",
+	DEF_MENUBUTTON_PADX, -1, Tk_Offset(TkMenuButton, padX),
+	0, 0, 0},
+    {TK_OPTION_PIXELS, "-pady", "padY", "Pad",
+	DEF_MENUBUTTON_PADY, -1, Tk_Offset(TkMenuButton, padY),
+	0, 0, 0},
+    {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
+	DEF_MENUBUTTON_RELIEF, -1, Tk_Offset(TkMenuButton, relief),
+        0, 0, 0},
+    {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
+         DEF_BUTTON_COMPOUND, -1, Tk_Offset(TkMenuButton, compound), 0,
+         (ClientData) compoundStrings, 0},
+    {TK_OPTION_STRING_TABLE, "-state", "state", "State",
+	DEF_MENUBUTTON_STATE, -1, Tk_Offset(TkMenuButton, state),
+	0, (ClientData) stateStrings, 0},
+    {TK_OPTION_STRING, "-takefocus", "takeFocus", "TakeFocus",
+	DEF_MENUBUTTON_TAKE_FOCUS, -1,
+        Tk_Offset(TkMenuButton, takeFocus), TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_STRING, "-text", "text", "Text",
+	DEF_MENUBUTTON_TEXT, -1, Tk_Offset(TkMenuButton, text), 0, 0, 0},
+    {TK_OPTION_OBJ, "-textvariable", "textVariable", "Variable",
+	DEF_MENUBUTTON_TEXT_VARIABLE, -1,
+        Tk_Offset(TkMenuButton, textVarName), TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_INT, "-underline", "underline", "Underline",
+	DEF_MENUBUTTON_UNDERLINE, -1, Tk_Offset(TkMenuButton, underline),
+         0, 0, 0},
+    {TK_OPTION_OBJ, "-width", "width", "Width",
+	DEF_MENUBUTTON_WIDTH, -1, Tk_Offset(TkMenuButton, widthString),
+        0, 0, 0},
+    {TK_OPTION_PIXELS, "-wraplength", "wrapLength", "WrapLength",
+	DEF_MENUBUTTON_WRAP_LENGTH, -1, Tk_Offset(TkMenuButton, wrapLength),
+        0, 0, 0},
+    {TK_OPTION_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, 0}
 };
 
+/*
+ * The following tables define the menubutton widget commands and map the
+ * indexes into the string tables into a single enumerated type used
+ * to dispatch the scale widget command.
+ */
+
+static CONST char *commandNames[] = {
+    "cget", "configure", (char *) NULL
+};
+
+enum command {
+    COMMAND_CGET, COMMAND_CONFIGURE
+};
+
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -178,20 +178,19 @@ static void		MenuButtonImageProc _ANSI_ARGS_((ClientData clientData,
 			    int imgHeight));
 static char *		MenuButtonTextVarProc _ANSI_ARGS_((
 			    ClientData clientData, Tcl_Interp *interp,
-			    Var name1, char *name2, int flags));
-static int		MenuButtonWidgetCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
+			    Var name1, CONST char *name2, int flags));
+static int		MenuButtonWidgetObjCmd _ANSI_ARGS_((
+                            ClientData clientData, Tcl_Interp *interp,
+			    int objc, Tcl_Obj *CONST objv[]));
 static int		ConfigureMenuButton _ANSI_ARGS_((Tcl_Interp *interp,
-			    TkMenuButton *mbPtr, int argc, char **argv,
-			    int flags));
+			    TkMenuButton *mbPtr, int objc,
+                            Tcl_Obj *CONST objv[]));
 static void		DestroyMenuButton _ANSI_ARGS_((char *memPtr));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
-
+
 /*
  *--------------------------------------------------------------
  *
- * Tk_MenubuttonCmd --
+ * Tk_MenubuttonObjCmd --
  *
  *	This procedure is invoked to process the "button", "label",
  *	"radiobutton", and "checkbutton" Tcl commands.  See the
@@ -207,20 +206,18 @@ static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
  */
 
 int
-Tk_MenubuttonCmd(clientData, interp, argc, argv)
-    ClientData clientData;	/* Main window associated with
-				 * interpreter. */
+Tk_MenubuttonObjCmd(clientData, interp, objc, objv)
+    ClientData clientData;	/* NULL. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     register TkMenuButton *mbPtr;
-    Tk_Window tkwin = (Tk_Window) clientData;
-    Tk_Window new;
+    Tk_OptionTable optionTable;
+    Tk_Window tkwin;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " pathName ?options?\"", (char *) NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?options?");
 	return TCL_ERROR;
     }
 
@@ -228,25 +225,35 @@ Tk_MenubuttonCmd(clientData, interp, argc, argv)
      * Create the new window.
      */
 
-    new = Tk_CreateWindowFromPath(interp, tkwin, argv[1], (char *) NULL);
-    if (new == NULL) {
+    tkwin = Tk_CreateWindowFromPath(interp, Tk_MainWindow(interp),
+	    Tcl_GetString(objv[1]), (char *) NULL);
+    if (tkwin == NULL) {
 	return TCL_ERROR;
     }
 
-    TkClassOption(new, "Menubutton",&argc,&argv);
-    mbPtr = TkpCreateMenuButton(new);
+    /*
+     * Create the option table for this widget class.  If it has already
+     * been created, the cached pointer will be returned.
+     */
 
-    TkSetClassProcs(new, &tkpMenubuttonClass, (ClientData) mbPtr);
+    optionTable = Tk_CreateOptionTable(interp, optionSpecs);
+
+    Tk_SetClass(tkwin, "Menubutton");
+    mbPtr = TkpCreateMenuButton(tkwin);
+
+    Tk_SetClassProcs(tkwin, &tkpMenubuttonClass, (ClientData) mbPtr);
 
     /*
      * Initialize the data structure for the button.
      */
 
-    mbPtr->tkwin = new;
-    mbPtr->display = Tk_Display (new);
+    mbPtr->tkwin = tkwin;
+    mbPtr->display = Tk_Display (tkwin);
     mbPtr->interp = interp;
-    mbPtr->widgetCmd = Tcl_CreateCommand(interp, Tk_PathName(mbPtr->tkwin),
-	    MenuButtonWidgetCmd, (ClientData) mbPtr, MenuButtonCmdDeletedProc);
+    mbPtr->widgetCmd = Tcl_CreateObjCommand(interp,
+            Tk_PathName(mbPtr->tkwin), MenuButtonWidgetObjCmd,
+            (ClientData) mbPtr, MenuButtonCmdDeletedProc);
+    mbPtr->optionTable = optionTable;
     mbPtr->menuName = NULL;
     mbPtr->text = NULL;
     mbPtr->underline = -1;
@@ -254,7 +261,7 @@ Tk_MenubuttonCmd(clientData, interp, argc, argv)
     mbPtr->bitmap = None;
     mbPtr->imageString = NULL;
     mbPtr->image = NULL;
-    mbPtr->state = TK_STATE_NORMAL;
+    mbPtr->state = STATE_NORMAL;
     mbPtr->normalBorder = NULL;
     mbPtr->activeBorder = NULL;
     mbPtr->borderWidth = 0;
@@ -286,39 +293,33 @@ Tk_MenubuttonCmd(clientData, interp, argc, argv)
     mbPtr->indicatorOn = 0;
     mbPtr->indicatorWidth = 0;
     mbPtr->indicatorHeight = 0;
+    mbPtr->direction = DIRECTION_FLUSH;
     mbPtr->cursor = None;
     mbPtr->takeFocus = NULL;
     mbPtr->flags = 0;
-    mbPtr->tile = mbPtr->activeTile = mbPtr->disabledTile = NULL;
-    mbPtr->tileGC = None;
-    mbPtr->tsoffset.flags = 0;
-    mbPtr->tsoffset.xoffset = 0;
-    mbPtr->tsoffset.yoffset = 0;
-    if (aboveUid == NULL) {
-	aboveUid = Tk_GetUid("above");
-	belowUid = Tk_GetUid("below");
-	leftUid = Tk_GetUid("left");
-	rightUid = Tk_GetUid("right");
-	flushUid = Tk_GetUid("flush");
-    }
-    mbPtr->direction = flushUid;
 
     Tk_CreateEventHandler(mbPtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    MenuButtonEventProc, (ClientData) mbPtr);
-    if (ConfigureMenuButton(interp, mbPtr, argc-2, argv+2, 0) != TCL_OK) {
+
+    if (Tk_InitOptions(interp, (char *) mbPtr, optionTable, tkwin) != TCL_OK) {
 	Tk_DestroyWindow(mbPtr->tkwin);
 	return TCL_ERROR;
     }
 
-    interp->result = Tk_PathName(mbPtr->tkwin);
+    if (ConfigureMenuButton(interp, mbPtr, objc-2, objv+2) != TCL_OK) {
+	Tk_DestroyWindow(mbPtr->tkwin);
+	return TCL_ERROR;
+    }
+
+    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tk_PathName(mbPtr->tkwin), -1);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
- * MenuButtonWidgetCmd --
+ * MenuButtonWidgetObjCmd --
  *
  *	This procedure is invoked to process the Tcl command
  *	that corresponds to a widget managed by this module.
@@ -334,67 +335,79 @@ Tk_MenubuttonCmd(clientData, interp, argc, argv)
  */
 
 static int
-MenuButtonWidgetCmd(clientData, interp, argc, argv)
+MenuButtonWidgetObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Information about button widget. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     register TkMenuButton *mbPtr = (TkMenuButton *) clientData;
-    int result;
-    size_t length;
-    int c;
+    int result, index;
+    Tcl_Obj *objPtr;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" option ?arg arg ...?\"", (char *) NULL);
+    if (objc < 2) {
+        Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
 	return TCL_ERROR;
     }
+    result = Tcl_GetIndexFromObj(interp, objv[1],
+            commandNames, "option", 0, &index);
+    if (result != TCL_OK) {
+        return result;
+    }
     Tcl_Preserve((ClientData) mbPtr);
-    c = argv[1][0];
-    length = strlen(argv[1]);
-    if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
-	    && (length >= 2)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " cget option\"",
-		    (char *) NULL);
-	    result = TCL_ERROR;
-	} else {
-	    result = Tk_ConfigureValue(interp, mbPtr->tkwin, configSpecs,
-		    (char *) mbPtr, argv[2], 0);
+
+    switch (index) {
+        case COMMAND_CGET: {
+	    if (objc != 3) {
+	        Tcl_WrongNumArgs(interp, 1, objv, "cget option");
+	    goto error;
+	    }
+
+	    objPtr = Tk_GetOptionValue(interp, (char *) mbPtr,
+                    mbPtr->optionTable, objv[2], mbPtr->tkwin);
+	    if (objPtr == NULL) {
+	        goto error;
+	    } else {
+	        Tcl_SetObjResult(interp, objPtr);
+	    }
+	    break;
 	}
-    } else if ((c == 'c') && (strncmp(argv[1], "configure", length) == 0)
-	    && (length >= 2)) {
-	if (argc == 2) {
-	    result = Tk_ConfigureInfo(interp, mbPtr->tkwin, configSpecs,
-		    (char *) mbPtr, (char *) NULL, 0);
-	} else if (argc == 3) {
-	    result = Tk_ConfigureInfo(interp, mbPtr->tkwin, configSpecs,
-		    (char *) mbPtr, argv[2], 0);
-	} else {
-	    result = ConfigureMenuButton(interp, mbPtr, argc-2, argv+2,
-		    TK_CONFIG_ARGV_ONLY);
+
+        case COMMAND_CONFIGURE: {
+	    if (objc <= 3) {
+		objPtr = Tk_GetOptionInfo(interp, (char *) mbPtr,
+			mbPtr->optionTable,
+			(objc == 3) ? objv[2] : (Tcl_Obj *) NULL,
+			mbPtr->tkwin);
+		if (objPtr == NULL) {
+		    goto error;
+		} else {
+		    Tcl_SetObjResult(interp, objPtr);
+		}
+	    } else {
+		result = ConfigureMenuButton(interp, mbPtr, objc-2,
+		        objv+2);
+	    }
+	    break;
 	}
-    } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
-		"\": must be cget or configure",
-		(char *) NULL);
-	result = TCL_ERROR;
     }
     Tcl_Release((ClientData) mbPtr);
     return result;
+
+    error:
+    Tcl_Release((ClientData) mbPtr);
+    return TCL_ERROR;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
  * DestroyMenuButton --
  *
  *	This procedure is invoked to recycle all of the resources
- *	associated with a button widget.  It is invoked as a
+ *	associated with a menubutton widget.  It is invoked as a
  *	when-idle handler in order to make sure that there is no
- *	other use of the button pending at the time of the deletion.
+ *	other use of the menubutton pending at the time of the deletion.
  *
  * Results:
  *	None.
@@ -410,6 +423,11 @@ DestroyMenuButton(memPtr)
     char *memPtr;		/* Info about button widget. */
 {
     register TkMenuButton *mbPtr = (TkMenuButton *) memPtr;
+    TkpDestroyMenuButton(mbPtr);
+
+    if (mbPtr->flags & REDRAW_PENDING) {
+        Tcl_CancelIdleCall(TkpDisplayMenuButton, (ClientData) mbPtr);
+    }
 
     /*
      * Free up all the stuff that requires special handling, then
@@ -417,8 +435,9 @@ DestroyMenuButton(memPtr)
      * stuff.
      */
 
+    Tcl_DeleteCommandFromToken(mbPtr->interp, mbPtr->widgetCmd);
     if (mbPtr->textVarName != NULL) {
-	Tcl_UntraceVar(mbPtr->interp, mbPtr->textVarName,
+	Lang_UntraceVar(mbPtr->interp, mbPtr->textVarName,
 		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		MenuButtonTextVarProc, (ClientData) mbPtr);
     }
@@ -431,51 +450,21 @@ DestroyMenuButton(memPtr)
     if (mbPtr->activeTextGC != None) {
 	Tk_FreeGC(mbPtr->display, mbPtr->activeTextGC);
     }
-    if (mbPtr->gray != None) {
-	Tk_FreeBitmap(mbPtr->display, mbPtr->gray);
-    }
     if (mbPtr->disabledGC != None) {
 	Tk_FreeGC(mbPtr->display, mbPtr->disabledGC);
     }
-    if (mbPtr->tile != NULL) {
-	Tk_FreeTile(mbPtr->tile);
+    if (mbPtr->gray != None) {
+	Tk_FreeBitmap(mbPtr->display, mbPtr->gray);
     }
-    if (mbPtr->activeTile != NULL) {
-	Tk_FreeTile(mbPtr->activeTile);
+    if (mbPtr->textLayout != NULL) {
+        Tk_FreeTextLayout(mbPtr->textLayout);
     }
-    if (mbPtr->disabledTile != NULL) {
-	Tk_FreeTile(mbPtr->disabledTile);
-    }
-    if (mbPtr->tileGC != None) {
-	Tk_FreeGC(mbPtr->display, mbPtr->tileGC);
-    }
-    Tk_FreeTextLayout(mbPtr->textLayout);
-    Tk_FreeOptions(configSpecs, (char *) mbPtr, mbPtr->display, 0);
-    ckfree((char *) mbPtr);
+    Tk_FreeConfigOptions((char *) mbPtr, mbPtr->optionTable,
+	    mbPtr->tkwin);
+    mbPtr->tkwin = NULL;
+    Tcl_EventuallyFree((ClientData) mbPtr, TCL_DYNAMIC);
 }
-
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *  None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;           /* Not used */
-{
-    register TkMenuButton *mbPtr = (TkMenuButton *) clientData;
 
-    ConfigureMenuButton(mbPtr->interp, mbPtr, 0, NULL, 0);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -487,7 +476,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as text string, colors, font,
@@ -498,140 +487,178 @@ TileChangedProc(clientData, tile, itemPtr)
  */
 
 static int
-ConfigureMenuButton(interp, mbPtr, argc, argv, flags)
+ConfigureMenuButton(interp, mbPtr, objc, objv)
     Tcl_Interp *interp;		/* Used for error reporting. */
-    register TkMenuButton *mbPtr;	/* Information about widget;  may or may
-				 * not already have values for some fields. */
-    int argc;			/* Number of valid entries in argv. */
-    char **argv;		/* Arguments. */
-    int flags;			/* Flags to pass to Tk_ConfigureWidget. */
+    register TkMenuButton *mbPtr;
+                                /* Information about widget;  may or may
+				 * not already have values for some
+				 * fields. */
+    int objc;			/* Number of valid entries in objv. */
+    Tcl_Obj *CONST objv[];	/* Arguments. */
 {
+    Tk_SavedOptions savedOptions;
+    Tcl_Obj *errorResult = NULL;
+    int error;
     Tk_Image image;
 
     /*
-     * Eliminate any existing trace on variables monitored by the menubutton.
+     * Eliminate any existing trace on variables monitored by the
+     * menubutton.
      */
 
     if (mbPtr->textVarName != NULL) {
-	Tcl_UntraceVar(interp, mbPtr->textVarName,
+	Lang_UntraceVar(interp, mbPtr->textVarName,
 		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		MenuButtonTextVarProc, (ClientData) mbPtr);
     }
 
-    if (Tk_ConfigureWidget(interp, mbPtr->tkwin, configSpecs,
-	    argc, argv, (char *) mbPtr, flags) != TCL_OK) {
-	return TCL_ERROR;
-    }
-
     /*
-     * A few options need special processing, such as setting the
-     * background from a 3-D border, or filling in complicated
-     * defaults that couldn't be specified to Tk_ConfigureWidget.
+     * The following loop is potentially executed twice.  During the
+     * first pass configuration options get set to their new values.
+     * If there is an error in this pass, we execute a second pass
+     * to restore all the options to their previous values.
      */
 
-    if ((mbPtr->state == TK_STATE_ACTIVE) && !Tk_StrictMotif(mbPtr->tkwin)) {
-	Tk_SetBackgroundFromBorder(mbPtr->tkwin, mbPtr->activeBorder);
-    } else {
-	Tk_SetBackgroundFromBorder(mbPtr->tkwin, mbPtr->normalBorder);
-    }
+    for (error = 0; error <= 1; error++) {
+	if (!error) {
+	    /*
+	     * First pass: set options to new values.
+	     */
 
-    if ((mbPtr->direction != aboveUid) && (mbPtr->direction != belowUid)
-	    && (mbPtr->direction != leftUid) && (mbPtr->direction != rightUid)
-	    && (mbPtr->direction != flushUid)) {
-	Tcl_AppendResult(interp, "bad direction value \"", mbPtr->direction,
-		"\": must be above, below, left, right, or flush",
-		(char *) NULL);
-	mbPtr->direction = belowUid;
-	return TCL_ERROR;
-    }
-    
-    if (mbPtr->highlightWidth < 0) {
-	mbPtr->highlightWidth = 0;
-    }
+	    if (Tk_SetOptions(interp, (char *) mbPtr,
+		    mbPtr->optionTable, objc, objv,
+		    mbPtr->tkwin, &savedOptions, (int *) NULL) != TCL_OK) {
+		continue;
+	    }
+	} else {
+	    /*
+	     * Second pass: restore options to old values.
+	     */
 
-    if (mbPtr->padX < 0) {
-	mbPtr->padX = 0;
-    }
-    if (mbPtr->padY < 0) {
-	mbPtr->padY = 0;
-    }
-
-    /*
-     * Get the image for the widget, if there is one.  Allocate the
-     * new image before freeing the old one, so that the reference
-     * count doesn't go to zero and cause image data to be discarded.
-     */
-
-    if (mbPtr->imageString != NULL) {
-	image = Tk_GetImage(mbPtr->interp, mbPtr->tkwin,
-		mbPtr->imageString, MenuButtonImageProc, (ClientData) mbPtr);
-	if (image == NULL) {
-	    return TCL_ERROR;
+	    errorResult = Tcl_GetObjResult(interp);
+	    Tcl_IncrRefCount(errorResult);
+	    Tk_RestoreSavedOptions(&savedOptions);
 	}
-    } else {
-	image = NULL;
+
+	/*
+	 * A few options need special processing, such as setting the
+	 * background from a 3-D border, or filling in complicated
+	 * defaults that couldn't be specified to Tk_SetOptions.
+	 */
+
+	if ((mbPtr->state == STATE_ACTIVE)
+		&& !Tk_StrictMotif(mbPtr->tkwin)) {
+	    Tk_SetBackgroundFromBorder(mbPtr->tkwin, mbPtr->activeBorder);
+	} else {
+	    Tk_SetBackgroundFromBorder(mbPtr->tkwin, mbPtr->normalBorder);
+	}
+
+	if (mbPtr->highlightWidth < 0) {
+	    mbPtr->highlightWidth = 0;
+	}
+
+	if (mbPtr->padX < 0) {
+	    mbPtr->padX = 0;
+	}
+	if (mbPtr->padY < 0) {
+	    mbPtr->padY = 0;
+	}
+
+	/*
+	 * Get the image for the widget, if there is one.  Allocate the
+	 * new image before freeing the old one, so that the reference
+	 * count doesn't go to zero and cause image data to be discarded.
+	 */
+
+	if (mbPtr->imageString != NULL) {
+	    image = Tk_GetImage(mbPtr->interp, mbPtr->tkwin,
+		    mbPtr->imageString, MenuButtonImageProc,
+		    (ClientData) mbPtr);
+	    if (image == NULL) {
+	        return TCL_ERROR;
+	    }
+	} else {
+	    image = NULL;
+	}
+	if (mbPtr->image != NULL) {
+	    Tk_FreeImage(mbPtr->image);
+	}
+	mbPtr->image = image;
+
+	/*
+	 * Recompute the geometry for the button.
+	 */
+
+	if ((mbPtr->bitmap != None) || (mbPtr->image != NULL)) {
+	    if (Tk_GetPixelsFromObj(interp, mbPtr->tkwin, mbPtr->widthString,
+                    &mbPtr->width) != TCL_OK) {
+                widthError:
+	        Tcl_AddErrorInfo(interp, "\n    (processing -width option)");
+		continue;
+	    }
+	    if (Tk_GetPixelsFromObj(interp, mbPtr->tkwin, mbPtr->heightString,
+		    &mbPtr->height) != TCL_OK) {
+	        heightError:
+	        Tcl_AddErrorInfo(interp, "\n    (processing -height option)");
+		continue;
+	    }
+	} else {
+	    if (Tcl_GetIntFromObj(interp, mbPtr->widthString, &mbPtr->width)
+		    != TCL_OK) {
+	        goto widthError;
+	    }
+	    if (Tcl_GetIntFromObj(interp, mbPtr->heightString, &mbPtr->height)
+		    != TCL_OK) {
+	        goto heightError;
+	    }
+	}
+	break;
     }
-    if (mbPtr->image != NULL) {
-	Tk_FreeImage(mbPtr->image);
+
+    if (!error) {
+      Tk_FreeSavedOptions(&savedOptions);
     }
-    mbPtr->image = image;
 
     if ((mbPtr->image == NULL) && (mbPtr->bitmap == None)
 	    && (mbPtr->textVarName != NULL)) {
-	/*
-	 * The menubutton displays a variable.  Set up a trace to watch
-	 * for any changes in it.
-	 */
 
-	char *value;
+      /*
+       * The menubutton displays the value of a variable.
+       * Set up a trace to watch for any changes in it, create
+       * the variable if it doesn't exist, and fetch its
+       * current value.
+       */
 
-	value = LangString(Tcl_GetVar(interp, mbPtr->textVarName, TCL_GLOBAL_ONLY));
-	if (value == NULL) {
-	    Tcl_SetVar(interp, mbPtr->textVarName, mbPtr->text,
-		    TCL_GLOBAL_ONLY);
-	} else {
-	    if (mbPtr->text != NULL) {
-		ckfree(mbPtr->text);
-	    }
-	    mbPtr->text = (char *) ckalloc((unsigned) (strlen(value) + 1));
-	    strcpy(mbPtr->text, value);
-	}
-	Tcl_TraceVar(interp, mbPtr->textVarName,
-		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
-		MenuButtonTextVarProc, (ClientData) mbPtr);
+      CONST char *value;
+
+      value = Tcl_GetString(Tcl_ObjGetVar2(interp, mbPtr->textVarName, NULL, TCL_GLOBAL_ONLY));
+      if (value == NULL) {
+	  Tcl_Obj *temp = Tcl_NewStringObj(mbPtr->text,-1);
+	  Tcl_ObjSetVar2(interp, mbPtr->textVarName, NULL, temp,
+		     TCL_GLOBAL_ONLY);
+	  Tcl_DecrRefCount(temp);
+      } else {
+	  if (mbPtr->text != NULL) {
+	      ckfree(mbPtr->text);
+	  }
+	  mbPtr->text = (char *) ckalloc((unsigned) (strlen(value) + 1));
+	  strcpy(mbPtr->text, value);
+      }
+      Lang_TraceVar(interp, mbPtr->textVarName,
+		   TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
+		   MenuButtonTextVarProc, (ClientData) mbPtr);
     }
 
-    /*
-     * Recompute the geometry for the button.
-     */
-
-    if ((mbPtr->bitmap != None) || (mbPtr->image != NULL)) {
-	if (Tk_GetPixels(interp, mbPtr->tkwin, LangString(mbPtr->widthString),
-		&mbPtr->width) != TCL_OK) {
-	    widthError:
-	    Tcl_AddErrorInfo(interp, "\n    (processing -width option)");
-	    return TCL_ERROR;
-	}
-	if (Tk_GetPixels(interp, mbPtr->tkwin, LangString(mbPtr->heightString),
-		&mbPtr->height) != TCL_OK) {
-	    heightError:
-	    Tcl_AddErrorInfo(interp, "\n    (processing -height option)");
-	    return TCL_ERROR;
-	}
-    } else {
-	if (Tcl_GetInt(interp, mbPtr->widthString, &mbPtr->width)
-		!= TCL_OK) {
-	    goto widthError;
-	}
-	if (Tcl_GetInt(interp, mbPtr->heightString, &mbPtr->height)
-		!= TCL_OK) {
-	    goto heightError;
-	}
-    }
     TkMenuButtonWorldChanged((ClientData) mbPtr);
-    return TCL_OK;
+    if (error) {
+	Tcl_SetObjResult(interp, errorResult);
+	Tcl_DecrRefCount(errorResult);
+        return TCL_ERROR;
+    } else {
+        return TCL_OK;
+    }
 }
-
+
 /*
  *---------------------------------------------------------------------------
  *
@@ -649,7 +676,7 @@ ConfigureMenuButton(interp, mbPtr, argc, argv, flags)
  *
  *---------------------------------------------------------------------------
  */
- 
+
 void
 TkMenuButtonWorldChanged(instanceData)
     ClientData instanceData;	/* Information about widget. */
@@ -658,38 +685,9 @@ TkMenuButtonWorldChanged(instanceData)
     GC gc;
     unsigned long mask;
     TkMenuButton *mbPtr;
-    Tk_Tile tile;
-    Pixmap pixmap;
 
     mbPtr = (TkMenuButton *) instanceData;
 
-    if ((mbPtr->state == TK_STATE_ACTIVE) && (mbPtr->activeTile != NULL)) {
-	tile = mbPtr->activeTile;
-    } else if ((mbPtr->state == TK_STATE_DISABLED) && (mbPtr->disabledTile != NULL)) {
-	tile = mbPtr->disabledTile;
-    } else {
-	tile = mbPtr->tile;
-    }
-    Tk_SetTileChangedProc(mbPtr->disabledTile, (Tk_TileChangedProc *) NULL,
-	    (ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(mbPtr->activeTile, (Tk_TileChangedProc *) NULL,
-	    (ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(mbPtr->tile, (Tk_TileChangedProc *) NULL, 
-	    (ClientData)NULL, (Tk_Item *)NULL);
-    Tk_SetTileChangedProc(tile, TileChangedProc,
-	    (ClientData) mbPtr, (Tk_Item *) NULL);
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	gc = Tk_GetGC(mbPtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else {
-	gc = Tk_GetGC(mbPtr->tkwin, 0, &gcValues);
-    }
-    if (mbPtr->tileGC != None) {
-	Tk_FreeGC(mbPtr->display, mbPtr->tileGC);
-    }
-    mbPtr->tileGC = gc;
-
     gcValues.font = Tk_FontId(mbPtr->tkfont);
     gcValues.foreground = mbPtr->normalFg->pixel;
     gcValues.background = Tk_3DBorderColor(mbPtr->normalBorder)->pixel;
@@ -753,7 +751,7 @@ TkMenuButtonWorldChanged(instanceData)
 	mbPtr->flags |= REDRAW_PENDING;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -788,15 +786,7 @@ MenuButtonEventProc(clientData, eventPtr)
 
 	goto redraw;
     } else if (eventPtr->type == DestroyNotify) {
-	TkpDestroyMenuButton(mbPtr);
-	if (mbPtr->tkwin != NULL) {
-	    mbPtr->tkwin = NULL;
-	    Tcl_DeleteCommandFromToken(mbPtr->interp, mbPtr->widgetCmd);
-	}
-	if (mbPtr->flags & REDRAW_PENDING) {
-	    Tcl_CancelIdleCall(TkpDisplayMenuButton, (ClientData) mbPtr);
-	}
-	Tcl_EventuallyFree((ClientData) mbPtr, DestroyMenuButton);
+        DestroyMenuButton((char *) mbPtr);
     } else if (eventPtr->type == FocusIn) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    mbPtr->flags |= GOT_FOCUS;
@@ -820,7 +810,7 @@ MenuButtonEventProc(clientData, eventPtr)
 	mbPtr->flags |= REDRAW_PENDING;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -854,11 +844,10 @@ MenuButtonCmdDeletedProc(clientData)
      */
 
     if (tkwin != NULL) {
-	mbPtr->tkwin = NULL;
 	Tk_DestroyWindow(tkwin);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -883,11 +872,11 @@ MenuButtonTextVarProc(clientData, interp, name1, name2, flags)
     ClientData clientData;	/* Information about button. */
     Tcl_Interp *interp;		/* Interpreter containing variable. */
     Var name1;			/* Name of variable. */
-    char *name2;		/* Second part of variable name. */
+    CONST char *name2;		/* Second part of variable name. */
     int flags;			/* Information about what happened. */
 {
     register TkMenuButton *mbPtr = (TkMenuButton *) clientData;
-    char *value;
+    CONST char *value;
 
     /*
      * If the variable is unset, then immediately recreate it unless
@@ -896,16 +885,18 @@ MenuButtonTextVarProc(clientData, interp, name1, name2, flags)
 
     if (flags & TCL_TRACE_UNSETS) {
 	if ((flags & TCL_TRACE_DESTROYED) && !(flags & TCL_INTERP_DESTROYED)) {
-	    Tcl_SetVar(interp, mbPtr->textVarName, mbPtr->text,
+	    Tcl_Obj *temp = Tcl_NewStringObj(mbPtr->text, -1);
+	    Tcl_ObjSetVar2(interp, mbPtr->textVarName, NULL, temp,
 		    TCL_GLOBAL_ONLY);
-	    Tcl_TraceVar(interp, mbPtr->textVarName,
+	    Tcl_DecrRefCount(temp);
+	    Lang_TraceVar(interp, mbPtr->textVarName,
 		    TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		    MenuButtonTextVarProc, clientData);
 	}
 	return (char *) NULL;
     }
 
-    value = LangString(Tcl_GetVar(interp, mbPtr->textVarName, TCL_GLOBAL_ONLY));
+    value = Tcl_GetString(Tcl_ObjGetVar2(interp, mbPtr->textVarName, NULL, TCL_GLOBAL_ONLY));
     if (value == NULL) {
 	value = "";
     }
@@ -923,7 +914,7 @@ MenuButtonTextVarProc(clientData, interp, name1, name2, flags)
     }
     return (char *) NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -961,3 +952,5 @@ MenuButtonImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
 	}
     }
 }
+
+
