@@ -1,16 +1,16 @@
-/* 
+/*
  * tkImage.c --
  *
  *	This file contains code that allows images to be
  *	nested inside text widgets.  It also implements the "image"
  *	widget command for texts.
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkTextImage.c 1.7 97/08/25 15:47:27
+ * RCS: @(#) $Id: tkTextImage.c,v 1.5 2002/08/05 04:30:40 dgp Exp $
  */
 
 #include "tk.h"
@@ -38,9 +38,9 @@
  */
 
 static int		AlignParseProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Tk_Window tkwin, Arg value,
-			    char *widgRec, int offset));
-static Arg		AlignPrintProc _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, Tk_Window tkwin, 
+			    Tcl_Obj *value, char *widgRec, int offset));
+static Tcl_Obj *	AlignPrintProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin, char *widgRec, int offset,
 			    Tcl_FreeProc **freeProcPtr));
 static TkTextSegment *	EmbImageCleanupProc _ANSI_ARGS_((TkTextSegment *segPtr,
@@ -62,7 +62,7 @@ static void		EmbImageDisplayProc _ANSI_ARGS_((
 static int		EmbImageLayoutProc _ANSI_ARGS_((TkText *textPtr,
 			    TkTextIndex *indexPtr, TkTextSegment *segPtr,
 			    int offset, int maxX, int maxChars,
-			    int noCharsYet, Tk_Uid wrapMode,
+			    int noCharsYet, TkWrapMode wrapMode,
 			    TkTextDispChunk *chunkPtr));
 static void		EmbImageProc _ANSI_ARGS_((ClientData clientData,
 			    int x, int y, int width, int height,
@@ -108,7 +108,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, 0}
 };
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -217,11 +217,11 @@ TkTextImageCmd(textPtr, interp, argc, argv)
 	/*
 	 * Don't allow insertions on the last (dummy) line of the text.
 	 */
-    
+
 	lineIndex = TkBTreeLineIndex(index.linePtr);
 	if (lineIndex == TkBTreeNumLines(textPtr->tree)) {
 	    lineIndex--;
-	    TkTextMakeIndex(textPtr->tree, lineIndex, 1000000, &index);
+	    TkTextMakeByteIndex(textPtr->tree, lineIndex, 1000000, &index);
 	}
 
 	/*
@@ -277,7 +277,7 @@ TkTextImageCmd(textPtr, interp, argc, argv)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -288,7 +288,7 @@ TkTextImageCmd(textPtr, interp, argc, argv)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message..
+ *	returned, then the interp's result contains an error message..
  *
  * Side effects:
  *	Configuration information for the embedded image changes,
@@ -347,7 +347,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
     	return TCL_OK;
     }
 
-    /* 
+    /*
      * Find a unique name for this image.  Use imageName (or imageString)
      * if available, otherwise tack on a #nn and use it.  If a name is already
      * associated with this image, delete the name.
@@ -384,7 +384,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
     Tcl_DStringAppend(&newName,name, -1);
 
     if (conflict) {
-    	char buf[10];
+    	char buf[4 + TCL_INTEGER_SPACE];
 	sprintf(buf, "#%d",count+1);
 	Tcl_DStringAppend(&newName,buf, -1);
     }
@@ -398,7 +398,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -423,12 +423,12 @@ AlignParseProc(clientData, interp, tkwin, valuearg, widgRec, offset)
     ClientData clientData;		/* Not used.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window for text widget. */
-    Arg valuearg;			/* Value of option. */
+    Tcl_Obj *valuearg;			/* Value of option. */
     char *widgRec;			/* Pointer to TkTextEmbWindow
 					 * structure. */
     int offset;				/* Offset into item (ignored). */
 {
-    char *value = LangString(valuearg);
+    char *value = Tcl_GetString(valuearg);
     register TkTextEmbImage *embPtr = (TkTextEmbImage *) widgRec;
 
     if (strcmp(value, "baseline") == 0) {
@@ -447,7 +447,7 @@ AlignParseProc(clientData, interp, tkwin, valuearg, widgRec, offset)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -478,8 +478,8 @@ AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
 					 * information about how to reclaim
 					 * storage for return string. */
-{   
-    Arg result = NULL; 
+{
+    Arg result = NULL;
     *freeProcPtr = (Tcl_FreeProc *) NULL;
     switch (((TkTextEmbImage *) widgRec)->align) {
 	case ALIGN_BASELINE:
@@ -499,7 +499,7 @@ AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
             return result;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -551,7 +551,7 @@ EmbImageDeleteProc(eiPtr, linePtr, treeGone)
     ckfree((char *) eiPtr);
     return 0;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -578,7 +578,7 @@ EmbImageCleanupProc(eiPtr, linePtr)
     eiPtr->body.ei.linePtr = linePtr;
     return eiPtr;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -612,8 +612,8 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
 				 * many characters. */
     int noCharsYet;		/* Non-zero means no characters have been
 				 * assigned to this line yet. */
-    Tk_Uid wrapMode;		/* Wrap mode to use for line: tkTextCharUid,
-				 * tkTextNoneUid, or tkTextWordUid. */
+    TkWrapMode wrapMode;	/* Wrap mode to use for line: TEXT_WRAPMODE_CHAR,
+				 * TEXT_WRAPMODE_NONE, or TEXT_WRAPMODE_WORD. */
     register TkTextDispChunk *chunkPtr;
 				/* Structure to fill in with information
 				 * about this chunk.  The x field has already
@@ -638,7 +638,7 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
 	height += 2*eiPtr->body.ei.padY;
     }
     if ((width > (maxX - chunkPtr->x))
-	    && !noCharsYet && (textPtr->wrapMode != tkTextNoneUid)) {
+	    && !noCharsYet && (textPtr->wrapMode != TEXT_WRAPMODE_NONE)) {
 	return 0;
     }
 
@@ -650,7 +650,7 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
     chunkPtr->undisplayProc = (Tk_ChunkUndisplayProc *) NULL;
     chunkPtr->measureProc = (Tk_ChunkMeasureProc *) NULL;
     chunkPtr->bboxProc = EmbImageBboxProc;
-    chunkPtr->numChars = 1;
+    chunkPtr->numBytes = 1;
     if (eiPtr->body.ei.align == ALIGN_BASELINE) {
 	chunkPtr->minAscent = height - eiPtr->body.ei.padY;
 	chunkPtr->minDescent = eiPtr->body.ei.padY;
@@ -667,7 +667,7 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
     eiPtr->body.ei.chunkCount += 1;
     return 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -698,7 +698,7 @@ EmbImageCheckProc(eiPtr, linePtr)
 	panic("EmbImageCheckProc: embedded image has size %d", eiPtr->size);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -760,7 +760,7 @@ EmbImageDisplayProc(chunkPtr, x, y, lineHeight, baseline, display, dst, screenY)
     Tk_RedrawImage(image, 0, 0, width, height, dst,
 	    imageX, imageY);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -828,7 +828,7 @@ EmbImageBboxProc(chunkPtr, index, y, lineHeight, baseline, xPtr, yPtr,
 	    break;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -852,7 +852,7 @@ EmbImageBboxProc(chunkPtr, index, y, lineHeight, baseline, xPtr, yPtr,
 int
 TkTextImageIndex(textPtr, name, indexPtr)
     TkText *textPtr;		/* Text widget containing image. */
-    char *name;			/* Name of image. */
+    CONST char *name;			/* Name of image. */
     TkTextIndex *indexPtr;	/* Index information gets stored here. */
 {
     Tcl_HashEntry *hPtr;
@@ -865,10 +865,10 @@ TkTextImageIndex(textPtr, name, indexPtr)
     eiPtr = (TkTextSegment *) Tcl_GetHashValue(hPtr);
     indexPtr->tree = textPtr->tree;
     indexPtr->linePtr = eiPtr->body.ei.linePtr;
-    indexPtr->charIndex = TkTextSegToOffset(eiPtr, indexPtr->linePtr);
+    indexPtr->byteIndex = TkTextSegToOffset(eiPtr, indexPtr->linePtr);
     return 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -901,6 +901,8 @@ EmbImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
 
     index.tree = eiPtr->body.ei.textPtr->tree;
     index.linePtr = eiPtr->body.ei.linePtr;
-    index.charIndex = TkTextSegToOffset(eiPtr, eiPtr->body.ei.linePtr);
+    index.byteIndex = TkTextSegToOffset(eiPtr, eiPtr->body.ei.linePtr);
     TkTextChanged(eiPtr->body.ei.textPtr, &index, &index);
 }
+
+
