@@ -6,12 +6,12 @@
  *	command procedures.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclEvent.c,v 1.3 1998/09/14 18:39:58 stanton Exp $
+ * RCS: @(#) $Id: tclEvent.c,v 1.23 2002/08/05 03:24:40 dgp Exp $
  */
 #include "tkPort.h"
 #include "tkInt.h"
@@ -28,8 +28,9 @@ typedef struct BgError {
     Tcl_Interp *interp;		/* Interpreter in which error occurred.  NULL
 				 * means this error report has been cancelled
 				 * (a previous report generated a break). */
-    char *errorMsg;		/* The error message (interp->result when
-				 * the error occurred).  Malloc-ed. */
+    char *errorMsg;		/* Copy of the error message (the interp's
+				 * result when the error occurred).
+				 * Malloc-ed. */
     char *errorInfo;		/* Value of the errorInfo variable
 				 * (malloc-ed). */
     char *errorCode;		/* Value of the errorCode variable
@@ -67,31 +68,41 @@ typedef struct ExitHandler {
 } ExitHandler;
 
 #ifdef TCL_EVENT_IMPLEMENT
+/*
+ * There is both per-process and per-thread exit handlers.
+ * The first list is controlled by a mutex.  The other is in
+ * thread local storage.
+ */
 
 static ExitHandler *firstExitPtr = NULL;
 				/* First in list of all exit handlers for
 				 * application. */
+TCL_DECLARE_MUTEX(exitMutex)
 
 /*
- * The following variable is a "secret" indication to Tcl_Exit that
- * it should dump out the state of memory before exiting.  If the
- * value is non-NULL, it gives the name of the file in which to
- * dump memory usage information.
- */
-
-char *tclMemDumpFileName = NULL;
-
-/*
- * This variable is set to 1 when Tcl_Exit is called, and at the end of
+ * This variable is set to 1 when Tcl_Finalize is called, and at the end of
  * its work, it is reset to 0. The variable is checked by TclInExit() to
  * allow different behavior for exit-time processing, e.g. in closing of
  * files and pipes.
  */
 
-static int tclInExit = 0;
+static int inFinalize = 0;
+static int subsystemsInitialized = 0;
 
-#endif
+typedef struct ThreadSpecificData {
+    ExitHandler *firstExitPtr;  /* First in list of all exit handlers for
+				 * this thread. */
+    int inExit;			/* True when this thread is exiting. This
+				 * is used as a hack to decide to close
+				 * the standard channels. */
+    Tcl_Obj *tclLibraryPath;	/* Path(s) to the Tcl library */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
+/*
+ * Common string for the library path for sharing across threads.
+ */
+char *tclLibraryPathStr;
 
 /*
  * Prototypes for procedures referenced only in this file:
@@ -100,10 +111,15 @@ static int tclInExit = 0;
 static void		BgErrorDeleteProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp));
 static void		HandleBgErrors _ANSI_ARGS_((ClientData clientData));
+#else
 static char *		VwaitVarProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Var name1, char *name2,
+			    Tcl_Interp *interp, Var name1, CONST char *name2,
 			    int flags));
-
+
+#endif
+#ifdef TCL_EVENT_IMPLEMENT
+
+
 #if 0
 /*
  *----------------------------------------------------------------------
@@ -131,8 +147,9 @@ Tcl_BackgroundError(interp)
 				 * occurred. */
 {
     BgError *errPtr;
-    char *errResult, *varValue;
+    CONST char *errResult, *varValue;
     ErrAssocData *assocPtr;
+    int length;
 
     /*
      * The Tcl_AddErrorInfo call below (with an empty string) ensures that
@@ -144,12 +161,12 @@ Tcl_BackgroundError(interp)
 
     Tcl_AddErrorInfo(interp, "");
 
-    errResult = TclGetStringFromObj(Tcl_GetObjResult(interp), (int *) NULL);
-	
+    errResult = Tcl_GetStringFromObj(Tcl_GetObjResult(interp), &length);
+
     errPtr = (BgError *) ckalloc(sizeof(BgError));
     errPtr->interp = interp;
-    errPtr->errorMsg = (char *) ckalloc((unsigned) (strlen(errResult) + 1));
-    strcpy(errPtr->errorMsg, errResult);
+    errPtr->errorMsg = (char *) ckalloc((unsigned) (length + 1));
+    memcpy(errPtr->errorMsg, errResult, (size_t) (length + 1));
     varValue = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
     if (varValue == NULL) {
 	varValue = errPtr->errorMsg;
@@ -189,7 +206,7 @@ Tcl_BackgroundError(interp)
     assocPtr->lastBgPtr = errPtr;
     Tcl_ResetResult(interp);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -212,8 +229,7 @@ HandleBgErrors(clientData)
     ClientData clientData;	/* Pointer to ErrAssocData structure. */
 {
     Tcl_Interp *interp;
-    char *command;
-    char *argv[2];
+    CONST char *argv[2];
     int code;
     BgError *errPtr;
     ErrAssocData *assocPtr = (ErrAssocData *) clientData;
@@ -243,11 +259,10 @@ HandleBgErrors(clientData)
 
 	argv[0] = "bgerror";
 	argv[1] = assocPtr->firstBgPtr->errorMsg;
-	command = Tcl_Merge(2, argv);
+
 	Tcl_AllowExceptions(interp);
         Tcl_Preserve((ClientData) interp);
-	code = Tcl_GlobalEval(interp, command);
-	ckfree(command);
+	code = TclGlobalInvoke(interp, 2, argv, 0);
 	if (code == TCL_ERROR) {
 
             /*
@@ -262,29 +277,11 @@ HandleBgErrors(clientData)
              */
 
             if (Tcl_IsSafe(interp)) {
-                Tcl_HashTable *hTblPtr;
-                Tcl_HashEntry *hPtr;
+		Tcl_SavedResult save;
 
-                hTblPtr = (Tcl_HashTable *) Tcl_GetAssocData(interp,
-                        "tclHiddenCmds", NULL);
-                if (hTblPtr == (Tcl_HashTable *) NULL) {
-                    goto doneWithInterp;
-                }
-                hPtr = Tcl_FindHashEntry(hTblPtr, "bgerror");
-                if (hPtr == (Tcl_HashEntry *) NULL) {
-                    goto doneWithInterp;
-                }
-
-                /*
-                 * OK, the hidden command "bgerror" exists, invoke it.
-                 */
-
-                argv[0] = "bgerror";
-                argv[1] = ckalloc((unsigned)
-                        strlen(assocPtr->firstBgPtr->errorMsg));
-                strcpy(argv[1], assocPtr->firstBgPtr->errorMsg);
-                (void) TclInvoke(interp, 2, argv, TCL_INVOKE_HIDDEN);
-                ckfree(argv[1]);
+		Tcl_SaveResult(interp, &save);
+                TclGlobalInvoke(interp, 2, argv, TCL_INVOKE_HIDDEN);
+		Tcl_RestoreResult(interp, &save);
 
                 goto doneWithInterp;
             }
@@ -296,22 +293,24 @@ HandleBgErrors(clientData)
 
             errChannel = Tcl_GetStdChannel(TCL_STDERR);
             if (errChannel != (Tcl_Channel) NULL) {
-                if (strcmp(interp->result,
-           "\"bgerror\" is an invalid command name or ambiguous abbreviation")
-                        == 0) {
-                    Tcl_Write(errChannel, assocPtr->firstBgPtr->errorInfo, -1);
-                    Tcl_Write(errChannel, "\n", -1);
+		char *string;
+		int len;
+
+		string = Tcl_GetStringFromObj(Tcl_GetObjResult(interp), &len);
+		if (Tcl_FindCommand(interp, "bgerror", NULL, TCL_GLOBAL_ONLY) == NULL) {
+                    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorInfo, -1);
+                    Tcl_WriteChars(errChannel, "\n", -1);
                 } else {
-                    Tcl_Write(errChannel,
+                    Tcl_WriteChars(errChannel,
                             "bgerror failed to handle background error.\n",
                             -1);
-                    Tcl_Write(errChannel, "    Original error: ", -1);
-                    Tcl_Write(errChannel, assocPtr->firstBgPtr->errorMsg,
+                    Tcl_WriteChars(errChannel, "    Original error: ", -1);
+                    Tcl_WriteChars(errChannel, assocPtr->firstBgPtr->errorMsg,
                             -1);
-                    Tcl_Write(errChannel, "\n", -1);
-                    Tcl_Write(errChannel, "    Error in bgerror: ", -1);
-                    Tcl_Write(errChannel, interp->result, -1);
-                    Tcl_Write(errChannel, "\n", -1);
+                    Tcl_WriteChars(errChannel, "\n", -1);
+                    Tcl_WriteChars(errChannel, "    Error in bgerror: ", -1);
+                    Tcl_WriteChars(errChannel, string, len);
+                    Tcl_WriteChars(errChannel, "\n", -1);
                 }
                 Tcl_Flush(errChannel);
             }
@@ -353,7 +352,7 @@ doneWithInterp:
 
     Tcl_Release((ClientData) assocPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -393,10 +392,8 @@ BgErrorDeleteProc(clientData, interp)
     Tcl_CancelIdleCall(HandleBgErrors, (ClientData) assocPtr);
     Tcl_EventuallyFree((ClientData) assocPtr, TCL_DYNAMIC);
 }
-#endif
-#ifdef TCL_EVENT_IMPLEMENT
 
-
+#endif
 /*
  *----------------------------------------------------------------------
  *
@@ -426,10 +423,12 @@ Tcl_CreateExitHandler(proc, clientData)
     exitPtr = (ExitHandler *) ckalloc(sizeof(ExitHandler));
     exitPtr->proc = proc;
     exitPtr->clientData = clientData;
+    Tcl_MutexLock(&exitMutex);
     exitPtr->nextPtr = firstExitPtr;
     firstExitPtr = exitPtr;
+    Tcl_MutexUnlock(&exitMutex);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -456,6 +455,7 @@ Tcl_DeleteExitHandler(proc, clientData)
 {
     ExitHandler *exitPtr, *prevPtr;
 
+    Tcl_MutexLock(&exitMutex);
     for (prevPtr = NULL, exitPtr = firstExitPtr; exitPtr != NULL;
 	    prevPtr = exitPtr, exitPtr = exitPtr->nextPtr) {
 	if ((exitPtr->proc == proc)
@@ -466,12 +466,88 @@ Tcl_DeleteExitHandler(proc, clientData)
 		prevPtr->nextPtr = exitPtr->nextPtr;
 	    }
 	    ckfree((char *) exitPtr);
+	    break;
+	}
+    }
+    Tcl_MutexUnlock(&exitMutex);
+    return;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_CreateThreadExitHandler --
+ *
+ *	Arrange for a given procedure to be invoked just before the
+ *	current thread exits.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Proc will be invoked with clientData as argument when the
+ *	application exits.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+Tcl_CreateThreadExitHandler(proc, clientData)
+    Tcl_ExitProc *proc;		/* Procedure to invoke. */
+    ClientData clientData;	/* Arbitrary value to pass to proc. */
+{
+    ExitHandler *exitPtr;
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+
+    exitPtr = (ExitHandler *) ckalloc(sizeof(ExitHandler));
+    exitPtr->proc = proc;
+    exitPtr->clientData = clientData;
+    exitPtr->nextPtr = tsdPtr->firstExitPtr;
+    tsdPtr->firstExitPtr = exitPtr;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_DeleteThreadExitHandler --
+ *
+ *	This procedure cancels an existing exit handler matching proc
+ *	and clientData, if such a handler exits.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	If there is an exit handler corresponding to proc and clientData
+ *	then it is cancelled;  if no such handler exists then nothing
+ *	happens.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+Tcl_DeleteThreadExitHandler(proc, clientData)
+    Tcl_ExitProc *proc;		/* Procedure that was previously registered. */
+    ClientData clientData;	/* Arbitrary value to pass to proc. */
+{
+    ExitHandler *exitPtr, *prevPtr;
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+
+    for (prevPtr = NULL, exitPtr = tsdPtr->firstExitPtr; exitPtr != NULL;
+	    prevPtr = exitPtr, exitPtr = exitPtr->nextPtr) {
+	if ((exitPtr->proc == proc)
+		&& (exitPtr->clientData == clientData)) {
+	    if (prevPtr == NULL) {
+		tsdPtr->firstExitPtr = exitPtr->nextPtr;
+	    } else {
+		prevPtr->nextPtr = exitPtr->nextPtr;
+	    }
+	    ckfree((char *) exitPtr);
 	    return;
 	}
     }
 }
 
-
 /*
  *----------------------------------------------------------------------
  *
@@ -495,29 +571,207 @@ Tcl_Exit(status)
 				 * 0 for normal return, 1 for error return. */
 {
     Tcl_Finalize();
-#ifdef TCL_MEM_DEBUG
-    if (tclMemDumpFileName != NULL) {
-	Tcl_DumpActiveMemory(tclMemDumpFileName);
-    }
-#endif
-    TclPlatformExit(status);
+    TclpExit(status);
 }
-
+
 /*
- *----------------------------------------------------------------------
+ *-------------------------------------------------------------------------
  *
- * Tcl_Finalize --
+ * TclSetLibraryPath --
  *
- *	Runs the exit handlers to allow Tcl to clean up its state prior
- *	to being unloaded. Called by Tcl_Exit and when Tcl was dynamically
- *	loaded and is now being unloaded.
+ *	Set the path that will be used for searching for init.tcl and
+ *	encodings when an interp is being created.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Whatever the exit handlers do. Also frees up storage associated
- *	with the Tcl object type table.
+ *	Changing the library path will affect what directories are
+ *	examined when looking for encodings for all interps from that
+ *	point forward.
+ *
+ *	The refcount of the new library path is incremented and the
+ *	refcount of the old path is decremented.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+#ifndef _LANG
+void
+TclSetLibraryPath(pathPtr)
+    Tcl_Obj *pathPtr;		/* A Tcl list object whose elements are
+				 * the new library path. */
+{
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+
+    if (pathPtr != NULL) {
+	Tcl_IncrRefCount(pathPtr);
+    }
+    if (tsdPtr->tclLibraryPath != NULL) {
+	Tcl_DecrRefCount(tsdPtr->tclLibraryPath);
+    }
+    tsdPtr->tclLibraryPath = pathPtr;
+
+    /*
+     *  No mutex locking is needed here as up the stack we're within
+     *  TclpInitLock().
+     */
+    tclLibraryPathStr = Tcl_GetStringFromObj(pathPtr, NULL);
+}
+
+#endif
+/*
+ *-------------------------------------------------------------------------
+ *
+ * TclGetLibraryPath --
+ *
+ *	Return a Tcl list object whose elements are the library path.
+ *	The caller should not modify the contents of the returned object.
+ *
+ * Results:
+ *	As above.
+ *
+ * Side effects:
+ *	None.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+#ifndef _LANG
+Tcl_Obj *
+TclGetLibraryPath()
+{
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+
+    if (tsdPtr->tclLibraryPath == NULL) {
+	/*
+	 * Grab the shared string and place it into a new thread specific
+	 * Tcl_Obj.
+	 */
+	tsdPtr->tclLibraryPath = Tcl_NewStringObj(tclLibraryPathStr, -1);
+
+	/* take ownership */
+	Tcl_IncrRefCount(tsdPtr->tclLibraryPath);
+    }
+    return tsdPtr->tclLibraryPath;
+}
+
+#endif
+/*
+ *-------------------------------------------------------------------------
+ *
+ * TclInitSubsystems --
+ *
+ *	Initialize various subsytems in Tcl.  This should be called the
+ *	first time an interp is created, or before any of the subsystems
+ *	are used.  This function ensures an order for the initialization
+ *	of subsystems:
+ *
+ *	1. that cannot be initialized in lazy order because they are
+ *	mutually dependent.
+ *
+ *	2. so that they can be finalized in a known order w/o causing
+ *	the subsequent re-initialization of a subsystem in the act of
+ *	shutting down another.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Varied, see the respective initialization routines.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+void
+TclInitSubsystems(argv0)
+    CONST char *argv0;		/* Name of executable from argv[0] to main()
+				 * in native multi-byte encoding. */
+{
+    ThreadSpecificData *tsdPtr;
+
+    if (inFinalize != 0) {
+	panic("TclInitSubsystems called while finalizing");
+    }
+
+    /*
+     * Grab the thread local storage pointer before doing anything because
+     * the initialization routines will be registering exit handlers.
+     * We use this pointer to detect if this is the first time this
+     * thread has created an interpreter.
+     */
+
+    tsdPtr = (ThreadSpecificData *) TclThreadDataKeyGet(&dataKey);
+
+    if (subsystemsInitialized == 0) {
+	/*
+	 * Double check inside the mutex.  There are definitly calls
+	 * back into this routine from some of the procedures below.
+	 */
+
+	TclpInitLock();
+	if (subsystemsInitialized == 0) {
+	    /*
+	     * Have to set this bit here to avoid deadlock with the
+	     * routines below us that call into TclInitSubsystems.
+	     */
+
+	    subsystemsInitialized = 1;
+
+#ifndef _LANG
+	    tclExecutableName = NULL;
+
+	    /*
+	     * Initialize locks used by the memory allocators before anything
+	     * interesting happens so we can use the allocators in the
+	     * implementation of self-initializing locks.
+	     */
+#if USE_TCLALLOC
+	    TclInitAlloc(); /* process wide mutex init */
+#endif
+#ifdef TCL_MEM_DEBUG
+	    TclInitDbCkalloc(); /* process wide mutex init */
+#endif
+
+#endif
+	    TclpInitPlatform(); /* creates signal handler(s) */
+	    TclInitObjSubsystem(); /* register obj types, create mutexes */
+	    TclInitIOSubsystem(); /* inits a tsd key (noop) */
+#ifndef _LANG
+	    TclInitEncodingSubsystem(); /* process wide encoding init */
+	    TclInitNamespaceSubsystem(); /* register ns obj type (mutexed) */
+#endif
+	}
+	TclpInitUnlock();
+    }
+
+    if (tsdPtr == NULL) {
+	/*
+	 * First time this thread has created an interpreter.
+	 * We fetch the key again just in case no exit handlers were
+	 * registered by this point.
+	 */
+
+	(void) TCL_TSD_INIT(&dataKey);
+	TclInitNotifier();
+     }
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_Finalize --
+ *
+ *	Shut down Tcl.  First calls registered exit handlers, then
+ *	carefully shuts down various subsystems.
+ *	Called by Tcl_Exit or when the Tcl shared library is being
+ *	unloaded.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Varied, see the respective finalization routines.
  *
  *----------------------------------------------------------------------
  */
@@ -525,39 +779,167 @@ Tcl_Exit(status)
 void
 Tcl_Finalize()
 {
+#ifndef _LANG
     ExitHandler *exitPtr;
+    ThreadSpecificData *tsdPtr;
 
-    /*
-     * Invoke exit handler first.
-     */
+    TclpInitLock();
+    if (subsystemsInitialized != 0) {
+	subsystemsInitialized = 0;
+
+	tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    tclInExit = 1;
-    for (exitPtr = firstExitPtr; exitPtr != NULL; exitPtr = firstExitPtr) {
 	/*
-	 * Be careful to remove the handler from the list before invoking
-	 * its callback.  This protects us against double-freeing if the
-	 * callback should call Tcl_DeleteExitHandler on itself.
+	 * Invoke exit handlers first.
 	 */
 
-	firstExitPtr = exitPtr->nextPtr;
-	(*exitPtr->proc)(exitPtr->clientData);
-	ckfree((char *) exitPtr);
+	Tcl_MutexLock(&exitMutex);
+	inFinalize = 1;
+	for (exitPtr = firstExitPtr; exitPtr != NULL; exitPtr = firstExitPtr) {
+	    /*
+	     * Be careful to remove the handler from the list before
+	     * invoking its callback.  This protects us against
+	     * double-freeing if the callback should call
+	     * Tcl_DeleteExitHandler on itself.
+	     */
+
+	    firstExitPtr = exitPtr->nextPtr;
+	    Tcl_MutexUnlock(&exitMutex);
+	    (*exitPtr->proc)(exitPtr->clientData);
+	    ckfree((char *) exitPtr);
+	    Tcl_MutexLock(&exitMutex);
+	}
+	firstExitPtr = NULL;
+	Tcl_MutexUnlock(&exitMutex);
+
+
+	/*
+	 * Clean up after the current thread now, after exit handlers.
+	 * In particular, the testexithandler command sets up something
+	 * that writes to standard output, which gets closed.
+	 * Note that there is no thread-local storage after this call.
+	 */
+
+	Tcl_FinalizeThread();
+
+	/*
+	 * Now finalize the Tcl execution environment.  Note that this
+	 * must be done after the exit handlers, because there are
+	 * order dependencies.
+	 */
+
+	TclFinalizeCompExecEnv();
+	TclFinalizeEnvironment();
+
+	TclFinalizeEncodingSubsystem();
+
+	if (tclExecutableName != NULL) {
+	    ckfree(tclExecutableName);
+	    tclExecutableName = NULL;
+	}
+	if (tclNativeExecutableName != NULL) {
+	    ckfree(tclNativeExecutableName);
+	    tclNativeExecutableName = NULL;
+	}
+	if (tclDefaultEncodingDir != NULL) {
+	    ckfree(tclDefaultEncodingDir);
+	    tclDefaultEncodingDir = NULL;
+	}
+
+	Tcl_SetPanicProc(NULL);
+
+	/*
+	 * Free synchronization objects.  There really should only be one
+	 * thread alive at this moment.
+	 */
+
+	TclFinalizeSynchronization();
+
+	/**
+	 * Finalizing the filesystem must come after anything which
+	 * might conceivably interact with the 'Tcl_FS' API.  This
+	 * will also unload any extensions which have been loaded.
+	 */
+	TclFinalizeFilesystem();
+
+	/*
+	 * There shouldn't be any malloc'ed memory after this.
+	 */
+
+	TclFinalizeMemorySubsystem();
+	inFinalize = 0;
     }
-
-    /*
-     * Now finalize the Tcl execution environment.  Note that this must be done
-     * after the exit handlers, because there are order dependencies.
-     */
-
-#if 0
-    TclFinalizeCompExecEnv();
-    TclFinalizeEnvironment();
-    TclpFinalize();
+    TclpInitUnlock();
 #endif
-    firstExitPtr = NULL;
-    tclInExit = 0;
 }
-
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_FinalizeThread --
+ *
+ *	Runs the exit handlers to allow Tcl to clean up its state
+ *	about a particular thread.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Varied, see the respective finalization routines.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+Tcl_FinalizeThread()
+{
+#ifndef _LANG
+    ExitHandler *exitPtr;
+    ThreadSpecificData *tsdPtr =
+	    (ThreadSpecificData *)TclThreadDataKeyGet(&dataKey);
+
+    if (tsdPtr != NULL) {
+	/*
+	 * Invoke thread exit handlers first.
+	 */
+
+	tsdPtr->inExit = 1;
+
+	/*
+	 * Clean up the library path now, before we invalidate thread-local
+	 * storage or calling thread exit handlers.
+	 */
+
+	if (tsdPtr->tclLibraryPath != NULL) {
+	    Tcl_DecrRefCount(tsdPtr->tclLibraryPath);
+	    tsdPtr->tclLibraryPath = NULL;
+	}
+
+	for (exitPtr = tsdPtr->firstExitPtr; exitPtr != NULL;
+		exitPtr = tsdPtr->firstExitPtr) {
+	    /*
+	     * Be careful to remove the handler from the list before invoking
+	     * its callback.  This protects us against double-freeing if the
+	     * callback should call Tcl_DeleteThreadExitHandler on itself.
+	     */
+
+	    tsdPtr->firstExitPtr = exitPtr->nextPtr;
+	    (*exitPtr->proc)(exitPtr->clientData);
+	    ckfree((char *) exitPtr);
+	}
+	TclFinalizeIOSubsystem();
+	TclFinalizeNotifier();
+	TclFinalizeAsync();
+
+	/*
+	 * Blow away all thread local storage blocks.
+	 */
+
+	TclFinalizeThreadData();
+    }
+#endif
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -577,17 +959,20 @@ Tcl_Finalize()
 int
 TclInExit()
 {
-    return tclInExit;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+	    TclThreadDataKeyGet(&dataKey);
+    if (tsdPtr == NULL) {
+	return inFinalize;
+    } else {
+	return tsdPtr->inExit;
+    }
 }
 
 #else /* TCL_EVENT_IMPLEMENT */
-
-
-
 /*
  *----------------------------------------------------------------------
  *
- * Tcl_VwaitCmd --
+ * Tcl_VwaitObjCmd --
  *
  *	This procedure is invoked to process the "vwait" Tcl command.
  *	See the user documentation for details on what it does.
@@ -603,23 +988,19 @@ TclInExit()
 
 	/* ARGSUSED */
 int
-Tcl_VwaitCmd(clientData, interp, argc, argv)
+Tcl_VwaitObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     int done, foundEvent;
-    Var variable;
 
-    if (argc != 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " name\"", (char *) NULL);
+    if (objc != 2) {
+        Tcl_WrongNumArgs(interp, 1, objv, "name");
 	return TCL_ERROR;
     }
-    if (LangSaveVar(interp, objv[1], &variable, TK_CONFIG_SCALARVAR) != TCL_OK)
-	return TCL_ERROR;
-    if (Tcl_TraceVar(interp, variable,
+    if (Lang_TraceVar(interp, objv[1],
 	    TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 	    VwaitVarProc, (ClientData) &done) != TCL_OK) {
 	return TCL_ERROR;
@@ -629,10 +1010,9 @@ Tcl_VwaitCmd(clientData, interp, argc, argv)
     while (!done && foundEvent) {
 	foundEvent = Tcl_DoOneEvent(TCL_ALL_EVENTS);
     }
-    Tcl_UntraceVar(interp, variable,
+    Lang_UntraceVar(interp, objv[1],
 	    TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 	    VwaitVarProc, (ClientData) &done);
-    LangFreeVar(variable);
 
     /*
      * Clear out the interpreter's result, since it may have been set
@@ -641,7 +1021,7 @@ Tcl_VwaitCmd(clientData, interp, argc, argv)
 
     Tcl_ResetResult(interp);
     if (!foundEvent) {
-	Tcl_AppendResult(interp, "can't wait for variable \"", argv[1],
+	Tcl_AppendResult(interp, "can't wait for variable \"", Tcl_GetString(objv[1]),
 		"\":  would wait forever", (char *) NULL);
 	return TCL_ERROR;
     }
@@ -654,7 +1034,7 @@ VwaitVarProc(clientData, interp, name1, name2, flags)
     ClientData clientData;	/* Pointer to integer to set to 1. */
     Tcl_Interp *interp;		/* Interpreter containing variable. */
     Var name1;			/* Name of variable. */
-    char *name2;		/* Second part of variable name. */
+    CONST char *name2;		/* Second part of variable name. */
     int flags;			/* Information about what happened. */
 {
     int *donePtr = (int *) clientData;
@@ -662,11 +1042,11 @@ VwaitVarProc(clientData, interp, name1, name2, flags)
     *donePtr = 1;
     return (char *) NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
- * Tcl_UpdateCmd --
+ * Tcl_UpdateObjCmd --
  *
  *	This procedure is invoked to process the "update" Tcl command.
  *	See the user documentation for details on what it does.
@@ -682,26 +1062,35 @@ VwaitVarProc(clientData, interp, name1, name2, flags)
 
 	/* ARGSUSED */
 int
-Tcl_UpdateCmd(clientData, interp, argc, argv)
+Tcl_UpdateObjCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Not used. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    int flags;
+    int optionIndex;
+    int flags = 0;		/* Initialized to avoid compiler warning. */
+    static CONST char *updateOptions[] = {"idletasks", (char *) NULL};
+    enum updateOptions {REGEXP_IDLETASKS};
 
-    if (argc == 1) {
+    if (objc == 1) {
 	flags = TCL_ALL_EVENTS|TCL_DONT_WAIT;
-    } else if (argc == 2) {
-	if (strncmp(argv[1], "idletasks", strlen(argv[1])) != 0) {
-	    Tcl_AppendResult(interp, "bad option \"", argv[1],
-		    "\": must be idletasks", (char *) NULL);
+    } else if (objc == 2) {
+	if (Tcl_GetIndexFromObj(interp, objv[1], updateOptions,
+		"option", 0, &optionIndex) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	flags = TCL_WINDOW_EVENTS|TCL_IDLE_EVENTS|TCL_DONT_WAIT;
+	switch ((enum updateOptions) optionIndex) {
+	    case REGEXP_IDLETASKS: {
+		flags = TCL_WINDOW_EVENTS|TCL_IDLE_EVENTS|TCL_DONT_WAIT;
+		break;
+	    }
+	    default: {
+		panic("Tcl_UpdateObjCmd: bad option index to UpdateOptions");
+	    }
+	}
     } else {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " ?idletasks?\"", (char *) NULL);
+        Tcl_WrongNumArgs(interp, 1, objv, "?idletasks?");
 	return TCL_ERROR;
     }
 
@@ -719,3 +1108,6 @@ Tcl_UpdateCmd(clientData, interp, argc, argv)
 }
 
 #endif /* TCL_EVENT_IMPLEMENT */
+
+
+
