@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclTimer.c,v 1.2 1998/09/14 18:40:02 stanton Exp $
+ * RCS: @(#) $Id: tclTimer.c,v 1.6 2002/03/01 06:22:31 hobbs Exp $
  */
 #include "tkPort.h"
 #include "Lang.h"
@@ -38,12 +38,6 @@ typedef struct TimerHandler {
 					 * end of queue. */
 } TimerHandler;
 
-static TimerHandler *firstTimerHandlerPtr = NULL;
-					/* First event in queue. */
-static int lastTimerId;			/* Timer identifier of most recently
-					 * created timer. */
-static int timerPending;		/* 1 if a timer event is in the queue. */
-
 /*
  * The data structure below is used by the "after" command to remember
  * the command to be executed later.  All of the pending "after" commands
@@ -55,8 +49,7 @@ typedef struct AfterInfo {
 				/* Pointer to the "tclAfter" assocData for
 				 * the interp in which command will be
 				 * executed. */
-    LangCallback *command;	/* Command to execute.  Malloc'ed, so must
-				 * be freed when structure is deallocated. */
+    Tcl_Obj *commandPtr;	/* Command to execute. */
     int id;			/* Integer identifier for command;  used to
 				 * cancel it. */
     Tcl_TimerToken token;	/* Used to cancel the "after" command.  NULL
@@ -97,16 +90,35 @@ typedef struct IdleHandler {
     struct IdleHandler *nextPtr;/* Next in list of active handlers. */
 } IdleHandler;
 
-static IdleHandler *idleList;
-				/* First in list of all idle handlers. */
-static IdleHandler *lastIdlePtr;
-				/* Last in list (or NULL for empty list). */
-static int idleGeneration;	/* Used to fill in the "generation" fields
+/*
+ * The timer and idle queues are per-thread because they are associated
+ * with the notifier, which is also per-thread.
+ *
+ * All static variables used in this file are collected into a single
+ * instance of the following structure.  For multi-threaded implementations,
+ * there is one instance of this structure for each thread.
+ *
+ * Notice that different structures with the same name appear in other
+ * files.  The structure defined below is used in this file only.
+ */
+
+typedef struct ThreadSpecificData {
+    TimerHandler *firstTimerHandlerPtr;	/* First event in queue. */
+    int lastTimerId;		/* Timer identifier of most recently
+				 * created timer. */
+    int timerPending;		/* 1 if a timer event is in the queue. */
+    IdleHandler *idleList;	/* First in list of all idle handlers. */
+    IdleHandler *lastIdlePtr;	/* Last in list (or NULL for empty list). */
+    int idleGeneration;		/* Used to fill in the "generation" fields
 				 * of IdleHandler structures.  Increments
 				 * each time Tcl_DoOneEvent starts calling
 				 * idle handlers, so that all old handlers
 				 * can be called without calling any of the
 				 * new ones created by old ones. */
+    int afterId;		/* For unique identifiers of after events. */
+} ThreadSpecificData;
+
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Prototypes for procedures referenced only in this file:
@@ -118,9 +130,9 @@ static void		AfterCleanupProc _ANSI_ARGS_((ClientData clientData,
 static void		AfterProc _ANSI_ARGS_((ClientData clientData));
 static void		FreeAfterPtr _ANSI_ARGS_((AfterInfo *afterPtr));
 static AfterInfo *	GetAfterEvent _ANSI_ARGS_((AfterAssocData *assocPtr,
-			    char *string));
+			    Tcl_Obj *commandPtr));
 #else
-static void		InitTimer _ANSI_ARGS_((void));
+static ThreadSpecificData *InitTimer _ANSI_ARGS_((void));
 static void		TimerExitProc _ANSI_ARGS_((ClientData clientData));
 static int		TimerHandlerEventProc _ANSI_ARGS_((Tcl_Event *evPtr,
 			    int flags));
@@ -128,10 +140,9 @@ static void		TimerCheckProc _ANSI_ARGS_((ClientData clientData,
 			    int flags));
 static void		TimerSetupProc _ANSI_ARGS_((ClientData clientData,
 			    int flags));
+
 #endif
 #ifdef TCL_EVENT_IMPLEMENT
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -140,7 +151,7 @@ static void		TimerSetupProc _ANSI_ARGS_((ClientData clientData,
  *	This function initializes the timer module.
  *
  * Results:
- *	None.
+ *	A pointer to the thread specific data.
  *
  * Side effects:
  *	Registers the idle and timer event sources.
@@ -148,21 +159,25 @@ static void		TimerSetupProc _ANSI_ARGS_((ClientData clientData,
  *----------------------------------------------------------------------
  */
 
-static void
+static ThreadSpecificData *
 InitTimer()
 {
-    initialized = 1;
-    lastTimerId = 0;
-    timerPending = 0;
-    idleGeneration = 0;
-    firstTimerHandlerPtr = NULL;
-    lastIdlePtr = NULL;
-    idleList = NULL;
+    ThreadSpecificData *tsdPtr =
+	(ThreadSpecificData *) TclThreadDataKeyGet(&dataKey);
 
-    Tcl_CreateEventSource(TimerSetupProc, TimerCheckProc, NULL);
-    Tcl_CreateExitHandler(TimerExitProc, NULL);
+    if (tsdPtr == NULL) {
+	tsdPtr = TCL_TSD_INIT(&dataKey);
+	Tcl_CreateEventSource(TimerSetupProc, TimerCheckProc, NULL);
+	Tcl_CreateThreadExitHandler(TimerExitProc, NULL);
+    }
+    return tsdPtr;
+}
+
+void *
+TkInitTimer()
+{
+ return InitTimer();
 }
-
 /*
  *----------------------------------------------------------------------
  *
@@ -175,7 +190,7 @@ InitTimer()
  *	None.
  *
  * Side effects:
- *	Removes the timer and idle event sources.
+ *	Removes the timer and idle event sources and remaining events.
  *
  *----------------------------------------------------------------------
  */
@@ -184,10 +199,21 @@ static void
 TimerExitProc(clientData)
     ClientData clientData;	/* Not used. */
 {
+    ThreadSpecificData *tsdPtr =
+	(ThreadSpecificData *) TclThreadDataKeyGet(&dataKey);
+
     Tcl_DeleteEventSource(TimerSetupProc, TimerCheckProc, NULL);
-    initialized = 0;
+    if (tsdPtr != NULL) {
+	register TimerHandler *timerHandlerPtr;
+	timerHandlerPtr = tsdPtr->firstTimerHandlerPtr;
+	while (timerHandlerPtr != NULL) {
+	    tsdPtr->firstTimerHandlerPtr = timerHandlerPtr->nextPtr;
+	    ckfree((char *) timerHandlerPtr);
+	    timerHandlerPtr = tsdPtr->firstTimerHandlerPtr;
+	}
+    }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -217,10 +243,9 @@ Tcl_CreateTimerHandler(milliseconds, proc, clientData)
 {
     register TimerHandler *timerHandlerPtr, *tPtr2, *prevPtr;
     Tcl_Time time;
+    ThreadSpecificData *tsdPtr;
 
-    if (!initialized) {
-	InitTimer();
-    }
+    tsdPtr = InitTimer();
 
     timerHandlerPtr = (TimerHandler *) ckalloc(sizeof(TimerHandler));
 
@@ -228,7 +253,7 @@ Tcl_CreateTimerHandler(milliseconds, proc, clientData)
      * Compute when the event should fire.
      */
 
-    TclpGetTime(&time);
+    Tcl_GetTime(&time);
     timerHandlerPtr->time.sec = time.sec + milliseconds/1000;
     timerHandlerPtr->time.usec = time.usec + (milliseconds%1000)*1000;
     if (timerHandlerPtr->time.usec >= 1000000) {
@@ -242,15 +267,15 @@ Tcl_CreateTimerHandler(milliseconds, proc, clientData)
 
     timerHandlerPtr->proc = proc;
     timerHandlerPtr->clientData = clientData;
-    lastTimerId++;
-    timerHandlerPtr->token = (Tcl_TimerToken) lastTimerId;
+    tsdPtr->lastTimerId++;
+    timerHandlerPtr->token = (Tcl_TimerToken) tsdPtr->lastTimerId;
 
     /*
      * Add the event to the queue in the correct position
      * (ordered by event firing time).
      */
 
-    for (tPtr2 = firstTimerHandlerPtr, prevPtr = NULL; tPtr2 != NULL;
+    for (tPtr2 = tsdPtr->firstTimerHandlerPtr, prevPtr = NULL; tPtr2 != NULL;
 	    prevPtr = tPtr2, tPtr2 = tPtr2->nextPtr) {
 	if ((tPtr2->time.sec > timerHandlerPtr->time.sec)
 		|| ((tPtr2->time.sec == timerHandlerPtr->time.sec)
@@ -260,15 +285,16 @@ Tcl_CreateTimerHandler(milliseconds, proc, clientData)
     }
     timerHandlerPtr->nextPtr = tPtr2;
     if (prevPtr == NULL) {
-	firstTimerHandlerPtr = timerHandlerPtr;
+	tsdPtr->firstTimerHandlerPtr = timerHandlerPtr;
     } else {
 	prevPtr->nextPtr = timerHandlerPtr;
     }
 
     TimerSetupProc(NULL, TCL_ALL_EVENTS);
+
     return timerHandlerPtr->token;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -294,15 +320,17 @@ Tcl_DeleteTimerHandler(token)
 				 * Tcl_DeleteTimerHandler. */
 {
     register TimerHandler *timerHandlerPtr, *prevPtr;
+    ThreadSpecificData *tsdPtr;
 
-    for (timerHandlerPtr = firstTimerHandlerPtr, prevPtr = NULL;
+    tsdPtr = InitTimer();
+    for (timerHandlerPtr = tsdPtr->firstTimerHandlerPtr, prevPtr = NULL;
 	    timerHandlerPtr != NULL; prevPtr = timerHandlerPtr,
 	    timerHandlerPtr = timerHandlerPtr->nextPtr) {
 	if (timerHandlerPtr->token != token) {
 	    continue;
 	}
 	if (prevPtr == NULL) {
-	    firstTimerHandlerPtr = timerHandlerPtr->nextPtr;
+	    tsdPtr->firstTimerHandlerPtr = timerHandlerPtr->nextPtr;
 	} else {
 	    prevPtr->nextPtr = timerHandlerPtr->nextPtr;
 	}
@@ -310,7 +338,7 @@ Tcl_DeleteTimerHandler(token)
 	return;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -335,9 +363,10 @@ TimerSetupProc(data, flags)
     int flags;			/* Event flags as passed to Tcl_DoOneEvent. */
 {
     Tcl_Time blockTime;
+    ThreadSpecificData *tsdPtr = InitTimer();
 
-    if (((flags & TCL_IDLE_EVENTS) && idleList)
-	    || ((flags & TCL_TIMER_EVENTS) && timerPending)) {
+    if (((flags & TCL_IDLE_EVENTS) && tsdPtr->idleList)
+	    || ((flags & TCL_TIMER_EVENTS) && tsdPtr->timerPending)) {
 	/*
 	 * There is an idle handler or a pending timer event, so just poll.
 	 */
@@ -345,14 +374,15 @@ TimerSetupProc(data, flags)
 	blockTime.sec = 0;
 	blockTime.usec = 0;
 
-    } else if ((flags & TCL_TIMER_EVENTS) && firstTimerHandlerPtr) {
+    } else if ((flags & TCL_TIMER_EVENTS) && tsdPtr->firstTimerHandlerPtr) {
 	/*
 	 * Compute the timeout for the next timer on the list.
 	 */
 
-	TclpGetTime(&blockTime);
-	blockTime.sec = firstTimerHandlerPtr->time.sec - blockTime.sec;
-	blockTime.usec = firstTimerHandlerPtr->time.usec - blockTime.usec;
+	Tcl_GetTime(&blockTime);
+	blockTime.sec = tsdPtr->firstTimerHandlerPtr->time.sec - blockTime.sec;
+	blockTime.usec = tsdPtr->firstTimerHandlerPtr->time.usec -
+		blockTime.usec;
 	if (blockTime.usec < 0) {
 	    blockTime.sec -= 1;
 	    blockTime.usec += 1000000;
@@ -364,10 +394,10 @@ TimerSetupProc(data, flags)
     } else {
 	return;
     }
-	
+
     Tcl_SetMaxBlockTime(&blockTime);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -393,15 +423,17 @@ TimerCheckProc(data, flags)
 {
     Tcl_Event *timerEvPtr;
     Tcl_Time blockTime;
+    ThreadSpecificData *tsdPtr = InitTimer();
 
-    if ((flags & TCL_TIMER_EVENTS) && firstTimerHandlerPtr) {
+    if ((flags & TCL_TIMER_EVENTS) && tsdPtr->firstTimerHandlerPtr) {
 	/*
 	 * Compute the timeout for the next timer on the list.
 	 */
 
-	TclpGetTime(&blockTime);
-	blockTime.sec = firstTimerHandlerPtr->time.sec - blockTime.sec;
-	blockTime.usec = firstTimerHandlerPtr->time.usec - blockTime.usec;
+	Tcl_GetTime(&blockTime);
+	blockTime.sec = tsdPtr->firstTimerHandlerPtr->time.sec - blockTime.sec;
+	blockTime.usec = tsdPtr->firstTimerHandlerPtr->time.usec -
+		blockTime.usec;
 	if (blockTime.usec < 0) {
 	    blockTime.sec -= 1;
 	    blockTime.usec += 1000000;
@@ -415,15 +447,16 @@ TimerCheckProc(data, flags)
 	 * If the first timer has expired, stick an event on the queue.
 	 */
 
-	if (blockTime.sec == 0 && blockTime.usec == 0 && !timerPending) {
-	    timerPending = 1;
+	if (blockTime.sec == 0 && blockTime.usec == 0 &&
+		!tsdPtr->timerPending) {
+	    tsdPtr->timerPending = 1;
 	    timerEvPtr = (Tcl_Event *) ckalloc(sizeof(Tcl_Event));
 	    timerEvPtr->proc = TimerHandlerEventProc;
 	    Tcl_QueueEvent(timerEvPtr, TCL_QUEUE_TAIL);
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -455,6 +488,7 @@ TimerHandlerEventProc(evPtr, flags)
     TimerHandler *timerHandlerPtr, **nextPtrPtr;
     Tcl_Time time;
     int currentTimerId;
+    ThreadSpecificData *tsdPtr = InitTimer();
 
     /*
      * Do nothing if timers aren't enabled.  This leaves the event on the
@@ -493,16 +527,16 @@ TimerHandlerEventProc(evPtr, flags)
      *	  appearing before later ones.
      */
 
-    timerPending = 0;
-    currentTimerId = lastTimerId;
-    TclpGetTime(&time);
+    tsdPtr->timerPending = 0;
+    currentTimerId = tsdPtr->lastTimerId;
+    Tcl_GetTime(&time);
     while (1) {
-	nextPtrPtr = &firstTimerHandlerPtr;
-	timerHandlerPtr = firstTimerHandlerPtr;
+	nextPtrPtr = &tsdPtr->firstTimerHandlerPtr;
+	timerHandlerPtr = tsdPtr->firstTimerHandlerPtr;
 	if (timerHandlerPtr == NULL) {
 	    break;
 	}
-	
+
 	if ((timerHandlerPtr->time.sec > time.sec)
 		|| ((timerHandlerPtr->time.sec == time.sec)
 			&& (timerHandlerPtr->time.usec > time.usec))) {
@@ -529,7 +563,7 @@ TimerHandlerEventProc(evPtr, flags)
     TimerSetupProc(NULL, TCL_TIMER_EVENTS);
     return 1;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -556,28 +590,25 @@ Tcl_DoWhenIdle(proc, clientData)
 {
     register IdleHandler *idlePtr;
     Tcl_Time blockTime;
-
-    if (!initialized) {
-	InitTimer();
-    }
+    ThreadSpecificData *tsdPtr = InitTimer();
 
     idlePtr = (IdleHandler *) ckalloc(sizeof(IdleHandler));
     idlePtr->proc = proc;
     idlePtr->clientData = clientData;
-    idlePtr->generation = idleGeneration;
+    idlePtr->generation = tsdPtr->idleGeneration;
     idlePtr->nextPtr = NULL;
-    if (lastIdlePtr == NULL) {
-	idleList = idlePtr;
+    if (tsdPtr->lastIdlePtr == NULL) {
+	tsdPtr->idleList = idlePtr;
     } else {
-	lastIdlePtr->nextPtr = idlePtr;
+	tsdPtr->lastIdlePtr->nextPtr = idlePtr;
     }
-    lastIdlePtr = idlePtr;
+    tsdPtr->lastIdlePtr = idlePtr;
 
     blockTime.sec = 0;
     blockTime.usec = 0;
     Tcl_SetMaxBlockTime(&blockTime);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -603,8 +634,9 @@ Tcl_CancelIdleCall(proc, clientData)
 {
     register IdleHandler *idlePtr, *prevPtr;
     IdleHandler *nextPtr;
+    ThreadSpecificData *tsdPtr = InitTimer();
 
-    for (prevPtr = NULL, idlePtr = idleList; idlePtr != NULL;
+    for (prevPtr = NULL, idlePtr = tsdPtr->idleList; idlePtr != NULL;
 	    prevPtr = idlePtr, idlePtr = idlePtr->nextPtr) {
 	while ((idlePtr->proc == proc)
 		&& (idlePtr->clientData == clientData)) {
@@ -612,18 +644,18 @@ Tcl_CancelIdleCall(proc, clientData)
 	    ckfree((char *) idlePtr);
 	    idlePtr = nextPtr;
 	    if (prevPtr == NULL) {
-		idleList = idlePtr;
+		tsdPtr->idleList = idlePtr;
 	    } else {
 		prevPtr->nextPtr = idlePtr;
 	    }
 	    if (idlePtr == NULL) {
-		lastIdlePtr = prevPtr;
+		tsdPtr->lastIdlePtr = prevPtr;
 		return;
 	    }
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -650,13 +682,14 @@ TclServiceIdle()
     IdleHandler *idlePtr;
     int oldGeneration;
     Tcl_Time blockTime;
+    ThreadSpecificData *tsdPtr = InitTimer();
 
-    if (idleList == NULL) {
+    if (tsdPtr->idleList == NULL) {
 	return 0;
     }
 
-    oldGeneration = idleGeneration;
-    idleGeneration++;
+    oldGeneration = tsdPtr->idleGeneration;
+    tsdPtr->idleGeneration++;
 
     /*
      * The code below is trickier than it may look, for the following
@@ -677,18 +710,18 @@ TclServiceIdle()
      *    change structure during the call.
      */
 
-    for (idlePtr = idleList;
+    for (idlePtr = tsdPtr->idleList;
 	    ((idlePtr != NULL)
 		    && ((oldGeneration - idlePtr->generation) >= 0));
-	    idlePtr = idleList) {
-	idleList = idlePtr->nextPtr;
-	if (idleList == NULL) {
-	    lastIdlePtr = NULL;
+	    idlePtr = tsdPtr->idleList) {
+	tsdPtr->idleList = idlePtr->nextPtr;
+	if (tsdPtr->idleList == NULL) {
+	    tsdPtr->lastIdlePtr = NULL;
 	}
 	(*idlePtr->proc)(idlePtr->clientData);
 	ckfree((char *) idlePtr);
     }
-    if (idleList) {
+    if (tsdPtr->idleList) {
 	blockTime.sec = 0;
 	blockTime.usec = 0;
 	Tcl_SetMaxBlockTime(&blockTime);
@@ -696,9 +729,8 @@ TclServiceIdle()
     return 1;
 }
 
-#else
-
-
+#endif
+#ifndef TCL_EVENT_IMPLEMENT
 /*
  *----------------------------------------------------------------------
  *
@@ -716,6 +748,12 @@ TclServiceIdle()
  *----------------------------------------------------------------------
  */
 
+#if ((defined(__STDC__) || defined(SABER)) && !defined(NO_PROTOTYPE)) || defined(__cplusplus) || defined(USE_PROTOTYPE)
+#   define ICONST const
+#else
+#   define ICONST
+#endif
+
 	/* ARGSUSED */
 int
 Tcl_AfterObjCmd(clientData, interp, objc, objv)
@@ -724,32 +762,24 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
 				 * hasn't been created yet.*/
     Tcl_Interp *interp;		/* Current interpreter. */
     int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument objects. */
+    Tcl_Obj *ICONST objv[];	/* Argument objects. */
 {
-    /*
-     * The variable below is used to generate unique identifiers for
-     * after commands.  This id can wrap around, which can potentially
-     * cause problems.  However, there are not likely to be problems
-     * in practice, because after commands can only be requested to
-     * about a month in the future, and wrap-around is unlikely to
-     * occur in less than about 1-10 years.  Thus it's unlikely that
-     * any old ids will still be around when wrap-around occurs.
-     */
-
-    static int nextId = 1;
     int ms;
     AfterInfo *afterPtr;
     AfterAssocData *assocPtr = (AfterAssocData *) clientData;
     Tcl_CmdInfo cmdInfo;
     int length;
-    char *arg;
-    int index, result;
-    static char *subCmds[] = {
-        "cancel", "idle", "info",
-        (char *) NULL};
+    char *argString;
+    int index;
+    char buf[16 + TCL_INTEGER_SPACE];
+    static CONST char *afterSubCmds[] = {
+	"cancel", "idle", "info", (char *) NULL
+    };
+    enum afterSubCmds {AFTER_CANCEL, AFTER_IDLE, AFTER_INFO};
+    ThreadSpecificData *tsdPtr = TkInitTimer();
 
     if (objc < 2) {
-	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
+	Tcl_WrongNumArgs(interp, 1, (Tcl_Obj **)objv, "option ?arg arg ...?");
 	return TCL_ERROR;
     }
 
@@ -768,7 +798,7 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
 		(ClientData) assocPtr);
 	cmdInfo.proc = NULL;
 	cmdInfo.clientData = (ClientData) NULL;
-	cmdInfo.objProc = Tcl_AfterObjCmd;
+	cmdInfo.objProc = (Tcl_ObjCmdProc *) Tcl_AfterObjCmd;
 	cmdInfo.objClientData = (ClientData) assocPtr;
 	cmdInfo.deleteProc = NULL;
 	cmdInfo.deleteData = (ClientData) assocPtr;
@@ -780,11 +810,16 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
      * First lets see if the command was passed a number as the first argument.
      */
 
-    arg = Tcl_GetStringFromObj(objv[1], &length);
-    if (isdigit(UCHAR(arg[0]))) {
+    if (TclObjGetType(objv[1]) == &tclIntType) {
+	ms = (int) TclObjInternal(objv[1])->longValue;
+	goto processInteger;
+    }
+    argString = Tcl_GetStringFromObj(objv[1], &length);
+    if (isdigit(UCHAR(argString[0]))) {	/* INTL: digit */
 	if (Tcl_GetIntFromObj(interp, objv[1], &ms) != TCL_OK) {
 	    return TCL_ERROR;
 	}
+processInteger:
 	if (ms < 0) {
 	    ms = 0;
 	}
@@ -795,128 +830,147 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
 	afterPtr = (AfterInfo *) ckalloc((unsigned) (sizeof(AfterInfo)));
 	afterPtr->assocPtr = assocPtr;
 	if (objc == 3) {
-	    afterPtr->command = LangMakeCallback(objv[2]);
+	    afterPtr->commandPtr = LangMakeCallback(objv[2]);
 	} else {
-	    Tcl_Obj *objPtr = Tcl_ConcatObj(objc-2, objv+2);
-	    afterPtr->command = LangMakeCallback(objPtr);
+	    Tcl_Obj *objPtr = Tcl_ConcatObj(objc-2, (Tcl_Obj **)objv+2);
+	    afterPtr->commandPtr = LangMakeCallback(objPtr);
 	    Tcl_DecrRefCount(objPtr);
 	}
-	afterPtr->id = nextId;
-	nextId += 1;
+	Tcl_IncrRefCount(afterPtr->commandPtr);
+	/*
+	 * The variable below is used to generate unique identifiers for
+	 * after commands.  This id can wrap around, which can potentially
+	 * cause problems.  However, there are not likely to be problems
+	 * in practice, because after commands can only be requested to
+	 * about a month in the future, and wrap-around is unlikely to
+	 * occur in less than about 1-10 years.  Thus it's unlikely that
+	 * any old ids will still be around when wrap-around occurs.
+	 */
+	afterPtr->id = tsdPtr->afterId;
+	tsdPtr->afterId += 1;
 	afterPtr->token = Tcl_CreateTimerHandler(ms, AfterProc,
 		(ClientData) afterPtr);
 	afterPtr->nextPtr = assocPtr->firstAfterPtr;
 	assocPtr->firstAfterPtr = afterPtr;
-	sprintf(interp->result, "after#%d", afterPtr->id);
+	sprintf(buf, "after#%d", afterPtr->id);
+	Tcl_AppendResult(interp, buf, (char *) NULL);
 	return TCL_OK;
     }
 
     /*
      * If it's not a number it must be a subcommand.
      */
-    result = Tcl_GetIndexFromObj(NULL, objv[1], subCmds, "option",
-            0, (int *) &index);
-    if (result != TCL_OK) {
-	Tcl_AppendResult(interp, "bad argument \"", arg,
+
+    if (Tcl_GetIndexFromObj(NULL, objv[1], afterSubCmds, "argument",
+            0, &index) != TCL_OK) {
+	Tcl_AppendResult(interp, "bad argument \"", argString,
 		"\": must be cancel, idle, info, or a number",
 		(char *) NULL);
 	return TCL_ERROR;
     }
+    switch ((enum afterSubCmds) index) {
+        case AFTER_CANCEL: {
+	    Tcl_Obj *commandPtr;
+	    char *command, *tempCommand;
+	    int tempLength;
 
-    switch (index) {
-        case 0:		/* cancel */
-	    {
-		char *arg;
-		Tcl_Obj *objPtr = NULL;
-
-		if (objc < 3) {
-		    Tcl_WrongNumArgs(interp, 2, objv, "id|command");
-		    return TCL_ERROR;
-		}
-		if (objc == 3) {
-		    objPtr = objv[2];
-		    Tcl_IncrRefCount(objPtr);
-		} else {
-		    objPtr = Tcl_ConcatObj(objc-2, objv+2);;
-		}
-		for (afterPtr = assocPtr->firstAfterPtr; afterPtr != NULL;
-		     afterPtr = afterPtr->nextPtr) {
-		    if (LangCmpCallback(afterPtr->command, objPtr)) {
-			break;
-		    }
-		}
-		if (afterPtr == NULL) {
-		    arg = Tcl_GetStringFromObj( objPtr, &length );
-		    afterPtr = GetAfterEvent(assocPtr, arg);
-		}
-		if (objPtr != NULL) {
-		    Tcl_DecrRefCount(objPtr);
-		}
-		if (afterPtr != NULL) {
-		    if (afterPtr->token != NULL) {
-			Tcl_DeleteTimerHandler(afterPtr->token);
-		    } else {
-			Tcl_CancelIdleCall(AfterProc, (ClientData) afterPtr);
-		    }
-		    FreeAfterPtr(afterPtr);
-		}
-		break;
-	    }
-	case 1:		/* idle */
 	    if (objc < 3) {
-		Tcl_WrongNumArgs(interp, 2, objv, "script script ...");
+		Tcl_WrongNumArgs(interp, 2, (Tcl_Obj **)objv, "id|command");
+		return TCL_ERROR;
+	    }
+	    if (objc == 3) {
+		commandPtr = objv[2];
+	    } else {
+		commandPtr = Tcl_ConcatObj(objc-2, (Tcl_Obj **)objv+2);;
+	    }
+	    command = Tcl_GetStringFromObj(commandPtr, &length);
+	    for (afterPtr = assocPtr->firstAfterPtr;  afterPtr != NULL;
+		    afterPtr = afterPtr->nextPtr) {
+		tempCommand = Tcl_GetStringFromObj(afterPtr->commandPtr,
+			&tempLength);
+		if ((length == tempLength)
+		        && (memcmp((void*) command, (void*) tempCommand,
+			        (unsigned) length) == 0)) {
+		    break;
+		}
+	    }
+	    if (afterPtr == NULL) {
+		afterPtr = GetAfterEvent(assocPtr, commandPtr);
+	    }
+	    if (objc != 3) {
+		Tcl_DecrRefCount(commandPtr);
+	    }
+	    if (afterPtr != NULL) {
+		if (afterPtr->token != NULL) {
+		    Tcl_DeleteTimerHandler(afterPtr->token);
+		} else {
+		    Tcl_CancelIdleCall(AfterProc, (ClientData) afterPtr);
+		}
+		FreeAfterPtr(afterPtr);
+	    }
+	    break;
+	}
+	case AFTER_IDLE:
+	    if (objc < 3) {
+		Tcl_WrongNumArgs(interp, 2, (Tcl_Obj **)objv, "script script ...");
 		return TCL_ERROR;
 	    }
 	    afterPtr = (AfterInfo *) ckalloc((unsigned) (sizeof(AfterInfo)));
 	    afterPtr->assocPtr = assocPtr;
 	    if (objc == 3) {
-		afterPtr->command = LangMakeCallback(objv[2]);
+		afterPtr->commandPtr = LangMakeCallback(objv[2]);
 	    } else {
-		Tcl_Obj *objPtr = Tcl_ConcatObj(objc-2, objv+2);
-		afterPtr->command = LangMakeCallback(objPtr);
+		Tcl_Obj *objPtr = Tcl_ConcatObj(objc-2, (Tcl_Obj **)objv+2);
+		afterPtr->commandPtr = LangMakeCallback(objPtr);
 		Tcl_DecrRefCount(objPtr);
 	    }
-	    afterPtr->id = nextId;
-	    nextId += 1;
+	    Tcl_IncrRefCount(afterPtr->commandPtr);
+	    afterPtr->id = tsdPtr->afterId;
+	    tsdPtr->afterId += 1;
 	    afterPtr->token = NULL;
 	    afterPtr->nextPtr = assocPtr->firstAfterPtr;
 	    assocPtr->firstAfterPtr = afterPtr;
 	    Tcl_DoWhenIdle(AfterProc, (ClientData) afterPtr);
-	    sprintf(interp->result, "after#%d", afterPtr->id);
+	    sprintf(buf, "after#%d", afterPtr->id);
+	    Tcl_AppendResult(interp, buf, (char *) NULL);
 	    break;
-	case 2:		/* info */
+	case AFTER_INFO: {
+	    Tcl_Obj *resultListPtr;
+
 	    if (objc == 2) {
-		char buffer[30];
-	
 		for (afterPtr = assocPtr->firstAfterPtr; afterPtr != NULL;
 		     afterPtr = afterPtr->nextPtr) {
 		    if (assocPtr->interp == interp) {
-			sprintf(buffer, "after#%d", afterPtr->id);
-			Tcl_AppendElement(interp, buffer);
+			sprintf(buf, "after#%d", afterPtr->id);
+			Tcl_AppendElement(interp, buf);
 		    }
 		}
 		return TCL_OK;
 	    }
 	    if (objc != 3) {
-		Tcl_WrongNumArgs(interp, 2, objv, "?id?");
+		Tcl_WrongNumArgs(interp, 2, (Tcl_Obj **)objv, "?id?");
 		return TCL_ERROR;
 	    }
-	    arg = Tcl_GetStringFromObj(objv[2], &length);
-	    afterPtr = GetAfterEvent(assocPtr, arg);
+	    afterPtr = GetAfterEvent(assocPtr, objv[2]);
 	    if (afterPtr == NULL) {
-		Tcl_AppendResult(interp, "event \"", arg,
+		Tcl_AppendResult(interp, "event \"", Tcl_GetString(objv[2]),
 			"\" doesn't exist", (char *) NULL);
 		return TCL_ERROR;
 	    }
-	    Tcl_ListObjAppendElement(interp, Tcl_GetObjResult(interp),
-               LangCallbackObj(afterPtr->command));
-	    Tcl_AppendElement(interp,
-		    (afterPtr->token == NULL) ? "idle" : "timer");
+	    resultListPtr = Tcl_GetObjResult(interp);
+ 	    Tcl_ListObjAppendElement(interp, resultListPtr, LangCallbackObj(afterPtr->commandPtr));
+ 	    Tcl_ListObjAppendElement(interp, resultListPtr, Tcl_NewStringObj(
+ 		(afterPtr->token == NULL) ? "idle" : "timer", -1));
+	    Tcl_SetObjResult(interp, resultListPtr);
 	    break;
+	}
+	default: {
+	    panic("Tcl_AfterObjCmd: bad subcommand index to afterSubCmds");
+	}
     }
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -927,7 +981,7 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
  *
  * Results:
  *	The return value is either a pointer to an AfterInfo structure,
- *	if one is found that corresponds to "string" and is for interp,
+ *	if one is found that corresponds to "cmdString" and is for interp,
  *	or NULL if no corresponding after event can be found.
  *
  * Side effects:
@@ -937,22 +991,24 @@ Tcl_AfterObjCmd(clientData, interp, objc, objv)
  */
 
 static AfterInfo *
-GetAfterEvent(assocPtr, string)
+GetAfterEvent(assocPtr, commandPtr)
     AfterAssocData *assocPtr;	/* Points to "after"-related information for
 				 * this interpreter. */
-    char *string;		/* Textual identifier for after event, such
-				 * as "after#6". */
+    Tcl_Obj *commandPtr;
 {
+    char *cmdString;		/* Textual identifier for after event, such
+				 * as "after#6". */
     AfterInfo *afterPtr;
     int id;
     char *end;
 
-    if (strncmp(string, "after#", 6) != 0) {
+    cmdString = Tcl_GetString(commandPtr);
+    if (strncmp(cmdString, "after#", 6) != 0) {
 	return NULL;
     }
-    string += 6;
-    id = strtoul(string, &end, 10);
-    if ((end == string) || (*end != 0)) {
+    cmdString += 6;
+    id = strtoul(cmdString, &end, 10);
+    if ((end == cmdString) || (*end != 0)) {
 	return NULL;
     }
     for (afterPtr = assocPtr->firstAfterPtr; afterPtr != NULL;
@@ -963,7 +1019,7 @@ GetAfterEvent(assocPtr, string)
     }
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -993,6 +1049,8 @@ AfterProc(clientData)
     AfterInfo *prevPtr;
     int result;
     Tcl_Interp *interp;
+    char *script;
+    int numBytes;
 
     /*
      * First remove the callback from our list of callbacks;  otherwise
@@ -1016,7 +1074,7 @@ AfterProc(clientData)
 
     interp = assocPtr->interp;
     Tcl_Preserve((ClientData) interp);
-    result = LangDoCallback(interp, afterPtr->command, 0, 0);
+    result = LangDoCallback(interp, afterPtr->commandPtr, 0, 0);
     if (result != TCL_OK) {
 	Tcl_AddErrorInfo(interp, "\n    (\"after\" script)");
 	Tcl_BackgroundError(interp);
@@ -1027,10 +1085,10 @@ AfterProc(clientData)
      * Free the memory for the callback.
      */
 
-    LangFreeCallback(afterPtr->command);
+    Tcl_DecrRefCount(afterPtr->commandPtr);
     ckfree((char *) afterPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1066,10 +1124,10 @@ FreeAfterPtr(afterPtr)
 	}
 	prevPtr->nextPtr = afterPtr->nextPtr;
     }
-    LangFreeCallback(afterPtr->command);
+    Tcl_DecrRefCount(afterPtr->commandPtr);
     ckfree((char *) afterPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1105,10 +1163,11 @@ AfterCleanupProc(clientData, interp)
 	} else {
 	    Tcl_CancelIdleCall(AfterProc, (ClientData) afterPtr);
 	}
-	LangFreeCallback(afterPtr->command);
+	Tcl_DecrRefCount(afterPtr->commandPtr);
 	ckfree((char *) afterPtr);
     }
     ckfree((char *) assocPtr);
 }
 
 #endif
+
