@@ -1,4 +1,7 @@
-/* 
+
+/*	$Id: tixTList.c,v 1.1.1.1 2000/05/17 11:08:42 idiscovery Exp $	*/
+
+/*
  * tixTList.c --
  *
  *	This module implements "TList" widgets.
@@ -137,7 +140,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_CALLBACK, "-sizecmd", "sizeCmd", "SizeCmd",
        DEF_TLIST_SIZE_COMMAND, Tk_Offset(WidgetRecord, sizeCmd),
        TK_CONFIG_NULL_OK},
- 
+
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_TLIST_TAKE_FOCUS, Tk_Offset(WidgetRecord, takeFocus),
 	TK_CONFIG_NULL_OK},
@@ -157,7 +160,7 @@ static Tk_ConfigSpec configSpecs[] = {
 
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
        (char *) NULL, 0, 0}
-};	
+};
 
 #define DEF_TLISTENTRY_STATE	 "normal"
 #define DEF_TLISTENTRY_DATA	 NULL
@@ -370,7 +373,7 @@ Tix_TListCmd(clientData, interp, argc, argv)
     interp->result = Tk_PathName(wPtr->dispData.tkwin);
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -533,7 +536,7 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -611,7 +614,7 @@ WidgetCommand(clientData, interp, argc, argv)
 
     return code;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -641,7 +644,7 @@ WidgetEventProc(clientData, eventPtr)
 	if (wPtr->dispData.tkwin != NULL) {
 	    wPtr->dispData.tkwin = NULL;
 	    wPtr->dispData.sizeChangedProc = NULL;
-	    Tcl_DeleteCommand(wPtr->dispData.interp, 
+	    Tcl_DeleteCommand(wPtr->dispData.interp,
 	        Tcl_GetCommandName(wPtr->dispData.interp, wPtr->widgetCmd));
 	}
 	CancelResizeWhenIdle(wPtr);
@@ -668,7 +671,7 @@ WidgetEventProc(clientData, eventPtr)
 	break;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -730,7 +733,7 @@ WidgetDestroy(clientData)
     Tk_FreeOptions(configSpecs, (char *) wPtr, wPtr->dispData.display, 0);
     ckfree((char *) wPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -762,12 +765,12 @@ WidgetCmdDeletedProc(clientData)
      */
     if (wPtr->dispData.tkwin != NULL) {
 	Tk_Window tkwin = wPtr->dispData.tkwin;
-	wPtr->dispData.tkwin = NULL;           
+	wPtr->dispData.tkwin = NULL;
 	wPtr->dispData.sizeChangedProc = NULL;
 	Tk_DestroyWindow(tkwin);
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -814,7 +817,7 @@ MakeGeomRequest(wPtr)
     WidgetPtr wPtr;
 {
     int reqW, reqH;
-    
+
     reqW = wPtr->width  * wPtr->scrollInfo[0].unit;
     reqH = wPtr->height * wPtr->scrollInfo[1].unit;
 
@@ -822,7 +825,7 @@ MakeGeomRequest(wPtr)
 }
 
 
-
+
 /*----------------------------------------------------------------------
  * DItemSizeChanged --
  *
@@ -841,7 +844,7 @@ Tix_TLDItemSizeChanged(iPtr)
 	ResizeWhenIdle(wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  * ResizeWhenIdle --
@@ -859,7 +862,7 @@ ResizeWhenIdle(wPtr)
 	Tcl_DoWhenIdle(WidgetComputeGeometry, (ClientData)wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  * ResizeWhenIdle --
@@ -876,7 +879,7 @@ ResizeNow(wPtr)
 	wPtr->resizing = 0;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  * CancelResizeWhenIdle --
@@ -891,7 +894,7 @@ CancelResizeWhenIdle(wPtr)
 	Tcl_CancelIdleCall(WidgetComputeGeometry, (ClientData)wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  * RedrawWhenIdle --
@@ -912,7 +915,7 @@ RedrawWhenIdle(wPtr)
 	Tcl_DoWhenIdle(WidgetDisplay, (ClientData)wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  * CancelRedrawWhenIdle --
@@ -927,7 +930,7 @@ CancelRedrawWhenIdle(wPtr)
 	Tcl_CancelIdleCall(WidgetDisplay, (ClientData)wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -971,10 +974,10 @@ WidgetDisplay(clientData)
     /* Draw the border */
     Tk_Draw3DRectangle(wPtr->dispData.tkwin, pixmap, wPtr->border,
 	wPtr->highlightWidth, wPtr->highlightWidth,
-	Tk_Width(tkwin)  - 2*wPtr->highlightWidth, 
+	Tk_Width(tkwin)  - 2*wPtr->highlightWidth,
 	Tk_Height(tkwin) - 2*wPtr->highlightWidth, wPtr->borderWidth,
 	wPtr->relief);
-  
+
     /* Draw the highlight */
     if (wPtr->highlightWidth > 0) {
 	GC gc;
@@ -982,7 +985,7 @@ WidgetDisplay(clientData)
 	if (wPtr->hasFocus) {
 	    gc = wPtr->highlightGC;
 	} else {
-	    gc = Tk_3DBorderGC(wPtr->dispData.tkwin, wPtr->border, 
+	    gc = Tk_3DBorderGC(wPtr->dispData.tkwin, wPtr->border,
 		TK_3D_FLAT_GC);
 	}
 	Tk_DrawFocusHighlight(tkwin, gc, wPtr->highlightWidth, pixmap);
@@ -997,7 +1000,7 @@ WidgetDisplay(clientData)
     Tk_FreePixmap(wPtr->dispData.display, pixmap);
 
 }
-
+
 /*----------------------------------------------------------------------
  *  AddElement  --
  *
@@ -1034,7 +1037,7 @@ AddElement(wPtr, chPtr, at)
 
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  *  AllocEntry  --
  *
@@ -1055,7 +1058,7 @@ AllocEntry(wPtr)
 
     return chPtr;
 }
-
+
 /*----------------------------------------------------------------------
  *  FreeEntry  --
  *
@@ -1116,9 +1119,9 @@ FreeEntry(wPtr, chPtr)
     Tk_FreeOptions(entryConfigSpecs, (char *)chPtr,wPtr->dispData.display, 0);
     ckfree((char*)chPtr);
 }
-
+
 /*----------------------------------------------------------------------
- * "insert" sub command -- 
+ * "insert" sub command --
  *
  *	Insert a new item into the list
  *----------------------------------------------------------------------
@@ -1145,7 +1148,7 @@ Tix_TLInsert(clientData, interp, argc, argv)
      */
 
     /* (1.1) Find out where */
-    if (Tix_TranslateIndex(wPtr, interp, args[0], &at, 1) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, objv[0], &at, 1) != TCL_OK) {
 	code = TCL_ERROR; goto done;
     }
 
@@ -1203,20 +1206,20 @@ Tix_TLInsert(clientData, interp, argc, argv)
     if (code == TCL_ERROR) {
 	if (chPtr != NULL) {
 	    if (added) {
-		Tix_LinkListFindAndDelete(&entListInfo, &wPtr->entList, 
+		Tix_LinkListFindAndDelete(&entListInfo, &wPtr->entList,
 		    (char*)chPtr, NULL);
 	    }
 	    FreeEntry(wPtr, chPtr);
 	}
     } else {
 	sprintf(buff, "%d", at);
-	Tcl_AppendResult(interp, buff, NULL);	
+	Tcl_AppendResult(interp, buff, NULL);
     }
 
     return code;
 }
 
-static int 
+static int
 Tix_TLSpecialEntryInfo(wPtr, interp, chPtr)
     WidgetPtr wPtr;
     Tcl_Interp *interp;
@@ -1248,7 +1251,7 @@ Tix_TLSpecialEntryInfo(wPtr, interp, chPtr)
     }
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  * "index" sub command
  *----------------------------------------------------------------------
@@ -1264,7 +1267,7 @@ Tix_TLIndex(clientData, interp, argc, argv)
     int index;
     char buff[100];
 
-    if (Tix_TranslateIndex(wPtr, interp, args[0], &index, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, objv[0], &index, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -1272,7 +1275,7 @@ Tix_TLIndex(clientData, interp, argc, argv)
     Tcl_AppendResult(interp, buff, NULL);
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  * "info" sub command
  *----------------------------------------------------------------------
@@ -1310,7 +1313,7 @@ Tix_TLInfo(clientData, interp, argc, argv)
 	for (chPtr=(ListEntry*)wPtr->entList.head, i=0;
 	     chPtr;
 	     chPtr=chPtr->next, i++) {
-	    
+
 	    if (chPtr->selected) {
 		Tcl_IntResults(interp,1,1,i);
 	    }
@@ -1329,7 +1332,7 @@ Tix_TLInfo(clientData, interp, argc, argv)
 	return Tix_TLGetNeighbor(wPtr, interp, TIX_UP,    argc-1, argv+1);
     }
     else {
-	Tcl_AppendResult(interp, "unknown option \"", argv[0], 
+	Tcl_AppendResult(interp, "unknown option \"", argv[0],
 	    "\": must be anchor or selection",
 	    NULL);
 	return TCL_ERROR;
@@ -1345,11 +1348,11 @@ Tix_TranslateIndex(wPtr, interp, string, index, isInsert)
     int isInsert;		/* Is this function called by an "insert"
 				 * operation? */
 {
-    if (strcmp(LangString(string), "end") == 0) {
+    if (strcmp(Tcl_GetString(string), "end") == 0) {
 	*index = wPtr->entList.numItems;
     }
-    else if (Tix_TLGetAt(wPtr, interp, LangString(string), index) != TCL_OK) {
-	if (Tcl_GetInt(interp, string, index) != TCL_OK) {
+    else if (Tix_TLGetAt(wPtr, interp, Tcl_GetString(string), index) != TCL_OK) {
+	if (Tcl_GetIntFromObj(interp, string, index) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 	else if (*index < 0) {
@@ -1408,7 +1411,7 @@ static int Tix_TLGetNeighbor(wPtr, interp, type, argc, argv)
 	Tix_ArgcError(interp, argc+3, argv-3, 3, "index");
     }
 
-    if (Tix_TranslateIndex(wPtr, interp, args[0], &index, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, objv[0], &index, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -1449,13 +1452,13 @@ static int Tix_TLGetNeighbor(wPtr, interp, type, argc, argv)
     }
 
     sprintf(buff, "%d", dst);
-    Tcl_AppendResult(interp, buff, NULL);	
+    Tcl_AppendResult(interp, buff, NULL);
 
     return TCL_OK;
 }
 
 
-
+
 /*----------------------------------------------------------------------
  * "cget" sub command --
  *----------------------------------------------------------------------
@@ -1472,7 +1475,7 @@ Tix_TLCGet(clientData, interp, argc, argv)
     return Tk_ConfigureValue(interp, wPtr->dispData.tkwin, configSpecs,
 	(char *)wPtr, argv[0], 0);
 }
-
+
 /*----------------------------------------------------------------------
  * "configure" sub command
  *----------------------------------------------------------------------
@@ -1497,7 +1500,7 @@ Tix_TLConfig(clientData, interp, argc, argv)
 	    TK_CONFIG_ARGV_ONLY);
     }
 }
-
+
 /*----------------------------------------------------------------------
  * "geometryinfo" sub command
  *----------------------------------------------------------------------
@@ -1516,10 +1519,10 @@ Tix_TLGeometryInfo(clientData, interp, argc, argv)
     int i;
 
     if (argc == 2) {
-	if (Tcl_GetInt(interp, argv[0], &qSize[0]) != TCL_OK) {
+	if (Tcl_GetIntFromObj(interp, argv[0], &qSize[0]) != TCL_OK) {
 	    return TCL_ERROR;
 	}
-	if (Tcl_GetInt(interp, argv[1], &qSize[1]) != TCL_OK) {
+	if (Tcl_GetIntFromObj(interp, argv[1], &qSize[1]) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     } else {
@@ -1535,12 +1538,17 @@ Tix_TLGeometryInfo(clientData, interp, argc, argv)
 	    &first[i], &last[i]);
     }
 
+#if 0
+    /* FIXME */
     sprintf(string, "{%f %f} {%f %f}", first[0], last[0], first[1], last[1]);
     Tcl_AppendResult(interp, string, NULL);
-
+#else
+    /* Not quite right - one list of four rather than two lists of two */
+    Tcl_DoubleResults(interp, 4, 1, first[0], last[0], first[1], last[1]);
+#endif
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  * "delete" sub command
  *----------------------------------------------------------------------
@@ -1609,7 +1617,7 @@ static int Tix_TLDeleteRange(wPtr, fromPtr, toPtr)
     return started;
 }
 
-
+
 /*----------------------------------------------------------------------
  * "entrycget" sub command
  *----------------------------------------------------------------------
@@ -1638,7 +1646,7 @@ Tix_TLEntryCget(clientData, interp, argc, argv)
     return Tix_ConfigureValue2(interp, wPtr->dispData.tkwin, (char *)chPtr,
 	entryConfigSpecs, chPtr->iPtr, argv[1], 0);
 }
-
+
 /*----------------------------------------------------------------------
  * "entryconfigure" sub command
  *----------------------------------------------------------------------
@@ -1675,7 +1683,7 @@ Tix_TLEntryConfig(clientData, interp, argc, argv)
 	    TK_CONFIG_ARGV_ONLY, 0);
     }
 }
-
+
 /*----------------------------------------------------------------------
  * "nearest" sub command
  *----------------------------------------------------------------------
@@ -1692,10 +1700,10 @@ Tix_TLNearest(clientData, interp, argc, argv)
     int index;
     char buff[100];
 
-    if (Tcl_GetInt(interp, argv[0], &posn[0]) != TCL_OK) {
+    if (Tcl_GetIntFromObj(interp, argv[0], &posn[0]) != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (Tcl_GetInt(interp, argv[1], &posn[1]) != TCL_OK) {
+    if (Tcl_GetIntFromObj(interp, argv[1], &posn[1]) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -1794,7 +1802,7 @@ static int Tix_TLGetNearest(wPtr, posn)
 
     return index;
 }
-
+
 /*----------------------------------------------------------------------
  * "selection" sub command
  * 	Modify the selection in this HList box
@@ -1817,11 +1825,11 @@ Tix_TLGetFromTo(interp, wPtr, argc, argv, fromPtr_ret, toPtr_ret)
     ListEntry * toPtr;
     int from, to, tmp;
 
-    if (Tix_TranslateIndex(wPtr, interp, args[0], &from, 0) != TCL_OK) {
+    if (Tix_TranslateIndex(wPtr, interp, objv[0], &from, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
     if (argc == 2) {
-	if (Tix_TranslateIndex(wPtr, interp, args[1], &to, 0) != TCL_OK) {
+	if (Tix_TranslateIndex(wPtr, interp, objv[1], &to, 0) != TCL_OK) {
 	    return TCL_ERROR;
 	}
     } else {
@@ -1961,7 +1969,7 @@ Tix_TLSelection(clientData, interp, argc, argv)
 	}
     }
     else {
-	Tcl_AppendResult(interp, "unknown option \"", argv[0], 
+	Tcl_AppendResult(interp, "unknown option \"", argv[0],
 	    "\": must be anchor, clear, includes or set", NULL);
 	code = TCL_ERROR;
     }
@@ -1973,7 +1981,7 @@ Tix_TLSelection(clientData, interp, argc, argv)
 
     return code;
 }
-
+
 /*----------------------------------------------------------------------
  * "see" command
  *----------------------------------------------------------------------
@@ -2004,7 +2012,7 @@ Tix_TLSee(clientData, interp, argc, argv)
 
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  * "anchor", "dragsite" and "dropsite" sub commands --
  *
@@ -2079,7 +2087,7 @@ Tix_TLSetSite(clientData, interp, argc, argv)
 
     return TCL_OK;
 }
-
+
 /*----------------------------------------------------------------------
  * "xview" and "yview" sub command
  *----------------------------------------------------------------------
@@ -2106,7 +2114,7 @@ Tix_TLView(clientData, interp, argc, argv)
 
 	Tix_GetScrollFractions((Tix_ScrollInfo*)&wPtr->scrollInfo[axis],
 	    &first, &last);
-                                  
+
 #if 0
 	sprintf(string, "{%f %f}", first, last);
 	Tcl_AppendResult(interp, string, NULL);
@@ -2115,7 +2123,7 @@ Tix_TLView(clientData, interp, argc, argv)
 #endif
 	return TCL_OK;
     }
-    else if (Tix_SetScrollBarView(interp, 
+    else if (Tix_SetScrollBarView(interp,
 	(Tix_ScrollInfo*)&wPtr->scrollInfo[axis], argc, argv, 0) != TCL_OK) {
 
 	return TCL_ERROR;
@@ -2159,7 +2167,7 @@ ConfigElement(wPtr, chPtr, argc, argv, flags, forced)
     }
     return TCL_OK;
 }
-
+
 static void
 Realloc(wPtr, new_size)
     WidgetPtr wPtr;
@@ -2175,7 +2183,7 @@ Realloc(wPtr, new_size)
                                       sizeof(ListRow)*new_size);
     wPtr->numRowAllocd = new_size;
 }
-
+
 static void ResizeRows(wPtr, winW, winH)
     WidgetPtr wPtr;
     int winW;			/* -1 == current width */
@@ -2208,7 +2216,7 @@ static void ResizeRows(wPtr, winW, winH)
 
     winSize[0] = winW;
     winSize[1] = winH;
-    
+
     if (wPtr->entList.numItems == 0) {
 	wPtr->rows[0].chPtr = NULL;
 	wPtr->rows[0].size[0] = 1;
@@ -2369,7 +2377,7 @@ RedrawRows(wPtr, pixmap)
 		size = wPtr->rows[r].size[i];
 		break;
 	    }
-	} 
+	}
 
 	if ((wPtr->scrollInfo[i].offset + windowSize) < (start + size)) {
 	    wPtr->scrollInfo[i].offset = start + size - windowSize;
@@ -2390,7 +2398,7 @@ RedrawRows(wPtr, pixmap)
 	    total += wPtr->rows[r].size[i];
 
 	    if (total > wPtr->scrollInfo[i].offset) {
-		p[i] -= wPtr->scrollInfo[i].offset - 
+		p[i] -= wPtr->scrollInfo[i].offset -
 		        (total - wPtr->rows[r].size[i]);
 		break;
 	    }
@@ -2416,7 +2424,7 @@ RedrawRows(wPtr, pixmap)
 		total += chPtr->iPtr->base.size[j];
 		if (total > wPtr->scrollInfo[j].offset) {
 		    /* Adjust for the shift due to partially visible elements*/
-		    p[j] -= wPtr->scrollInfo[j].offset - 
+		    p[j] -= wPtr->scrollInfo[j].offset -
 		      (total - chPtr->iPtr->base.size[j]);
 		    break;
 		}
@@ -2464,7 +2472,7 @@ RedrawRows(wPtr, pixmap)
 	p[i]+= wPtr->rows[r].size[i];
     }
 }
-
+
 /*----------------------------------------------------------------------
  *  UpdateScrollBars
  *----------------------------------------------------------------------
