@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWinCursor.c --
  *
  *	This file contains Win32 specific cursor related routines.
@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinCursor.c,v 1.2 1998/09/14 18:23:59 stanton Exp $
+ * RCS: @(#) $Id: tkWinCursor.c,v 1.10 2002/08/05 04:30:41 dgp Exp $
  */
 
 #include "tkWinInt.h"
@@ -61,16 +61,16 @@ static struct CursorName {
 
 #define TK_DEFAULT_CURSOR	IDC_ARROW
 
-
+
 /*
  *----------------------------------------------------------------------
  *
  * TkGetCursorByName --
  *
- *	Retrieve a system cursor by name.  
+ *	Retrieve a system cursor by name.
  *
  * Results:
- *	Returns a new cursor, or NULL on errors.  
+ *	Returns a new cursor, or NULL on errors.
  *
  * Side effects:
  *	Allocates a new cursor.
@@ -79,51 +79,94 @@ static struct CursorName {
  */
 
 TkCursor *
-TkGetCursorByName(interp, tkwin, arg)
+TkGetCursorByName(interp, tkwin, string)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    Arg arg;			/* Description of cursor.  See manual entry
+    Tk_Uid string;		/* Description of cursor.  See manual entry
 				 * for details on legal syntax. */
 {
     struct CursorName *namePtr;
     TkWinCursor *cursorPtr;
-    char *string = LangString(arg);
+    int argc;
+    Tcl_Obj **objv = NULL;
 
     /*
-     * Check for the cursor in the system cursor set.
+     * All cursor names are valid lists of one element (for
+     * Unix-compatability), even unadorned system cursor names.
      */
 
-    for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
-	if (strcmp(namePtr->name, string) == 0) {
-	    break;
-	}
+    Tcl_Obj *tmp = Tcl_NewStringObj(string,-1);
+    if (Tcl_ListObjGetElements(interp, tmp, &argc, &objv) != TCL_OK) {
+	Tcl_DecrRefCount(tmp);
+	return NULL;
     }
+    if (argc != 1) {
+	goto badCursorSpec;
+    }
+    string = (Tk_Uid) Tcl_GetString(objv[0]);
 
     cursorPtr = (TkWinCursor *) ckalloc(sizeof(TkWinCursor));
     cursorPtr->info.cursor = (Tk_Cursor) cursorPtr;
     cursorPtr->winCursor = NULL;
-    if (namePtr->name != NULL) {
-	cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
-	cursorPtr->system = 1;
-#ifdef __OPEN32__
+    cursorPtr->system = 0;
+
+    if (string[0] == '@') {
+	/*
+	 * Check for system cursor of type @<filename>, where only
+	 * the name is allowed.  This accepts any of:
+	 *	-cursor @/winnt/cursors/globe.ani
+	 *	-cursor @C:/Winnt/cursors/E_arrow.cur
+	 *	-cursor {@C:/Program\ Files/Cursors/bart.ani}
+	 *      -cursor {{@C:/Program Files/Cursors/bart.ani}}
+	 *	-cursor [list @[file join "C:/Program Files" Cursors bart.ani]]
+	 */
+
+	if (Tcl_IsSafe(interp)) {
+	    Tcl_AppendResult(interp, "can't get cursor from a file in",
+		    " a safe interpreter", (char *) NULL);
+	    ckfree((char *) cursorPtr);
+	    return NULL;
+	}
+	cursorPtr->winCursor = LoadCursorFromFile(&(string[1]));
     } else {
-	cursorPtr->winCursor = os2LoadCursor(TkWinGetTkModule(), string);
+	/*
+	 * Check for the cursor in the system cursor set.
+	 */
+	for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
+	    if (strcmp(namePtr->name, string) == 0) {
+		cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
+		break;
+	    }
+	}
+
+	if (cursorPtr->winCursor == NULL) {
+	    /*
+	     * Hmm, it is not in the system cursor set.  Check to see
+	     * if it is one of our application resources.
+	     */
+#ifdef __OPEN32__
+	    cursorPtr->winCursor = os2LoadCursor(TkWinGetTkModule(), string);
+#else
+	    cursorPtr->winCursor = LoadCursor(Tk_GetHINSTANCE(), string);
 #endif
+	} else {
+	    cursorPtr->system = 1;
+	}
     }
+
     if (cursorPtr->winCursor == NULL) {
-	cursorPtr->winCursor = LoadCursor(Tk_GetHINSTANCE(), string);
-	cursorPtr->system = 0;
-    }
-    if (cursorPtr->winCursor == NULL) {
-	ckfree((char *)cursorPtr);
+	ckfree((char *) cursorPtr);
+    badCursorSpec:
 	Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"",
 		(char *) NULL);
+	Tcl_DecrRefCount(tmp);
 	return NULL;
     } else {
+	Tcl_DecrRefCount(tmp);
 	return (TkCursor *) cursorPtr;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -144,8 +187,8 @@ TkCursor *
 TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 	fgColor, bgColor)
     Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *source;		/* Bitmap data for cursor shape. */
-    char *mask;			/* Bitmap data for cursor mask. */
+    CONST char *source;		/* Bitmap data for cursor shape. */
+    CONST char *mask;		/* Bitmap data for cursor mask. */
     int width, height;		/* Dimensions of cursor. */
     int xHot, yHot;		/* Location of hot-spot in cursor. */
     XColor fgColor;		/* Foreground color for cursor. */
@@ -153,11 +196,11 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
 {
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
- * TkFreeCursor --
+ * TkpFreeCursor --
  *
  *	This procedure is called to release a cursor allocated by
  *	TkGetCursorByName.
@@ -172,13 +215,12 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
  */
 
 void
-TkFreeCursor(cursorPtr)
+TkpFreeCursor(cursorPtr)
     TkCursor *cursorPtr;
 {
     TkWinCursor *winCursorPtr = (TkWinCursor *) cursorPtr;
-    ckfree((char *) winCursorPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -213,3 +255,4 @@ TkpSetCursor(cursor)
 	SetCursor(hcursor);
     }
 }
+
