@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWinDraw.c --
  *
  *	This file contains the Xlib emulation functions pertaining to
@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinDraw.c,v 1.2 1998/09/14 18:24:00 stanton Exp $
+ * RCS: @(#) $Id: tkWinDraw.c,v 1.11 2001/12/07 05:20:52 chengyemao Exp $
  */
 
 #include "tkWinInt.h"
@@ -106,11 +106,16 @@ static int bltModes[] = {
 typedef BOOL (CALLBACK *WinDrawFunc) _ANSI_ARGS_((HDC dc,
 			    CONST POINT* points, int npoints));
 
+typedef struct ThreadSpecificData {
+    POINT *winPoints;    /* Array of points that is reused. */
+    int nWinPoints;	/* Current size of point array. */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
+
 /*
  * Forward declarations for procedures defined in this file:
  */
 
-static HPEN		SetUpGraphicsPort _ANSI_ARGS_((GC gc));
 static POINT *		ConvertPoints _ANSI_ARGS_((XPoint *points, int npoints,
 			    int mode, RECT *bbox));
 static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
@@ -120,7 +125,8 @@ static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
 static void		RenderObject _ANSI_ARGS_((HDC dc, GC gc,
 			    XPoint* points, int npoints, int mode, HPEN pen,
 			    WinDrawFunc func));
-
+static HPEN		SetUpGraphicsPort _ANSI_ARGS_((GC gc));
+
 /*
  *----------------------------------------------------------------------
  *
@@ -151,7 +157,7 @@ TkWinGetDrawableDC(display, d, state)
 
     if (twdPtr->type == TWD_WINDOW) {
 	TkWindow *winPtr = twdPtr->window.winPtr;
-    
+
  	dc = GetDC(twdPtr->window.handle);
 	if (winPtr == NULL) {
 	    cmap = DefaultColormap(display, DefaultScreen(display));
@@ -167,9 +173,10 @@ TkWinGetDrawableDC(display, d, state)
 	cmap = twdPtr->bitmap.colormap;
     }
     state->palette = TkWinSelectPalette(dc, cmap);
+    state->bkmode  = GetBkMode(dc);
     return dc;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -193,6 +200,7 @@ TkWinReleaseDrawableDC(d, dc, state)
     TkWinDCState *state;
 {
     TkWinDrawable *twdPtr = (TkWinDrawable *)d;
+    SetBkMode(dc, state->bkmode);
     SelectPalette(dc, state->palette, TRUE);
     RealizePalette(dc);
     if (twdPtr->type == TWD_WINDOW) {
@@ -201,7 +209,7 @@ TkWinReleaseDrawableDC(d, dc, state)
 	DeleteDC(dc);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -213,7 +221,8 @@ TkWinReleaseDrawableDC(d, dc, state)
  *	Returns the converted array of POINTs.
  *
  * Side effects:
- *	Allocates a block of memory that should not be freed.
+ *	Allocates a block of memory in thread local storage that
+ *      should not be freed.
  *
  *----------------------------------------------------------------------
  */
@@ -225,8 +234,8 @@ ConvertPoints(points, npoints, mode, bbox)
     int mode;			/* CoordModeOrigin or CoordModePrevious. */
     RECT *bbox;			/* Bounding box of points. */
 {
-    static POINT *winPoints = NULL; /* Array of points that is reused. */
-    static int nWinPoints = -1;	    /* Current size of point array. */
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     int i;
 
     /*
@@ -234,45 +243,45 @@ ConvertPoints(points, npoints, mode, bbox)
      * we reuse the last array if it is large enough.
      */
 
-    if (npoints > nWinPoints) {
-	if (winPoints != NULL) {
-	    ckfree((char *) winPoints);
+    if (npoints > tsdPtr->nWinPoints) {
+	if (tsdPtr->winPoints != NULL) {
+	    ckfree((char *) tsdPtr->winPoints);
 	}
-	winPoints = (POINT *) ckalloc(sizeof(POINT) * npoints);
-	if (winPoints == NULL) {
-	    nWinPoints = -1;
+	tsdPtr->winPoints = (POINT *) ckalloc(sizeof(POINT) * npoints);
+	if (tsdPtr->winPoints == NULL) {
+	    tsdPtr->nWinPoints = -1;
 	    return NULL;
 	}
-	nWinPoints = npoints;
+	tsdPtr->nWinPoints = npoints;
     }
 
     bbox->left = bbox->right = points[0].x;
     bbox->top = bbox->bottom = points[0].y;
-    
+
     if (mode == CoordModeOrigin) {
 	for (i = 0; i < npoints; i++) {
-	    winPoints[i].x = points[i].x;
-	    winPoints[i].y = points[i].y;
-	    bbox->left = MIN(bbox->left, winPoints[i].x);
-	    bbox->right = MAX(bbox->right, winPoints[i].x);
-	    bbox->top = MIN(bbox->top, winPoints[i].y);
-	    bbox->bottom = MAX(bbox->bottom, winPoints[i].y);
+	    tsdPtr->winPoints[i].x = points[i].x;
+	    tsdPtr->winPoints[i].y = points[i].y;
+	    bbox->left = MIN(bbox->left, tsdPtr->winPoints[i].x);
+	    bbox->right = MAX(bbox->right, tsdPtr->winPoints[i].x);
+	    bbox->top = MIN(bbox->top, tsdPtr->winPoints[i].y);
+	    bbox->bottom = MAX(bbox->bottom, tsdPtr->winPoints[i].y);
 	}
     } else {
-	winPoints[0].x = points[0].x;
-	winPoints[0].y = points[0].y;
+	tsdPtr->winPoints[0].x = points[0].x;
+	tsdPtr->winPoints[0].y = points[0].y;
 	for (i = 1; i < npoints; i++) {
-	    winPoints[i].x = winPoints[i-1].x + points[i].x;
-	    winPoints[i].y = winPoints[i-1].y + points[i].y;
-	    bbox->left = MIN(bbox->left, winPoints[i].x);
-	    bbox->right = MAX(bbox->right, winPoints[i].x);
-	    bbox->top = MIN(bbox->top, winPoints[i].y);
-	    bbox->bottom = MAX(bbox->bottom, winPoints[i].y);
+	    tsdPtr->winPoints[i].x = tsdPtr->winPoints[i-1].x + points[i].x;
+	    tsdPtr->winPoints[i].y = tsdPtr->winPoints[i-1].y + points[i].y;
+	    bbox->left = MIN(bbox->left, tsdPtr->winPoints[i].x);
+	    bbox->right = MAX(bbox->right, tsdPtr->winPoints[i].x);
+	    bbox->top = MIN(bbox->top, tsdPtr->winPoints[i].y);
+	    bbox->bottom = MAX(bbox->bottom, tsdPtr->winPoints[i].y);
 	}
     }
-    return winPoints;
+    return tsdPtr->winPoints;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -328,7 +337,7 @@ XCopyArea(display, src, dest, gc, src_x, src_y, width, height, dest_x, dest_y)
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -420,7 +429,7 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
 	    /*
 	     * Case 3: two arbitrary bitmaps.  Copy the source rectangle
 	     * into a color pixmap.  Use the result as a brush when
-	     * copying the clip mask into the destination.	 
+	     * copying the clip mask into the destination.
 	     */
 
 	    HDC memDC, maskDC;
@@ -476,7 +485,7 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -504,7 +513,7 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
     Drawable d;				/* Destination drawable. */
     GC gc;
     XImage* image;			/* Source image. */
-    int src_x, src_y;			/* Offset of subimage. */      
+    int src_x, src_y;			/* Offset of subimage. */
     int dest_x, dest_y;			/* Position of subimage origin in
 					 * drawable.  */
     unsigned int width, height;		/* Dimensions of subimage. */
@@ -538,55 +547,25 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 	}
 	SetTextColor(dc, gc->foreground);
 	SetBkColor(dc, gc->background);
-    } else {    
+    } else {
 	int i, usePalette;
 
 	/*
 	 * Do not use a palette for TrueColor images.
 	 */
-	
+
 	usePalette = (image->bits_per_pixel < 16);
-	
+
 	if (usePalette) {
 	    infoPtr = (BITMAPINFO*) ckalloc(sizeof(BITMAPINFOHEADER)
 		    + sizeof(RGBQUAD)*ncolors);
 	} else {
 	    infoPtr = (BITMAPINFO*) ckalloc(sizeof(BITMAPINFOHEADER));
 	}
-	
+
 	infoPtr->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
 	infoPtr->bmiHeader.biWidth = image->width;
-
-	/*
-	 * The following code works around a bug in Win32s.  CreateDIBitmap
-	 * fails under Win32s for top-down images.  So we have to reverse the
-	 * order of the scanlines.  If we are not running under Win32s, we can
-	 * just declare the image to be top-down.
-	 */
-
-#ifdef __OPEN32__
-	if (1) {			/* Bug in Open32. */
-#else
-	if (tkpIsWin32s) {
-#endif
-	    int y;
-	    char *srcPtr, *dstPtr, *temp;
-
-	    temp = ckalloc((unsigned) image->bytes_per_line);
-	    srcPtr = image->data;
-	    dstPtr = image->data+(image->bytes_per_line * (image->height - 1));
-	    for (y = 0; y < (image->height/2); y++) {
-		memcpy(temp, srcPtr, image->bytes_per_line);
-		memcpy(srcPtr, dstPtr, image->bytes_per_line);
-		memcpy(dstPtr, temp, image->bytes_per_line);
-		srcPtr += image->bytes_per_line;
-		dstPtr -= image->bytes_per_line;
-	    }
-	    ckfree(temp);
-	    infoPtr->bmiHeader.biHeight = image->height; /* Bottom-up order */
-	} else {
-	    infoPtr->bmiHeader.biHeight = -image->height; /* Top-down order */
-	}
+	infoPtr->bmiHeader.biHeight = -image->height; /* Top-down order */
 	infoPtr->bmiHeader.biPlanes = 1;
 	infoPtr->bmiHeader.biBitCount = image->bits_per_pixel;
 	infoPtr->bmiHeader.biCompression = BI_RGB;
@@ -610,13 +589,19 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 		image->data, infoPtr, DIB_RGB_COLORS);
 	ckfree((char *) infoPtr);
     }
+    if(!bitmap) {
+	panic("Fail to allocate bitmap\n");
+	DeleteDC(dcMem);
+    	TkWinReleaseDrawableDC(d, dc, &state);
+	return;
+    }
     bitmap = SelectObject(dcMem, bitmap);
     BitBlt(dc, dest_x, dest_y, width, height, dcMem, src_x, src_y, SRCCOPY);
     DeleteObject(SelectObject(dcMem, bitmap));
     DeleteDC(dcMem);
     TkWinReleaseDrawableDC(d, dc, &state);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -671,7 +656,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	/*
 	 * Select stipple pattern into destination dc.
 	 */
-	
+
 	stipple = CreatePatternBrush(twdPtr->bitmap.handle);
 	SetBrushOrgEx(dc, gc->ts_x_origin, gc->ts_y_origin, NULL);
 	oldBrush = SelectObject(dc, stipple);
@@ -703,7 +688,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	    SelectObject(dcMem, oldBitmap);
 	    DeleteObject(bitmap);
 	}
-	
+
 	DeleteDC(dcMem);
 	SelectObject(dc, oldBrush);
 	DeleteObject(stipple);
@@ -717,7 +702,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
     DeleteObject(brush);
     TkWinReleaseDrawableDC(d, dc, &state);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -749,7 +734,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
     HPEN oldPen;
     HBRUSH oldBrush;
     POINT *winPoints = ConvertPoints(points, npoints, mode, &rect);
-    
+
     if ((gc->fill_style == FillStippled
 	    || gc->fill_style == FillOpaqueStippled)
 	    && gc->stipple != None) {
@@ -760,21 +745,19 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	HBITMAP oldBitmap;
 	int i;
 	HBRUSH oldMemBrush;
-	
+
 	if (twdPtr->type != TWD_BITMAP) {
 	    panic("unexpected drawable type in stipple");
 	}
 
 	/*
-	 * Grow the bounding box enough to account for wide lines.
+	 * Grow the bounding box enough to account for line width.
 	 */
 
-	if (gc->line_width > 1) {
-	    rect.left -= gc->line_width;
-	    rect.top -= gc->line_width;
-	    rect.right += gc->line_width;
-	    rect.bottom += gc->line_width;
-	}
+	rect.left -= gc->line_width;
+	rect.top -= gc->line_width;
+	rect.right += gc->line_width;
+	rect.bottom += gc->line_width;
 
 	width = rect.right - rect.left;
 	height = rect.bottom - rect.top;
@@ -782,7 +765,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	/*
 	 * Select stipple pattern into destination dc.
 	 */
-	
+
 	SetBrushOrgEx(dc, gc->ts_x_origin, gc->ts_y_origin, NULL);
 	oldBrush = SelectObject(dc, CreatePatternBrush(twdPtr->bitmap.handle));
 
@@ -790,7 +773,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	 * Create temporary drawing surface containing a copy of the
 	 * destination equal in size to the bounding box of the object.
 	 */
-	
+
 	dcMem = CreateCompatibleDC(dc);
 	oldBitmap = SelectObject(dcMem, CreateCompatibleBitmap(dc, width,
 		height));
@@ -799,7 +782,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 
 	/*
 	 * Translate the object for rendering in the temporary drawing
-	 * surface. 
+	 * surface.
 	 */
 
 	for (i = 0; i < npoints; i++) {
@@ -850,7 +833,7 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
     }
     DeleteObject(SelectObject(dc, oldBrush));
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -879,7 +862,7 @@ XDrawLines(display, d, gc, points, npoints, mode)
     HPEN pen;
     TkWinDCState state;
     HDC dc;
-    
+
     if (d == None) {
 	return;
     }
@@ -887,15 +870,14 @@ XDrawLines(display, d, gc, points, npoints, mode)
     dc = TkWinGetDrawableDC(display, d, &state);
 
     pen = SetUpGraphicsPort(gc);
+    SetBkMode(dc, TRANSPARENT);
     RenderObject(dc, gc, points, npoints, mode, pen, Polyline);
     DeleteObject(pen);
-    
+
     TkWinReleaseDrawableDC(d, dc, &state);
 }
 
 
-
-#if 1
 void XDrawPoints(display, d, gc, points, npoints, mode)
     Display*		display;
     Drawable		d;
@@ -912,11 +894,6 @@ void XDrawPoints(display, d, gc, points, npoints, mode)
     }
 }
 
-#endif
-
-
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -958,7 +935,7 @@ XFillPolygon(display, d, gc, points, npoints, shape, mode)
 
     TkWinReleaseDrawableDC(d, dc, &state);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -997,6 +974,7 @@ XDrawRectangle(display, d, gc, x, y, width, height)
     dc = TkWinGetDrawableDC(display, d, &state);
 
     pen = SetUpGraphicsPort(gc);
+    SetBkMode(dc, TRANSPARENT);
     oldPen = SelectObject(dc, pen);
     oldBrush = SelectObject(dc, GetStockObject(NULL_BRUSH));
     SetROP2(dc, tkpWinRopModes[gc->function]);
@@ -1007,7 +985,7 @@ XDrawRectangle(display, d, gc, x, y, width, height)
     SelectObject(dc, oldBrush);
     TkWinReleaseDrawableDC(d, dc, &state);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1040,7 +1018,7 @@ XDrawArc(display, d, gc, x, y, width, height, start, extent)
 
     DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1073,7 +1051,7 @@ XFillArc(display, d, gc, x, y, width, height, start, extent)
 
     DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1167,6 +1145,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
 	 * it's only supported under Windows NT.
 	 */
 
+	SetBkMode(dc, TRANSPARENT);
 	Arc(dc, x, y, x+width+1, y+height+1, xstart, ystart, xend, yend);
     } else {
 	brush = CreateSolidBrush(gc->foreground);
@@ -1181,7 +1160,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
     DeleteObject(SelectObject(dc, oldPen));
     TkWinReleaseDrawableDC(d, dc, &state);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1198,16 +1177,6 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
  *----------------------------------------------------------------------
  */
 
-#ifndef TkWinCreatePen
-#define TkWinCreatePen(gc, style, width, foreground) \
-	CreatePen(style, width, foreground)
-#endif
-
-#ifndef TkWinExtCreatePen
-#define TkWinExtCreatePen(gc, style, width, brush, count, lp) \
-	ExtCreatePen(style, width, brush, count, lp)
-#endif
-
 static HPEN
 SetUpGraphicsPort(gc)
     GC gc;
@@ -1222,7 +1191,7 @@ SetUpGraphicsPort(gc)
 	 * Below is a simple translation of serveral dash patterns
 	 * to valid windows pen types. Far from complete,
 	 * but I don't know how to do it better.
-	 * Any ideas: <mailto:Jan.Nijtmans@wxs.nl>
+	 * Any ideas: <mailto:j.nijtmans@chello.nl>
 	 */
 
 	if (p[1] && p[2]) {
@@ -1241,11 +1210,8 @@ SetUpGraphicsPort(gc)
     } else {
 	style = PS_SOLID;
     }
-    if (tkpIsWin32s || (gc->line_width < 2)) {
-	if (gc->line_width > 1) {
-	    style = PS_SOLID;
-	}
-	return TkWinCreatePen(gc, style, gc->line_width, gc->foreground);
+    if (gc->line_width < 2) {
+	return CreatePen(style, gc->line_width, gc->foreground);
     } else {
 	LOGBRUSH lb;
 
@@ -1257,30 +1223,30 @@ SetUpGraphicsPort(gc)
 	switch (gc->cap_style) {
 	    case CapNotLast:
 	    case CapButt:
-		style |= PS_ENDCAP_FLAT; 
+		style |= PS_ENDCAP_FLAT;
 		break;
 	    case CapRound:
-		style |= PS_ENDCAP_ROUND; 
+		style |= PS_ENDCAP_ROUND;
 		break;
 	    default:
-		style |= PS_ENDCAP_SQUARE; 
+		style |= PS_ENDCAP_SQUARE;
 		break;
 	}
 	switch (gc->join_style) {
-	    case JoinMiter: 
-		style |= PS_JOIN_MITER; 
+	    case JoinMiter:
+		style |= PS_JOIN_MITER;
 		break;
 	    case JoinRound:
-		style |= PS_JOIN_ROUND; 
+		style |= PS_JOIN_ROUND;
 		break;
 	    default:
-		style |= PS_JOIN_BEVEL; 
+		style |= PS_JOIN_BEVEL;
 		break;
 	}
-	return TkWinExtCreatePen(gc, style, gc->line_width, &lb, 0, NULL);
+	return ExtCreatePen(style, gc->line_width, &lb, 0, NULL);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1318,7 +1284,7 @@ TkScrollWindow(tkwin, gc, x, y, width, height, dx, dy, damageRgn)
     return (ScrollWindowEx(hwnd, dx, dy, &scrollRect, NULL, (HRGN) damageRgn,
 	    NULL, 0) == NULLREGION) ? 0 : 1;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1355,3 +1321,37 @@ TkWinFillRect(dc, x, y, width, height, pixel)
     ExtTextOut(dc, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
     SetBkColor(dc, oldColor);
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpDrawHighlightBorder --
+ *
+ *	This procedure draws a rectangular ring around the outside of
+ *	a widget to indicate that it has received the input focus.
+ *
+ *      On Windows, we just draw the simple inset ring.  On other sytems,
+ *      e.g. the Mac, the focus ring is a little more complicated, so we
+ *      need this abstraction.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	A rectangle "width" pixels wide is drawn in "drawable",
+ *	corresponding to the outer area of "tkwin".
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpDrawHighlightBorder(tkwin, fgGC, bgGC, highlightWidth, drawable)
+    Tk_Window tkwin;
+    GC fgGC;
+    GC bgGC;
+    int highlightWidth;
+    Drawable drawable;
+{
+    TkDrawInsetFocusHighlight(tkwin, fgGC, highlightWidth, drawable, 0);
+}
+
