@@ -1,4 +1,4 @@
-/* 
+/*
  * tkWinEmbed.c --
  *
  *	This file contains platform specific procedures for Windows platforms
@@ -6,12 +6,12 @@
  *	one application can use as its main window an internal window from
  *	another application).
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1996-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinEmbed.c,v 1.2 1998/09/14 18:24:00 stanton Exp $
+ * RCS: @(#) $Id: tkWinEmbed.c,v 1.7 2002/08/05 04:30:41 dgp Exp $
  */
 
 #include "tkWinInt.h"
@@ -38,9 +38,11 @@ typedef struct Container {
 					 * this process. */
 } Container;
 
-static Container *firstContainerPtr = NULL;
-					/* First in list of all containers
+typedef struct ThreadSpecificData {
+    Container *firstContainerPtr;       /* First in list of all containers
 					 * managed by this process.  */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 static void		CleanupContainerList _ANSI_ARGS_((
     			    ClientData clientData));
@@ -51,7 +53,7 @@ static void		EmbeddedEventProc _ANSI_ARGS_((
 static void		EmbedGeometryRequest _ANSI_ARGS_((
     			    Container*containerPtr, int width, int height));
 static void		EmbedWindowDeleted _ANSI_ARGS_((TkWindow *winPtr));
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -74,16 +76,18 @@ CleanupContainerList(clientData)
     ClientData clientData;
 {
     Container *nextPtr;
-    
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
     for (;
-        firstContainerPtr != (Container *) NULL;
-        firstContainerPtr = nextPtr) {
-        nextPtr = firstContainerPtr->nextPtr;
-        ckfree((char *) firstContainerPtr);
+        tsdPtr->firstContainerPtr != (Container *) NULL;
+        tsdPtr->firstContainerPtr = nextPtr) {
+        nextPtr = tsdPtr->firstContainerPtr->nextPtr;
+        ckfree((char *) tsdPtr->firstContainerPtr);
     }
-    firstContainerPtr = (Container *) NULL;
+    tsdPtr->firstContainerPtr = (Container *) NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -106,11 +110,11 @@ TkpTestembedCmd(clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp *interp;
     int argc;
-    char **argv;
+    CONST char **argv;
 {
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -126,7 +130,7 @@ TkpTestembedCmd(clientData, interp, argc, argv)
  *	The return value is normally TCL_OK. If an error occurred (such as
  *	if the argument does not identify a legal Windows window handle),
  *	the return value is TCL_ERROR and an error message is left in the
- *	interp->result if interp is not NULL.
+ *	the interp's result if interp is not NULL.
  *
  * Side effects:
  *	None.
@@ -134,19 +138,22 @@ TkpTestembedCmd(clientData, interp, argc, argv)
  *----------------------------------------------------------------------
  */
 
-int 
+int
 TkpUseWindow(interp, tkwin, string)
     Tcl_Interp *interp;		/* If not NULL, used for error reporting
 				 * if string is bogus. */
     Tk_Window tkwin;		/* Tk window that does not yet have an
 				 * associated X window. */
-    Arg string;			/* String identifying an X window to use
+    Tcl_Obj *string;		/* String identifying an X window to use
 				 * for tkwin;  must be an integer value. */
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
+    TkWindow *usePtr;
     int id;
     HWND hwnd;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (winPtr->window != None) {
         panic("TkpUseWindow: Already assigned a window");
@@ -159,7 +166,8 @@ TkpUseWindow(interp, tkwin, string)
 
     /*
      * Check if the window is a valid handle. If it is invalid, return
-     * TCL_ERROR and potentially leave an error message in interp->result.
+     * TCL_ERROR and potentially leave an error message in the interp's
+     * result.
      */
 
     if (!IsWindow(hwnd)) {
@@ -170,6 +178,15 @@ TkpUseWindow(interp, tkwin, string)
         return TCL_ERROR;
     }
 
+    usePtr = (TkWindow *) Tk_HWNDToWindow(hwnd);
+    if (usePtr != NULL) {
+        if (!(usePtr->flags & TK_CONTAINER)) {
+	    Tcl_AppendResult(interp, "window \"", usePtr->pathName,
+                    "\" doesn't have -container option set", (char *) NULL);
+	    return TCL_ERROR;
+	}
+    }
+
     /*
      * Store the parent window in the platform private data slot so
      * TkWmMapWindow can use it when creating the wrapper window.
@@ -190,10 +207,10 @@ TkpUseWindow(interp, tkwin, string)
      * things will get cleaned up at finalization.
      */
 
-    if (firstContainerPtr == (Container *) NULL) {
+    if (tsdPtr->firstContainerPtr == (Container *) NULL) {
         Tcl_CreateExitHandler(CleanupContainerList, (ClientData) NULL);
     }
-    
+
     /*
      * Save information about the container and the embedded window
      * in a Container structure.  If there is already an existing
@@ -201,8 +218,8 @@ TkpUseWindow(interp, tkwin, string)
      * app. are in the same process.
      */
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
-	    containerPtr = containerPtr->nextPtr) {
+    for (containerPtr = tsdPtr->firstContainerPtr;
+            containerPtr != NULL; containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->parentHWnd == hwnd) {
 	    winPtr->flags |= TK_BOTH_HALVES;
 	    containerPtr->parentPtr->flags |= TK_BOTH_HALVES;
@@ -213,8 +230,8 @@ TkpUseWindow(interp, tkwin, string)
 	containerPtr = (Container *) ckalloc(sizeof(Container));
 	containerPtr->parentPtr = NULL;
 	containerPtr->parentHWnd = hwnd;
-	containerPtr->nextPtr = firstContainerPtr;
-	firstContainerPtr = containerPtr;
+	containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+	tsdPtr->firstContainerPtr = containerPtr;
     }
 
     /*
@@ -232,7 +249,7 @@ TkpUseWindow(interp, tkwin, string)
 
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -258,16 +275,18 @@ TkpMakeContainer(tkwin)
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * If this is the first container, register an exit handler so that
      * things will get cleaned up at finalization.
      */
 
-    if (firstContainerPtr == (Container *) NULL) {
+    if (tsdPtr->firstContainerPtr == (Container *) NULL) {
         Tcl_CreateExitHandler(CleanupContainerList, (ClientData) NULL);
     }
-    
+
     /*
      * Register the window as a container so that, for example, we can
      * find out later if the embedded app. is in the same process.
@@ -279,8 +298,8 @@ TkpMakeContainer(tkwin)
     containerPtr->parentHWnd = Tk_GetHWND(Tk_WindowId(tkwin));
     containerPtr->embeddedHWnd = NULL;
     containerPtr->embeddedPtr = NULL;
-    containerPtr->nextPtr = firstContainerPtr;
-    firstContainerPtr = containerPtr;
+    containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+    tsdPtr->firstContainerPtr = containerPtr;
     winPtr->flags |= TK_CONTAINER;
 
     /*
@@ -297,7 +316,7 @@ TkpMakeContainer(tkwin)
     Tk_CreateEventHandler(tkwin, StructureNotifyMask,
 	ContainerEventProc, (ClientData) containerPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -328,7 +347,7 @@ EmbeddedEventProc(clientData, eventPtr)
 	EmbedWindowDeleted(winPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -358,12 +377,14 @@ TkWinEmbeddedEventProc(hwnd, message, wParam, lParam)
     LPARAM lParam;
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the Container structure associated with the parent window.
      */
 
-    for (containerPtr = firstContainerPtr;
+    for (containerPtr = tsdPtr->firstContainerPtr;
 	    containerPtr->parentHWnd != hwnd;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr == NULL) {
@@ -386,12 +407,12 @@ TkWinEmbeddedEventProc(hwnd, message, wParam, lParam)
 
 	break;
       case TK_GEOMETRYREQ:
-	EmbedGeometryRequest(containerPtr, wParam, lParam);
+	EmbedGeometryRequest(containerPtr, (int) wParam, lParam);
 	break;
     }
     return 1;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -420,7 +441,7 @@ EmbedGeometryRequest(containerPtr, width, height)
     int width, height;		/* Size that the child has requested. */
 {
     TkWindow * winPtr = containerPtr->parentPtr;
-    
+
     /*
      * Forward the requested size into our geometry management hierarchy
      * via the container window.  We need to send a Configure event back
@@ -441,7 +462,7 @@ EmbedGeometryRequest(containerPtr, width, height)
 	    0, 0, winPtr->changes.width, winPtr->changes.height, SWP_NOZORDER);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -482,7 +503,7 @@ ContainerEventProc(clientData, eventPtr)
 	EmbedWindowDeleted(containerPtr->parentPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -508,8 +529,10 @@ TkpGetOtherWindow(winPtr)
 				 * embedded window. */
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
+    for (containerPtr = tsdPtr->firstContainerPtr; containerPtr != NULL;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    return containerPtr->parentPtr;
@@ -520,7 +543,7 @@ TkpGetOtherWindow(winPtr)
     panic("TkpGetOtherWindow couldn't find window");
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -551,7 +574,7 @@ TkpClaimFocus(topLevelPtr, force)
     HWND hwnd = GetParent(Tk_GetHWND(topLevelPtr->window));
     SendMessage(hwnd, TK_CLAIMFOCUS, (WPARAM) force, 0);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -583,7 +606,7 @@ TkpRedirectKeyEvent(winPtr, eventPtr)
 {
     /* not implemented */
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -608,15 +631,19 @@ EmbedWindowDeleted(winPtr)
 				 * was deleted. */
 {
     Container *containerPtr, *prevPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the Container structure for this window work.  Delete the
      * information about the embedded application and free the container's
      * record.
+     * The main container may be null. [Bug #476176]
      */
 
     prevPtr = NULL;
-    containerPtr = firstContainerPtr;
+    containerPtr = tsdPtr->firstContainerPtr;
+    if (containerPtr == NULL) return;
     while (1) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    containerPtr->embeddedHWnd = NULL;
@@ -636,10 +663,11 @@ EmbedWindowDeleted(winPtr)
     if ((containerPtr->embeddedPtr == NULL)
 	    && (containerPtr->parentPtr == NULL)) {
 	if (prevPtr == NULL) {
-	    firstContainerPtr = containerPtr->nextPtr;
+	    tsdPtr->firstContainerPtr = containerPtr->nextPtr;
 	} else {
 	    prevPtr->nextPtr = containerPtr->nextPtr;
 	}
 	ckfree((char *) containerPtr);
     }
 }
+
