@@ -4,27 +4,22 @@
  *	Xlib emulation routines for Windows related to creating,
  *	displaying and destroying windows.
  *
- * Copyright (c) 1995 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinWindow.c,v 1.4 1998/09/14 18:24:01 stanton Exp $
+ * RCS: @(#) $Id: tkWinWindow.c,v 1.10 2002/06/14 22:25:12 jenglish Exp $
  */
 
 #include "tkWinInt.h"
 
-/*
- * The windowTable maps from HWND to Tk_Window handles.
- */
-
-static Tcl_HashTable windowTable;
-
-/*
- * Have statics in this module been initialized?
- */
-
-static int initialized = 0;
+typedef struct ThreadSpecificData {
+    int initialized;            /* 0 means table below needs initializing. */
+    Tcl_HashTable windowTable;  /* The windowTable maps from HWND to
+				 * Tk_Window handles. */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Forward declarations for procedures defined in this file:
@@ -32,9 +27,7 @@ static int initialized = 0;
 
 static void		NotifyVisibility _ANSI_ARGS_((XEvent *eventPtr,
 			    TkWindow *winPtr));
-static void 		StackWindow _ANSI_ARGS_((Window w, Window sibling,
-			    int stack_mode));
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -61,10 +54,12 @@ Tk_AttachHWND(tkwin, hwnd)
     int new;
     Tcl_HashEntry *entryPtr;
     TkWinDrawable *twdPtr = (TkWinDrawable *) Tk_WindowId(tkwin);
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    if (!initialized) {
-	Tcl_InitHashTable(&windowTable, TCL_ONE_WORD_KEYS);
-	initialized = 1;
+    if (!tsdPtr->initialized) {
+	Tcl_InitHashTable(&tsdPtr->windowTable, TCL_ONE_WORD_KEYS);
+	tsdPtr->initialized = 1;
     }
 
     /*
@@ -77,7 +72,7 @@ Tk_AttachHWND(tkwin, hwnd)
 	twdPtr->type = TWD_WINDOW;
 	twdPtr->window.winPtr = (TkWindow *) tkwin;
     } else if (twdPtr->window.handle != NULL) {
-	entryPtr = Tcl_FindHashEntry(&windowTable,
+	entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable,
 		(char *)twdPtr->window.handle);
 	Tcl_DeleteHashEntry(entryPtr);
     }
@@ -87,13 +82,13 @@ Tk_AttachHWND(tkwin, hwnd)
      */
 
     twdPtr->window.handle = hwnd;
-    entryPtr = Tcl_CreateHashEntry(&windowTable, (char *)hwnd, &new);
+    entryPtr = Tcl_CreateHashEntry(&tsdPtr->windowTable, (char *)hwnd, &new);
     Tcl_SetHashValue(entryPtr, (ClientData)tkwin);
 
 
     return (Window)twdPtr;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -116,17 +111,20 @@ Tk_HWNDToWindow(hwnd)
     HWND hwnd;
 {
     Tcl_HashEntry *entryPtr;
-    if (!initialized) {
-	Tcl_InitHashTable(&windowTable, TCL_ONE_WORD_KEYS);
-	initialized = 1;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+    if (!tsdPtr->initialized) {
+	Tcl_InitHashTable(&tsdPtr->windowTable, TCL_ONE_WORD_KEYS);
+	tsdPtr->initialized = 1;
     }
-    entryPtr = Tcl_FindHashEntry(&windowTable, (char*)hwnd);
+    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, (char*)hwnd);
     if (entryPtr != NULL) {
 	return (Tk_Window) Tcl_GetHashValue(entryPtr);
     }
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -155,7 +153,7 @@ Tk_GetHWND(window)
 	return GetDesktopWindow();
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -181,9 +179,13 @@ TkpPrintWindowId(buf, window)
     Window window;		/* Window to be printed into buffer. */
 {
     HWND hwnd = (window) ? Tk_GetHWND(window) : 0;
-    sprintf(buf, "0x%x", (unsigned int) hwnd);
+    /*
+     * Use pointer representation, because Win64 is P64 (*not* LP64).
+     * Windows doesn't print the 0x for %p, so we do it.
+     */
+    sprintf(buf, "0x%p", hwnd);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -196,7 +198,7 @@ TkpPrintWindowId(buf, window)
  *	The return value is normally TCL_OK;  in this case *idPtr
  *	will be set to the X Window id equivalent to string.  If
  *	string is improperly formed then TCL_ERROR is returned and
- *	an error message will be left in interp->result.  If the
+ *	an error message will be left in the interp's result.  If the
  *	number does not correspond to a Tk Window, then *idPtr will
  *	be set to None.
  *
@@ -209,16 +211,25 @@ TkpPrintWindowId(buf, window)
 int
 TkpScanWindowId(interp, string, idPtr)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
-    Arg string;			/* String containing a (possibly signed)
+    Tcl_Obj *string;		/* String containing a (possibly signed)
 				 * integer in a form acceptable to strtol. */
-    int *idPtr;			/* Place to store converted result. */
+    Window *idPtr;		/* Place to store converted result. */
 {
-    int number;
     Tk_Window tkwin;
+    Window number;
 
-    if (Tcl_GetInt(interp, string, &number) != TCL_OK) {
+    /*
+     * We want sscanf for the 64-bit check, but if that doesn't work,
+     * then Tcl_GetInt manages the error correctly.
+     */
+    if (
+#ifdef _WIN64
+	(sscanf(string, "0x%p", &number) != 1) &&
+#endif
+	Tcl_GetInt(interp, string, (int *)&number) != TCL_OK) {
 	return TCL_ERROR;
     }
+
     tkwin = Tk_HWNDToWindow((HWND)number);
     if (tkwin) {
 	*idPtr = Tk_WindowId(tkwin);
@@ -227,7 +238,7 @@ TkpScanWindowId(interp, string, idPtr)
     }
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -288,7 +299,7 @@ TkpMakeWindow(winPtr, parent)
 		    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
     return Tk_AttachHWND((Tk_Window)winPtr, hwnd);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -315,6 +326,8 @@ XDestroyWindow(display, w)
     TkWinDrawable *twdPtr = (TkWinDrawable *)w;
     TkWindow *winPtr = TkWinGetWinPtr(w);
     HWND hwnd = Tk_GetHWND(w);
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     display->request++;
 
@@ -325,7 +338,7 @@ XDestroyWindow(display, w)
 
     TkPointerDeadWindow(winPtr);
 
-    entryPtr = Tcl_FindHashEntry(&windowTable, (char*)hwnd);
+    entryPtr = Tcl_FindHashEntry(&tsdPtr->windowTable, (char*)hwnd);
     if (entryPtr != NULL) {
 	Tcl_DeleteHashEntry(entryPtr);
     }
@@ -341,7 +354,7 @@ XDestroyWindow(display, w)
 	DestroyWindow(hwnd);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -370,7 +383,7 @@ XMapWindow(display, w)
 
     display->request++;
 
-    ShowWindow(TkWinGetHWND(w), SW_SHOWNORMAL);
+    ShowWindow(Tk_GetHWND(w), SW_SHOWNORMAL);
     winPtr->flags |= TK_MAPPED;
 
     /*
@@ -379,13 +392,13 @@ XMapWindow(display, w)
      * its mapped children have just become visible.
      */
 
-    if (!(winPtr->flags & TK_TOP_LEVEL)) {
+    if (!(winPtr->flags & TK_TOP_HIERARCHY)) {
 	for (parentPtr = winPtr->parentPtr; ;
 	        parentPtr = parentPtr->parentPtr) {
 	    if ((parentPtr == NULL) || !(parentPtr->flags & TK_MAPPED)) {
 		return;
 	    }
-	    if (parentPtr->flags & TK_TOP_LEVEL) {
+	    if (parentPtr->flags & TK_TOP_HIERARCHY) {
 		break;
 	    }
 	}
@@ -413,7 +426,7 @@ XMapWindow(display, w)
     event.xvisibility.state = VisibilityUnobscured;
     NotifyVisibility(&event, winPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -450,7 +463,7 @@ NotifyVisibility(eventPtr, winPtr)
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -483,10 +496,10 @@ XUnmapWindow(display, w)
      * it will be cleared before XUnmapWindow is called.
      */
 
-    ShowWindow(TkWinGetHWND(w), SW_HIDE);
+    ShowWindow(Tk_GetHWND(w), SW_HIDE);
     winPtr->flags &= ~TK_MAPPED;
 
-    if (winPtr->flags & TK_TOP_LEVEL) {
+    if (winPtr->flags & TK_WIN_MANAGED) {
 	event.type = UnmapNotify;
 	event.xunmap.serial = display->request;
 	event.xunmap.send_event = False;
@@ -497,7 +510,7 @@ XUnmapWindow(display, w)
 	Tk_HandleEvent(&event);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -524,9 +537,9 @@ XMoveResizeWindow(display, w, x, y, width, height)
     unsigned int height;
 {
     display->request++;
-    MoveWindow(TkWinGetHWND(w), x, y, width, height, TRUE);
+    MoveWindow(Tk_GetHWND(w), x, y, width, height, TRUE);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -554,10 +567,10 @@ XMoveWindow(display, w, x, y)
 
     display->request++;
 
-    MoveWindow(TkWinGetHWND(w), x, y, winPtr->changes.width,
+    MoveWindow(Tk_GetHWND(w), x, y, winPtr->changes.width,
 	    winPtr->changes.height, TRUE);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -585,10 +598,10 @@ XResizeWindow(display, w, width, height)
 
     display->request++;
 
-    MoveWindow(TkWinGetHWND(w), winPtr->changes.x, winPtr->changes.y, width,
+    MoveWindow(Tk_GetHWND(w), winPtr->changes.x, winPtr->changes.y, width,
 	    height, TRUE);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -610,10 +623,10 @@ XRaiseWindow(display, w)
     Display* display;
     Window w;
 {
-    HWND window = TkWinGetHWND(w);
-    TkWinSetWindowPos(window, NULL, Above); 
+    HWND window = Tk_GetHWND(w);
+    TkWinSetWindowPos(window, NULL, Above);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -634,10 +647,10 @@ XLowerWindow(display, w)
     Display* display;
     Window w;
 {
-    HWND window = TkWinGetHWND(w);
-    TkWinSetWindowPos(window, NULL, Below); 
+    HWND window = Tk_GetHWND(w);
+    TkWinSetWindowPos(window, NULL, Below);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -665,7 +678,7 @@ XConfigureWindow(display, w, value_mask, values)
     XWindowChanges* values;
 {
     TkWindow *winPtr = TkWinGetWinPtr(w);
-    HWND hwnd = TkWinGetHWND(w);
+    HWND hwnd = Tk_GetHWND(w);
 
     display->request++;
 
@@ -692,7 +705,7 @@ XConfigureWindow(display, w, value_mask, values)
 	TkWinSetWindowPos(hwnd, sibling, values->stack_mode);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -718,7 +731,7 @@ XClearWindow(display, w)
     HBRUSH brush;
     HPALETTE oldPalette, palette;
     TkWindow *winPtr;
-    HWND hwnd = TkWinGetHWND(w);
+    HWND hwnd = Tk_GetHWND(w);
     HDC dc = GetDC(hwnd);
 
     palette = TkWinGetPalette(display->screens[0].cmap);
@@ -738,7 +751,7 @@ XClearWindow(display, w)
     SelectPalette(dc, oldPalette, TRUE);
     ReleaseDC(hwnd, dc);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -768,7 +781,7 @@ XChangeWindowAttributes(display, w, valueMask, attributes)
 	XDefineCursor(display, w, attributes->cursor);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -814,7 +827,7 @@ TkWinSetWindowPos(hwnd, siblingHwnd, pos)
     SetWindowPos(hwnd, siblingHwnd, 0, 0, 0, 0,
 	    SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -862,3 +875,4 @@ XGetWindowAttributes(display, w, attrib)
 }
 
 
+
