@@ -1,4 +1,4 @@
-/* 
+/*
  * ptkCanvGroup.c --
  *
  *	This file implements grid items for canvas
@@ -35,7 +35,7 @@ typedef struct GroupItem  {
 
 /*
  * Information used for parsing configuration specs:
- */       
+ */
 
 static int		MembersParseProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, Tk_Window tkwin,
@@ -46,8 +46,8 @@ static Arg		MembersPrintProc _ANSI_ARGS_((ClientData clientData,
 
 
 static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
+    TkStateParseProc,
+    TkStatePrintProc, (ClientData) 3
 };
 static Tk_CustomOption tagsOption = {
     Tk_CanvasTagsParseProc,
@@ -83,10 +83,10 @@ static void		ComputeGroupBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    GroupItem *groupPtr));
 static int		ConfigureGroup _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    Arg *args, int flags));
+			    Tcl_Obj *CONST *args, int flags));
 static int		CreateGroup _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, Arg *args));
+			    int argc, Tcl_Obj *CONST *args));
 static void		DeleteGroup _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, Display *display));
 static void		DisplayGroup _ANSI_ARGS_((Tk_Canvas canvas,
@@ -94,7 +94,7 @@ static void		DisplayGroup _ANSI_ARGS_((Tk_Canvas canvas,
 			    int x, int y, int width, int height));
 static int		GroupCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    Arg *args));
+			    Tcl_Obj *CONST *args));
 static int		GroupToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
 static int		GroupToArea _ANSI_ARGS_((Tk_Canvas canvas,
@@ -108,7 +108,7 @@ static void		TranslateGroup _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double deltaX, double deltaY));
 static int		GroupIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, Tcl_Obj *indexString,
-			    int *indexPtr));         
+			    int *indexPtr));
 static int		GroupInsert _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, int beforeThis, Tcl_Obj *string));
 static void		GroupInsertProc _ANSI_ARGS_((Tk_Canvas canvas,
@@ -137,15 +137,15 @@ Tk_ItemType ptkCanvGroupType = {
     ScaleGroup,				/* scaleProc */
     TranslateGroup,			/* translateProc */
     GroupIndex,				/* indexProc */
-    (Tk_ItemCursorProc *) NULL,		/* icursorProc */  /* Abuse to set active? */   
+    (Tk_ItemCursorProc *) NULL,		/* icursorProc */  /* Abuse to set active? */
     (Tk_ItemSelectionProc *) NULL,	/* selectionProc */
     GroupInsertProc,			/* insertProc */
     GroupDChars,			/* dTextProc */
     (Tk_ItemType *) NULL,		/* nextPtr */
     (Tk_ItemBboxProc *) ComputeGroupBbox,/* bboxProc */
     Tk_Offset(Tk_VisitorType, visitGroup), /* acceptProc */
-    (Tk_ItemGetCoordProc *) NULL,	/* getCoordPtr */
-    (Tk_ItemSetCoordProc *) NULL	/* setCoordPtr */
+    NULL,	/* getCoordPtr */
+    NULL	/* setCoordPtr */
 };
 
 
@@ -171,7 +171,7 @@ ShowMembers(char *f,GroupItem *groupPtr)
 }
 
 
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -199,11 +199,11 @@ CreateGroup(interp, canvas, itemPtr, argc, args)
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
     int argc;				/* Number of arguments in args. */
-    Arg *args;				/* Arguments describing group. */
+    Tcl_Obj *CONST *args;		/* Arguments describing group. */
 {
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     int i;
-                                                     
+
     if (argc==1) {
 	i = 1;
     } else {
@@ -251,7 +251,7 @@ CreateGroup(interp, canvas, itemPtr, argc, args)
     DeleteGroup(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -278,7 +278,7 @@ GroupCoords(interp, canvas, itemPtr, argc, args)
 					 * read or modified. */
     int argc;				/* Number of coordinates supplied in
 					 * args. */
-    Arg *args;				/* Array of coordinates: x1, y1,
+    Tcl_Obj *CONST *args;		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     GroupItem *groupPtr = (GroupItem *) itemPtr;
@@ -295,7 +295,7 @@ GroupCoords(interp, canvas, itemPtr, argc, args)
 	double newX;
 	double newY;
  	if (argc==1) {
-	    if (Tcl_ListObjGetElements(interp, args[0], &argc, &args) != TCL_OK) {
+	    if (Tcl_ListObjGetElements(interp, args[0], &argc, (Tcl_Obj ***)&args) != TCL_OK) {
 		return TCL_ERROR;
 	    } else if (argc != 2) {
 		sprintf(c0,"%d",argc);
@@ -319,7 +319,7 @@ GroupCoords(interp, canvas, itemPtr, argc, args)
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -346,14 +346,14 @@ ConfigureGroup(interp, canvas, itemPtr, argc, args, flags)
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Rectangle item to reconfigure. */
     int argc;			/* Number of elements in args.  */
-    Arg *args;		/* Arguments describing things to configure. */
+    Tcl_Obj *CONST *args;	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;       
+    int i;
     Tk_Window tkwin = Tk_CanvasTkwin(canvas);
 
     if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, args,
@@ -372,7 +372,7 @@ ConfigureGroup(interp, canvas, itemPtr, argc, args, flags)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -401,24 +401,24 @@ DeleteGroup(canvas, itemPtr, display)
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;       
+    int i;
 
     canvasPtr->activeGroup = itemPtr;
     for (i=groupPtr->numMembers-1; i >= 0; i--) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	TkGroupRemoveItem(subitemPtr);  
+	TkGroupRemoveItem(subitemPtr);
 	
 #ifdef DELETE_GROUP_DELETES_MEMBERS
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    (*subitemPtr->typePtr->deleteProc)(canvas, subitemPtr, display);
 	}
 #endif
     }
     canvasPtr->activeGroup = saveGroup;
     if (groupPtr->members) {
-	ckfree((char *) groupPtr->members);      
+	ckfree((char *) groupPtr->members);
     }
-}              
+}
 
 
 void
@@ -432,20 +432,20 @@ Tk_Item *itemPtr;
 	    if (groupPtr->members[i] == itemPtr) {
 		int j;
 		for (j=i+1; j < groupPtr->numMembers; j++) {
-		    groupPtr->members[j-1] = groupPtr->members[j];   
+		    groupPtr->members[j-1] = groupPtr->members[j];
 		}
 		itemPtr->redraw_flags |= FORCE_REDRAW;
 		groupPtr->numMembers--;
 		itemPtr->group = NULL;
 		return;
-	    }     
+	    }
 	}
     }
   itemPtr->group = NULL;
   LangDebug("Cannot find %d in %d\n",itemPtr->id, groupPtr->header.id);
 }
 
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -476,38 +476,36 @@ ComputeGroupBbox(canvas, groupPtr)
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, &groupPtr->header);
     int seen = 0;
-    int i;       
+    int i;
 
     canvasPtr->activeGroup = &groupPtr->header;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    if (Tk_GetItemState(canvas, subitemPtr) == TK_STATE_HIDDEN) {
 		continue;
 	    }
-	    /* FIXME - do we really need to call bboxProc ? */
-	    (*subitemPtr->typePtr->bboxProc)(canvas, subitemPtr);
 	    if (seen++ == 0) {
 		groupPtr->header.x1 = subitemPtr->x1;
 		groupPtr->header.y1 = subitemPtr->y1;
 		groupPtr->header.x2 = subitemPtr->x2;
 		groupPtr->header.y2 = subitemPtr->y2;
 	    } else {
-		if (subitemPtr->x1 < groupPtr->header.x1) { 
+		if (subitemPtr->x1 < groupPtr->header.x1) {
 		     groupPtr->header.x1 = subitemPtr->x1;
 		}
-		if (subitemPtr->y1 < groupPtr->header.y1) { 
+		if (subitemPtr->y1 < groupPtr->header.y1) {
 		     groupPtr->header.y1 = subitemPtr->y1;
 		}
-		if (subitemPtr->x2 > groupPtr->header.x2) { 
+		if (subitemPtr->x2 > groupPtr->header.x2) {
 		     groupPtr->header.x2 = subitemPtr->x2;
 		}
-		if (subitemPtr->y2 > groupPtr->header.y2) { 
+		if (subitemPtr->y2 > groupPtr->header.y2) {
 		     groupPtr->header.y2 = subitemPtr->y2;
 		}
 	    }
 	}
-    }          
+    }
     canvasPtr->activeGroup = saveGroup;
 
     /* If all items were hidden then have a "null" bbox */
@@ -519,7 +517,7 @@ ComputeGroupBbox(canvas, groupPtr)
 	groupPtr->header.y2 = groupPtr->header.y1;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -552,27 +550,35 @@ DisplayGroup(canvas, itemPtr, display, drawable, x, y, width, height)
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;       
+    int i;
     if (state == TK_STATE_HIDDEN) {
 	return;
     }
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    if (Tk_GetItemState(canvas, subitemPtr) == TK_STATE_HIDDEN) {
 		continue;
 	    }
-	    if (drawable != None || 
-		(subitemPtr->typePtr->flags & TK_ITEM_ALWAYS_REDRAW)) {
-		(*subitemPtr->typePtr->displayProc)(canvas, subitemPtr, display, 
+	    if (drawable != None ||
+		(subitemPtr->typePtr->alwaysRedraw & 1)) {
+		if (subitemPtr->updateCmd) {
+		    if (canvasPtr->updateCmds == NULL) {
+			canvasPtr->updateCmds = Tcl_NewListObj(0,NULL);
+		    }
+		    Tcl_IncrRefCount(subitemPtr->updateCmd);
+		    Tcl_ListObjAppendElement(canvasPtr->interp,canvasPtr->updateCmds,
+				subitemPtr->updateCmd);
+		}
+		(*subitemPtr->typePtr->displayProc)(canvas, subitemPtr, display,
 		    drawable, x, y, width, height);
 	    }
 	}
     }
     canvasPtr->activeGroup = saveGroup;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -585,7 +591,7 @@ DisplayGroup(canvas, itemPtr, display, drawable, x, y, width, height)
  *	The return value is 0 if the point whose x and y coordinates
  *	are coordPtr[0] and coordPtr[1] is inside the group.  If the
  *	point isn't inside the rectangle then the return value is the
- *	distance from the point to the group. 
+ *	distance from the point to the group.
  *
  * Side effects:
  *	None.
@@ -604,7 +610,7 @@ GroupToPoint(canvas, itemPtr, pointPtr)
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;       
+    int i;
 
     double best = 1.0e36;
 
@@ -612,10 +618,15 @@ GroupToPoint(canvas, itemPtr, pointPtr)
 	return best;
     }
 
+    /* If the group is active it is invisible to picking */
+    if (state == TK_STATE_ACTIVE) {
+	return best;
+    }
+
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    double try = (*subitemPtr->typePtr->pointProc)(canvas, subitemPtr, pointPtr);
 	    if (try < best) {
 		best = try;
@@ -628,8 +639,8 @@ GroupToPoint(canvas, itemPtr, pointPtr)
     canvasPtr->activeGroup = saveGroup;
     return best;
 }
-
-
+
+
 /*
  *--------------------------------------------------------------
  *
@@ -663,19 +674,24 @@ GroupToArea(canvas, itemPtr, areaPtr)
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;                                            
+    int i;
 #define ALL_OUTSIDE 1
 #define ALL_INSIDE  2
-    int seen    = ALL_INSIDE|ALL_OUTSIDE;          
+    int seen    = ALL_INSIDE|ALL_OUTSIDE;
 
     if (state == TK_STATE_HIDDEN) {
 	return -1;
     }
 
+    /* If the group is active it is invisible to picking */
+    if (state == TK_STATE_ACTIVE) {
+	return -1;
+    }
+
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    int inner =  (*subitemPtr->typePtr->areaProc)(canvas, subitemPtr, areaPtr);
 	    if (inner < 0)   /* outside */
 		seen &= ~ALL_INSIDE;  /* clear the inside option */
@@ -690,17 +706,17 @@ GroupToArea(canvas, itemPtr, areaPtr)
     canvasPtr->activeGroup = saveGroup;
 
     switch (seen) {
-      case 0 : 
+      case 0 :
        return 0;
-      case ALL_INSIDE  : 
+      case ALL_INSIDE  :
        return 1;
       default:
-      case ALL_OUTSIDE : 
+      case ALL_OUTSIDE :
        return -1;
     }
 }
-
-
+
+
 /*
  *--------------------------------------------------------------
  *
@@ -734,7 +750,7 @@ ScaleGroup(canvas, itemPtr, originX, originY, scaleX, scaleY)
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;       
+    int i;
 
     groupPtr->posn[0] = originX + scaleX*(groupPtr->posn[0] - originX);
     groupPtr->posn[1] = originY + scaleY*(groupPtr->posn[1] - originY);
@@ -742,7 +758,7 @@ ScaleGroup(canvas, itemPtr, originX, originY, scaleX, scaleY)
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    (*subitemPtr->typePtr->scaleProc)(canvas, subitemPtr, originX, originY, scaleX, scaleY);
 	}
     }
@@ -750,7 +766,7 @@ ScaleGroup(canvas, itemPtr, originX, originY, scaleX, scaleY)
 
     ComputeGroupBbox(canvas, groupPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -780,20 +796,20 @@ TranslateGroup(canvas, itemPtr, deltaX, deltaY)
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
-    int i;       
+    int i;
     groupPtr->posn[0] += deltaX;
     groupPtr->posn[1] += deltaY;
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
+	if (subitemPtr != NULL) {
 	    (*subitemPtr->typePtr->translateProc)(canvas, subitemPtr, deltaX, deltaY);
 	}
     }
     canvasPtr->activeGroup = saveGroup;
     ComputeGroupBbox(canvas, groupPtr);
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -830,12 +846,20 @@ GroupToPostscript(interp, canvas, itemPtr, prepass)
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
     int code = TCL_OK;
-    int i;       
+    int i;
+
+    if (state == TK_STATE_HIDDEN) {
+	return code;
+    }
+
     canvasPtr->activeGroup = itemPtr;
     for (i=0; i < groupPtr->numMembers; i++) {
 	Tk_Item *subitemPtr = groupPtr->members[i];
-	if (subitemPtr != NULL) {            
-	    code = (*subitemPtr->typePtr->postscriptProc)(interp, canvas, 
+	if (subitemPtr != NULL) {
+	    if (Tk_GetItemState(canvas, subitemPtr) == TK_STATE_HIDDEN) {
+		continue;
+	    }
+	    code = (*subitemPtr->typePtr->postscriptProc)(interp, canvas,
 			subitemPtr, prepass);
 	    if (code != TCL_OK) {
 		break;
@@ -858,7 +882,7 @@ int *indexPtr;
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i;  
+    int i;
     int length;
     int id;
     char *string;
@@ -873,7 +897,7 @@ int *indexPtr;
 	&& Tk_CanvasGetCoordFromObj(interp, canvas, objv[0], &point[0]) == TCL_OK
 	&& Tk_CanvasGetCoordFromObj(interp, canvas, objv[1], &point[1]) == TCL_OK) {
 	goto doxy;
-    } 
+    }
 
     string = Tcl_GetStringFromObj(obj, &length);
     if (string[0] == 'e') {
@@ -902,7 +926,7 @@ int *indexPtr;
 	point[1] = strtod(p, &end);
 	if ((end == p) || (*end != 0)) {
 	    goto badIndex;
-	}                 
+	}
      doxy:
 	bestDist = 1.0e36;
 	*indexPtr = 0;
@@ -931,7 +955,7 @@ int *indexPtr;
 	}
     }
     return TCL_OK;
-}         
+}
 
 static int
 GroupInsert(canvas, itemPtr, beforeThis, string)
@@ -939,7 +963,7 @@ Tk_Canvas canvas;
 Tk_Item *itemPtr;
 int beforeThis;
 Tcl_Obj *string;
-{  
+{
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
@@ -947,7 +971,7 @@ Tcl_Obj *string;
     Tcl_Obj **objv;
     int argc;
     int i;
-    int id;                            
+    int id;
     if (Tcl_ListObjGetElements(groupPtr->interp,string,&argc,&objv) == TCL_OK) {
 	int count = 0;
 	for (i=0; i < argc; i++) {
@@ -955,15 +979,15 @@ Tcl_Obj *string;
 		Tcl_HashEntry *entryPtr = Tcl_FindHashEntry(&canvasPtr->idTable, (char *) id);
 		if (entryPtr != NULL) {
 		    Tk_Item *subitemPtr = (Tk_Item *) Tcl_GetHashValue(entryPtr);
-		    if (subitemPtr == NULL 
+		    if (subitemPtr == NULL
 			|| subitemPtr == itemPtr
 			|| subitemPtr->group == itemPtr) {
 			continue;
 		    }
 		    if (subitemPtr->group != NULL) {
 			TkGroupRemoveItem(subitemPtr);
-		    }       
-		    count++;                        
+		    }
+		    count++;
 		}
 	    } else {
 		return TCL_ERROR;
@@ -974,7 +998,7 @@ Tcl_Obj *string;
 	    if (groupPtr->members == NULL) {
 		groupPtr->members = (Tk_Item **)ckalloc(i*sizeof(Tk_Item *));
 	    } else {
-		groupPtr->members = (Tk_Item **)ckrealloc((char *)groupPtr->members, 
+		groupPtr->members = (Tk_Item **)ckrealloc((char *)groupPtr->members,
 					i*sizeof(Tk_Item *));
 	    }
 	    if (groupPtr->members != NULL) {
@@ -984,12 +1008,12 @@ Tcl_Obj *string;
 		groupPtr->numSlots   = 0;
 		Tcl_SetResult(groupPtr->interp,"Out of memory",TCL_STATIC);
 		return TCL_ERROR;
-	    }   
+	    }
 	}
 	/* Move tail up */
 	for (i=groupPtr->numMembers-1; i >= beforeThis; i--) {
 		groupPtr->members[i+count] = groupPtr->members[i];
-	}                                 
+	}
 	/* Fill in slots */
 	groupPtr->numMembers += count;
 	for (i=0; i < argc; i++) {
@@ -998,7 +1022,7 @@ Tcl_Obj *string;
 		Tcl_HashEntry *entryPtr = Tcl_FindHashEntry(&canvasPtr->idTable, (char *) id);
 		if (entryPtr != NULL) {
 		    Tk_Item *subitemPtr = (Tk_Item *) Tcl_GetHashValue(entryPtr);
-		    if (subitemPtr == NULL 
+		    if (subitemPtr == NULL
 			|| subitemPtr == itemPtr
 			|| subitemPtr->group == itemPtr) {
 			continue;
@@ -1008,9 +1032,9 @@ Tcl_Obj *string;
 		    groupPtr->members[beforeThis] = subitemPtr;
 		    beforeThis++;
 		    count--;
-		} 
-	    } 
-	} 
+		}
+	    }
+	}
 	if (count != 0) {
 	   abort();
 	}
@@ -1044,17 +1068,17 @@ int last;
     GroupItem *groupPtr = (GroupItem *) itemPtr;
     Tk_Item *saveGroup  = canvasPtr->activeGroup;
     Tk_State state = Tk_GetItemState(canvas, itemPtr);
-    int i; 
+    int i;
 
     if (first < 0) {
 	first = 0;
-    }           
+    }
     if (last >=  groupPtr->numMembers) {
 	last = groupPtr->numMembers-1;
     }
     if (first > last) {
 	return;
-    }         
+    }
     for (i=last; i >= first; i--) {
 	TkGroupRemoveItem(groupPtr->members[i]);
     }
@@ -1064,31 +1088,31 @@ int last;
 static int
 MembersParseProc(clientData,interp,tkwin,value,recordPtr,offset)
 ClientData clientData;
-Tcl_Interp *interp; 
+Tcl_Interp *interp;
 Tk_Window tkwin;
-Arg value; 
-char *recordPtr; 
+Arg value;
+char *recordPtr;
 int offset;
-{          
-    Tk_Item *itemPtr    = (Tk_Item *) recordPtr;   
-    GroupItem *groupPtr = (GroupItem *) itemPtr;   
+{
+    Tk_Item *itemPtr    = (Tk_Item *) recordPtr;
+    GroupItem *groupPtr = (GroupItem *) itemPtr;
     int code = TCL_OK;
     Tk_CanvasEventuallyRedraw(groupPtr->canvas, itemPtr->x1, itemPtr->y1, itemPtr->x2, itemPtr->y2);
-    GroupDChars(groupPtr->canvas, itemPtr, 0, groupPtr->numMembers-1); 
+    GroupDChars(groupPtr->canvas, itemPtr, 0, groupPtr->numMembers-1);
     code = GroupInsert(groupPtr->canvas, itemPtr, 0, value);
     Tk_CanvasEventuallyRedraw(groupPtr->canvas, itemPtr->x1, itemPtr->y1, itemPtr->x2, itemPtr->y2);
     return code;
 }
 
 static Arg
-MembersPrintProc(clientData,tkwin,recordPtr,offset,freeProcPtr) 
+MembersPrintProc(clientData,tkwin,recordPtr,offset,freeProcPtr)
 ClientData clientData;
 Tk_Window tkwin;
 char *recordPtr;
 int offset;
 Tcl_FreeProc **freeProcPtr;
 {
-    GroupItem *groupPtr = (GroupItem *) recordPtr;   
+    GroupItem *groupPtr = (GroupItem *) recordPtr;
     Tcl_Obj *result = Tcl_NewListObj(0,NULL);
     int i;
     for (i=0; i < groupPtr->numMembers; i++) {
